; $Id: dcs_cryst_align.pro,v 1.15 2003/10/10 14:11:40 qiuym Exp $
; This program plots crystal alignment files
; Yiming Qiu (Yiming.Qiu@nist.gov) 4/4/2003

;circle symbol
pro dcs_cryst_circle,radius=radius,fill=fill,thick=thick
    if n_elements(radius) eq 0 then radius=1
    if n_elements(fill) eq 0 then fill=0
    if n_elements(thick) eq 0 then thick=1
    x = findgen(16)*(!pi*2/15)
    usersym,radius*cos(x),radius*sin(x),fill=fill,thick=thick
end

;plotting
pro dcs_cryst_align_Plot,pState,chars=chars,thick=thick
    xtit = ['Rotation (!6!Eo!n)','Height (mm)','X-translation (mm)','Y-translation (mm)','X-tilt (!6!Eo!n)','Y-tilt (!6!Eo!n)',$
            'File Index']
    xtit = xtit[(*pState).xaxis]
    ytit = 'Neutron Counts/'+strtrim(string((*pState).duration,format='(i5)'),2)+' Secs'
    n_file = n_elements((*(*pState).yPtr)[0,*])
    y    = fltarr(n_file) & dy=y
    for i=0,n_file-1 do begin
       y[i]  =total((*(*pState).yPtr)[*(*pState).detPtr,i])
       dy[i] =sqrt(total((*(*pState).dyPtr)[*(*pState).detPtr,i]^2))
    endfor
    xmax=max((*(*pState).xPtr)[(*pState).xaxis,*],min=xmin)
    ymax=max(y+dy,imax) & imax=imax[0]
    ymin=min(y-dy)
    if total(finite((*pState).xran)) ne 2 then begin
       (*pState).xran=[xmin,xmax]+[-0.05,0.05]*(xmax-xmin+0.01)
       (*pState).yran=[ymin,ymax]+[-0.05,0.05]*(ymax-ymin)
    endif
    if n_elements(thick) eq 0 then thick   = 1.5
    if n_elements(chars) eq 0 then chars   = 1.2
    dcs_cryst_circle,thick=thick
    ;adjust xmargin according to the ticktext
    otmp=obj_new('idlgrAxis',1,/exact,range=(*pState).yran)
    otmp->getproperty,ticktext=otick
    otick->getproperty,strings=ticktext
    max_len=max(dm_strlen(ticktext))
    obj_destroy,otmp
    xmargin    =  [max_len+6,3]
    plot,(*(*pState).xPtr)[(*pState).xaxis,*],y,xrange=(*pState).xran,yrange=(*pState).yran,/xst,/yst,$
       title='!6',xtitle=xtit,ytitle=ytit,linestyle=0,thick=thick,psym=8,xthick=thick,ythick=thick,xminor=2,yminor=2,$
       chars=chars,xmargin=xmargin
    ;the following two lines are for multiple dcs_cryst_align running at the same time
    *(*pState).xaxisPtr=!x
    *(*pState).yaxisPtr=!y
    if float(!version.release) ge 5.6 then $
       errplot,(*(*pState).xPtr)[(*pState).xaxis,*],y-dy,y+dy,thick=thick $
    else $
       errplot,(*(*pState).xPtr)[(*pState).xaxis,*],y-dy,y+dy
    index  =    uniq((*(*pState).xPtr)[(*pState).xaxis,*],sort((*(*pState).xPtr)[(*pState).xaxis,*]))
    outs   = (*pState).det_tit
    if ((*pState).fit eq 1) and (n_elements(index) ne 1) and (n_elements(y) gt 4)then begin
       fwhm  =    total(y-ymin)/(ymax-ymin)*$
              abs((*(*pState).xPtr)[(*pState).xaxis,1]-(*(*pState).xPtr)[(*pState).xaxis,0])
       estimates   = [ymax,(*(*pState).xPtr)[(*pState).xaxis,imax],fwhm/sqrt(8.*alog(2)),ymin]
       estimates[2]= estimates[2]/2.
       result=  gaussfit((*(*pState).xPtr)[(*pState).xaxis,*],y,params,estimates=estimates,nterms=n_elements(estimates))
       xx    =  xmin+findgen(100)/99*(xmax-xmin)
       yy    =  params[0]*exp(-0.5*((xx-params[1])/params[2])^2)+params[3]
       oplot,xx,yy,lines=0,thick=thick
       xyouts,(*pState).xran[0]+((*pState).xran[1]-(*pState).xran[0])*0.05,$
            (*pState).yran[0]+((*pState).yran[1]-(*pState).yran[0])*0.9,$
         'peak!l !n='+strtrim(string(params[1],format='(f8.2)'),2),chars=chars
       xyouts,(*pState).xran[0]+((*pState).xran[1]-(*pState).xran[0])*0.05,$
            (*pState).yran[0]+((*pState).yran[1]-(*pState).yran[0])*0.8,$
         'fwhm='+strtrim(string(params[2]*sqrt(8.*alog(2)),format='(f9.3)'),2),chars=chars
    endif
    xyouts,0.6,0.95,outs,/norm,chars=chars/1.2,alignment=0.5
end



pro dcs_cryst_align_zoom,event
    widget_control,event.top,get_uvalue = pState
    if n_elements(*(*pState).xPtr) eq 0 then return
    device,decomposed=1
    case event.type of
       0:    begin   ; button press
             (*pState).mouse = event.press
             if (*pState).mouse eq 1 then begin
                (*pState).xbox[*] = event.x
                (*pState).ybox[*] = event.y
                wset,(*pState).winVis
                device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                empty
                widget_control,(*pState).plotDraw,/draw_motion_events
             endif
         end
       1:    begin  ; button release
             widget_control,(*pState).plotDraw,draw_motion_events = 0
             if (*pState).mouse eq 1 then begin
                xll = (*pState).xbox[0] < (*pState).xbox[1]
                yll = (*pState).ybox[0] < (*pState).ybox[1]
                w = abs((*pState).xbox[1] - (*pState).xbox[0])
                h = abs((*pState).ybox[1] - (*pState).ybox[0])
                if (w eq 0) or (h eq 0) then break
                xur = xll + w
                yur = yll + h
                ;the following two lines are for multiple dcs_cryst_align running at the same time
                !x=*(*pState).xaxisPtr
                !y=*(*pState).yaxisPtr
                ll = convert_coord(xll,yll,/device,/to_data)
                ur = convert_coord(xur,yur,/device,/to_data)
                if (ll[0] eq ur[0]) or (ll[1] eq ur[1]) then break
                (*pState).xran = [ll[0],ur[0]]
                (*pState).yran = [ll[1],ur[1]]
                wset,(*pState).winPix
                dcs_cryst_align_plot,pState
                wset,(*pState).winVis
                device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                (*pState).mouse = 0B
             endif
             if (*pState).mouse eq 4 then begin
                (*pState).xran[*]=!values.f_nan
                (*pState).yran[*]=!values.f_nan
                wset,(*pState).winPix
                dcs_cryst_align_plot,pState
                wset,(*pState).winVis
                device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                (*pState).mouse = 0B
             endif
         end
       2:    begin  ; mouse motion
              if (*pState).mouse eq 1 then begin
                (*pState).xbox[1] = event.x
                (*pState).ybox[1] = event.y
                xc = [(*pState).xbox[0],event.x,event.x,(*pState).xbox[0],(*pState).xbox[0]]
                yc = [(*pState).ybox[0],(*pState).ybox[0],event.y,event.y,(*pState).ybox[0]]
                wset,(*pState).winVis
                device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                plots,xc,yc,/device,color=256L*255
                empty
              endif
         end
       else:
    endcase
    device,decomposed=(*pState).decompose
end

pro dcs_cryst_align_Exit,tlb
    widget_control,tlb,get_uvalue=pState,/no_copy
    tvlct,(*(*pState).rgb).r,(*(*pState).rgb).g,(*(*pState).rgb).b ;restore entry color map
    device,decomposed=(*pState).decompose  ;restore entry decompose info
    obj_destroy,(*pState).filesel
    cd,(*pState).root
    ptr_free,(*pState).detPtr,(*pState).xPtr,(*pState).yPtr,(*pState).dyPtr,(*pState).rgb,(*pState).xaxisPtr,(*pState).yaxisPtr
    wdelete,(*pState).winPix
    ptr_free,pState
    widget_control,tlb,/destroy
end



pro dcs_cryst_align_event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    WIDGET_CONTROL,/HOURGLASS
    widget_control,event.handler,get_uvalue=pState,/no_copy
    device,decomposed=1
    widget_type=widget_info(event.id,/type)
    case widget_type of
       8: tmp_value  =widget_info(event.id,/droplist_select)  ;droplist
       12:tmp_value  =event.index                  ;combobox
       else: tmp_value=0
    endcase

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok=dialog_message(dialog_parent=(*pState).tlb,!error_state.msg,/error)
       catch,/cancel
       widget_control,event.handler,set_uvalue=pState,/no_copy
       return
    end

    case (event.id) of
       (*pState).tlb:    widget_control,event.id,scr_xsize=(*pState).geom[0],scr_ysize=(*pState).geom[1]
       (*pState).loadBut: begin
          (*pState).fit=0
          (*pState).filesel->getproperty,path=open_path,file=open_file
          if n_elements(open_file) eq 0 then begin
              ok=dialog_message('Please select DCS files first.',/error,dialog_parent=event.handler)
              break
          endif
          dm_load_dcsdata,open_path,open_file,ang=x,qty=y,dqty=dy,t_chan=(*pState).t_chan,parent=(*pState).tlb,$
              duration=duration,/raw
          *(*pState).xPtr     =temporary(x)
          *(*pState).yPtr     =temporary(y)
          *(*pState).dyPtr    =temporary(dy)
          (*pState).duration  =fix(duration,type=3)
          (*pState).xran[*]   =!values.f_nan
          (*pState).yran[*]   =!values.f_nan
          wset,(*pState).winPix
          dcs_cryst_align_plot,pState
          wset,(*pState).winVis
          device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
         end
       (*pState).fitBut:  begin
          if n_elements(*(*pState).yPtr) le 4 then break
          (*pState).fit=1
          (*pState).xran[*]=!values.f_nan
          (*pState).yran[*]=!values.f_nan
          wset,(*pState).winPix
          dcs_cryst_align_plot,pState
          wset,(*pState).winVis
          device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
         end
       (*pState).saveBut: begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          mydevice=!d.name
          set_plot,'ps',/copy
          deviceKeywords=PSConfig(cancel=canceled,group_leader=(*pState).tlb,xsize=7,ysize=5,xoffset=0.75,/inch,$
              directory=(*pState).workDir)
          if canceled ne 1 then begin
              deviceKeywords.color=0 ;black and white
              device,_extra=deviceKeywords
              dcs_cryst_align_plot,pState,chars=1.6
              device,/close_file
          endif
          set_plot,mydevice
         end
       (*pState).printBut: begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          ok=dialog_printersetup(dialog_parent=(*pState).tlb)
          if not ok then break
          mydevice=!d.name
          set_plot,'printer'
          device,xsize=7,ysize=5,xoffset=0.75,/inch
          dcs_cryst_align_plot,pState,chars=1.6
          device,/close_document
          set_plot,mydevice
         end
       (*pState).exitBut: begin
          widget_control,event.handler,set_uvalue=pState,/no_copy
          widget_control,event.handler,/destroy
          return
         end
       (*pState).detList: begin
          old =   *(*pState).detPtr
          case tmp_value of
              0: begin
                 *(*pState).detPtr =indgen((*pState).ndet)
                 (*pState).det_tit ='!6det=all'
                 end
              1: begin
                 n_index   =n_elements(*(*pState).detPtr)
                 if n_index eq (*pState).ndet then default='1:'+strtrim(string((*pState).ndet),2) $
                 else default=string(*(*pState).detPtr+1,format='('+string(n_index)+'(i3,","))')
                 if strmid(default,0,/reverse_offset) eq ',' then default=strmid(default,0,strlen(default)-1)
                 tmp=dm_dialog_input(dialog_parent=event.handler,'Det num:',default=default,title='Specifying detectors',$
                   info='Detector number starts from 1. eg:371,374 or 370:374 c',xsize=200)
                 (*pState).det_tit ='!6det='+tmp
                 if strlen(tmp) eq 0 then det_tmp=-1 $
                 else begin
                   status   =''
                   index    =strpos(tmp,'c',/reverse_search)
                   if index[0] ne -1 then status='c'
                   index    =strpos(tmp,'C',/reverse_search)
                   if index[0] ne -1 then status='c'
                   index    =strpos(tmp,'u',/reverse_search)
                   if index[0] ne -1 then status='u'
                   index    =strpos(tmp,'U',/reverse_search)
                   if index[0] ne -1 then status='u'
                   index    =strpos(tmp,'l',/reverse_search)
                   if index[0] ne -1 then status='l'
                   index    =strpos(tmp,'L',/reverse_search)
                   if index[0] ne -1 then status='l'

                   ind0 =strsplit(tmp,'[a-zA-Z '+string(9b)+',:-]',/regex)
                   ind1 =strsplit(tmp,':-')
                   index    =-1L
                   for i=1,n_elements(ind1)-1 do begin
                    ind2=where(ind0 eq ind1[i],count)
                    if count ne 0 then index=[index,ind2]
                   endfor
                   n_index  =n_elements(index)
                   if n_index ne 1 then index=index[1:n_index-1]-1
                   tmp   =strsplit(tmp,'[a-zA-Z '+string(9b)+',:-]',/regex,/extract)
                   det_tmp  =    -1L
                   for i=0,n_elements(tmp)-1 do begin
                    tmp1=dm_to_number(tmp[i],/int)
                    if finite(tmp1) then begin
                        ind0   = where(index eq i,count)
                        if count ne 0 then begin
                           i=i+1
                           tmp2=dm_to_number(tmp[i],/int)
                           if finite(tmp2) then det_tmp=[det_tmp,tmp1+findgen(abs(tmp2-tmp1)+1)] $
                           else det_tmp=[det_tmp,tmp1]
                        endif else det_tmp=[det_tmp,tmp1]
                    endif
                   endfor
                   index    =  where(det_tmp gt 0,count)
                   if count ne 0 then det_tmp=fix(det_tmp[index],type=3)-1 $
                   else det_tmp=indgen((*pState).ndet)
                   case status of
                    'c': det_tmp=dm_common(det_tmp,dm_mask_detbanks(0),/no_copy)
                    'l': det_tmp=dm_common(det_tmp,dm_mask_detbanks(-1),/no_copy)
                    'u': det_tmp=dm_common(det_tmp,dm_mask_detbanks(1),/no_copy)
                    else:det_tmp=dm_common(det_tmp,indgen((*pState).ndet),/no_copy)
                   endcase
                 endelse
                 if (det_tmp[0] eq -1) or (n_elements(det_tmp) eq (*pState).ndet) then begin
                   *(*pState).detPtr    =indgen((*pState).ndet)
                   (*pState).det_tit    ='!6det=all'
                   dm_set_droplist,(*pState).detList,select=0
                 endif else begin
                   *(*pState).detPtr    =det_tmp
                 endelse
                 end
              2: begin
                 *(*pState).detPtr =dm_mask_detbanks(0)
                 (*pState).det_tit ='!6det=central'
                 end
              3: begin
                 *(*pState).detPtr =dm_mask_detbanks(1)
                 (*pState).det_tit ='!6det=upper'
                 end
              4: begin
                 *(*pState).detPtr =dm_mask_detbanks(-1)
                 (*pState).det_tit ='!6det=lower'
                 end
              else:begin
                 *(*pState).detPtr =tmp_value-5
                 (*pState).det_tit =strtrim(string(tmp_value-4,format='("!6det=",i3)'),2)
                 end
          endcase
          n_old   =n_elements(old)
          n_new   =n_elements(*(*pState).detPtr)
          comm    =dm_common(old,*(*pState).detPtr)
          same=(n_old eq n_new) and (n_old eq n_elements(comm)) and (comm[0] ne -1)
          old=-1 & comm=-1
          if (n_elements(*(*pState).yPtr) ne 0) and (1-same) then begin   ;reload data
              (*pState).xran[*]=!values.f_nan
              (*pState).yran[*]=!values.f_nan
              wset,(*pState).winPix
              dcs_cryst_align_plot,pState
              wset,(*pState).winVis
              device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
          endif
         end
       (*pState).timeList:   if (*pState).t_chan ne tmp_value then begin
          (*pState).t_chan=tmp_value
          (*pState).filesel->getproperty,file=open_file
          if n_elements(open_file) ne 0 then begin ;reload data
              tlb    =(*pState).tlb
              load   =(*pState).loadBut
              widget_control,event.handler,set_uvalue=pState,/no_copy
              dcs_cryst_align_event,{WIDGET_BUTTON,ID:load,TOP:tlb,HANDLER:tlb,SELECT:1}
              return
          endif
         endif
       (*pState).xaxisList:begin
          (*pState).xaxis =tmp_value
          (*pState).fit=0
          if n_elements(*(*pState).xPtr) ne 0 then begin
              (*pState).xran[*]=!values.f_nan
              (*pState).yran[*]=!values.f_nan
              wset,(*pState).winPix
              dcs_cryst_align_plot,pState
              wset,(*pState).winVis
              device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
          endif
         end
       else:
    endcase
    device,decomposed=(*pState).decompose
    widget_control,event.handler,set_uvalue=pState,/no_copy
end





pro dcs_cryst_align,event
    state={group_leader:            0L,$        ;group leader
           tlb:                     0L,$        ;top level base
           rgb:ptr_new(/allocate_heap),$
           geom:               [0L,0L],$        ;geometry
           filesel:          obj_new(),$        ;file selector
           root:                    '',$        ;starting directory
           loadBut:                 0L,$        ;load file button
           fitBut:                  0L,$        ;gauss fit button
           fit:                     0b,$        ;flag for fit
           saveBut:                 0L,$        ;save plot button
           printBut:                0L,$        ;print plot button
           exitBut:                 0L,$        ;exit button
           ndet:                    0L,$        ;number of detectors
           detList:                 0L,$        ;detector droplist
           detPtr:ptr_new(/allocate_heap),$     ;pointer to detector array
           timeList:                0L,$        ;time channel droplist
           t_chan:                  0s,$        ;0-all 1-elastic
           xaxisList:               0L,$        ;x-axis droplist
           xaxis:                   0s,$        ;xaxis number
           xPtr:ptr_new(/allocate_heap),$       ;pointer to x data
           yPtr:ptr_new(/allocate_heap),$       ;pointer to y data
           dyPtr:ptr_new(/allocate_heap),$      ;pointer to dy data
           xaxisPtr:ptr_new(/allocate_heap),$   ;pointer to store !x
           yaxisPtr:ptr_new(/allocate_heap),$   ;pointer to store !y
           winVis:                  0L,$        ;visible window
           winPix:                  0L,$        ;buffer invisible window
           plotDraw:                0L,$        ;draw widget
           mouse:                   0b,$        ;mouse state
           xbox:               [0e,1e],$        ;zoom box
           ybox:               [0e,1e],$        ;zoom box
           xran:             fltarr(2),$        ;xrange
           yran:             fltarr(2),$        ;yrange
           duration:                0L,$        ;duration of measurement in secs
           det_tit:                 '',$        ;output detector string
           workDir:                 '',$        ;work directory
           datDir:                  '',$        ;data directory
           decompose:               0l $
    }
    registerName='dcs_cryst_align'
;   if xregistered(registerName) then return  ;only allow one copy to be running at one time
    ;save the entry decompose information
    device,get_decomposed=tmp & if n_elements(tmp) eq 1 then state.decompose=(0>(tmp)<1)
    device,decomposed=1
    tvlct,r,g,b,/get
    *state.rgb={r:temporary(r),g:temporary(g),b:temporary(b)}  ;save entry color map
    loadct,0,/silent
    cd,current=current
    state.root     =current
    state.workDir  =current
    state.datDir   =current

    ;get the detector information
    alldetPtr=dm_load_detpos()

    if n_elements(event) ne 0 then begin
       state.group_leader    =  event.top
       state.tlb =widget_base(title='DCS Single Crystal Alignment Utility',/row,kill_notify='dcs_cryst_align_Exit',/tlb_size_event,$
         group_leader=event.top)
    endif else $
       state.tlb =widget_base(title='DCS Single Crystal Alignment Utility',/row,kill_notify='dcs_cryst_align_Exit',/tlb_size_event)
    left   =widget_base(state.tlb,/col)
    state.filesel=obj_new('dm_filesel',parent=left,xsize=160,ysize=20,/frame,filter='.dcs')
    defsysv,'!dave_defaults',exists=exists
    if exists then $
       if ptr_valid(!dave_defaults) then begin
         state.datDir =   (*!dave_defaults).datDir
         state.workDir    =  (*!dave_defaults).workDir
       endif
    state.filesel->set_path,state.datDir
    if !version.os_family eq 'unix' then $
       center    =widget_base(state.tlb,/grid_layout,/col,/align_bottom,xpad=0,ypad=0,space=0) $
    else $
       center    =widget_base(state.tlb,/grid_layout,/col,/align_bottom)
    ;construct the detector droplist values
    state.ndet =   n_elements((*alldetPtr).two_theta)
    tmp         =    string((*alldetPtr).two_theta,format='(f9.2)')
    tmp         =    string(indgen(state.ndet)+1,format='(i3)')+tmp
    index   =   dm_mask_detbanks(0)
    tmp[index] =   tmp[index]+'  c' & index=-1  ;central bank
    index   =   dm_mask_detbanks(1)
    tmp[index] =   tmp[index]+'  u' & index=-1  ;upper bank
    index   =   dm_mask_detbanks(-1)
    tmp[index] =   tmp[index]+'  l' & index=-1  ;lower bank
    tmp         =    ['All','Specify','Central Bank','Upper Bank','Lower Bank',tmp]
    ptr_free,alldetPtr
    *state.detPtr  =indgen(state.ndet)  ;default all detectors
    state.det_tit  ='det=all'
    void      =widget_label(center,value='Det. Num')
    state.detList  =dm_widget_droplist(center,value=tmp)
    void      =widget_label(center,value='Time Chan.')
    state.timeList     =dm_widget_droplist(center,value=['All','Elastic'])
    void      =widget_label(center,value='X-axis')
    state.xaxisList    =dm_widget_droplist(center,value=['Rotation','Height','X-translation','Y-translation','X-tilt','Y-tilt','File Index'])
    void      =widget_label(center,value='')
    state.loadBut  =widget_button(center,value='Load')
    state.fitBut   =widget_button(center,value='Guass Fit')
    state.saveBut  =widget_button(center,value='Save')
    state.printBut =widget_button(center,value='Print')
    state.exitBut  =widget_button(center,value='Exit')
    geom   =widget_info(state.exitBut,/geometry)

    widget_control,state.detList,xsize=geom.scr_xsize
    widget_control,state.timeList,xsize=geom.scr_xsize
    widget_control,state.xaxisList,xsize=geom.scr_xsize

    geom   =widget_info(left,/geometry)
    ysize  =geom.scr_ysize
    xsize  =fix(1.3*ysize,type=3)
    state.plotDraw =widget_draw(state.tlb,xsize = xsize,ysize = ysize,/button_events,event_pro='dcs_cryst_align_zoom')


    dm_center_kid, state.tlb   ;centering the top level base
    widget_control,state.tlb,/realize

    widget_control,state.plotDraw,get_value = winVis
    window,/free,/pixmap,xsize = xsize,ysize = ysize
    winPix = !d.window
    state.winVis   =winVis
    state.winPix   =winPix

    geom      =widget_info(state.tlb,/geometry)
    state.geom     =[geom.scr_xsize,geom.scr_ysize]

    tlb          =state.tlb
    device,decomposed=state.decompose
    pState =   ptr_new(state,/no_copy)
    widget_control,tlb,set_uvalue=pState,/no_copy
    xmanager,registerName,tlb,cleanup='dcs_cryst_align_Exit',/no_block
end



