; $Id: dm_basis_r.pro,v 1.4 2003/09/12 19:09:03 qiuym Exp $
; construct basis vectors from the direct lattice parameters
; usuage:
;   abc_r=dm_basis_r([a,b,c,alpha,beta,gamma],error=error)
; parameter:
;   dir_parm: [a,b,c,alpha,beta,gamma]
; keyword:
;   error:    1 if error encountered
; return:
;   abc_r:    reciprocal lattice vectors in a reference where a//(100) and axb //(001)
function dm_basis_r,dir_parm,error=error
    abc=dblarr(3,3)
    abc_r=dblarr(3,3)
    dirparm=double(dir_parm)
    error=0    ;no error
    cosabg=dblarr(3)
    sinabg=dblarr(3)
    mydtor=!dpi/180
    for i=0,2 do begin
       cosabg[i]=cos(mydtor*dirparm[3+i])
       sinabg[i]=sin(mydtor*dirparm[3+i])
    endfor
    itmp   = where(abs(cosabg) lt 1e-15,count)
    if count ne 0 then cosabg[itmp]=0d
    itmp   = where(abs(sinabg) lt 1e-15,count)
    if count ne 0 then sinabg[itmp]=0d
    vol=dirparm[0]*dirparm[1]*dirparm[2]*sqrt(1.+$
       2.*cosabg[0]*cosabg[1]*cosabg[2]-cosabg[0]*cosabg[0]-$
       cosabg[1]*cosabg[1]-cosabg[2]*cosabg[2])
    if abs(vol) lt 1e-10 then begin
       error=1
       ok=dialog_message(['The volume of the given lattice cell is 0.','Please check lattice parameters.'],/error)
       return,abc_r
    endif
    abc[*,0]=dirparm[0]*[1.0d,0.0d,0.0d]
    abc[*,1]=dirparm[1]*[cosabg[2],sinabg[2],0.0d]
    abc[0,2]=dirparm[2]*cosabg[1]
    abc[1,2]=dirparm[2]*(cosabg[0]-cosabg[1]*cosabg[2])/sinabg[2];
    abc[2,2]=vol/(dirparm[0]*dirparm[1]*sinabg[2])
    tmp=where(abs(abc) le 1e-15,count)
    if count ne 0 then abc[tmp]=0.0d
    for i=0,2 do $
       for j=0,2 do $
         abc_r[j,i]=2.*!dpi/vol*(abc[(j+1) mod 3,(i+1) mod 3]*abc[(j+2) mod 3,(i+2) mod 3]- $
                        abc[(j+2) mod 3,(i+1) mod 3]*abc[(j+1) mod 3,(i+2) mod 3])
    tmp=where(abs(abc_r) le 1e-15,count)
    if count ne 0 then abc_r[tmp]=0.0d
    return,abc_r
end


