; $Id: dm_calc_projection.pro,v 1.18.2.1 2003/12/18 15:08:01 azuah Exp $
; calculate projection
; apply masking, rebinning, and detector efficiency adjustment to data
pro dcs_mslice::dm_calc_projection,debug=debug
    self.error=1  ;initialize the error flag, it will be cleared at the end of the program
    if keyword_set(debug) then begin
       print,'**Calculate projection...'
       current=systime(/sec)
    end

    if self.samp_typ eq 2 then begin
       case self.e_bin[2] of
          0:    en=0.0                                      ;use elastic energy for all time channels
          1:    en=0.0                                      ;use elastic energy for elastic peak
          2:    en=(self.e_range[0]+self.e_range[1])/2.0    ;use the mean energy for specified energy range
          else: en=0.0
       endcase
    endif

    if not ptr_valid(self.dataStrPtr) then begin       ;no data loaded yet
       ;ask for incident energy
       if self.samp_typ eq 2 then info=['E='+dm_to_string(en)+' meV'] $
       else info=['-3Ei< E <Ei']
       ei    = dm_dialog_input('Ei(meV):',/float,default=5,dialog_parent=self.tlb,info=info,/e2lambda)
       while (not finite(ei)) do $
         ei  = dm_dialog_input('Ei(meV):',/float,default=ei,dialog_parent=self.tlb,info=info,/e2lambda)
       if self.samp_typ eq 2 then begin
         if en gt ei then begin
            ok  = dialog_message('Incident energy is less than the energy transfer. E=0 is used.',/error)
            en  = 0.0
         endif
         a2  = dm_dialog_input(['Start:','End:','Step:'],title='Psi range:',default=[0,50,0.5],/float,dialog_parent=self.tlb)
         while (total(finite(a2)) ne 3 ) do $
           a2= dm_dialog_input(['Start:','End:','Step:'],title='Psi range:',default=a2,/float,dialog_parent=self.tlb)
         min_a=min(a2[0:1],max=max_a)
         if a2[2] le 0 then begin
            a2[2]=1.0
            ok=dialog_message('Invalid step. Step=1.0 is used.',/error)
         endif
         a2  = min_a+findgen(fix((max_a-min_a)/a2[2])+1)*a2[2]
         n_file=n_elements(a2)
       endif else en = (findgen(101)/25.-3)*ei                    ;[-3e,...,e]
       qty   = 0
       err   = 0
    endif else begin
       ei    = (*self.dataStrPtr).ei
       while (not finite(ei)) do begin
         ei  = dm_dialog_input('Ei(meV):',/float,default=ei,dialog_parent=self.tlb,/e2lambda)
         (*self.dataStrPtr).ei=ei
       endwhile
       if self.samp_typ eq 2 then begin                     ;diffuse scattering type
         a2  = (*self.dataStrPtr).rotation                 ;angle 2-rotation
         a2  = a2+self.orie_psi-a2[0]                      ;adjust the offset
         weight=(*self.dataStrPtr).weight                   ;weight
       endif else $
         en  = (*self.dataStrPtr).e                        ;[emin,...,emax]
       qty   = ((*self.dataStrPtr).qty)*(self.oifac)
       err   = ((*self.dataStrPtr).err)*(self.oifac)
       if ptr_valid(self.bgdata) then begin
          bgqty= (*self.bgdata).qty
          bgerr= (*self.bgdata).err
       endif
       n_file= size(qty)
       if n_file[0] le 1 then n_file=1 else n_file=n_file[2] ;used for diffuse scattering type
    endelse
    det_tt   = (*self.detPosPtr).two_theta
    det_psi  = (*self.detPosPtr).psi
    det_dtt  = (*self.detPosPtr).dtwo_theta
    det_dpsi = (*self.detPosPtr).dpsi
    nen      = n_elements(en)
    ndet     = n_elements(det_tt)
    widget_control,/hourglass
    if ptr_valid(self.dataStrPtr) then begin
       ;subtract overall background rate
       if finite(self.bgrate) then begin
         if ptr_valid(self.eff) then tmp_eff=(*self.eff) else tmp_eff=fltarr(ndet)+1.
         tmp_value=self.bgrate/3600./(self.monrate)*(self.oifac)*temporary(tmp_eff)
         if self.samp_typ eq 2 then begin    ;diffuse scattering type
          tmp_value=tmp_value*((*self.dataStrPtr).ntchan)
          for i=0,n_file-1 do qty[*,i]=qty[*,i]-tmp_value
          if ptr_valid(self.bgdata) then $
              bgqty=(*self.bgdata).qty-tmp_value
         endif else begin
          for i=0,nen-1 do $
              qty[i,*]=qty[i,*]-tmp_value
          if ptr_valid(self.bgdata) then $
              for i=0,nen-1 do $
                 bgqty[i,*]=(*self.bgdata).qty[i,*]-tmp_value
         endelse
         tmp_value=1       ;free up the memory
       endif
       ;subtract background file
       if ptr_valid(self.bgdata) then begin
         if finite(self.ssfac) then ssfac=self.ssfac $
         else ssfac=1.0   ;self shielding factor
         if self.samp_typ eq 2 then begin    ;diffuse scattering type
          for i=0,n_file-1 do begin
              qty[*,i]=qty[*,i]-bgqty*ssfac
              err[*,i]=sqrt(err[*,i]^2+(bgerr)^2*(ssfac*ssfac))
          endfor
         endif else begin
          qty = temporary(qty)-(bgqty)*ssfac
          err = sqrt(temporary(err)^2+(bgerr)^2*(ssfac*ssfac))
         endelse
         bgqty=0 & bgerr=0    ;free up the memory
       endif
       ;efficicency adjustment
       if ptr_valid(self.eff) then begin
         if self.samp_typ eq 2 then begin    ;diffuse scattering type
          for i=0,n_file-1 do begin
              qty[*,i]=qty[*,i]/(*self.eff)
              err[*,i]=err[*,i]/(*self.eff)
          endfor
         endif else begin
          for i=0,nen-1 do begin
              qty[i,*]=qty[i,*]/(*self.eff)
              err[i,*]=err[i,*]/(*self.eff)
          endfor
         endelse
       endif
       ;kfactor adjustment
       if self.is_dcs then begin
         case self.intn_typ of
          0:   tmp=(*self.kfactor)^4
          1:  begin
               ;ask for a sample temperature
               old   =(*self.dataStrPtr).info.temperature
               notvalid=1
               while notvalid do begin
                 temp=dm_dialog_input('sample temperature:',/float,default=old,dialog_parent=self.tlb,$
                    title='Detailed blance calculation')
                 widget_control,/hourglass
                 if finite(temp) then $
                    if temp gt 0 then begin
                        (*self.dataStrPtr).info.temperature=temp
                        notvalid=0 ;valid temperature given
                    endif
               endwhile
               ;calculate detailed balance factor
               kb  = 8.617342e-2    ;Boltzmann constant in mev.k^-1
               dbal= exp(-en/(2.0*kb*temp))
               tmp = (*self.kfactor)^4*dbal
              end
          2:  tmp  = (*self.kfactor)^3
          else:tmp = (*self.kfactor)^4    ;S(Q,omega)
         endcase
         ;energy depedent detector efficiency adjustment
         if self.edep_yn eq 1 then begin
          det_thick   = 1.0     ; assumed detector thickness in cm.
          sigma_acoeff= 0.4466  ; (from Jeremy in cm-1/lambda_f) absorption factor of He3 gas
          sigma_s     = 0.0008  ; (from Jeremy in cm-1) sigma_s+
          wl0  = (*self.dataStrPtr).info.ch_wl
          sumtd= sigma_acoeff*wl0*(*self.kfactor)+sigma_s
          ;deteff_inv=1.0/(1.0-exp(-det_thick*sumtd))
          deteff_inv=sumtd/(sumtd-sigma_s)/(1.0-exp(-det_thick*sumtd))    ;from Jeremy's Equation 9
          sumtd= 0
          tmp  = temporary(tmp)*temporary(deteff_inv)
         endif
         if self.samp_typ eq 2 then begin
            qty= temporary(qty)*tmp    ;tmp is a scalor here,qty is a [ndet,nfile] arrray
         endif else begin
            for i=0,ndet-1 do begin    ;tmp is a [nen] array,qty is a [nen,ndet] array
              qty[*,i] = qty[*,i]*tmp
              err[*,i] = err[*,i]*tmp
            endfor
         endelse
         tmp=0   ;free up the memory
       endif

       ;energy rebin
       if (self.e_bin[self.samp_typ] gt 1) and (self.samp_typ ne 2) then begin
         dm_grid_ebin,self.e_bin[self.samp_typ],qty,err,en,debug=debug,avgsum=self.bin_avgsum
       endif

       ;absorption correction
       if ptr_valid(self.absp_info) then begin
          info= [(*(self.absp_info))[0],(*(self.absp_info))[2:(n_elements(*(self.absp_info))-1)]]
          tmp = dm_calc_shielding(info,ei,en,det_tt,interp=(*(self.absp_info))[1],group_leader=self.tlb)
          if self.samp_typ eq 2 then begin    ;diffuse scattering type
            for i=0,n_file-1 do begin
              qty[*,i]=qty[*,i]*tmp
              err[*,i]=err[*,i]*tmp
            endfor
          endif else begin
            qty = temporary(qty)*tmp
            err = temporary(err)*temporary(tmp)
          endelse
          widget_control,/hourglass
       endif
    endif

    ;mask detectors
    if ptr_valid(self.mask) then begin
       tmp = bytarr(ndet)
       tmp[*self.mask]=1b
       ind = where(tmp eq 0,count) & tmp=0
       if count ne 0 then begin
         if ptr_valid(self.dataStrPtr) then begin
          if self.samp_typ eq 2 then begin
              qty = qty[ind,*]
              err = err[ind,*]
          endif else begin
              qty = qty[*,ind]
              err = err[*,ind]
          endelse
         endif
         det_tt   = det_tt[ind]
         det_dtt  = det_dtt[ind]
         det_psi  = det_psi[ind]
         det_dpsi = det_dpsi[ind]
         ndet     = count
       endif
    endif

    ;check if the common input parameters are all set for single crystal and diffuse scattering type
    if self.samp_typ ne 1 then begin
       if total(finite(self.latt_parm)) ne 6 then begin
         ok=dialog_message('Missing lattice parameters.',/error,dialog_parent=self.tlb)
         return
       endif
       if total([finite(self.orie_u),finite(self.orie_v),finite(self.orie_psi)]) ne 7 then begin
         ok=dialog_message('Missing crystal orientation parameters.',/error,dialog_parent=self.tlb)
         return
       endif
       ;direct -> reciprocal basis vectors
       abc_r    = dm_basis_r(self.latt_parm,error=error)
       if error then return  ;error encountered, no further calcuation
       ;orthogonalize and normalize the crystal orientation axes
       if norm(crossp(self.orie_u,self.orie_v)) eq 0 then begin    ;parallel
         ok=dialog_message('Please check cyrstal orientation.',/error,dialog_parent=self.tlb)
         return          ;no further calculation
       endif
       uvw      = dblarr(3,3)
       uvw[*,0] = abc_r#self.orie_u
       uvw[*,1] = abc_r#self.orie_v
       uvw[*,0] = uvw[*,0]/norm(uvw[*,0],/double)
       uvw[*,1] = uvw[*,1]-uvw[*,0]#(transpose(uvw[*,1])#uvw[*,0])
       uvw[*,1] = uvw[*,1]/norm(uvw[*,1],/double)
       for i=0,2 do uvw[i,2]=uvw[(i+1) mod 3,0]*uvw[(i+2) mod 3,1]-uvw[(i+2) mod 3,0]*uvw[(i+1) mod 3,1]
                 ;cross product, not using crossp() for higher precision
       ;rewrite reciprocal lattice parameters in the (u,v,w) frame
       abc_r    = transpose(uvw)#(temporary(abc_r))
    endif

    case self.samp_typ of
     0:begin  ;single crystal
       ;check if the inputs in viewing axes are all set
       if total([finite(self.view_u1),finite(self.view_u2),finite(self.view_u3)]) ne 12 then begin
         ok=dialog_message('Missing viewing axes parameters.',/error,dialog_parent=self.tlb)
         return
       endif
       ;reconstruct the viewing axis
       U1       = [abc_r#self.view_u1[0:2],self.view_u1[3]]
       U2       = [abc_r#self.view_u2[0:2],self.view_u2[3]]
       U3       = [abc_r#self.view_u3[0:2],self.view_u3[3]]
       norm1=norm(U1,/double) & norm2=norm(U2,/double) & norm3=norm(U3,/double)
       if self.view_ortharbi eq 0 then begin    ;check orthogonality
         if max(abs([transpose(U1)#U2/norm1/norm2,transpose(U2)#U3/norm2/norm3,$
           transpose(U3)#U1/norm3/norm1])) gt 1e-8 then begin   ;not orthogonal
           ok=dialog_message('Viewing axes are not orthogonal.',/error,dialog_parent=self.tlb)
           return          ;no further calculation
         endif
       endif else begin
         ;make sure no viewing axes are parale to each other
         tmp       = [[self.view_u1[0:2]],[self.view_u2[0:2]],[self.view_u3[0:2]]]
         for i=0,2 do begin
           if total(abs(tmp[*,i])) eq 0 then begin
             tmp[*,i]=crossp(tmp[*,(i+1) mod 3],tmp[*,(i+2) mod 3])
             if norm(tmp[*,i]) eq 0 then begin
                    ok=dialog_message('Parallel viewing axes',/error,dialog_parent=self.tlb)
                    return  ;no further calculation
                endif
             break
           endif
         endfor
       endelse
       Q        = dm_proj_spec_samp(ei,en,det_tt,det_dtt,det_psi,det_dpsi,psi=self.orie_psi,type=0,debug=debug,is_spe=1-self.is_dcs)
                                                                ;Q(3,ne,ndet) in sample frame
       V        = dm_proj_view(Q,en,U1,U2,U3,debug=debug)       ;project onto viewing axis
       hkl      = dm_q2hkl(Q,abc_r,debug=debug)                 ;hkl values
       ptr_free,self.projStrPtr
       self.projStrPtr=ptr_new({Q:temporary(Q),V:temporary(V),hkl:temporary(hkl),en:temporary(en),$
         two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err)})
       end
     1:begin  ;powder
       ;combine equivalent detectors
       det_tt   = abs(temporary(det_tt))
       ind      = sort(det_tt)
       ndat     = n_elements(det_tt)
       if ptr_valid(self.dataStrPtr) then begin
         qty    = qty[*,ind]
         err    = err[*,ind]
       endif
       det_tt   = det_tt[ind]
       det_dtt  = det_dtt[ind]
       det_psi  = det_psi[ind]
       det_dpsi = det_dpsi[ind]
       ind      = uniq(det_tt)
       nun      = n_elements(ind)
       j        = 0L
       nen      = n_elements(qty[*,0])
       for i=0L,nun-1 do begin
         if ptr_valid(self.dataStrPtr) then begin
            qtmp= fltarr(nen)
            etmp= fltarr(nen)
         endif
         itmp   = 0L
         while(det_tt[j] eq det_tt[ind[i]]) do begin
          if det_dtt[i] lt det_dtt[j] then det_dtt[i]=det_dtt[j]
          if det_dpsi[i] lt det_dpsi[j] then det_dpsi[j]=det_dpsi[j]
          if ptr_valid(self.dataStrPtr) then begin
           qtmp = temporary(qtmp)+qty[*,j]
           etmp = temporary(etmp)+err[*,j]*err[*,j]
           itmp = temporary(itmp)+1
          endif
          j     = temporary(j)+1
          if j eq ndat then break
         endwhile
         if ptr_valid(self.dataStrPtr) then begin
          qty[*,i] = qtmp/(itmp>1)
          err[*,i] = sqrt(etmp)/(itmp>1)
         endif
       endfor
       if ptr_valid(self.dataStrPtr) then begin
         qtmp=0 & etmp=0  ;free up the memory
         qty    = qty[*,0:nun-1]
         err    = err[*,0:nun-1]
       endif
       det_dtt  = det_dtt[0:nun-1]
       det_dpsi = det_dpsi[0:nun-1]
       det_tt   = det_tt[ind]
       det_psi  = det_psi[ind]
       Q =   dm_proj_spec_samp(ei,en,det_tt,det_dtt,det_psi,det_dpsi,type=1,debug=debug)   ;Q[nen,ndet]
       ptr_free,self.projStrPtr
       self.projStrPtr=ptr_new({Q:temporary(Q),en:temporary(en),two_theta:temporary(det_tt),$
         qty:temporary(qty),err:temporary(err)})
       end
     2:begin  ;diffuse scattering
       ;check if the inputs in viewing axes are all set
       if total([finite(self.view_u1[0:2]),finite(self.view_u2[0:2])]) ne 6  then begin
         ok=dialog_message('Missing viewing axes parameters.',/error,dialog_parent=self.tlb)
         return
       endif
       ;reconstruct the viewing axis and check orthogonality
       U1       = [abc_r#self.view_u1[0:2]]
       U2       = [abc_r#self.view_u2[0:2]]
       norm1=norm(U1,/double) & norm2=norm(U2,/double)
       if self.view_ortharbi eq 0 then begin    ;check orthogonality
         if abs(transpose(U1)#U2/norm1/norm2) gt 1e-8 then begin      ;not orthogonal
           ok=dialog_message('Viewing axes are not orthogonal.',/error,dialog_parent=self.tlb)
           return          ;no further calculation
         endif
       endif else begin
         ;make sure the viewing axes are not parallel to each other
         if norm(crossp(self.view_u1[0:2],self.view_u2[0:2])) eq 0 then begin
              ok=dialog_message('Parallel viewing axes',/error,dialog_parent=self.tlb)
              return  ;no further calculation
         endif
       endelse
       ;combine data with the same A2 angles
       if ptr_valid(self.dataStrPtr) then begin
         ind    = sort(a2)
         ndat   = n_elements(a2)
         qty    = qty[*,ind]
         err    = err[*,ind]
         a2     = a2[ind]
         ind    = uniq(a2)
         nun    = n_elements(ind)
         j      = 0L
         ndet   = n_elements(qty[*,0])
         for i=0L,nun-1 do begin
           qtmp = fltarr(ndet)
           etmp = fltarr(ndet)
           wgt  = 0.0
           while(a2[j] eq a2[ind[i]]) do begin
            qtmp= temporary(qtmp)+qty[*,j]*weight[j]
            etmp= temporary(etmp)+err[*,j]*err[*,j]*weight[j]*weight[j]
            wgt = wgt+weight[j]
            j   = temporary(j)+1
            if j eq ndat then break
           endwhile
           if wgt eq 0 then wgt=1.0
           qty[*,i] = qtmp/wgt
           err[*,i] = sqrt(etmp)/wgt
         endfor
         qtmp=0 & etmp=0  ;free up the memory
         qty    = qty[*,0:nun-1]
         err    = err[*,0:nun-1]
         a2     = a2[ind]
       endif
       Q        = dm_proj_spec_samp(ei,en,det_tt,det_dtt,det_psi,det_dpsi,psi=a2,type=2,debug=debug)
                                                                                 ;Q[3,ndet,nfile] in sample frame
       V        = dm_proj_view(Q,en,U1,U2,[0.0,0.0,0.0],/diffuse,debug=debug)    ;project onto viewing axis
       hkl      = dm_q2hkl(Q,abc_r,debug=debug)                                  ;hkl values
       ptr_free,self.projStrPtr,self.powd_line
       self.projStrPtr=ptr_new({Q:temporary(Q),V:temporary(V),hkl:temporary(hkl),en:temporary(en),$
         two_theta:temporary(det_tt),qty:temporary(qty),err:temporary(err)})
       ;Al or Cu powder line calculation
       case self.powd_typ of
            1:  q_value=[sqrt(3),2,sqrt(8),sqrt(11)]*2*!pi/4.04912  ;Al a=4.04912 FCC
            2:  q_value=[sqrt(3),2,sqrt(8),sqrt(11)]*2*!pi/3.6148   ;Cu a=3.6148  FCC
            else:
       endcase
       cos_th   =   cos(findgen(361)*!dtor)
       sin_th   =   sin(findgen(361)*!dtor)
       for i=0,n_elements(q_value)-1 do begin
           x    =   q_value[i]/norm1*cos_th
           y    =   q_value[i]/norm2*sin_th
           if n_elements(lines) ne 0 then lines=[[[lines]],[[[x],[y]]]] $
           else lines=[[x],[y]]
       endfor
       if n_elements(lines) ne 0 then self.powd_line=ptr_new(lines,/no_copy)
       end
     else:
    endcase
    self.error=0       ;clear the error flag
    ;allow plots
    if not keyword_set(debug) then begin
       if self.view[2] eq 1 then          self->my_widget_control,'plotTra',self.trajBas,sensitive=1
       if ptr_valid(self.dataStrPtr) then self->set_display_button,sensitive=1
    endif else begin
       print,'**Projection calculation finished in ',systime(/sec)-current,' sec.'
    endelse
    if ptr_valid(self.dataStrPtr) and (self.bin_avgsum eq 1) then $
        ok=dialog_message('The current binning method is SUM.',dialog_parent=self.tlb)
end