; $Id: dm_choose_cutdata.pro,v 1.9 2003/09/12 19:09:03 qiuym Exp $
; choose cut data
; parameters:
;   None
; keywords:
;   xdat,ydat,yerr:     data to be returned
;   (xy)tit:       label of the data
;   ylog:          =self.log_yn if ydat is intensity, 0 otherwise

pro dcs_mslice::dm_choose_cutdata,xdat=xdat,ydat=ydat,yerr=yerr,xtit=xtit,ytit=ytit,debug=debug,ylog=ylog
    xp =   [self.view_u1typ, self.view_u2typ]  ;powder x-axis
    x  =    [6+self.cut_x[0],xp[self.cut_x[1]],6+self.cut_x[2]]
    if keyword_set(debug) then begin
       print,'^^choose cut data...'
       current=systime(/sec)
    endif

    ;xdat
    self->dm_choose_trajdata,x[self.samp_typ],data=xdat,label=xtit
    if (xtit eq 'E (meV)') or (xtit eq 'Energy') then uniq_E=xdat[*,0]
    if keyword_set(debug) then print,'choose x done',systime(/sec)-current,' sec'
    ;thickness
    case self.samp_typ of
       0:begin     ;single crystal
         t1_id    =  6+((self.cut_x[0]+1) mod 3)
         t2_id    =  6+((self.cut_x[0]+2) mod 3)
         self->dm_choose_trajdata,t1_id,index=index,range=self.cut_t1ran,/leave_empty
         self->dm_choose_trajdata,t2_id,index=index1,range=self.cut_t2ran,/leave_empty
         if self.disp_fold then begin
          ;fold axis
          x_id    =  6+((self.disp_perp+1) mod 3)
          if x_id eq t1_id then begin
            self->dm_choose_trajdata,t1_id,index=tmp,range=[-self.cut_t1ran[1],-self.cut_t1ran[0]],/leave_empty
            index=[index,tmp]
          endif else if x_id eq t2_id then begin
            self->dm_choose_trajdata,t2_id,index=tmp,range=[-self.cut_t2ran[1],-self.cut_t2ran[0]],/leave_empty
            index1=[index1,tmp]
          endif
         endif
         index = dm_common(index,index1,/no_copy,count)
         end
       1:begin     ;powder
         self->dm_choose_trajdata,xp[(self.cut_x[1]+1) mod 2],index=index,range=self.cut_t1ran
         end
       2:begin     ;diffuse scattering
         self->dm_choose_trajdata,6+((self.cut_x[2]+1) mod 2),index=index,range=self.cut_t1ran
         end
       else:
    endcase

    ;ydat
    self->dm_choose_trajdata, self.cut_y[self.samp_typ]-1,data=ydat,error=yerr,label=ytit
    ylog   =(self.cut_y[self.samp_typ] eq 0)?(self.log_yn):0
    ok_err =(n_elements(yerr) ne 0)
    if index[0] ne -1 then begin
       xdat=xdat[index]
       ydat=ydat[index]
       if ok_err then yerr=yerr[index]
       ;index=-1
    endif else $
       ok=dialog_message('The thickness is out of range. All possilbe data will be plotted.',dialog_parent=self.tlb)

    ;xrange
    if total(finite(self.cut_ran)) eq 2 then begin
       if self.cut_ran[1] gt self.cut_ran[0] then begin
         index=where((xdat le self.cut_ran[1]) and (xdat ge self.cut_ran[0]),count)
         if count gt 0 then begin
          xdat=xdat[index]
          ydat=ydat[index]
          if ok_err then yerr=yerr[index]
          index=-1
         endif
       endif
    endif else begin
       if finite(self.cut_ran[0]) then begin
         index=where(xdat ge self.cut_ran[0],count)
         if count gt 0 then begin
          xdat=xdat[index]
          ydat=ydat[index]
          if ok_err then yerr=yerr[index]
          index=-1
         endif
       endif
       if finite(self.cut_ran[1]) then begin
         index=where(xdat le self.cut_ran[1],count)
         if count gt 0 then begin
          xdat=xdat[index]
          ydat=ydat[index]
          if ok_err then yerr=yerr[index]
          index=-1
         endif
       endif
    endelse

    if keyword_set(debug) then help,xdat,ydat
    if (xtit eq 'E (meV)') or (xtit eq 'Energy') then $    ;rebin if x-axis is E
       dm_step_bin,1e-5,xdat,ydat=ydat,yerr=yerr,uniq_val=uniq_E,debug=debug,avgsum=self.bin_avgsum

    ;rebinning according to the steps
    if finite(self.cut_step) then $
      if (self.cut_step ne 0) then $
       dm_step_bin,self.cut_step,xdat,ydat=ydat,yerr=yerr,debug=debug,avgsum=self.bin_avgsum

    if keyword_set(debug) then print,'^^choose cut data finished in ',systime(/sec)-current,' sec.'
end