; $Id: dm_dcs2spe.pro,v 1.19 2003/11/24 16:24:16 qiuym Exp $
; this program converts dcs files into spe files
; Yiming Qiu (yiming.qiu@nist.gov)
; 9/26/2002

; convert open_file to a spe file and saved in save_path
; parameters:
;   open_path:      opening file path
;   open_file:      file name, excluding the directory
;   bg_rate:        background rate to be subtracted,in cts/hr
;   bg_error:       error of above
;   ebin_size:      E rebin size
; keywords:
;   qty,err,weight: data
;   save_file:      file name(including directory) to be saved
;   parent:         group leader, message is centered on the parent widget
;   bgdataPtr:      pointer to backgound file data
;   deteffPtr:      pointer to detector efficiency data
;   savedet:        if set, detector profile will be saved
;   maskPtr:        detector mask pointer
;   powder:         if set or equal to 1 then powder mode
pro dm_convert_dcs2spe,open_path,open_file,bg_rate,bg_error,ebin_size,qty=qty,dqty=dqty,weight=weight,save_file=save_file,$
    parent=parent,bgdataPtr=bgdataPtr,deteffPtr=deteffPtr,savedet=savedet,detPtr=detPtr,maskPtr=maskPtr,powder=powder
    dm_load_dcsdata,open_path,open_file,qty=qty,dqty=dqty,weight=weight,monrate=monrate,kfactor=kfactor,parent=parent,emean=emean,$
       error=error
    ndet=n_elements(qty[0,*])
    nen =n_elements(qty[*,0])
    if error eq 1 then return
    mesg=widget_base(title='converting...',/col,/base_align_center,group_leader=parent,map=0)
    label=widget_label(mesg,value=' ',/dynamic_resize,/align_center)
    widget_control,mesg,/realize,map=1

    ;subtract overall background
    if finite(bg_rate) then begin
       widget_control,label,set_value='subtracting overall background...'
       dm_center_kid,mesg,parent
       tmp_value = bg_rate/3600./monrate*(fltarr(nen)+1.)
       if ptr_valid(deteffPtr) then $
         tmp_value = tmp_value#(*deteffPtr) $
       else $
         tmp_value = tmp_value#(fltarr(ndet)+1.)
       qty=temporary(qty)-temporary(tmp_value)
       if finite(bg_error) then begin
         tmp_value = bg_error/3600./monrate*(fltarr(nen)+1.)
         if ptr_valid(deteffPtr) then $
          tmp_value = tmp_value#(*deteffPtr) $
         else $
          tmp_value = tmp_value#(fltarr(ndet)+1.)
         dqty = sqrt(temporary(dqty)^2+(temporary(tmp_value))^2)
       endif
    endif

    ;subtract background file data
    if ptr_valid(bgdataPtr) then begin
       widget_control,label,set_value='subtracting background file data...'
       qty  = temporary(qty)-(*bgdataPtr).qty
       dqty = sqrt(temporary(dqty)^2+((*bgdataPtr).err)^2)
    endif


    ;detector efficiency adjustment
    if ptr_valid(deteffPtr) then begin
       eff  = (fltarr(nen)+1.0)#(*deteffPtr)
       qty  = temporary(qty)/eff
       dqty = temporary(dqty)/eff
    endif

    ;kfactor adjustment
    tmp=kfactor^4
    for i=0,ndet-1 do begin
        qty[*,i]    = qty[*,i]*tmp
        dqty[*,i]   = dqty[*,i]*tmp
    endfor

    if keyword_set(save_file) then begin
       ;rebin E
       if ebin_size gt 1 then dm_grid_ebin,ebin_size,qty,dqty,emean

       tt   = (*detPtr).two_theta
       dtt  = (*detPtr).dtwo_theta
       psi  = (*detPtr).psi
       dpsi = (*detPtr).dpsi

       ;mask detectors
       if ptr_valid(maskPtr) then begin
         widget_control,label,set_value='masking detectors...'
         ndet = n_elements(tt)
         tmp  = intarr(ndet)
         tmp[*maskPtr] = -1
         ind  = where(tmp eq 0,count)
         if count ne 0 then begin
          qty = qty[*,ind]
          dqty= dqty[*,ind]
          tt  = tt[ind]
          dtt = dtt[ind]
          psi = psi[ind]
          dpsi= dpsi[ind]
         endif
       end

       if keyword_set(powder) then begin
         ;combine equivalent detectors
         widget_control,label,set_value='combining equivalent detectors...'
         tt   = abs(temporary(tt))
         ind  = sort(tt)
         ndet = n_elements(tt)
         qty  = qty[*,ind]
         dqty = dqty[*,ind]
         tt   = tt[ind]
         dtt  = dtt[ind]
         psi  = psi[ind]
         dpsi = dpsi[ind]
         ind  = uniq(tt)
         nun  = n_elements(ind)
         j    = 0L
         nen  = n_elements(qty[*,0])
         for i= 0L,nun-1 do begin
          qtmp= dblarr(nen)
          etmp= dblarr(nen)
          itmp= 0L
          while(tt[j] eq tt[ind[i]]) do begin
              if dtt[i] lt dtt[j] then dtt[i]=dtt[j]
              if dpsi[i] lt dpsi[j] then dpsi[i]=dpsi[j]
              qtmp = temporary(qtmp)+qty[*,j]
              etmp = temporary(etmp)+dqty[*,j]
              itmp = itmp+1
              j    = j+1
              if j eq ndet then break
          endwhile
          qty[*,i] = qtmp/itmp
          dqty[*,i]= etmp/itmp
         endfor
         qty  = qty[*,0:nun-1]
         dqty = dqty[*,0:nun-1]
         dtt  = dtt[0:nun-1]
         dpsi = dpsi[0:nun-1]
         tt   = tt[ind]
         psi  = psi[ind]
       endif else begin
         ;convert two theta and psi to spe compatible format
         delta= max(psi)                    ;=7.6285 degree is the upper and lower row angle
         ndet = n_elements(tt)
         sind = sin(delta*!dtor)
         for i=0, ndet-1 do begin
            bank = fix(psi[i]/delta*1.2)    ;-1:lower 0:middle 1:upper
            if bank eq 0 then begin         ;middle bank
                if tt[i] lt 0 then psi[i]=0.0 else psi[i]=180.0
            endif else begin
                ang = asin(sind/sin(tt[i]*!dtor))/!dtor
                ang = (ang ge 0)*180.0-ang
                if bank eq 1 then psi[i]=ang else psi[i]=360.0-ang
            endelse
         endfor
         tt   = abs(tt)
         dtt  = abs(dtt)
       endelse
       widget_control,label,set_value='saving as '+save_file[0]
       openw,unit,save_file,/get_lun,error=openerr
       if openerr ne 0 then begin
         ok=dialog_message("Can't write in "+save_file[0],/error,dialog_parent=parent)
         widget_control,mesg,/destroy
         return
       endif
       dm_write_spe,unit,tt,emean,qty,dqty
       free_lun,unit

       if n_elements(savedet) eq 1 then begin
         if savedet then begin
          detfile=dialog_pickfile(title='Please Select a File for Writing the Detector Profile File',$
              dialog_parent=parent,/write,filter='*.phx',file='det.phx')
          if strlen(detfile) ne 0 then begin
              tmp=strmid(detfile,3,4,/reverse_offset)
              if strlowcase(tmp) ne '.phx' then detfile=detfile+'.phx'
              widget_control,label,set_value='saving detetor profile as '+detfile
              dm_write_phx,detfile,tt,dtt,psi,dpsi
          endif
         endif
       endif
    end
    ; Clear any floating-point underflows that has occurred
    tmp=check_math(mask=32)
    widget_control,mesg,/destroy
end


;save current settings in file dcs2spe.cfg
pro dm_save_cfg,state
    tmp1 = state.open_filesel->getproperty(/path)
    tmp2 = state.save_filesel->getproperty(/path)
    cd,state.root
    cfg  = state.root+state.pathsep+'dcs2spe.cfg'
    openw,unit,cfg,/get_lun,error=err
    if err ne 0 then return
    printf,unit,'opendir:'+tmp1
    printf,unit,'savedir:'+tmp2
    printf,unit,'bg_rate:'+dm_to_string(state.bg_rate)
    printf,unit,'bg_erro:'+dm_to_string(state.bg_error)
    printf,unit,'ebin_sz:'+dm_to_string(state.ebin_size)
    free_lun,unit
end


;load default settings
pro dm_load_cfg,state
    cd,current = current
    state.root = current
    cfg = current+state.pathsep+'dcs2spe.cfg'
    tmp = file_test(cfg)
    if tmp eq 0 then begin ;doesn't exist
       state.bg_rate  = 0.0
       state.bg_error = 0.0
       state.ebin_size= 1
       state.open_filesel->set_path,current
       state.save_filesel->set_path,current
       defsysv,'!dave_defaults',exists=exists
       if exists then $
         if ptr_valid(!dave_defaults) then begin
         state.open_fileSel->set_path,(*!dave_defaults).datDir
         state.save_filesel->set_path,(*!dave_defaults).workDir
         endif
       return
    endif
    openr,unit,cfg,/get_lun
    tmp=' '
    while(not eof(unit)) do begin
       readf,unit,tmp
       head = strmid(tmp,0,8)
       tmp  = strmid(tmp,8,strlen(tmp)-8)
       case head of
          'opendir:':state.open_filesel->set_path,tmp
          'savedir:':state.save_filesel->set_path,tmp
          'bg_rate:':state.bg_rate=float(tmp)
          'bg_erro:':state.bg_error=float(tmp)
          'ebin_sz:':state.ebin_size=fix(tmp)
          else:
       endcase
    endwhile
    free_lun,unit
end

;clear all objects and widget hierarchy
pro dm_dcs2spe_Exit,tlb
    widget_control,tlb,get_uvalue=state,/no_copy
    ptr_free,state.bgdata,state.eff,state.mask,state.detPosPtr
    dm_save_cfg,state
    obj_destroy,state.open_filesel
    obj_destroy,state.save_filesel
    widget_control,tlb,/destroy
end

;event handler
pro dm_dcs2spe_event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    WIDGET_CONTROL,/HOURGLASS
    widget_control,event.handler,get_uvalue=state,/no_copy
    state.open_filesel->getproperty,path=open_path,file=open_file
    state.save_filesel->getproperty,path=save_path,file=save_file

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok=dialog_message(dialog_parent=state.tlb,!error_state.msg,/error)
       catch,/cancel
       widget_control,event.handler,set_uvalue=state,/no_copy
       return
    end

    case (event.id) of
       state.tlb:    widget_control,event.id,scr_xsize=state.geom[0],scr_ysize=state.geom[1]
       state.loadBut:begin
          if n_elements(open_file) eq 0 then begin
              ok=dialog_message('Please select DCS files first.',/error,dialog_parent=event.handler)
              break
          endif
          dm_convert_dcs2spe,open_path,open_file,state.bg_rate,state.bg_error,state.ebin_size,$
                 qty=qty,dqty=dqty,weight=weight,parent=event.handler
          state.open_filesel->clear_file
          if n_elements(qty) le 1 then break
          ptr_free,state.bgdata
          state.bgdata=ptr_new({qty:temporary(qty),err:temporary(dqty)})
          widget_control,state.clearBut,sensitive=1
         end
       state.clearBut:begin
          widget_control,state.clearBut,sensitive=0
          ptr_free,state.bgdata
         end
       state.loadV:begin
          if n_elements(open_file) eq 0 then begin
              ok=dialog_message('Please select DCS files first.',/error,dialog_parent=event.handler)
              break
          endif
          dm_convert_dcs2spe,open_path,open_file,state.bg_rate,state.bg_error,state.ebin_size,$
                 qty=qty,dqty=dqty,weight=weight,parent=event.handler,bgdataPtr=state.bgdata
          state.open_filesel->clear_file
          if n_elements(qty) le 1 then break
          ptr_free,state.eff
          eff = dm_calc_deteff(qty,/fit)
          state.eff = ptr_new(eff,/no_copy)
          widget_control,state.clearV,sensitive=1
         end
       state.clearV:begin
          widget_control,state.clearV,sensitive=0
          ptr_free,state.eff
         end
       state.combBut:begin
          if n_elements(open_file) lt 2 then begin
              ok = dialog_message('Please choose 2 or more data files.',/error,dialog_parent=event.handler)
              break
          endif
          if n_elements(save_file) eq 0 then save_file='combine.spe' else begin
              if save_file eq '.spe' or strlen(save_file) eq 0 then save_file='combine.spe' else begin
                  tmp = strmid(save_file,3,4,/reverse_offset)
                  if strlowcase(tmp) ne '.spe' then save_file=save_file+'.spe'
              endelse
          endelse
          state.save_filesel->set_file,filename=save_file
          save_file = save_path+state.pathsep+save_file
          dm_convert_dcs2spe,open_path,open_file,state.bg_rate,state.bg_error,state.ebin_size,$
              save_file=save_file,parent=event.handler,bgdataPtr=state.bgdata,deteffPtr=state.eff,$
              savedet=state.yn,detPtr=state.detPosPtr,maskPtr=state.mask,powder=state.samptyp
          state.open_filesel->clear_file
          state.save_filesel->set_path
          state.save_filesel->set_file,filename=''
          end
       state.convBut:begin
         if n_elements(open_file) eq 0 then begin
          ok=dialog_message('Please select DCS files first.',/error,dialog_parent=event.handler)
          break
         endif
         nfile= n_elements(open_file)
         yn   = intarr(nfile)
         yn[nfile-1]=state.yn
         for i=0,nfile-1 do begin
          ;generate the output file name according to input file name
          save_file = strsplit(open_file[i],'.',/extract)
          state.save_filesel->set_file,filename=save_file[0]+'.spe'
          save_file = save_path+state.pathsep+save_file[0]+'.spe'
          dm_convert_dcs2spe,open_path,open_file[i],state.bg_rate,state.bg_error,state.ebin_size,$
              save_file=save_file,parent=event.handler,bgdataPtr=state.bgdata,deteffPtr=state.eff,$
              savedet=yn[i],detPtr=state.detPosPtr,maskPtr=state.mask,powder=state.samptyp
          state.save_filesel->set_path    ;update the save directory
          state.save_filesel->set_file,filename=''
         endfor
         state.open_filesel->clear_file
          end
       state.doneBut:begin
         widget_control,event.handler,set_uvalue=state,/no_copy
         widget_control,event.handler,/destroy
         return
          end
       state.mkcnBut:begin   ;view central banks only
         ptr_free,state.mask
         mask = dm_mask_detbanks([-1,1])
         state.mask = ptr_new(mask,/no_copy)
         dm_toggle_menubut,state.mkcnBut,[state.mkloBut,state.mkupBut]
         widget_control,state.mkclBut,sensitive=1
         end
       state.mkloBut:begin   ;view lower banks only
         ptr_free,state.mask
         mask = dm_mask_detbanks([0,1])
         state.mask = ptr_new(mask,/no_copy)
         dm_toggle_menubut,state.mkloBut,[state.mkcnBut,state.mkupBut]
         widget_control,state.mkclBut,sensitive=1
         end
       state.mkupBut:begin   ;view upper banks only
         ptr_free,state.mask
         mask = dm_mask_detbanks([-1,0])
         state.mask = ptr_new(mask,/no_copy)
         dm_toggle_menubut,state.mkupBut,[state.mkloBut,state.mkcnBut]
         widget_control,state.mkclBut,sensitive=1
         end
       state.mkstBut:begin             ;set mask
         ptr_free,state.mask
         widget_control,state.mkclBut,sensitive=0
         dm_toggle_menubut,state.mkcnBut,[state.mkcnBut,state.mkloBut,state.mkupBut]
         mask = dm_dialog_input(dialog_parent=event.handler,'Det num:',title='Masking detectors',$
          info='Detector number starts from 0.',xsize=200)
         mask = dm_to_number(strsplit(mask,', '+string(9b),/extract),/int)
         ind  = where(finite(mask) eq 1,count)
         if count ne 0 then begin
          mask= mask[ind]
          ind = where((mask lt 913) and (mask ge 0),count)
          if count ne 0 then begin
              state.mask = ptr_new(mask[ind])
              widget_control,state.mkclBut,sensitive=1
          endif
         endif
         end
       state.mkclBut:begin
         ptr_free,state.mask  ;clear the mask
         dm_toggle_menubut,state.mkcnBut,[state.mkcnBut,state.mkloBut,state.mkupBut]
         widget_control,state.mkclBut,sensitive=0
         end
       state.crstBut:begin
          state.samptyp = 0
          dm_toggle_menubut,state.crstBut,state.powdBut
         end
       state.powdBut:begin
          state.samptyp = 1
          dm_toggle_menubut,state.powdBut,state.crstBut
         end
       state.hpctBut:void=launch_help(state.helpfile,tlb=event.top)
       state.bgInfo:begin
         widget_control,event.id,get_value=tmp
         state.bg_rate = dm_to_number(tmp,/float)
          end
       state.bgError:begin
         widget_control,event.id,get_value=tmp
         state.bg_error = dm_to_number(tmp,/float)
          end
       state.eBin:begin
         widget_control,event.id,get_value=tmp
         state.ebin_size = abs(fix(tmp))
         if (abs(state.ebin_size-float(tmp)) ge 0.05) or (state.ebin_size gt 1000) then begin
          if state.ebin_size gt 1000 then state.ebin_size=1000
          widget_control,event.id,set_value=dm_to_string(state.ebin_size)
         end
          end
       state.yesBut:begin
         state.yn = 1
         dm_toggle_menubut,state.yesBut,state.noBut
          end
       state.noBut:begin
         dm_toggle_menubut,state.noBut,state.yesBut
         state.yn = 0
          end
       else: ;nothing
    endcase
    widget_control,event.handler,set_uvalue=state,/no_copy
end

;main program
pro dm_dcs2spe,event
    state={ group_leader:   0L,$     ;group leader
         tlb:               0L,$     ;top level base
         open_filesel:obj_new(),$    ;open file selector
         save_filesel:obj_new(),$    ;save file selector
         bg_rate:           0E,$     ;overall background in cts/hour
         bg_error:          0E,$     ;associated error
         ebin_size:         0L,$     ;E binning size
         bgInfo:            0L,$     ;fast neutron background text box
         bgError:           0L,$     ;associated error text box
         eBin:              0L,$     ;E binning size text box
         loadBut:           0L,$     ;load background file
         clearBut:          0L,$     ;clear background file
         loadV:             0L,$     ;load vanadium file
         clearV:            0L,$     ;clear vanadium file
         combBut:           0L,$     ;combine files into a single spe file button
         convBut:           0L,$     ;convert button
         doneBut:           0L,$     ;done button
         mkupBut:           0L,$     ;mask central and lower bank button
         mkcnBut:           0L,$     ;mask top and lower bank button
         mkloBut:           0L,$     ;mask top and central bank button
         mkstBut:           0L,$     ;set mask button
         mkclBut:           0L,$     ;clear the mask button
         yesBut:            0L,$     ;create detector file
         noBut:             0L,$     ;don't create detector file
         yn:                0s,$     ;0 no 1 yes
         hpctBut:           0L,$     ;help content button
         crstBut:           0L,$     ;single crystal button
         powdBut:           0L,$     ;powder button
         samptyp:           0s,$     ;0-single crystal, 1-powder
         root:              '',$     ;root directory
         pathsep:           '',$     ;path seperator
         helpfile: 'dcs_mslice.pdf',$;help file
         geom:   [0L,0L,0L,0L],$     ;xsize,ysize,xoffset,yoffset
         bgdata:     ptr_new(),$     ;background data
         eff:        ptr_new(),$     ;detector efficiency
         mask:       ptr_new(),$     ;mask pointer
         detPosPtr:   ptr_new()$     ;detector positions
    }
    registerName    ='dm_dcs2spe'
    if xregistered(registerName) then return   ;only allow one copy to be running at one time
    ;get the correct path separator
    if float(!version.release) ge 5.5 then state.pathsep=call_function("path_sep") $
    else begin
       defsysv,'!delimiter',exists=exists
       if not exists then begin
         case !version.os_family of
          'Windows':  defsysv,'!delimiter','\'
          'unix':     defsysv,'!delimiter','/'
          'MacOS':    defsysv,'!delimiter',':'
          else:       defsysv,'!delimiter','/'
         endcase
       endif
       state.pathsep=!delimiter
    endelse
    if n_elements(event) ne 0 then begin
       state.group_leader=event.top
       state.tlb    = widget_base(title='Convert DCS data files to SPE format',/col,kill_notify='dcs2spe_Exit',$
                group_leader=event.top,/tlb_size_event,mbar=mbar)
       ;widget_control,event.top,sensitive=0
    endif else $
       state.tlb    = widget_base(title='Convert DCS data files to SPE format',/col,kill_notify='dcs2spe_Exit',/tlb_size_event,mbar=mbar)
    ;menu bar
    filemenu        = widget_button(mbar,value='File',/menu)
    maskmenu        = widget_button(mbar,value='Mask',/menu)
    sampmenu        = widget_button(mbar,value='Sample',/menu)
    helpmenu        = widget_button(mbar,value='Help',/menu)
    state.helpfile  = file_which(state.helpfile,/include)
    state.detPosPtr = dm_load_detpos()  ;load detector informations
    state.doneBut   = widget_button(filemenu,value='Exit',uvalue='doneBut',uname='DONE')
    state.mkcnBut   = dm_widget_button(maskmenu,value='Mask upper and lower banks',uvalue='mkcnBut',uname='VIEWCENTRALBANK')
    state.mkupBut   = dm_widget_button(maskmenu,value='Mask central and lower banks',uvalue='mkupBut',uname='VIEWUPPERBANK')
    state.mkloBut   = dm_widget_button(maskmenu,value='Mask upper and central banks',uvalue='mkloBut',uname='VIEWLOWERBANK')
    state.mkstBut   = widget_button(maskmenu,value='Set your own mask',uvalue='mkstBut',uname='SETMASK')
    state.mkclBut   = widget_button(maskmenu,value='Clear the mask',uvalue='mkclBut',uname='CLEARMASK',sensitive=0,/separator)
    state.crstBut   = dm_widget_button(sampmenu,value='Single Crystal',uvalue='crstBut',uname='SINGLECRYSTAL')
    state.powdBut   = dm_widget_button(sampmenu,value='Powder',uvalue='powdBut',uname='POWDER')
    state.samptyp   = 0                        ;0-single crystal, 1-powder
    dm_toggle_menubut,state.crstBut,state.powdBut  ;default single crystal

    tmp             = widget_button(sampmenu,value='Create Detector File',/menu,/separator)
    state.yesBut    = dm_widget_button(tmp,value='Yes')
    state.noBut     = dm_widget_button(tmp,value='No')
    state.yn        = 0                       ;default not saving
    dm_toggle_menubut,state.noBut,state.yesBut

    state.hpctBut   = widget_button(helpmenu,value='Help',uvalue='hpctBut',uname='HELPCONTENT')

    ;menu bar separator for windows system
    if !version.os_family eq 'Windows' then $
       mbar_sep     = widget_label(state.tlb,sensitive=0,/dynamic_resize)
    row             = widget_base(state.tlb,/row)
    openCol         = widget_base(row,/col)
    cmdCol          = widget_base(row,/col,/align_center,ypad=0)
    saveCol         = widget_base(row,/col)

    label           = widget_label(openCol,value='DCS file directory')
    label           = widget_label(saveCol,value='SPE file directory')

    state.open_filesel = obj_new('dm_filesel',parent=openCol,ysize=11,/frame,filter='.dcs')
    state.save_filesel = obj_new('dm_filesel',parent=saveCol,ysize=9,/write,/frame,filter='.spe')

    dm_load_cfg,state   ;load previous saved settings

    label           = widget_label(cmdcol,value='Overall background rate')
    label           = widget_label(cmdcol,value='(cts/hr/det):')
    state.bgInfo    = widget_text(cmdcol,uvalue='bgInfo',value=dm_to_string(state.bg_rate),/editable,/all_events)
    label           = widget_label(cmdcol,value='Associated error:')
    state.bgError   = widget_text(cmdcol,uvalue='bgError',value=dm_to_string(state.bg_error),/editable,/all_events)
    label           = widget_label(cmdcol,value='Energy binning size:')
    state.eBin      = widget_text(cmdcol,uvalue='eBin',value=dm_to_string(state.ebin_size),/editable,/all_events)
    geom            = widget_info(state.eBin,/geom)
    line            = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0)
    state.loadBut   = widget_button(line,value='Load BG',uvalue='loadBut',uname='LOADBGFILE',xsize=geom.scr_xsize/2)
    state.clearBut  = widget_button(line,value='Clear BG',uvalue='clearBut',uname='CLEARBGFILE',xsize=geom.scr_xsize/2,sensitive=0)
    line            = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0)
    state.loadV     = widget_button(line,value='Load V',uvalue='loadV',uname='LOADVFILE',xsize=geom.scr_xsize/2)
    state.clearV    = widget_button(line,value='Clear V',uvalue='clearV',uname='CLEARVFILE',xsize=geom.scr_xsize/2,sensitive=0)
    state.convBut   = widget_button(cmdCol,value='Convert',uvalue='convBut',uname='CONVERT')
    state.combBut   = widget_button(cmdCol,value='Combine',uvalue='combBut',uname='COMBINE')

    dm_center_kid, state.tlb   ;centering the top level base
    widget_control,state.tlb,/realize
    geom            = widget_info(state.tlb,/geometry)
    state.geom      = [geom.scr_xsize,geom.scr_ysize,geom.xoffset,geom.yoffset]
    ;show the horizontal bar style menu bar separator
    if n_elements(mbar_sep) ne 0 then begin
       sep          = bytarr(1000) & sep[*]=byte(175) &sep =string(sep)
       widget_control,mbar_sep,set_value=sep,xsize=state.geom[0]
       widget_control,state.tlb,scr_xsize=state.geom[0],scr_ysize=state.geom[1]
    endif

    tlb =state.tlb
    widget_control,state.tlb,set_uvalue=state,/no_copy
    xmanager,registerName,tlb,cleanup='dm_dcs2spe_Exit',/no_block
end