; $Id: dm_filesel__define.pro,v 1.15 2003/11/24 16:24:16 qiuym Exp $
; ******************************************************************
; This program defines a file selction object tailored for dcs2spe *
; program.                                                         *
; Usuage: filesel=obj_new('dm_filesel',parent=parent,path=path,$   *
;           multi=multi,write=write,filter=filter,frame=frame,$    *
;           ysize=ysize,ypad=ypad,xsize=xsize)                     *
; Yiming Qiu (yiming.qiu@nist.gov)                                 *
; 9/28/2002                                                        *
; ******************************************************************

@cw_filesel ;use its cw_fileSel_GetDirs,cwfileSel_atTop function
            ;usuage: dFiles=cw_fileSel_GetDirs(filter='.dcs,.dcs.gz')


pro dm_filesel_Exit,tlb
    ;leave the job to obj_destroy
end


;*******************************************************************
;standard error handling functionality
;copied from Fanning
;*******************************************************************
function dm_filesel_error_message,theMessage,Error=error,Informational=information,Traceback=traceback,NoName=noname,$
    Title=title,_Extra=extra

    On_Error, 2
    ; Check for presence and type of message.
    IF N_Elements(theMessage) EQ 0 THEN theMessage = !Error_State.Msg
    s = Size(theMessage)
    messageType = s[s[0]+1]
    IF messageType NE 7 THEN BEGIN
       Message, "The message parameter must be a string.",_Extra=extra
    ENDIF
    ; Get the call stack and the calling routine's name.
    Help, Calls=callStack
    IF Float(!Version.Release) GE 5.2 THEN $
       callingRoutine = (StrSplit(StrCompress(callStack[1])," ",/Extract))[0] $
    ELSE $
       callingRoutine = (Str_Sep(StrCompress(callStack[1])," "))[0]
    ; Are widgets supported?
    widgetsSupported = ((!D.Flags AND 65536L) NE 0)
    IF widgetsSupported THEN BEGIN
    ;If this is an error produced with the MESSAGE command,
    ;it is a trapped error and will have the name "IDL_M_USER_ERR".
      IF !ERROR_STATE.NAME EQ "IDL_M_USER_ERR" THEN BEGIN
       IF N_Elements(title) EQ 0 THEN title = 'Trapped Error'
       ; If the message has the name of the calling routine in it,
       ; it should be stripped out.
       ;Can you find a colon in the string?
       colon = StrPos(theMessage, ":")
       IF colon NE -1 THEN BEGIN
         ; Extract the text up to the colon. Is this the same as
         ; the callingRoutine? If so, strip it.
         IF StrMid(theMessage, 0, colon) EQ callingRoutine THEN $
          theMessage = StrMid(theMessage, colon+1)
       ENDIF
       ; Add the calling routine's name, unless NONAME is set.
       IF Keyword_Set(noname) THEN BEGIN
         answer = Dialog_Message(theMessage, Title=title, $
          _Extra=extra, Error=error, Information=information)
       ENDIF ELSE BEGIN
         answer = Dialog_Message(StrUpCase(callingRoutine) + ": " $
         + theMessage, Title=title, _Extra=extra, Error=error, Information=information)
       ENDELSE
      ENDIF ELSE BEGIN
       ; Otherwise, this is an IDL system error.
       IF N_Elements(title) EQ 0 THEN title = 'System Error'
       IF StrUpCase(callingRoutine) EQ "$MAIN$" THEN $
         answer = Dialog_Message(theMessage, _Extra=extra,Title=title, Error=error, $
             Information=information) $
       ELSE $
       IF Keyword_Set(noname) THEN BEGIN
         answer=Dialog_Message(theMessage, _Extra=extra, $
         Title=title,Error=error, Information=information)
       ENDIF ELSE BEGIN
         answer = Dialog_Message(StrUpCase(callingRoutine) + $
         "--> " + theMessage, _Extra=extra, Title=title, $
         Error=error, Information=information)
       ENDELSE
      ENDELSE
    ENDIF ELSE BEGIN
       Message, theMessage, /Continue, /NoPrint, /NoName,/NoPrefix, _Extra=extra
       Print, '%' + callingRoutine + ': ' + theMessage
       answer = 'OK'
    ENDELSE

    ; Provide traceback information if requested.
    IF Keyword_Set(traceback) THEN BEGIN
       Help, /Last_Message, Output=traceback
       Print,''
       Print, 'Traceback Report from ' +StrUpCase(callingRoutine) + ':'
       Print, ''
       FOR j=0,N_Elements(traceback)-1 DO Print, "     " + traceback[j]
    ENDIF
    RETURN, answer
end

;widget event handler
pro dm_filesel_event,event
    widget_control,event.handler,get_uvalue=self
    if obj_valid(self) then self->event,event
end

;*******************************************************************
;clear the file selection
;syntax:
;   filesel->clear_file
;arguments:
;   None
;keywords:
;   bottom: if set, go to the bottom of the file after clearing the selection
;*******************************************************************
pro dm_filesel::clear_file,bottom=bottom
    fileList=widget_info(self.tlb,find_by_uname='fileList')
    if ptr_valid(self.file) then begin
       ptr_free,self.file
       widget_control,fileList,set_list_select=-1
       if self.write then $
            widget_control,widget_info(self.tlb,find_by_uname='saveFile'),set_value=''
    endif
    if keyword_set(bottom) then begin
       if ptr_valid(self.dFiles) then nFiles=n_elements(*self.dFiles)    $
       else nFiles=0
       geom=widget_info(fileList,/geom)
       widget_control,fileList,set_list_top=nFiles-geom.ysize+1
    endif
    cd,self.root
end

;*******************************************************************
;object event handler
;*******************************************************************
pro dm_filesel::event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    widget_control,/hourglass
    ;error handling
    catch, anyerror
    if anyerror ne 0 then begin
       catch,/cancel
       ok=dm_filesel_error_message(traceback=1,/error)
       return
    end
    eventname=widget_info(event.id,/uname)
    case eventname of
       'driPull': begin
            drives=get_drive_list()
            if (drives[0] ne '') and (self -> isDir(drives[event.index],/showmesg)) then begin
                cd,drives[event.index]
                cd,current=currdir
                self->set_path,currdir
                self->clear_file
            endif
          end
       'upBut':begin
            cd,self.path
            cd,self.upDir
            cd,current=current
            self->set_path,current
            self->clear_file
          end
       'refreshBut':begin
            self->set_path
            self->clear_file,/bottom
          end
       'pathLab':begin
            widget_control,event.id,get_value=newPath
            self->set_path,newPath[0]
            self->clear_file
          end
       'limitFrom':begin
            widget_control,event.id,get_value=tmp
            tmp=strtrim(tmp,2)
            if self.limitfrom ne tmp then begin
                self.limitfrom=tmp
                self->set_path
            endif
          end
       'limitTo':begin
            widget_control,event.id,get_value=tmp
            tmp=strtrim(tmp,2)
            if self.limitto ne tmp then begin
                self.limitto=tmp
                self->set_path
            endif
          end
       'fileList':begin
            dFiles  =*self.dFiles
            selected=widget_info(event.id,/list_select) & n_item=n_elements(selected)
            if selected[0] eq -1 then break  ;nothing selected
            isDir   =self->isDir(self.path+self.pathsep+dFiles[selected])
            case(event.clicks) of
            1:begin ;single click
                self->set_file  ;clear file pointer
                index=where(isDir eq 0,count)
                if count ne 0 then self->set_file,selected[index]
              end
            2:begin ;double click
                if isDir[0] then begin
                    cd,self->getproperty(/path)
                    cd,dFiles[event.index]
                    cd,current=currDir
                    self->set_path,currDir
                    self->clear_file
                endif else self->set_file,event.index
              end
            endcase
          end
       'saveFile':begin
            widget_control,event.id,get_value=newfile
            self->set_file,filename=newfile[0],/noupdate
          end
       'okBut':begin
            if ptr_valid(self.file) then $
                widget_control,self.tlb,/destroy
          end
       'cancelBut':begin
            self->clear_file
            widget_control,self.tlb,/destroy
          end
       else:
    endcase
end

;*******************************************************************
;get the vairous properties
;syntax:
;   file=filesel->getproperty(/file)    ;returns '' if nonexistent
;arguments:
;   None
;keywords:
;   dFiles:     return list of files in the selection window
;   file:       return selected files
;   limitfrom:  return from limit
;   limitto:    return to limit
;   path:       return path of selected files
;   sep:        return path separator
;*******************************************************************
function dm_filesel::getproperty,dFiles=dFiles,file=file,limitfrom,limitto,path=path,sep=sep
    if keyword_set(file) then $
       if ptr_valid(self.file) then  begin
          result=strtrim(*self.file,2)
          if n_elements(result) eq 1 then result=result[0]
          return,result
       endif
    if keyword_set(path) then           return, strtrim(self.path,2)
    if keyword_set(dFiles) then $
       if ptr_valid(self.dFiles) then begin
          result=strtrim(*self.dFiles,2)
          if n_elements(result) eq 1 then result=result[0]
          return,result
       endif
    if keyword_set(sep) then            return,self.pathsep
    if keyword_set(limitfrom) then      return,self.limitfrom
    if keyword_set(limitto) then        return,self.limitto
    return,''
end

;*******************************************************************
;get the vairous properties
;syntax:
;   filesel->getproperty,file=file,path=path,dFiles=dFiles,sep=sep
;arguments:
;   None
;keywords:
;   dFiles:     return list of files in the selection window
;   file:       return selected files
;   limitfrom:  return from limit
;   limitto:    return to limit
;   path:       return path of selected files
;   sep:        return path separator
;*******************************************************************
pro dm_filesel::getproperty,dFiles=dFiles,file=file,limitfrom=limitfrom,limitto=limitto,path=path,sep=sep
    if arg_present(file) then $
       if ptr_valid(self.file) then begin
            file     = strtrim(*self.file,2)
            if n_elements(file) eq 1 then file   = file[0]
       endif
    if arg_present(path) then           path     = strtrim(self.path,2)
    if arg_present(dFiles) then $
        if ptr_valid(self.dFiles) then begin
            dFiles   = strtrim(*self.dFiles,2)
            if n_elements(dFiles) eq 1 then dFiles=dFiles[0]
        endif
    if arg_present(sep) then            sep      = self.pathsep
    if arg_present(limitfrom) then      limitfrom= self.limitfrom
    if arg_present(limitto) then        limitto  = self.limitto
end

;*******************************************************************
;check whether a path is a valid directory
;syntax:
;   result=filesel->isDir(path,/showmesg)
;arguments:
;   path: a string of the path to be checked
;keywords:
;   nomesg: if set the error message is not displayed
;*******************************************************************
function dm_filesel::isDir,path,showmesg=showmesg
    nitem   =   n_elements(path)
    if nitem eq 0 then return,0b
    tmp     =   file_info(path,/noexpand_path)
    result  =   tmp[*].directory and tmp[*].exists
    if keyword_set(showmesg) then begin
        index=where(result eq 0,count)
        if count ne 0 then begin
            mesg=path[index]+' is not a valid directory.'
            if widget_info(self.tlb,/valid_id) then $
                ok  =dialog_message(mesg,dialog_parent=self.tlb) $
            else $
                ok  =dialog_message(mesg)
        endif
    endif
    return,(nitem eq 1)?result[0]:result
end

;*******************************************************************
;setting self.file , not supposed to be called by outside user
;syntax:
;   filesel->set_file,[selected],filename=filename
;arguments:
;   selected:   an array of index
;keywords:
;   filename:   a string
;   noupdate:   if not set and if self.write eq 1 then (*self.file) will be shown in saveas textbox
;*******************************************************************
pro dm_filesel::set_file,selected,filename=filename,noupdate=noupdate,highlight=highlight
    ptr_free,self.file
    if n_elements(selected) eq 0 then begin
     if n_elements(filename) ne 0 then begin
       tmp=where(['all files','*.*','.*'] eq strlowcase(self.filter),count)
       if count eq 0 then begin
         tmp1=strsplit(self.filter,'.',/extract)
         if n_elements(tmp1) gt 1 then tmp1=tmp1[1]
         tmp2=strsplit(filename,'.',/extract)
         if tmp2[0] ne '' then filename=tmp2[0]+'.'+tmp1
       endif
       self.file=ptr_new(filename)
     endif
    endif else begin
        if ptr_valid(self.dFiles) then begin
            self.file=ptr_new((*self.dFiles)[selected])
            if keyword_set(highlight) then begin
                filelist=widget_info(self.tlb,find_by_uname='fileList')
                widget_control,filelist,set_list_select=selected
            endif
        endif
    endelse
    if self.write and (not keyword_set(noupdate)) then begin
      if ptr_valid(self.file) then tmp=*self.file else tmp=''
      widget_control,widget_info(self.tlb,find_by_uname='saveFile'),set_value=tmp
    endif
end

;*******************************************************************
;go to a specified path
;syntax:
;   filesel->set_path,newpath
;arguments:
;   newapth: string of the new path to be set
;keywords:
;   None
;*******************************************************************
pro dm_filesel::set_path,newpath
    if n_elements(newpath) eq 0 then newpath=''
    if (strlen(newpath[0]) ne 0) and (newpath[0] ne self.path) then $
        if (self->isDir(newPath[0],/showmesg)) then self.path=newpath[0]
    cd,self.path
    widget_control,widget_info(self.tlb,find_by_uname='pathLab'),set_value=self.path
    atTop=cwfileSel_atTop(self.path)
    widget_control,widget_info(self.tlb,find_by_uname='upBut'),sensitive=1-atTop
    ptr_free,self.dFiles
    dFiles=cw_filesel_GetDirs(filter=self.filter)
    if(size(dFiles,/type) ne 7) then $  ;empty directory
        widget_control,widget_info(self.tlb,find_by_uname='fileList'),sensitive=0,set_value='' $
    else begin
        dFiles=dFiles(sort(dFiles))
        n_item=n_elements(dFiles)
        isDir =self->isDir(self.path+self.pathsep+dFiles)
        ind_file=where(isDir eq 0,nfile,complement=ind_dir,ncomplement=ndir)
        if ndir ne 0 then    dir =dFiles[ind_dir]   ;directories
        if nfile ne 0 then files =dFiles[ind_file]
        if self.limitfile then begin
            if nfile ne 0 then begin
                if self.limitfrom ne '' then begin
                    ind=where(files ge self.limitfrom,count)
                    if count ne 0 then files=files[ind] else tmp=temporary(files)
                endif
                if (self.limitto ne '') and (n_elements(files) ne 0) then begin
                    ind=where(files le self.limitto+'~',count)
                    if count ne 0 then files=files[ind] else tmp=temporary(files)
                endif
            endif
        endif
        if n_elements(files) ne 0 then $
            if n_elements(dir) ne 0 then dir=[dir,files] else dir=files
        if n_elements(dir) ne 0 then dFiles=dir else tmp=temporary(dFiles)
        if n_elements(dFiles) ne 0 then begin
            self.dFiles=ptr_new(dFiles)
            widget_control,widget_info(self.tlb,find_by_uname='fileList'),sensitive=1,set_value=dFiles
        endif else begin
            widget_control,widget_info(self.tlb,find_by_uname='fileList'),sensitive=0,set_value=''
        endelse
    endelse
    driPull=widget_info(self.tlb,find_by_uname='driPull')
    if driPull ne 0 then begin
      drives=get_drive_list()
      case !version.os_family of
       'Windows':drive_loc=(WHERE(STRUPCASE(STRMID(self.path,0,STRPOS(self.path,'\')+1)) eq STRUPCASE(drives)) > 0)[0]
       'MacOS'  :drive_loc=(WHERE(STRMID(self.path,0,STRPOS(self.path,':')+1) eq drives) > 0)[0]
      endcase
      widget_control,driPull,set_droplist_select=drive_loc
    endif
end

;*******************************************************************
;destroy the object and widget hierarchy, life cycle method
;syntax:
;   filesel->destroy
;arguments:
;   None
;keywords:
;   None
;*******************************************************************
pro dm_filesel::Cleanup
    cd,self.root
    if widget_info(self.extbase,/valid_id) then widget_control,self.extbase,/destroy
    if widget_info(self.tlb,/valid_id) then widget_control,self.tlb,/destroy
    ptr_free,self.dFiles,self.file
end

;*******************************************************************
;class constructor, life cycle method
;
;syntax:object constructor,called automatically
;parameter:
;   None
;keywords:
;   groupleader:groupleader id
;   parent:     parent ID of this object, if present, it looks like a widget component
;   path:       a string of starting path, default is current directory
;   multi:      if set, allow multiple file choice, only in opening files, default is 1
;   write:      if set, the fileselction is for saving files, defalut is 0
;   filter:     file selection filter, default is 'All Files'
;   frame:      if set, there is a frame around the widget
;   xsize:      xsize of the list widget
;   ysize:      row number of the filelist widget
;   limitfile:  if set, allow restrictions on start and end of file
;   limitfrom:  limit from
;   limitto:    limit to
;*******************************************************************
function dm_filesel::Init,parent=parent,group_leader=group_leader,path=path,multi=multi,write=write,$
    filter=filter,frame=frame,ysize=ysize,ypad=ypad,xpad=xpad,xsize=xsize,limitfile=limitfile,$
    limitfrom=limitfrom,limitto=limitto
    ;error handling
    catch, anyerror
    if anyerror ne 0 then begin
       catch,/cancel
       ok=dm_filesel_error_message(traceback=1,/error)
       return,0
    end
    if n_elements(multi) eq 0 then self.multi=1 else self.multi=0>(multi)<1
    if n_elements(write) eq 0 then self.write=0 else self.write=0>(write)<1
    if n_elements(limitfile) eq 0 then self.limitfile=0 else self.limitfile=0>(limitfile)<1
    if self.write then begin
        self.multi=0
        self.limitfile=0
    endif
    if n_elements(filter) eq 0 then self.filter='All Files' else self.filter=filter
    if n_elements(ysize) eq 0 then ysize=8
    if n_elements(xsize) eq 0 then xsize=240
    if n_elements(ypad)  eq 0 then ypad=1
    if n_elements(xpad)  eq 0 then xpad=3
    if n_elements(space) eq 0 then space=2
    if n_elements(limitfrom) ne 0 then self.limitfrom=strtrim(limitfrom[0],2)
    if n_elements(limitto) ne 0 then self.limitto=strtrim(limitto[0],2)
    cd,current=currdir
    self.path=currdir
    self.root=currdir
    if keyword_set(path)  then $
       if self->isDir(path) then self.path=path

    if n_elements(frame) eq 0 then frame=0

    if n_elements(parent) eq 0 then begin
       self.extbase =widget_base(group_leader=group_leader,map=0)
       self.tlb=widget_base(group_leader=self.extbase,/column,frame=frame,event_pro='dm_filesel_event',uname='filesel',$
            title='Please Select '+((self.multi eq 1)?'Files':'a File'),/modal,xpad=xpad,ypad=ypad,space=space)
    endif else begin
       self.tlb=widget_base(parent,/column,frame=frame,event_pro='dm_filesel_event',uname='filesel',xpad=xpad,ypad=ypad,space=space)
    endelse
    self.upDir='..'
    ;check operation systems
    case !VERSION.OS_FAMILY of
    'Windows': begin
       driveVol = 'Drive:'
       drives  =get_drive_list()
       self.pathsep='\'
       end
    'MacOS': begin
       self.upDir='::'
       driveVol = 'Volume:'
       drives=get_drive_list()
       self.pathsep=':'
       end
    'vms': begin
       self.upDir='[-]'
       driveVol = ''
       self.pathsep='/'
       end
    else:begin
       driveVol =''
       self.pathsep='/'
       end
    endcase
    space   =   (widget_info(self.tlb,/geometry)).space
    if (driveVol ne '') then begin
       row   =widget_base(self.tlb,/row,xpad=xpad,ypad=ypad,space=space)
       label =widget_label(row,value=driveVol)
       void  =widget_droplist(row,value=drives,uname='driPull',xsize=100)
    endif
    row  =widget_base(self.tlb,/row,xpad=xpad,ypad=ypad,space=space)
    bitmap_filename=file_which('refresh.bmp',/include_current_dir)
    refreshBut=widget_button(row,value=bitmap_filename,uname='refreshBut',/bitmap,/dynamic_resize)
    geom1=widget_info(refreshBut,/geometry)
    ;bitmap_filename = FILEPATH('up1lvl.bmp', ROOT_DIR = IDL_DIR,SUBDIRECTORY = ['resource','bitmaps'])
    bitmap_filename=file_which('up1lvl.bmp',/include_current_dir)
    upBut=widget_button(row,value=bitmap_filename,uname='upBut',/bitmap,/dynamic_resize)
    geom =widget_info(upBut,/geometry)
    if float(!version.release) ge 5.6 then begin
       widget_control,refreshBut,tooltip='Refresh'
       widget_control,upBut,tooltip='Up One Level'
    endif
    tmp  =   geom.scr_xsize+geom1.scr_xsize+2*geom.margin+3*(xpad+space)+space
    void =widget_text(row,value=self.path,scr_xsize=xsize-tmp,ysize=1,/editable,uname='pathLab')
    if self.limitfile then begin
        row     =   widget_base(self.tlb,/row,xpad=xpad,ypad=ypad,space=space)
        label   =   widget_label(row,value='from:  ')
        geom    =   widget_info(label,/geometry)
        tmp     =   geom.scr_xsize+2.*geom.margin+(xpad+space)+space
        void    =   widget_text(row,value=self.limitfrom,scr_xsize=xsize-tmp,ysize=1,/editable,uname='limitFrom',/all_events)
        row     =   widget_base(self.tlb,/row,xpad=xpad,ypad=ypad,space=space)
        label   =   widget_label(row,value=' to:   ',scr_xsize=geom.scr_xsize)
        void    =   widget_text(row,value=self.limitto,scr_xsize=xsize-tmp,ysize=1,/editable,uname='limitTo',/all_events)
    end
    fileList =widget_list(self.tlb,uname='fileList',multiple=self.multi,scr_xsize=xsize)
    self->set_path,self.path
    if self.write then begin
       row   =widget_base(self.tlb,/row,xpad=xpad,ypad=ypad,space=space)
       label =widget_label(row,value='save as:  ')
       geom  =widget_info(label,/geometry)
       tmp   =geom.scr_xsize+xpad+2*space+2*geom.margin
       void  =widget_text(row,scr_xsize=xsize-tmp,ysize=1,/editable,uname='saveFile',/all_events)
    endif
    widget_control,fileList,ysize=ysize
    geom =widget_info(fileList,/geometry)
    if n_elements(parent) eq 0 then begin
       row  =   widget_base(self.tlb,/row,/align_center,xpad=xpad,ypad=ypad,space=space)
       void =   widget_button(row,value=(self.write eq 1)?'Save':'Open',uname='okBut')
       void =   widget_label(row,value='',xsize=15)
       void =   widget_button(row,value='Cancel',uname='cancelBut')
       geom =   widget_info(row,/geom)
       dm_center_kid,self.tlb,group_leader
       widget_control,self.tlb,/realize
    end
    widget_control,self.tlb,set_uvalue=self
    if n_elements(parent) eq 0 then $
       xmanager,'dm_filesel',self.tlb,cleanup='dm_filesel_Exit'
    return,1
end

;*******************************************************************
;filesel class definition
;*******************************************************************
pro dm_filesel__define
    void={dm_filesel          ,$    ;the object class name
       extbase:             0L,$    ;group leader of the modal widget
       tlb:                 0L,$    ;identifier of top level base
       dFiles:       PTR_NEW(),$    ;diretory listing
       file:         PTR_NEW(),$    ;selected files
       path:                '',$    ;current directory
       root:                '',$    ;starting directory
       pathsep:             '',$    ;path separator
       filter:              '',$    ;filter
       upDir:               '',$    ;up directory
       multi:               0s,$    ;multiple file selection flag,not allowed if write is set
       write:               0s,$    ;for saving files
       limitfile:           0s,$    ;for limitting file selection range
       limitfrom:           '',$    ;limit file selection
       limitto:             '' $    ;limit file selection
    }
end