; $Id: dm_plot__define.pro,v 1.41.2.1 2003/12/18 15:08:26 azuah Exp $
; an object of data plotting for dcs_mslice.pro
; Yiming Qiu (yiming.qiu@nist.gov 5/2003)

;if widget is killed
pro dm_plot_Exit,tlb
    widget_control,tlb,get_uvalue=self
    if obj_valid(self) then begin
       self->getproperty,group_leader=group_leader,tlb=tlb,parenthandler=parenthandler,parentobj=parentobj
       sendevent={plot_kill,ID:tlb,TOP:group_leader,HANDLER:tlb,object:self}
       if obj_valid(parentobj) then $     ;notify parent object
            call_method,parenthandler,parentobj,sendevent $
       else if group_leader ne 0 then $   ;notify the group leader
            widget_control,group_leader,send_event=sendevent,/no_copy
       obj_destroy,self
    endif
end

;event handler
pro dm_plot_event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(!error_state.msg,/error)
       return
    end

    widget_control,/hourglass
    widget_control,event.handler,get_uvalue=self
    eventname=widget_info(event.id,/uname)
    case eventname of
       'tlb':        self->resize,event
       'exitBut':    obj_destroy,self
       'readBut':    self->read1dfile
       'save':       self->saveas,'ps'
       'saveps':     self->saveas,'ps'
       'savejpg':    self->saveas,'jpeg'
       'savetif':    self->saveas,'tiff'
       'savebmp':    self->saveas,'bmp'
       'savepng':    self->saveas,'png'
       'saveppm':    self->saveas,'ppm'
       'saveasc':    self->saveas,'ascii'
       'blackbg':    self->setproperty,misscol=0
       'whitebg':    self->setproperty,misscol=1
       'printBut':   self->saveas,'printer'
       else:         self->event,event  ;events that need to access internal variables
    endcase
end

;add a plot to the object
pro dm_plot::add_plot,xdat,ydat,zdat,yerr=yerr,color=color,psym=psym,linestyle=linestyle,legend=legend,symsize=symsize,thick=thick,$
    coltable=coltable,shading=shading,hideintbar=hideintbar,surfplot=surfplot,usepolygon=usepolygon,layer=layer,no_copy=no_copy
    if float(!version.release) ge 6.0 then v6p0_extra= {register_properties:1}  ;register_properties keyword for IDL6.0 or later
    if n_elements(color) eq 0  then color='white'
    mycolor =   self->getcolor(color)
    white   =   self->getcolor('white')
    ;check keywords
    if keyword_set(surfplot) then cshift=0.5 else cshift=0.0
    if n_params() lt 2 then begin
       ok=dialog_message('Wrong number of parameters for dm_plot::add_plot.',/error,dialog_parent=self.tlb)
       return
    endif
    if total(finite(self.xran[*,0])) ne 2 then begin
       xmax=max(xdat,min=xmin,/nan)
       self.xran[*,0]=[temporary(xmin),temporary(xmax)]
       if self.xran[0,0] eq self.xran[1,0] then begin
         tmp  =    (self.xran[0,0] eq 0)? 0.5:abs(self.xran[0,0]/2.)
         self.xran[0,0]=self.xran[0,0]-tmp
         self.xran[1,0]=self.xran[1,0]+tmp
       endif
    endif
    if total(finite(self.xran[*,2])) ne 2 then begin   ;set the correct x log range from the data
       poselts=where(xdat gt 0,count)
       if count gt 0 then begin
         xmax=max(xdat[temporary(poselts)],min=xmin)
       endif else begin
         xmin=1.0 & xmax=10.0
       endelse
       self.xran[0,2]=(xmin gt self.xran[0,0])?((self.xran[0,0] gt 0)?(self.xran[0,0]):(xmin)):(self.xran[0,0])
       self.xran[1,2]=(xmax gt self.xran[1,0])?((self.xran[1,0] gt 0)?(self.xran[1,0]):(xmax)):(self.xran[1,0])
       if self.xran[1,2] le self.xran[0,2] then self.xran[1,2]=self.xran[0,2]*10.0
    endif
    if total(finite(self.yran[*,0])) ne 2 then begin
       if n_elements(yerr) eq n_elements(ydat) then begin
         ymin=min(ydat-yerr,/nan)
         ymax=max(ydat+yerr,/nan)
       endif else begin
         ymax=max(ydat,min=ymin,/nan)
       endelse
       self.yran[*,0]=[temporary(ymin),temporary(ymax)]
       if self.yran[0,0] eq self.yran[1,0] then begin
         tmp  =    (self.yran[0,0] eq 0)? 0.5:abs(self.yran[0,0]/2.)
         self.yran[0,0]=self.yran[0,0]-tmp
         self.yran[1,0]=self.yran[1,0]+tmp
       endif
    endif
    if total(finite(self.yran[*,2])) ne 2 then begin   ;set the correct y log range from the data
       poselts=where(ydat gt 0,count)
       if count gt 0 then begin
         ymax=max(ydat[temporary(poselts)],min=ymin)
       endif else begin
         ymin=1.0 & ymax=10.0
       endelse
       self.yran[0,2]=(ymin gt self.yran[0,0])?((self.yran[0,0] gt 0)?(self.yran[0,0]):(ymin)):(self.yran[0,0])
       self.yran[1,2]=(ymax gt self.yran[1,0])?((self.yran[1,0] gt 0)?(self.yran[1,0]):(ymax)):(self.yran[1,0])
       if self.yran[1,2] le self.yran[0,2] then self.yran[1,2]=self.yran[0,2]*10.0
    endif
    ;create x & y axis if nonexistent
    if not obj_valid(self.drawxAxis[0]) then begin
       ;x axis
       self.drawxAxis[0] =   obj_new('IDLgrAxis',0,range=self.xran[*,0],title=self.xtit,_extra=v6p0_extra,$
         color=white,ticklen=self.default[0]*1.5,/exact,location=[0,-cshift,0],name='axis') ;bottom
       self.drawGroup->add,self.drawxAxis[0]
       if not keyword_set(surfplot) then begin
         self.title->getproperty,strings=title & obj_destroy,self.title
         self.title=obj_new('IDLgrText',title,recompute_dimensions=2,font=self.font[4],/enable,_extra=v6p0_extra,$
            color=white,name='titl')
         self.drawxAxis[1]   = obj_new('IDLgrAxis',0,range=self.xran[*,0],color=white,_extra=v6p0_extra,$
         ticklen=self.default[0]*1.5,/tickdir,location=[1,1,0],/exact,name='axis',title=self.title,textpos=1,textalignments=[0.5,0])  ;top
         self.drawGroup->add,self.drawxAxis[1]
       endif
       ;y axis
       self.drawyAxis[0] =   obj_new('IDLgrAxis',1,range=self.yran[*,0],title=self.ytit,_extra=v6p0_extra,$
         color=white,ticklen=self.default[0],location=[-cshift,0,0],/exact,name='axis') ;left
       self.drawGroup->add,self.drawyAxis[0]
       if not keyword_set(surfplot) then begin
         self.drawyAxis[1]   = obj_new('IDLgrAxis',1,range=self.yran[*,0],color=white,_extra=v6p0_extra,$
         ticklen=self.default[0],/tickdir,location=[1,1,0],/exact,/notext,name='axis')    ;right
         self.drawGroup->add,self.drawyAxis[1]
       endif
    endif
    if n_params() eq 3  then begin        ;2d data
       if n_elements(hideintbar) eq 1 then self.hideIntbar=0>(fix(hideintbar[0]))<1
       if n_elements(shading) ne 0 then  shading=0>(fix(shading[0]))<1 else shading=fix(self.default[3]) ;default flat shading
       if n_elements(coltable) ne 0 then coltable=0>(fix(coltable[0]))<40 else coltable=fix(self.default[4]) ;default rainbow color table
       drawPal=self->getpalette(coltable)
       if total(finite(self.zran[*,0])) ne 2 then begin
         zmax=max(zdat,min=zmin,/nan)
         self.zran[*,0]=[zmin,zmax]
         if self.zran[0,0] eq self.zran[1,0] then begin
          tmp =   (self.zran[0,0] eq 0)? 0.5:abs(self.zran[0,0]/2.)
          self.zran[0,0]=self.zran[0,0]-tmp
          self.zran[1,0]=self.zran[1,0]+tmp
         endif
       endif
       if total(finite(self.zran[*,2])) ne 2 then begin  ;set the correct z log range from the data
         finelts=where(finite(zdat),count)
         if count gt 0 then poselts=where(zdat[finelts] gt 0,count)
         if count gt 0 then begin
          zmax=max(zdat[finelts[poselts]],min=zmin)
         endif else begin
          zmin=1.0 & zmax=10.0
         endelse
         finelts=-1 & poselts=-1
         self.zran[0,2]=(zmin gt self.zran[0,0])?((self.zran[0,0] gt 0)?(self.zran[0,0]):(zmin)):(self.zran[0,0])
         self.zran[1,2]=(zmax gt self.zran[1,0])?((self.zran[1,0] gt 0)?(self.zran[1,0]):(zmax)):(self.zran[1,0])
         if self.zran[1,2] le self.zran[0,2] then self.zran[1,2]=self.zran[0,2]*10.0
       endif
       if not finite(self.zmin) then self.zmin=self.zran[0,2*self.zlog]
       if not finite(self.zmax) then self.zmax=self.zran[1,2*self.zlog]
       if keyword_set(surfplot) then begin
         type=2   ;surface plot
         ;z axis
         if not obj_valid(self.drawzAxis) then begin
          self.drawzAxis  =    obj_new('IDLgrAxis',2,range=self.zran[*,0],title=self.ztit,name='axis',_extra=v6p0_extra,$
              color=white,ticklen=self.default[0],location=[-cshift,1-cshift,1],/exact)
          self.drawGroup->add,self.drawzAxis
         endif
         drawPlot =   obj_new('idlgrsurface',style=2,shading=shading,palette=drawPal,name='twod',_extra=v6p0_extra)
         self.title->getproperty,strings=title & obj_destroy,self.title
         self.title   = obj_new('IDLgrText',title,recompute_dimensions=2,font=self.font[4],/enable,_extra=v6p0_extra,$
          color=white,alignment=0.5,location=[0,1.05,0],name='titl')
         self.drawModel->add,self.title
       endif else begin
         type=1   ;contour plot
         if keyword_set(usepolygon) then begin
           ;use idlgrpolygon object instead of idlgrcontour object to speed up plotting
           drawPlot=obj_new('IDLgrPolygon',palette=drawPal,shading=shading,style=2,shade_range=[1,255],name='twod',$
            _extra=v6p0_extra)
         endif else begin
           drawPlot=obj_new('IDLgrContour',/fill,name='twod',c_color=1b+bindgen(255),palette=drawPal,shade_range=[1,255],$
              shading=shading,_extra=v6p0_extra)
         endelse
       endelse
       newplotInfo={dm_plot_info,obj:drawPlot,type:type,coltable:coltable,shading:shading,legend:'',color:mycolor,psym:'',symsize:0e,$
         thick:0e,linestyle:0,xPtr:ptr_new(xdat,no_copy=no_copy),yPtr:ptr_new(ydat,no_copy=no_copy),$
         zPtr:ptr_new(zdat,no_copy=no_copy),hide:0b}
       ;colorbar
       ;check if the colorbar axis exists
       if not obj_valid(self.drawxAxis[2]) then begin
         ;x
         self.drawxAxis[2]    =  obj_new('IDLgrAxis',0,range=[1.1,1.2]-cshift/2.,hide=self.hideIntbar,_extra=v6p0_extra,$
           color=white,ticklen=0.0,/exact,location=[1.1-cshift/2,-cshift/4.,0],/notext,name='cbar')     ;bottom
         self.drawxAxis[3]    =  obj_new('IDLgrAxis',0,range=[1.1,1.2]-cshift/2.,hide=self.hideIntbar,_extra=v6p0_extra,$
           color=white,ticklen=0.0,location=[1.2-cshift/2.,1-cshift/4.,0],/exact,name='cbar',/notext)   ;top
         self.drawModel->add,self.drawxAxis[2]
         self.drawModel->add,self.drawxAxis[3]
           ;y
         self.drawyAxis[2]    =  obj_new('IDLgrAxis',1,range=self.zran[*,0],/notext,minor=0,hide=self.hideIntbar,$
          color=white,ticklen=self.default[0],location=[1.1-cshift/2.,-cshift/4.,0],/exact,$
          _extra=v6p0_extra,name='cbar');left
         self.drawyAxis[3]    =  obj_new('IDLgrAxis',1,range=self.zran[*,0],color=white,$
          hide=self.hideIntbar,ticklen=self.default[0],/tickdir,location=[1.2-cshift/2.,-cshift/4.,0],/exact,$
          name='cbar',minor=0,textpos=1,_extra=v6p0_extra) ;right
         self.drawModel->add,self.drawyAxis[2]
         self.drawModel->add,self.drawyAxis[3]
         i_level  =self.zran[0,0]+findgen(256)/255.0*(self.zran[1,0]-self.zran[0,0])
         self.colBar=obj_new('IDLgrContour',transpose([[i_level],[i_level]]),geomy=i_level,geomz=-0.001,$
          geomx=[1.1,1.2]-cshift/2.,/plan,/fill,name='cbar',c_color=1b+bindgen(255),palette=drawPal,shading=shading,$
          hide=self.hideIntbar,_extra=v6p0_extra)
         self.drawModel->add,self.colBar
       endif
    endif else begin               ;1d data
       type=0    ;1d plot
       if n_elements(symsize) eq 0 then symsize=self.default[1] else symsize=(0.0>(symsize[0])<1.0)
       if n_elements(linestyle) eq 0 then linestyle=0 else linestyle=self->getlinestyle(linestyle[0])
       if n_elements(thick) eq 0 then thick=self.default[2] else thick=1.0>(thick[0])<10.0
       if n_elements(psym) eq 0 then psym='no symbol' else psym=self->getpsym(psym,/name)
       if n_elements(legend) eq 0 then legend=''
       if strlen(legend) eq 0 then legend=' '+(dm_to_string(color))[0]+' '+psym else legend=' '+legend
       if n_elements(layer) eq 0 then begin
           layer=n_elements(*self.plotInfo)
           if layer gt 0 then onedplot=where((*self.plotInfo)[*].type eq 0,layer)
       endif
       drawPsym=self->getpsym(psym,symsize,thick,mycolor)
       drawPlot=obj_new('IDLgrPlot',xdat,ydat,color=mycolor,symbol=drawPsym,linestyle=linestyle,zvalue=-0.0001*(1+layer),$
         /use_zvalue,thick=thick,xrange=self.xran[*,0],yrange=self.yran[*,0],name='plot',_extra=v6p0_extra)
       newplotInfo={dm_plot_info,obj:drawPlot,type:type,coltable:-1,shading:-1,legend:legend,color:mycolor,psym:psym,$
         symsize:symsize,thick:thick,linestyle:linestyle,xPtr:ptr_new(xdat),yPtr:ptr_new(ydat),zPtr:ptr_new(layer),hide:0b}
       if n_elements(yerr) eq n_elements(ydat) then begin
         nx     =n_elements(xdat)
         x      =fltarr(nx*2)
         y      =x
         con   =lonarr(nx*3)
         idx   =lindgen(nx)
         x[idx*2L]    =xdat
         x[idx*2L+1]  =xdat
         y[idx*2L]    =ydat-yerr
         y[idx*2L+1]  =ydat+yerr
         con[idx*3L]  =2
         con[idx*3L+1]=2L*idx
         con[idx*3L+2]=2L*idx+1      &idx=-1
         errorPlot=obj_new('idlgrpolyline',x,y,fltarr(nx*2)-0.0001*(0.5+layer),color=mycolor,$
          polylines=temporary(con),thick=thick,name='error',_extra=v6p0_extra)
         self.drawGroup->add,errorPlot
         newplotInfo=[newplotInfo,{dm_plot_info,obj:errorPlot,type:-1,coltable:-1,shading:-1,legend:'',color:mycolor,psym:'',$
          symsize:0e,thick:thick,linestyle:0,xPtr:ptr_new(x,/no_copy),yPtr:ptr_new(y,/no_copy),zPtr:ptr_new(layer),hide:0b}]
       endif
    endelse
    self.drawGroup->add,drawPlot
    if keyword_set(no_copy) then begin ;destroy xdat and ydat and zdat
       xdat=0 & tmp=temporary(xdat)
       ydat=0 & tmp=temporary(ydat)
       zdat=0 & tmp=temporary(zdat)
    endif
    if n_elements(*self.plotInfo) eq 0 then $
       *self.plotInfo=newplotInfo $
    else $
       *self.plotInfo=[*self.plotInfo,newplotInfo]
    if type eq 2 then begin    ;initial rotation for surface plot
       self.drawGroup->rotate,[1,0,0],-90
       self.drawGroup->rotate,[0,1,0],30
       self.drawGroup->rotate,[1,0,0],30
    endif

    onedplot=where((*self.plotInfo)[*].type eq 0,count0)
    twodplot=where((*self.plotInfo)[*].type gt 0,count1)
    sufplot =where((*self.plotInfo)[*].type eq 2,count2)
    ;allow(disallow) changing ztitle,zran,colortable, disallow(allow) changing color,psym,and linestyle
    sensitive  =    (count2 gt 0)
    id=widget_info(self.tlb,find_by_uname='ztitBut')
    if id gt 0 then widget_control,id,sensitive=sensitive
    sensitive  =    (count1 gt 0)
    id=widget_info(self.tlb,find_by_uname='zranBut')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='zlogBut')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='cs_coltable')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='cs_shade')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='cs_hidecbar')
    if id gt 0 then widget_control,id,sensitive=sensitive
    if sensitive then dm_set_button,id,self.hideIntbar,onstring='Show Colorbar',offstring='Hide Colorbar'
    sensitive  =    (count0 gt 0)
    id=widget_info(self.tlb,find_by_uname='pcolBut')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='psymBut')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='lineBut')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='lgndBut')
    if id gt 0 then widget_control,id,sensitive=sensitive
    id=widget_info(self.tlb,find_by_uname='cs_hidelegend')
    if id gt 0 then widget_control,id,sensitive=sensitive
    if sensitive then begin
       self.drawLegend->setproperty,hide=self.hideLegend
       dm_set_button,id,self.hideLegend,onstring='Show Legend',offstring='Hide Legend'
    endif else $
       self.drawLegend->setproperty,/hide
end


;draw the data using object graphics
pro dm_plot::draw,view=view
    widget_control,/hourglass
    if keyword_set(view) then begin ;show the view only
        self.drawWin->draw,self.drawView
        return
    endif
    widget_control,self.tlb,map=1
    widget_control,self.plotWin,/DRAW_EXPOSE_EVENTS
    nplot=n_elements(*self.plotInfo)
    if nplot eq 0 then return
    surfplot=where((*self.plotInfo)[*].type eq 2,count)
    if count gt 0 then cshift=[-0.5,0] else cshift=[0,0]

    if self.xlog eq 1 then xran=alog10(self.xran[*,2]) else xran=self.xran[*,0]
    if self.ylog eq 1 then yran=alog10(self.yran[*,2]) else yran=self.yran[*,0]
    if self.zlog eq 1 then zran=alog10(self.zran[*,2]) else zran=self.zran[*,0]

    ;set x,y coordinate conversions
    oChildArr = self.drawModel->IDL_Container::Get(/ALL, COUNT=nKids)
    for i=0,nKids-1 do begin
       if obj_isa(oChildArr[i],'IDLgrAxis') then begin   ;axis
         oChildArr[i]->getproperty,direction=dir,name=name,ticktext=tick
         tick->setproperty,font=self.font[0],recompute_dimensions=2
         oChildArr[i]->setproperty,ticktext=tick,thick=self.axisthick
         if dir eq 1 then $
          oChildArr[i]->setproperty,ycoord_conv=norm_coord(zran)+cshift/4.,range=self.zran[*,2*self.zlog],$
                 minor=(self.zlog eq 1)?(-1):0,log=self.zlog
       endif else if obj_isa(oChildArr[i],'IDLgrContour') then begin ;contour
               i_level    =zran[0]+findgen(256)/255.0*(zran[1]-zran[0])
         oChildArr[i]->setproperty,ycoord_conv=norm_coord(zran[*])+cshift/4.,$
          data_value=transpose([[i_level],[i_level]]),c_value=i_level,geomy=i_level,shading=max((*self.plotInfo)[*].shading)
       endif else if obj_isa(oChildArr[i],'IDLgrText') then begin    ;text
         oChildArr[i]->setproperty,font=self.font[4]
       endif
    endfor

    oChildArr = self.drawGroup->IDL_Container::Get(/ALL, COUNT=nKids)
    clipplane=[[-1,0,0,xran[0]],[1,0,0,-xran[1]],[0,-1,0,yran[0]],[0,1,0,-yran[1]]]
    for i=0,nKids-1 do begin
       if obj_isa(oChildArr[i],'IDLgrAxis') then begin
         oChildArr[i]->getproperty,direction=dir,name=name,ticktext=tick
         tick->setproperty,recompute_dimensions=2,font=self.font[0]
         oChildArr[i]->setproperty,ticktext=tick,thick=self.axisthick
         if dir eq 0 then begin
          oChildArr[i]->setproperty,xcoord_conv=norm_coord(xran)+cshift,range=self.xran[*,2*self.xlog],log=self.xlog
         endif else if dir eq 1 then begin
          oChildArr[i]->setproperty,ycoord_conv=norm_coord(yran)+cshift,range=self.yran[*,2*self.ylog],log=self.ylog
         endif else begin
          oChildArr[i]->setproperty,zcoord_conv=norm_coord(zran),range=self.zran[*,2*self.zlog],log=self.zlog
         endelse
       endif
    endfor
    for i=0,n_elements(*self.plotInfo)-1 do begin
       case (*self.plotInfo)[i].type of
         ;error bars,IDLgrPolyline object
         -1:  begin
          x   = (*(*self.plotInfo)[i].xPtr)
          y   = (*(*self.plotInfo)[i].yPtr)
          layer=  (*(*self.plotInfo)[i].zPtr)
          if (float(!version.release) lt 5.6) or (self.xlog eq 1) then begin
              index_x    =where((x ge self.xran[0,2*self.xlog]) and (x le self.xran[1,2*self.xlog]),count_x)
              if count_x gt 0 then begin
                 x =   x[index_x]
                 y =   y[index_x] & index_x=-1
              endif else begin
                 x =   (self.xran[1,2*self.xlog]+1e2)*[1.0,1.0]
                 y =   (self.yran[1,2*self.ylog])*[1.0,1.0]
                 con   = [2,0,1]
              endelse
          endif
          if (float(!version.release) lt 5.6) or (self.ylog eq 1) then begin
              index_y=where(y gt self.yran[1,2*self.ylog],count_y)
              if count_y ne 0 then y[index_y]=self.yran[1,2*self.ylog]
              index_y=where(y lt self.yran[0,2*self.ylog],count_y)
              if count_y ne 0 then y[index_y]=self.yran[0,2*self.ylog] & index_y=-1
          endif
          nx  =    n_elements(x)/2
          z   = fltarr(nx*2)-0.0001*(0.5+layer)
          con =   lonarr(nx*3)
          idx =   lindgen(nx)
          con[idx*3]=2
          con[idx*3+1]=2*idx
          con[idx*3+2]=2*idx+1
          if self.xlog eq 1 then x=alog10(x)
          if self.ylog eq 1 then y=alog10(y)
          x=transpose(x) & y=transpose(y) & z=transpose(z)
          (*self.plotInfo)[i].obj->setproperty,xcoord_conv=norm_coord(xran)+cshift,polylines=temporary(con),$
              ycoord_conv=norm_coord(yran)+cshift,data=[temporary(x),temporary(y),temporary(z)]
          end
         ;1d plot,IDLgrPlot object
         0:   begin
          x   = (*(*self.plotInfo)[i].xPtr)
          y   = (*(*self.plotInfo)[i].yPtr)
          if self.xlog eq 1 then begin
              index_x    =where((x ge self.xran[0,2*self.xlog]) and (x le self.xran[1,2*self.xlog]),count_x)
              if count_x gt 0 then begin
                 x =   x[index_x]
                 y =   y[index_x] & index_x=-1
              endif else begin
                 x =   self.xran[1,2]*10 ;fake data
                 y =   self.yran[1,2]*10
              endelse
          endif
          if self.ylog eq 1 then begin
              index_y    =where((y ge self.yran[0,2*self.ylog]) and (y le self.yran[1,2*self.ylog]),count_y)
              if count_y gt 0 then begin
                 x =   x[index_y]
                 y =   y[index_y] & index_y=-1
              endif else begin
                 x =   self.xran[1,2]*10 ;fake data
                 y =   self.yran[1,2]*10
              endelse
          endif
          if self.xlog eq 1 then x=alog10(x)
          if self.ylog eq 1 then y=alog10(y)
          (*self.plotInfo)[i].obj->setproperty,xcoord_conv=norm_coord(xran)+cshift,$
              ycoord_conv=norm_coord(yran)+cshift,xran=xran,yran=yran,datax=temporary(x),datay=temporary(y)
          end
         ;2d plot, IDLgrContour or IDLgrPolygon object
         1:   begin
          xdat    =  (*(*self.plotInfo)[i].xPtr)
          ydat    =  (*(*self.plotInfo)[i].yPtr)
          zdat    =  (*(*self.plotInfo)[i].zPtr)
          if (float(!version.release) lt 5.6) or (self.xlog eq 1) or (self.ylog eq 1) then begin
              ind_x  =    where((xdat ge self.xran[0,2*self.xlog]) and (xdat le self.xran[1,2*self.xlog]),count_x)
              ind_y  =    where((ydat ge self.yran[0,2*self.ylog]) and (ydat le self.yran[1,2*self.ylog]),count_y)
              if count_x le 1 or count_y le 1 then begin
                 xdat=self.xran[*,self.xlog*2]
                 ydat=self.yran[*,self.ylog*2]
                 zdat=fltarr(2,2) & zdat[*]=!values.f_nan
              endif else begin
                 xdat=xdat[ind_x]
                 ydat=ydat[ind_y]
                 zdat=zdat[ind_x,*] &zdat=zdat[*,ind_y]
              endelse
          endif
          if self.xlog eq 1 then xdat=alog10(xdat)
              if self.ylog eq 1 then ydat=alog10(ydat)
              if self.zlog eq 1 then begin
                tmp   =   where(finite(zdat),count)
                if count gt 0 then index=where(zdat[tmp] le 0,count)
                if count gt 0 then zdat[tmp[index]]=!values.f_nan
                zdat=alog10(zdat)
              endif
          if obj_isa((*self.plotInfo)[i].obj,'IDLgrPolygon') then begin
              nx =   n_elements(xdat)+1
                ny   = n_elements(ydat)+1
                xdat=    [xdat[0],(xdat[0:(0>(nx-3L))]+xdat[1:(nx-2L)])/2.,xdat[nx-2L]]
                ydat=    [ydat[0],(ydat[0:(0>(ny-3L))]+ydat[1:(ny-2L)])/2.,ydat[ny-2L]]
                conn=    lonarr(5L*(nx-1L)*(ny-1L))
                conn_idx =   lindgen((nx-1L)*(ny-1L))*5L
                vert_idx =   reform((lindgen(nx,ny))[0:(nx-2L),0:(ny-2L)],(nx-1L)*(ny-1L))
                conn[conn_idx]   = 4L
                conn[conn_idx+1]=    0L+vert_idx
                conn[conn_idx+2]=    1L+vert_idx
                conn[conn_idx+3]=    1L+vert_idx+nx
                conn[temporary(conn_idx)+4]= 0L+temporary(vert_idx)+nx
                dattype  =    max([size(xdat[0],/type),size(ydat[0],/type)])
                if dattype eq size(0d,/type) then begin  ;double
                 xdat=xdat#(dblarr(ny)+1d) & ydat=(dblarr(nx)+1d)#ydat
                endif else begin                  ;float
                 xdat=xdat#(fltarr(ny)+1e) & ydat=(fltarr(nx)+1e)#ydat
                endelse
                data=[[reform(temporary(xdat),nx*ny)],[reform(temporary(ydat),nx*ny)],[fltarr(nx*ny)-0.001]]
                data=transpose(data)
              zdat=  [[zdat],[zdat[*,(ny-2L)]]]
                zdat=    [zdat,zdat[(nx-2L),*]]
              vc =   intarr(n_elements(zdat))
              ;finite index
              index=where(finite(zdat),count)
              if count ne 0 then begin
                 vc[index]=1+round((zdat[index]-zran[0])/(zran[1]-zran[0])*254.)
                 index0=where(zdat[index] ge zran[1],count)
                 if count ne 0 then vc[index[index0]]=255
                 index0=where(zdat[index] lt zran[0],count)
                 if count ne 0 then vc[index[index0]]=0
                 index=-1 & index0=-1
              endif
              zdat   = -1
              (*self.plotInfo)[i].obj->setproperty,xcoord_conv=norm_coord(xran)+cshift,$
                 ycoord_conv=norm_coord(yran)+cshift,vert_colors=temporary(vc),data=temporary(data),polygons=temporary(conn)
          endif else begin    ;IDLgrContour
              nx =   n_elements(xdat)
                ny   = n_elements(ydat)
                xdat=(temporary(xdat))#(fltarr(ny)+1.) & ydat=(fltarr(nx)+1.)#(temporary(ydat))
              i_level    =zran[0]+findgen(256)/255.0*(zran[1]-zran[0])
                      i_level[255]=(self.zlog eq 1)?(alog10((self.zmax gt 0)?(self.zmax):(self.zran[1,2]))):(self.zmax)
                      (*self.plotInfo)[i].obj->setproperty,xcoord_conv=norm_coord(xran)+cshift,ycoord_conv=norm_coord(yran)+cshift,$
                         c_value=i_level,geomx=temporary(xdat),geomy=temporary(ydat),data_values=temporary(zdat),geomz=-0.001,/planar
          endelse
          end
         ;2d plot, IDLgrSurface object
         2:   begin
          xdat    =  (*(*self.plotInfo)[i].xPtr)
          ydat    =  (*(*self.plotInfo)[i].yPtr)
          zdat    =  (*(*self.plotInfo)[i].zPtr)
          if (float(!version.release) lt 5.6) or (self.xlog eq 1) or (self.ylog eq 1) then begin
              ind_x  =    where((xdat ge self.xran[0,2*self.xlog]) and (xdat le self.xran[1,2*self.xlog]),count_x)
              ind_y  =    where((ydat ge self.yran[0,2*self.ylog]) and (ydat le self.yran[1,2*self.ylog]),count_y)
              if count_x le 1 or count_y le 1 then begin
                 xdat=self.xran[*,self.xlog*2]
                 ydat=self.yran[*,self.ylog*2]
                 zdat=fltarr(2,2) & zdat[*]=!values.f_nan
              endif else begin
                 xdat=xdat[ind_x]
                 ydat=ydat[ind_y]
                 zdat=zdat[ind_x,*] &zdat=zdat[*,ind_y]
              endelse
          endif
          if self.xlog eq 1 then xdat=alog10(xdat)
              if self.ylog eq 1 then ydat=alog10(ydat)
              if self.zlog eq 1 then zdat=alog10(zdat)
          vc  =    intarr(n_elements(zdat))
          ;finite index
          index=where(finite(zdat),count)
          if count ne 0 then begin
              vc[index]=1+round((zdat[index]-zran[0])/(zran[1]-zran[0])*254.)
              index0=where(zdat[index] ge zran[1],count)
              if count ne 0 then vc[index[index0]]=255
              index0=where(zdat[index] lt zran[0],count)
              if count ne 0 then vc[index[index0]]=0
              index=-1 & index0=-1
          endif
          (*self.plotInfo)[i].obj->setproperty,xcoord_conv=norm_coord(xran)+cshift,ycoord_conv=norm_coord(yran)+cshift,$
              zcoord_conv=norm_coord(zran),vert_colors=temporary(vc),min_value=zran[0] ,max_value=zran[1],$
              datax=temporary(xdat),datay=temporary(ydat),dataz=temporary(zdat)
          end
         else:
       endcase
       if float(!version.release) ge 5.6 then $
         (*self.plotInfo)[i].obj->setproperty,clip_planes=clipplane
    endfor

    ;update the drawlegend
    onedplot=where((*self.plotInfo)[*].type eq 0,count)
    if count gt 0 then begin
        show=where((*self.plotInfo)[onedplot].hide eq 0,count)
        if count gt 0 then begin
            item_name    =(*self.plotInfo)[onedplot[show]].legend
            item_thick   =(*self.plotInfo)[onedplot[show]].thick
            item_linesty =(*self.plotInfo)[onedplot[show]].linestyle
            item_color   =(*self.plotInfo)[onedplot[show]].color
            item_object  =(*self.plotInfo)[onedplot[show]].obj
            for i=0,count-1 do begin
                item_object[i]->getproperty,symbol=tmp
                item_object[i]=tmp
            endfor
            self.drawLegend->setproperty,item_name=item_name,item_type=intarr(count),item_linestyle=item_linesty,$
                item_thick=item_thick,item_color=item_color,item_object=item_object
        endif
    endif; else self.drawLegend->setproperty,item_object=[obj_new()]

    ;set boundary
    self->set_view

    ;draw it
    self.drawWin->draw,self.drawView
end

;handle all events that are passed from dm_plot_event and require access to the internal variables
pro dm_plot::event,event
    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(!error_state.msg,/error)
       return
    end
    eventname=widget_info(event.id,/uname)
    case eventname of
       'keepBut':    begin
          self->setproperty,keep=1b
          sendevent={plot_keep,ID:self.tlb,TOP:self.group_leader,HANDLER:self.tlb,object:self}
          if obj_valid(self.parentobj) then $
                call_method,self.parenthandler,self.parentobj,sendevent $
          else if self.group_leader ne 0 then $
                widget_control,self.group_leader,send_event=sendevent,/no_copy
         end
       'mkcrBut':  begin
          self->setproperty,keep=0b
          sendevent={plot_make_current,ID:self.tlb,TOP:self.group_leader,HANDLER:self.tlb,object:self}
          if obj_valid(self.parentobj) then $
                call_method,self.parenthandler,self.parentobj,sendevent $
          else if self.group_leader ne 0 then $    ; send an event there
                widget_control,self.group_leader,send_event=sendevent,/no_copy
         end
       'show_all':   begin
          for i=0,n_elements(*self.plotInfo)-1 do begin
            (*self.plotInfo)[i].obj->setproperty,hide=0
            (*self.plotInfo)[i].hide=0
          endfor
          if obj_valid(self.title) then self.title->setproperty,hide=0
          self->draw
          widget_control,event.id,sensitive=0
         end
       'titlBut':    begin
          self.title->getproperty,strings=title
          new=dm_dialog_input('title:',title='Set title',xsize=150,default=title,dialog_parent=self.tlb)
          if new ne title then self->setproperty,title=new
         end
       'xranBut':    begin
          new=dm_dialog_input(['from:','to:'],title='Set xrange',xsize=100,default=float(self.xran[*,2*self.xlog]),/float,dialog_parent=self.tlb)
          if total(finite(new)) eq 2 then $
              if total(abs(new-self.xran[*,0])) ne 0 then $
                 if new[1] gt new[0] then  self->setproperty,xran=new
         end
       'xlogBut':    self->setproperty,xlog=(1b)-self.xlog
       'xtitBut':    begin
          self.xtit->getproperty,strings=xtit
          new=dm_dialog_input('x tit:',title='Set xtitle',xsize=150,default=xtit,dialog_parent=self.tlb)
          if new ne xtit then self->setproperty,xtit=new
         end
       'yranBut':    begin
          new=dm_dialog_input(['from:','to:'],title='Set yrange',xsize=100,default=float(self.yran[*,2*self.ylog]),/float,dialog_parent=self.tlb)
          if total(finite(new)) eq 2 then $
              if total(abs(new-self.yran[*,0])) ne 0 then $
                 if new[1] gt new[0] then self->setproperty,yran=new
         end
       'ylogBut':    self->setproperty,ylog=(1b)-self.ylog
       'ytitBut':    begin
          self.ytit->getproperty,strings=ytit
          new=dm_dialog_input('y tit:',title='Set ytitle',xsize=150,default=ytit,dialog_parent=self.tlb)
          if new ne ytit then self->setproperty,ytit=new
         end
       'zranBut':    begin
          new=dm_dialog_input(['from:','to:'],title='Set zrange',xsize=100,default=float(self.zran[*,2*self.zlog]),/float,dialog_parent=self.tlb)
          if total(finite(new)) eq 2 then $
              if total(abs(new-self.zran[*,0])) ne 0 then $
                 if new[1] gt new[0] then self->setproperty,zran=new
         end
       'zlogBut':    self->setproperty,zlog=(1b)-self.zlog
       'psymBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot=where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          all_psym=['no symbol','circle','bullet','plus','star','dot','diamond','triangle','square','cross','greater','less']
          name    =strarr(count*2) & default=fltarr(count*2) & is_droplist=bytarr(count*2) & old=strarr(count*2)
          for i=0,count-1 do begin
              name[(2*i):(2*i+1)]   =['line '+dm_to_string(i)+': symbol name','       '+'symbol size']
              default[2*i]          =0>(where(all_psym eq (*self.plotInfo)[onedplot[i]].psym))
              default[2*i+1]        =(*self.plotInfo)[onedplot[i]].symsize
              old[2*i]              =(*self.plotInfo)[onedplot[i]].psym
              old[2*i+1]            =dm_to_string((*self.plotInfo)[onedplot[i]].symsize)
              is_droplist[(2*i):(2*i+1)]=[1,0]
          endfor
          new=dm_dialog_input(name,title='Set plot symbol',default=default,is_droplist=is_droplist,$
              droplist_content=all_psym,xsize=100,dialog_parent=self.tlb)
          index=where(strlen(new) eq 0,count0)
          if count0 gt 0 then new[index]=old[index]
          changeid=-1
          for i=0,count-1 do $
              if (new[2*i] ne old[2*i]) or (new[2*i+1] ne old[2*i+1]) then begin
                 (*self.plotInfo)[onedplot[i]].psym=new[2*i]
                 (*self.plotInfo)[onedplot[i]].symsize=0.0>(dm_to_number(new[2*i+1]))<1.0
                 changeid=[changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'lineBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot=where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          all_lines=['solid','dotted','dashed','dash dot','dash dot dot dot','long dash','no line']
          name=strarr(count*2) & default=fltarr(count*2) & is_droplist=bytarr(count*2)+1b & old=strarr(count*2)
          for i=0,count-1 do begin
              name[(2*i):(2*i+1)]   =['line '+dm_to_string(i)+': linestyle','       '+'thickness']
              default[2*i]          =(*self.plotInfo)[onedplot[i]].linestyle
              default[2*i+1]        =(*self.plotInfo)[onedplot[i]].thick
              old[2*i]              =dm_to_string((*self.plotInfo)[onedplot[i]].linestyle)
              old[2*i+1]            =dm_to_string((*self.plotInfo)[onedplot[i]].thick)
              is_droplist[(2*i):(2*i+1)]=[1,0]
          endfor
          new=dm_dialog_input(name,title='Set line style',default=default,is_droplist=is_droplist,$
              droplist_content=all_lines,xsize=100,dialog_parent=self.tlb,/return_number)
          index=where(strlen(new) eq 0,count0)
          if count0 gt 0 then new[index]=old[index]
          changeid=-1
          for i=0,count-1 do $
              if (new[2*i] ne old[2*i]) or (new[2*i+1] ne old[2*i+1]) then begin
                 (*self.plotInfo)[onedplot[i]].linestyle=dm_to_number(new[2*i],/int)
                 (*self.plotInfo)[onedplot[i]].thick=1.0>(dm_to_number(new[2*i+1]))<10.0
                 changeid=[changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'pcolBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot=where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          all_col =['white','red','blue','magenta','green','cyan','yellow','grey','dark grey','dark red','dark blue',$
              'dark magenta','dark green','dark cyan','dark yellow','user define']
          name=strarr(count) & default=intarr(count) & is_droplist=bytarr(count)+1b
          old =self->getcolor((*self.plotInfo)[onedplot].color,/name)
          define=where(old eq 'user define',count0)
          if count0 ne 0 then define=(*self.plotInfo)[onedplot[define[0]]].color else define=[255,255,255]
          for i=0,count-1 do begin
              name[i]    = 'line '+dm_to_string(i)
              default[i] = 0>(where(all_col eq old[i]))
          endfor
          new=dm_dialog_input(name,title='Set plot color',default=default,is_droplist=is_droplist,$
              droplist_content=all_col,xsize=100,dialog_parent=self.tlb)
          changeid=-1
          for i=0,count-1 do $
              if new[i] ne old[i] then begin
                 (*self.plotInfo)[onedplot[i]].color=self->getcolor(new[i],define=define)
                 changeid=[changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'lgndBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot=where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          name='line '+dm_to_string(indgen(count)) & old=(*self.plotInfo)[onedplot].legend
          new=dm_dialog_input(name,title='Set legend labels',default=old,xsize=100,dialog_parent=self.tlb)
          changeid=-1
          for i=0,count-1 do $
              if strtrim(new[i],2) ne strtrim(old[i],2) then begin
                 (*self.plotInfo)[onedplot[i]].legend=' '+strtrim(new[i],2)
                 changeid=[changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'pnsetBut':   ok=dialog_printersetup(self.printer,dialog_parent=self.tlb)
       'plotWin':    begin
          if event.type eq 4 then self.drawWin->draw,self.drawView else begin
              if n_elements((*self.plotInfo)) eq 0 then break    ; no plots
              if event.release eq 4 then begin   ;right mouse button clicked
                 self.select   = obj_new()
                 tmp=self.drawWin->select(self.drawView,[event.x,event.y])
                 if obj_valid(tmp[0]) then begin
                   self.select=tmp[0]
                   tmp[0]->getproperty,name=name
                   if name eq 'error' then begin    ;special care to error plot
                    name=''
                    index=where((*self.plotInfo).obj eq self.select,count)
                    if count ne 0 then $
                        if (*self.plotInfo)[index[0]-1].type eq 0 then begin ;double check
                           name='plot'
                           self.select=(*self.plotInfo)[index[0]-1].obj
                        endif
                   endif
                   if strlen(name) ne 0 then begin
                    if self.select eq self.drawxAxis[1] then begin
                        name='titl' & self.select=self.title
                    endif
                    menuid=widget_info(event.id, find_by_uname='cs_'+name+'Menu')
                    logid =widget_info(menuid,find_by_uname='cs_logscale')
                    setlog=0
                    case name of    ;for axis and cbar, need to set properly the log scale button
                        'axis':begin
                           setlog=1
                           self.select->getproperty,direction=direction
                           case direction of
                             0:log_yn=self.xlog
                             1:log_yn=self.ylog
                             else:log_yn=self.zlog
                           endcase
                          end
                        'cbar':begin
                           setlog=1
                           log_yn=self.zlog
                            end
                        else:
                    endcase
                    if setlog eq 1 then dm_set_button,logid,log_yn,onstring='Use Normal Scale',offstring='Use Log Scale'
                    widget_displaycontextmenu,event.id,event.x,event.y,menuid
                   endif
                 endif else begin  ;turn off zoom
                   tmp  =    max((*self.plotInfo)[*].type)
                   if tmp eq 2 then begin
                    self.drawGroup->reset
                    self.drawModel->reset
                    self.drawGroup->rotate,[1,0,0],-90
                    self.drawGroup->rotate,[0,1,0],30
                    self.drawGroup->rotate,[1,0,0],30
                    self.drawWin->draw,self.drawView
                   endif
                   if (total(finite(self.xran[*,2*self.xlog+1])) eq 2) then begin
                    if tmp eq 2 then begin
                        self->setproperty,xran=self.xran[*,2*self.xlog+1],yran=self.yran[*,2*self.ylog+1],$
                           zran=self.zran[*,2*self.zlog+1]
                        self.zran[*,2*self.zlog+1]=[!values.f_nan,!values.f_nan]
                    endif else $
                        self->setproperty,xran=self.xran[*,2*self.xlog+1],yran=self.yran[*,2*self.ylog+1]
                    self.xran[*,2*self.xlog+1]=[!values.f_nan,!values.f_nan]
                    self.yran[*,2*self.ylog+1]=[!values.f_nan,!values.f_nan]
                   endif
                 endelse
              endif else begin
                 if max((*self.plotInfo)[*].type) eq 2 then $
                   self->rotate,event $
                 else $
                   self->zoom,event
              endelse
          endelse
         end
       'cs_title':   begin
          if obj_isa(self.select,'IDLgrAxis') then begin
              self.select->getproperty,title=title,direction=direction
              case direction of
                 0:   name='xtitle'
                 1:   name='ytitle'
                 else:name='ztitle'
              endcase
              if title eq self.title then name='title'
          endif else begin
              title  =self.select
              name   ='title'
          endelse
          if obj_valid(title) then begin
              title->getproperty,strings=old
              new    =dm_dialog_input(name+':',title='Set '+name,xsize=150,default=old,dialog_parent=self.tlb)
              if new ne old then begin
                 if name eq 'title' then self->setproperty,title=new $
                 else begin
                   title->setproperty,string=new
                   self.drawWin->draw,self.drawView
                 endelse
              end
          endif
         end
       'cs_range':   begin
          self.select->getproperty,name=name
          if name eq 'axis' then begin
              self.select->getproperty,direction=direction
              case direction of
                 0:type='x'
                 1:type='y'
                 else:type='z'
              endcase
          endif else  type='z'
          old=call_function("dm_to_string",(type eq 'x')?(self.xran[*,2*self.xlog]):$
            ((type eq 'y')?(self.yran[*,2*self.ylog]):(self.zran[*,2*self.zlog])))
          new=dm_dialog_input(['from:','to:'],title='Set '+type+'range',xsize=100,default=old,dialog_parent=self.tlb)
          if (old[0] ne new[0]) or (old[1] ne new[1]) then begin
              new=dm_to_number(new)
              if total(finite(new)) eq 2 then $
                 if new[1] gt new[0] then begin
                    case type of
                        'x':    self->setproperty,xran=new
                        'y':    self->setproperty,yran=new
                        else:   self->setproperty,zran=new
                    endcase
                 endif
          endif
         end
       'cs_logscale':begin
          self.select->getproperty,name=name
          if name eq 'axis' then begin
              self.select->getproperty,direction=direction
              case direction of
                 0:   type='xlog'
                 1:   type='ylog'
                 else:type='zlog'
              endcase
          endif else  type='zlog'
          case type of
             'xlog':  self->setproperty,xlog=1-self.xlog
             'ylog':  self->setproperty,ylog=1-self.ylog
             else:    self->setproperty,zlog=1-self.zlog
          endcase
         end
       'cs_axisthick':begin
          old=dm_to_string(self.axisthick)
          new=dm_dialog_input('thick:',title='Set axis thickness',xsize=100,default=old,dialog_parent=self.tlb)
          if old ne new then begin
              new=dm_to_number(new)
              if finite(new) then $
                 self->setproperty,axisthick=new
          end
         end
       'cs_font':begin
          index=0 ;for ticktext
          self.select->getproperty,name=name
          if name eq 'axis' then begin
              if self.select eq self.drawxAxis[0] then index=    1  $
              else if self.select eq self.drawxAxis[1] then index=4 $
              else if self.select eq self.drawyAxis[0] then index=2 $
              else if self.select eq self.drawzAxis then index=3
          endif else if name eq 'titl' then index=4 $
          else if name eq 'legend' then index=5
          self.font[index]->getproperty,name=old
          old      =strlowcase(old)
          all_fonts=['helvetica','courier','times','symbol','monospace symbol','hershey']
          default  =[0>(where(all_fonts eq old)),self.fsize[index]]
          old      =[old,dm_to_string(self.fsize[index])]
          new      =dm_dialog_input(['font name:','font size:'],title='Set font',default=default,is_droplist=[1,0],$
                 droplist_content=all_fonts,xsize=100,dialog_parent=self.tlb)
          ind      =where(strlen(new) eq 0,count)
          if count gt 0 then new[ind]=old[ind]
          redraw   =0
          if new[0] ne old[0] then begin
              self.font[index]->setproperty,name=new[0],substitute='Hershey'
              self.font[index]->getproperty,name=tmp
              if strmid(strlowcase(tmp),0,7) eq 'hershey' then self.fscale[index]=1.2 else self.fscale[index]=1.0
              redraw=1
          endif
          if new[1] ne old[1] then begin
              self.fsize[index]=(8>(dm_to_number(new[1],/int)))
              self.font[index]->setproperty,size=self.fsize[index]
              redraw=1
          endif
          if redraw then self->draw
         end
       'cs_symbol':begin
          index   = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          all_psym=['no symbol','circle','bullet','plus','star','dot','diamond','triangle','square','cross','greater','less']
          default =[0>(where(all_psym eq (*self.plotInfo)[index].psym)),(*self.plotInfo)[index].symsize]
          old     =[(*self.plotInfo)[index].psym,dm_to_string((*self.plotInfo)[index].symsize)]
          new=dm_dialog_input(['symbol name:','symbol size:'],title='Set plot symbol',default=default,$
              is_droplist=[1,0],droplist_content=all_psym,xsize=100,dialog_parent=self.tlb,$
              info='0.0<symbol size<1.0')
          if (new[0] ne old[0]) or (new[1] ne old[1]) then begin
              (*self.plotInfo)[index].psym=new[0]
              (*self.plotInfo)[index].symsize=0.0>(dm_to_number(new[1]))<1.0
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_color':   begin
          index   = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          all_col = ['white','red','blue','magenta','green','cyan','yellow','grey','dark grey','dark red','dark blue',$
              'dark magenta','dark green','dark cyan','dark yellow','user define']
          old     = self->getcolor((*self.plotInfo)[index].color,/name)
          new     = dm_dialog_input('color:',title='Set plot color',default=0>(where(all_col eq old)),$
              is_droplist=[1],droplist_content=all_col,xsize=100,dialog_parent=self.tlb,parentobject=self,$
              object=self.select,property='color',apply=apply)
          if (apply eq 1) or (new[0] ne old[0]) then begin
              (*self.plotInfo)[index].color=self->getcolor(new[0],define=(*self.plotInfo)[index].color)
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_linestyle':begin
          index   = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          all_lines=['solid','dotted','dashed','dash dot','dash dot dot dot','long dash','no line']
          old  = (*self.plotInfo)[index].linestyle
          new  = dm_dialog_input('linestyle:',title='Set plot linestyle',default=old,$
              is_droplist=[1],droplist_content=all_lines,xsize=100,dialog_parent=self.tlb,parentobject=self,$
              object=self.select,property='linestyle',apply=apply,/return_number)
          if (apply eq 1) or (new[0] ne old[0]) then begin
              (*self.plotInfo)[index].linestyle=new[0]
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_linethick':begin
          index   = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          old     =dm_to_string((*self.plotInfo)[index].thick)
          new     =dm_dialog_input('thickness:',title='Set plot thickness',default=(*self.plotInfo)[index].thick,$
                   xsize=70,dialog_parent=self.tlb,info='1.0<thick<10.0')
          if (new[0] ne old[0]) then begin
              (*self.plotInfo)[index].thick=1.0>(dm_to_number(new[0]))<10.0
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_legend':  begin
          index   = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          old     = strtrim((*self.plotInfo)[index].legend,2)
          new     = dm_dialog_input('legend:',title='Set plot legend',default=old,xsize=100,dialog_parent=self.tlb)
          if (new[0] ne old[0]) then begin
              (*self.plotInfo)[index].legend=' '+new[0]
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_layer':     begin
          index   = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          old     = (*(*self.plotInfo)[index].zPtr)
          new     = dm_dialog_input('layer:',title='Set plot layer',default=old,xsize=60,dialog_parent=self.tlb)
          if strlen(new) eq 0 then break
          if new ne old then begin
              new = dm_to_number(new,/int)
              old = dm_to_number(old,/int)
              nplots  = n_elements(*self.plotInfo)
              ;check if the new layer is already used, if yes, switch that one to old
              onedplot= where((*self.plotInfo)[*].type eq 0,count)
              index0  = -1
              for i=0,count-1 do if *(*self.plotInfo)[onedplot[i]].zPtr eq new then index0=onedplot[i]
              if index0 ne -1 then begin
                 *(*self.plotInfo)[index0].zPtr=old
                 changeid=[index0,index]
              endif else changeid=index
              *(*self.plotInfo)[index].zPtr=new
              self->setproperty,plotInfo=(*self.plotInfo),changeid=changeid
          endif
         end
       'cs_coltable':begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          twodplot=where((*self.plotInfo)[*].type gt 0,count) & if count eq 0 then break
          all_ctb =['B-W LINEAR','BLUE/WHITE','GRN-RED-BLU-WHT','RED TEMPERATURE','BLUE/GREEN/RED/YELLOW','STD GAMMA-II','PRISM',$
              'RED-PURPLE','GREEN/WHITE LINEAR','GRN/WHT EXPONENTIAL','GREEN-PINK','BLUE-RED','16 LEVEL','RAINBOW','STEPS',$
              'STERN SPECIAL','Haze','Blue-Pastel-Red','Pastels','Hue Sat Lightness 1','Hue Sat Lightness 2','Hue Sat Value 1',$
              'Hue Sat Value 2','Purple-Red+Stripes','Beach','Mac Style','Eos A','Eos B','Hardcandy','Nature','Ocean',$
              'Peppermint','plasma','Blue-Red','Rainbow','Blue Waves','Volcano','Waves','Rainbow18','Rainbow+white',$
              'Rainbow+black']
          all_ctb= dm_to_string(indgen(n_elements(all_ctb)))+'  '+all_ctb
          name   = 'plot '+dm_to_string(indgen(count)) & is_droplist=bytarr(count)+1b & old=(*self.plotInfo)[twodplot].coltable
          if count eq 1 then name=['color table']
          new    = dm_dialog_input(name,title='Set color table',default=old,is_droplist=is_droplist,droplist_content=all_ctb,$
              xsize=220,/return_number,/int,dialog_parent=self.tlb,parentobject=self,$
              object=(*self.plotInfo)[twodplot].obj,property='coltable',apply=apply,/coltable)
          changeid=-1
          for i=0,count-1 do begin
              if (apply eq 1) or (new[i] ne old[i]) then begin
                 (*self.plotInfo)[twodplot[i]].coltable=new[i]
                 changeid=[changeid,twodplot[i]]
              endif
          endfor
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'cs_shade':  begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          twodplot= where((*self.plotInfo)[*].type gt 0,count)        & if count eq 0 then break
          choice  = ['Flat','Gouraud']
          name    = 'plot '+dm_to_string(indgen(count)) & is_droplist=bytarr(count)+1b & old=(*self.plotInfo)[twodplot].shading
          if count eq 1 then name=['shading']
          new     = dm_dialog_input(name,title='Set shading',default=old,is_droplist=is_droplist,droplist_content=choice,$
                    xsize=100,/return_number,/int,dialog_parent=self.tlb)
          changeid= -1
          for i=0,count-1 do $
              if new[i] ne old[i] then begin
                 (*self.plotInfo)[twodplot[i]].shading=new[i]
                 changeid=[changeid,twodplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'cs_hidecbar':self->setproperty,hideintbar=(1b)-self.hideIntbar
       'cs_hidelegend':self->setproperty,hidelegend=(1b)-self.hideLegend
       'cs_outline':begin
          widget_control,event.id,get_value=value
          if value eq 'Show Outline' then begin
              value  ='Hide Outline'
              show   =1
          endif else begin
              value  ='Show Outline'
              show   =0
          endelse
          widget_control,event.id,set_value=value
          self.select->setproperty,show_outline=show
          self.drawWin->draw,self.drawView
         end
       'cs_lgndgap':begin
          self.select->getproperty,gap=old
          new=dm_dialog_input('legend gap:',title='Set Legend Gap',default=old,/float,dialog_parent=self.tlb,$
              info='border gap=2*gap')
          if finite(new) then $
              if new ne old then begin
                 self.select->setproperty,gap=new,border_gap=new*2
                 self.drawWin->draw,self.drawView
              endif
         end
       'cs_lgndwidth':begin
          self.select->getproperty,glyph_width=old
          new=dm_dialog_input('legend glyph width:',title='Set Legend Glyph Width',default=old,/float,dialog_parent=self.tlb)
          if finite(new) then $
              if new ne old then begin
                 self.select->setproperty,glyph_width=new
                 self.drawWin->draw,self.drawView
              endif
         end
       'cs_propsheet':begin
          invisible=total((*self.plotinfo)[*].hide)
          self.drawLegend->getproperty,item_color=item_color,item_linestyle=item_linestyle,hide=hide
          propsheet=obj_new('dm_propsheet',self.select,parentobject=self,/view) ;a modal interactive dialog widet
          obj_destroy,propsheet
          case strlowcase(obj_class(self.select)) of
            'idlgrplot':begin
                self.select->getproperty,symbol=symbol,color=color,linestyle=linestyle,thick=thick,hide=hide
                symbol->getproperty,data=data
                if obj_valid(data) then data->setproperty,color=color
                symbol->setproperty,color=color     ;change symbol color
                ;change legend properties
                onedplot= where((*self.plotInfo)[*].type eq 0)
                show    = where((*self.plotInfo)[onedplot].hide eq 0)
                onedplot= onedplot[show]
                changeid= where((*self.plotInfo)[*].obj eq self.select,count)
                if count eq 0 then break
                (*self.plotInfo)[changeid].color    =   color
                (*self.plotInfo)[changeid].linestyle=   linestyle
                (*self.plotInfo)[changeid].thick    =   thick
                (*self.plotInfo)[changeid].hide     =   hide
                nplots  = n_elements(*self.plotInfo)
                i_num   = where(onedplot eq changeid[0],count)
                if count ne 0 then begin
                 item_color[*,i_num[0]]   = color
                 item_linestyle[i_num[0]] = linestyle
                endif
                ;change the color and thickness of the error plot if any
                if changeid[0]+1 lt nplots then $
                 if (*self.plotInfo)[changeid[0]+1].type eq -1 then begin
                   (*self.plotInfo)[changeid[0]+1].obj->setproperty,thick=thick,color=color,hide=hide
                   (*self.plotInfo)[changeid[0]+1].color    =   color
                   (*self.plotInfo)[changeid[0]+1].linestyle=   linestyle
                   (*self.plotInfo)[changeid[0]+1].thick    =   thick
                   (*self.plotInfo)[changeid[0]+1].hide     =   hide
                 endif
                end
            'idlgrlegend':begin
                ;check if the show status has been changed
                self.select->getproperty,hide=newhide
                if newhide ne hide then self->setproperty,hidelegend=newhide
                end
            'idlgrtext':begin
                self.select->getproperty,hide=hide
                if hide eq 1 then begin
                    wid=widget_info(self.tlb,find_by_uname='show_all')
                    if wid ne 0 then widget_control,wid,sensitive=1
                endif
                end
            else:
          endcase
          self.drawLegend->setproperty,item_color=item_color,item_linestyle=item_linestyle
          newinvisible=total((*self.plotinfo)[*].hide)
          if newinvisible ne invisible then begin
            self->draw
            wid=widget_info(self.tlb,find_by_uname='show_all')
            if wid ne 0 then widget_control,wid,sensitive=1-(newinvisible eq 0)
          endif else $
            self->draw,/view
         end
       else:
    endcase
end

function dm_plot::getcolor,color,scalar=scalar,name=name,define=define
    ncolor=n_elements((keyword_set(name))?(color[0,*]):color)
    if keyword_set(name) then begin
        if ncolor eq 0 then return,'user define'
        result=strarr(ncolor)
        for i=0,ncolor-1 do begin
            index=color[0,i]+256L*color[1,i]+256L*256L*color[2,i]
            case index of
                0L:         name='black'
                16777215L:  name='white'
                255L:       name='red'
                16711680L:  name='blue'
                16711935L:  name='magenta'
                65280L:     name='green'
                16776960L:  name='cyan'
                65535L:     name='yellow'
                12632256L:  name='grey'
                8421504L:   name='dark grey'
                128L:       name='dark red'
                8388608L:   name='dark blue'
                8388736L:   name='dark magenta'
                32768L:     name='dark green'
                8421376L:   name='dark cyan'
                32896L:     name='dark yellow'
                else:       name='user define'
            endcase
            result[i]=name
        endfor
        return,(ncolor eq 1)?result[0]:result
    endif else begin
        if n_elements(define) ne 3 then define=[255b,255b,255b] else define=byte(define)
        if ncolor eq 0 then $
            if keyword_set(scalar) then return,16777215L else return,define
        if size(color,/type) ne 7 then begin
            if keyword_set(scalar) and n_elements(color) eq 3 then return,color[0]+256L*color[1]+256L*256L*color[2] $
            else return,color
        endif
        if keyword_set(scalar) then result=lonarr(ncolor) else result=bytarr(3,ncolor)
        for i=0,ncolor-1 do begin
            case strlowcase(strtrim(color[i],2)) of
                'white'   :      tmp=[255b,255b,255b]
                'red' :          tmp=[255b,0b,0b]
                'blue'    :      tmp=[0b,0b,255b]
                'magenta':       tmp=[255b,0b,255b]
                'green':         tmp=[0b,255b,0b]
                'cyan':          tmp=[0b,255b,255b]
                'yellow':        tmp=[255b,255b,0b]
                'grey':          tmp=[192b,192b,192b]
                'dark grey':     tmp=[128b,128b,128b]
                'dark red':      tmp=[128b,0b,0b]
                'dark blue':     tmp=[0b,0b,128b]
                'dark magenta':  tmp=[128b,0b,128b]
                'dark green':    tmp=[0b,128b,0b]
                'dark cyan':     tmp=[0b,128b,128b]
                'dark yellow':   tmp=[128b,128b,0b]
                'black':         tmp=[0b,0b,0b]
                'user define':   tmp=define
                else:            tmp=[255b,255b,255b]
            endcase
            if keyword_set(scalar) then result[i]=tmp[0]+256L*tmp[1]+256L*256L*tmp[2] else result[*,i]=tmp
        endfor
    endelse
    if keyword_set(scalar) or keyword_set(name) then return,(ncolor eq 1)?result[0]:result $
    else return, (ncolor eq 1)?result[*,0]:result
end

function dm_plot::getlinestyle,linestyle,name=name
    nline  =    n_elements(linestyle)
    if nline eq 0 then return,keyword_set(name)?'solid':0
    if keyword_set(name) then begin
        lines=strarr(nline)
        for i=0,nline-1 do begin
          case linestyle[i] of
            0:                    lines[i]='solid'
            1:                    lines[i]='dotted'
            2:                    lines[i]='dashed'
            3:                    lines[i]='dash dot'
            4:                    lines[i]='dash dot dot dot'
            5:                    lines[i]='long dash'
            6:                    lines[i]='no line'
            else:                 lines[i]='solid'
          endcase
        endfor
    endif else begin
        if size(linestyle,/type) ne 7 then return,linestyle
        lines  =    lonarr(nline)
        for i=0,nline-1 do begin
          case strlowcase(strtrim(linestyle[i],2)) of
            'solid':              lines[i]=0L
            'dotted':             lines[i]=1L
            'dashed':             lines[i]=2L
            'dash dot':           lines[i]=3L
            'dash dot dot dot':   lines[i]=4L
            'long dash':          lines[i]=5L
            'no line':            lines[i]=6L
            else:                 lines[i]=0L
          endcase
        endfor
    endelse
    return,(nline eq 1)?lines[0]:lines
end

function dm_plot::getpalette,coltable,background=background,oldpal=oldpal
    if n_elements(oldpal) ne 0 then drawPal=oldpal $
    else drawPal=obj_new('IDLgrPalette',name='palette')
    ;save the old color table and restore it later to avoid affecting other programs running at the same time
    tvlct,old_r,old_g,old_b,/get
    loadct,coltable,/silent
    stretch,1,255  ;0 left for background color
    tvlct,r,g,b,/get
    if n_elements(background) eq 3 then begin
       r[0]=fix(background[0],type=1)
       g[0]=fix(background[0],type=1)
       b[0]=fix(background[0],type=1)
    endif else begin
       r[0]=0b
       g[0]=0b
       b[0]=0b
    endelse
    drawPal->setproperty,red_values=temporary(r),green_values=temporary(g),blue_values=temporary(b)
    ;restore the old color table
    tvlct,old_r,old_g,old_b
    return,drawPal
end

function dm_plot::getpsym,symbol,symsize,thick,color,oldsymbol=oldsymbol,name=name
    if keyword_set(name) then begin
        if size(symbol,/type) eq 7 then return,symbol $
        else begin
            case symbol[0] of
                0:      result='no symbol'
                1:      result='plus'
                2:      result='star'
                3:      result='dot'
                4:      result='diamond'
                5:      result='triangle'
                6:      result='square'
                7:      result='cross'
                8:      result='greater'
                9:      result='less'
                else:   result='no symbol'
            endcase
        endelse
        return,result
    endif
    if n_elements(oldsymbol) ne 0 then begin
       psym=oldsymbol
       psym->getproperty,data=tmp
       if obj_valid(tmp) then obj_destroy,tmp
    endif else begin
        psym=obj_new('IDLgrSymbol')
    endelse
    if n_elements(symsize) eq 0 then symsize=self.default[1]
    ;modify the symbol size to achieve isotropic effect
    self.drawView->getproperty,dimensions=dim, viewplane_rect=view_rect
    dim       = dim/view_rect[2:3]
    if self.xlog eq 1 then xran=alog10(self.xran[*,2]) else xran=self.xran[*,0]
    if self.ylog eq 1 then yran=alog10(self.yran[*,2]) else yran=self.yran[*,0]
    tmpsize    =  symsize*[xran[1]-xran[0],yran[1]-yran[0]]*max(dim)/dim
    if n_elements(thick) eq 0 then thick=self.default[2]
    if n_elements(color) eq 0 then color='white'
    if size(symbol,/type) ne 7 then data=symbol[0] $
    else begin
        case strlowcase(strtrim(symbol,2)) of
            'circle':   data=obj_new('idlgrplot',cos(findgen(41)/40.0*!pi*2),sin(findgen(41)/40.0*!pi*2),$
                            linestyle=0,color=self->getcolor(color),thick=thick)
            'bullet':   data=obj_new('idlgrcontour',fltarr(41)+1.0,geomx=cos(findgen(41)/40.0*!pi*2),geomz=0,/plan,$
                            geomy=sin(findgen(41)/40.0*!pi*2),c_color=0,color=self->getcolor(color),c_value=[0,1],/fill)
            'plus':     data=1
            'star':     data=2
            'dot':      data=3
            'diamond':  data=4
            'triangle': data=5
            'square':   data=6
            'cross':    data=7
            'greater':  data=8
            'less':     data=9
            else:       data=0  ;no symbol
        endcase
    endelse
    psym->setproperty,data=data,size=tmpsize,thick=thick
    return,psym
end

function dm_plot::getproperty,group_leader=group_leader,keep=keep,parenthandler=parenthandler,parentobj=parentobj,tlb=tlb,error=error
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(dialog_parrent=self.tlb,!error_state.msg,/error)
       error=1  ;indicating that an error occurs
       return,0
    end
    error=0
    if keyword_set(group_leader) then return, self.group_leader
    if keyword_set(keep) then return, self.keep
    if keyword_set(parentobj) then return,self.parentobj
    if keyword_set(parenthandler) then return,self.parenthandler
    if keyword_set(tlb) then return, self.tlb
    error=1
end

pro dm_plot::getproperty,group_leader=group_leader,keep=keep,parenthandler=parenthandler,parentobj=parentobj,tlb=tlb,error=error
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(dialog_parrent=self.tlb,!error_state.msg,/error)
       error=1  ;indicating that an error occurs
       return
    end
    error=0
    if arg_present(group_leader) then group_leader=self.group_leader
    if arg_present(keep) then keep=self.keep
    if arg_present(parenthandler) then parenthandler=self.parenthandler
    if arg_present(parentobj) then parentobj=self.parentobj
    if arg_present(tlb) then tlb=self.tlb
end

;read in a one d 2 or 3-column file and add it to the plot
pro dm_plot::read1dfile
    widget_control,/hourglass
    defsysv,'!dave_defaults',exists=exists
    if exists then $
       if ptr_valid(!dave_defaults) then cd,(*!dave_defaults).workDir

    file=dialog_pickfile(dialog_parent=self.tlb,/read,/must_exist)
    if strlen(file) eq 0 then return

    openr,unit,file,/get_lun,error=error
    if error ne 0 then begin
       ok=dialog_message("Can't read file "+file+'.',dialog_parent=self.tlb)
       return
    endif

    ;catch and clear possible io errors
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(dialog_parent=self.tlb,!error_state.msg,/error)
       if n_elements(unit) ne 0 then free_lun,unit
       return
    end

    legend=strsplit(file,':\/',/extract)
    legend=legend[n_elements(legend)-1]

    tmp=''
    while not eof(unit) do begin
       readf,unit,tmp
       tmp1=strsplit(tmp,'   ,',/extract)
       tmp1=dm_to_number(tmp1,/float)
       if total(finite(tmp1)) ne 0 then $
         if n_elements(data) eq 0 then data=tmp1 else data=[[data],[tmp1]]
    endwhile
    free_lun,unit
    if n_elements(data) ne 0 then begin
       if n_elements(data[*,0]) eq 2 then $
         self->add_plot,reform(data[0,*]),reform(data[1,*]),legend=legend $
       else if n_elements(data[*,0]) ge 3 then $
         self->add_plot,reform(data[0,*]),reform(data[1,*]),yerr=reform(data[2,*]),legend=legend
       self->draw
    endif
end

pro dm_plot::rotate,event
    if self.moveLegend eq 0 then begin
       haveTransform=self.trackBall->update(event,transform=qmat)
       if (haveTransform ne 0) then begin
         self.drawGroup->getproperty,transform=t
         self.drawGroup->setproperty,transform=t#qmat
         self.drawWin->draw,self.drawView
       endif
    endif
    case event.type of
       0:    begin  ;button press
         case event.press of
          1:  begin    ;left button down
                 widget_control,self.plotWin,draw_motion_events=1
                 self.drawView->getproperty,dimension=dim
                 xstart    =  self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
                 ystart    =  self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
                 tmp=self.drawWin->select(self.drawView,[event.x,event.y])
                 if obj_valid(tmp[0]) then $
                   if obj_isa(tmp[0],'IDLgrLegend') then begin
                    self.moveLegend=1b
                    self.drawLegend->getproperty,xcoord_conv=x,ycoord_conv=y
                    self.legdOffset=[x[0]-xstart,y[0]-ystart]
                    break
                   endif
              end
          else:
         endcase
         end
       1:    begin   ;button release
          widget_control,self.plotWin,draw_motion_events=0
          self.moveLegend=0b
         end
       2:    if self.moveLegend then begin
          self.drawView->getproperty,dimension=dim
          xend    =  self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
          yend    =  self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
          self.drawLegend->setproperty,xcoord_conv=[xend+self.legdOffset[0],1],ycoord_conv=[yend+self.legdOffset[1],1]
          self.drawWin->draw,self.drawView
         endif
       else:
    endcase
end

pro dm_plot::resize,event
    mbar_y=0
    if widget_info(self.mbarSep,/valid_id) then begin
       widget_control,self.mbarSep,xsize=event.x
       mbar_y    =  widget_info(self.mbarSep,/geometry)
       mbar_y    =  mbar_y.ysize
    endif
    widget_control,self.plotWin,xsize=event.x,ysize=event.y-mbar_y
    self.drawView->setproperty,dimension=[event.x,event.y-mbar_y]
    self.tranView->setproperty,dimension=[event.x,event.y-mbar_y]
    ;self->draw
    self->set_view
    self.drawWin->draw,self.drawView
end

;save plot as images or print
pro dm_plot::saveas,type
    self.drawView->getproperty,dimension=dim
    self.drawView->setproperty,dimension=self.default[5:6]
    self->set_view,/save   ;reverse black and white color
    resolution=[0.018,0.018]
    if (type ne 'printer') and (type ne 'ps') then begin
       buffer    =  obj_new('IDLgrBuffer',dimension=self.default[5:6],color_model=0,resolution=resolution)
       buffer->draw,self.drawView
       buffer->getproperty,image_data=image
       obj_destroy,buffer
    endif
    defsysv,'!dave_defaults',exists=exists
    if exists then $
       if ptr_valid(!dave_defaults) then cd,(*!dave_defaults).workDir
    case type of
       'printer':    begin
          ok=dialog_printersetup(self.printer,dialog_parent=self.tlb)
          if not ok then break
          self.printer->getproperty,dimensions=printerDim
          windowAspect    =float(dim[1])/dim[0]
          printerAspect   =float(printerDim[1])/printerDim[0]
          margin=0.1
          if windowAspect le printerAspect then begin
              xstart=margin
              ystart=0.5-(0.5-margin)*(windowAspect/printerAspect)
              xend  =1.0-margin
              yend  =1.0-ystart
          endif else begin
              xstart=0.5-(0.5-margin)*(printerAspect/windowAspect)
              ystart=margin
              xend  =1.0-xstart
              yend  =1.0-margin
          endelse
          self.drawView->setproperty,dimensions=[xend-xstart,yend-ystart],$
              location=[xstart,ystart],units=3
          self.printer->newpage
          self.printer->draw,self.drawView
          self.printer->newdocument
          end
       'ps': begin
          cd,current=current
          defsysv,'!dave_defaults',exists=exists
              if exists then $
                if ptr_valid(!dave_defaults) then current=(*!dave_defaults).workDir
          deviceKeywords=PSConfig(cancel=canceled,group_leader=self.tlb,/inch,dir=current,xsize=5.5,xoffset=1.5)
          if canceled ne 1 then begin
              if deviceKeywords.inches eq 0 then begin
                 deviceKeywords.inches =1
                 deviceKeywords.xsize  =deviceKeywords.xsize/2.54   ;cm->inch
                 deviceKeywords.xoffset    =deviceKeywords.xoffset/2.54
                 deviceKeywords.ysize  =deviceKeywords.ysize/2.54   ;cm->inch
                 deviceKeywords.yoffset    =deviceKeywords.yoffset/2.54
              endif
              if deviceKeywords.encapsulated eq 1 then begin
                 deviceKeywords.xoffset    =0.0
                 deviceKeywords.yoffset    =0.0
              endif
              location   =[deviceKeywords.xoffset,deviceKeywords.yoffset]/$
                     [deviceKeywords.xsize+deviceKeywords.xoffset, $
                      deviceKeywords.ysize+deviceKeywords.yoffset]
              self.drawView->setproperty,dimensions=[1.0,1.0]-location,location=location,units=3
              dimensions =[deviceKeywords.xsize+deviceKeywords.xoffset,$
                      deviceKeywords.ysize+deviceKeywords.yoffset]*2.54/resolution
              widget_control,/hourglass  ;busy signal
              buffer =   obj_new('IDLgrClipboard',quality=2,dimensions=dimensions,resolution=resolution)
              buffer->draw,self.drawView,/vector,/postscript,filename=deviceKeywords.filename
              obj_destroy,buffer
          endif
          end
       'ascii': begin ;ascii file
          twodplot=where((*self.plotInfo)[*].type gt 0,count)
          if count eq 1 then begin
            save_file=dialog_pickfile(/write,filter='*.spe',dialog_parent=self.tlb)
            openw,unit,save_file,/get_lun,error=openerr
            if openerr ne 0 then begin
                ok=dialog_message("Can't write in "+save_file[0],/error,dialog_parent=parent)
            endif else begin
                x   =   *(*self.plotInfo[twodplot[0]]).xPtr
                y   =   *(*self.plotInfo[twodplot[0]]).yPtr
                z   =   *(*self.plotInfo[twodplot[0]]).zPtr
                itmp=   where(finite(z) eq 0,count)
                if count ne 0 then z[itmp]=0.0
                dz  =   z*0.05
                widget_control,/hourglass  ;busy signal
                help,x,y,z
                dm_write_spe,unit,y,x,z,dz
                free_lun,unit
            endelse
          endif else begin
            ok=dialog_message(dialog_parent=self.tlb,'There is no 2-d data in the plot.')
          endelse
          end
       else:     begin
          filter='*.'+type
          if type eq 'jpeg' then filter=[filter,'*.jpg']
          file    =  dialog_pickfile(/write,filter=filter,dialog_parent=self.tlb)
          if strlen(strtrim(file,2)) eq 0 then $
              file='idl.'+type $
          else begin
              tmp=strsplit(file,'.',/extract)
              if n_elements(tmp) eq 1 then file=tmp[0]+'.'+type
          endelse
          id=where(['bmp','jpeg','png','ppm','srf','tiff','dicom'] eq type,count)
          widget_control,/hourglass   ;busy signal
          if count ne 0 then write_image,file,type,image
          end
    endcase
    self.drawView->setproperty,dimension=dim,unit=0,location=[0,0]
    self->set_view
end

;set property
;usuage:
;   dm_plot->setproperty,property=value
;     or
;   dm_plot->setproperty,object=object,property=property,value=value
;parameters:
;   NONE
;keyword:
;   axisthick:          axis thickness for all axes
;   changeid:           changed id of plotinfo,internal call
;   color:              plot color
;   coltable:           color table
;   error:              1 if error occurs    0 operation successful
;   font:               font name
;   fsize:              font size
;   hideintnbar:        if set, hide the intensity bar
;   hidelegend:         if set, plot legends are not shown
;   iso:                1:isotropic plot 0:ratioed to window
;   keep:               1 or 0
;   layer:              layer of 1d plot
;   legend:             legend for the current plot
;   linestyle:          linestyle for the current plot
;   misscol:            missing data color for saving 2d plots 0:black, 1:white
;   no_copy:            to save memory
;   nodraw:             if set, don't draw the view
;   object:             for setting properties from dm_dialog_input apply button
;   plotInfo:           plot inforamtion,internal call
;   psym:               symbol number or symbol object
;   shading:            shading for object graphics
;   surfplot:           if set, 2d data are plot as surface rather than contour
;   symsize:            symbol size
;   thick:              line thickness for plots and symbols
;   title:              title of the plot
;   usepolygon:         if set, use idlgrpolygon to draw contour plots instead of idlgrcontour
;   xdat,ydat,zdat:     x,y,z data
;   xlog,ylog,zlog:     log10 scale x,y,z respectively
;   xran,yran,zran:     x,y,z range
;   xtit,ytit,ztit:     x,y,z axis title string
;   yerr:               error bar of y
;   zmin,zmax:          zmin, zmax for contour plots
pro dm_plot::setproperty,axisthick=axisthick,changeid=changeid,color=color,coltable=coltable,font=font,fsize=fsize,$
    hideintbar=hideintbar,hideLegend=hideLegend,iso=iso,keep=keep,layer=layer,legend=legend,linestyle=linestyle,misscol=misscol,$
    no_copy=no_copy,nodraw=nodraw,object=object,plotInfo=plotInfo,psym=psym,shading=shading,surfplot=surfplot,symsize=symsize,$
    thick=thick,title=title,usepolygon=usepolygon,xdat=xdat,ydat=ydat,zdat=zdat,yerr=yerr,xlog=xlog,ylog=ylog,zlog=zlog,$
    xran=xran,yran=yran,zran=zran,xtit=xtit,ytit=ytit,ztit=ztit,zmin=zmin,zmax=zmax,error=error,_extra=extra
    error=0.0
    redraw=1
    widget_control,/hourglass
    if (n_elements(object) ne 0) then begin
        if n_elements(coltable) ne 0 then begin
            for i=0,n_elements(object)-1 do begin
                object->getproperty,palette=drawPal
                drawPal=self->getpalette(dm_to_number(coltable[0],/int),oldpal=drawPal)
            endfor
        endif
        if n_elements(color) ne 0 then begin
            for i=0,n_elements(object)-1 do begin
                object->setproperty,color=self->getcolor(color)
            endfor
        endif
        if n_elements(linestyle) ne 0 then begin
            for i=0,n_elements(object)-1 do begin
                object->setproperty,linestyle=self->getlinestyle(linestyle)
            endfor
        endif
        self.drawWin->draw,self.drawView
        return
    endif
    if n_elements(keep) ne 0 then begin
       mkmenu = widget_info(self.tlb,find_by_uname='mkcrmenu')
       mkbut  = widget_info(self.tlb,find_by_uname='mkcrBut')
       kpmenu = widget_info(self.tlb,find_by_uname='keepmenu')
       kpbut  = widget_info(self.tlb,find_by_uname='keepBut')
       self.keep=(0b)>(fix(keep,type=1))<(1b)
       if self.keep then begin
         widget_control,kpbut,sensitive=0
         widget_control,kpmenu,sensitive=0
         widget_control,mkmenu,sensitive=1
         widget_control,mkbut,sensitive=1
       endif else begin
         widget_control,kpmenu,sensitive=1
         widget_control,kpbut,sensitive=1
         widget_control,mkbut,sensitive=0
         widget_control,mkmenu,sensitive=0
       endelse
       redraw=0  ;no need to redraw
    end
    if n_elements(misscol) ne 0 then begin
       blackbut=widget_info(self.tlb,find_by_uname='blackbg')
       whitebut=widget_info(self.tlb,find_by_uname='whitebg')
       self.misscol=0>(misscol)<1
       if self.misscol eq 0 then dm_toggle_menubut,blackbut,whitebut $
       else dm_toggle_menubut,whitebut,blackbut
       redraw=0  ;no need to redraw
    endif
    if n_elements(xlog) ne 0 then begin
       new=(0b>(byte(xlog))<1b)
       if self.xlog ne new then begin
         self.xlog= new
         logid    = widget_info(self.tlb,find_by_uname='xlogBut')
         if logid gt 0 then dm_set_button,logid,new,onstring='X Normal Scale',offstring='X Log Scale'
         redraw   = 2 ;need to reset view
       endif
    endif
    if n_elements(ylog) ne 0 then begin
       new=(0b>(byte(ylog))<1b)
       if self.ylog ne new then begin
         self.ylog= new
         logid    = widget_info(self.tlb,find_by_uname='ylogBut')
         if logid gt 0 then dm_set_button,logid,new,onstring='Y Normal Scale',offstring='Y Log Scale'
         redraw   = 2 ;need to reset view
       endif
    endif
    if n_elements(zlog) ne 0 then begin
       new=(0b>(byte(zlog))<1b)
       if self.zlog ne new then begin
         self.zlog= new
         logid    = widget_info(self.tlb,find_by_uname='zlogBut')
         if logid gt 0 then dm_set_button,logid,new,onstring='Z Normal Scale',offstring='Z Log Scale'
         redraw   = 2 ;need to reset view
       endif
    endif
    if n_elements(plotInfo) ne 0 then begin
       nplots  = n_elements(plotInfo)
       onedplot= where(plotInfo[*].type eq 0)
       if n_elements(changeid) eq 0 then changeid=indgen(nplots)
       self.drawLegend->getproperty,item_color=item_color,item_linestyle=item_linestyle,item_name=item_name,item_thick=item_thick
       for i=0,n_elements(changeid)-1 do begin
         switch plotInfo[changeid[i]].type of
          ;error plot
          -1: begin
              plotInfo[changeid[i]].obj->setproperty,thick=plotInfo[changeid[i]].thick,$
                 color=plotInfo[changeid[i]].color
              break
              end
          ;1d plot
          0:  begin
              plotInfo[changeid[i]].obj->getproperty,symbol=drawPsym
              newcolor   =   plotInfo[changeid[i]].color
              newlinesty =   plotInfo[changeid[i]].linestyle
              drawPsym   =   self->getpsym(plotInfo[changeid[i]].psym,plotInfo[changeid[i]].symsize,plotInfo[changeid[i]].thick,$
                 plotInfo[changeid[i]].color,old=drawPsym)
              plotInfo[changeid[i]].obj->setproperty,color=newcolor,thick=plotInfo[changeid[i]].thick,linestyle=newlinesty,$
                 zvalue=-0.0001*(1+(*plotInfo[changeid[i]].zPtr))
              ;change the legend name, linestyle, and color in the legend
              i_num  =    where(onedplot eq changeid[i],count)
              if count ne 0 then begin
                 item_color[*,i_num]     =newcolor
                 item_linestyle[i_num]   =newlinesty
                 item_name[i_num]        =plotInfo[changeid[i]].legend
              endif
              ;change the color and thickness of the error plot if any
              if changeid[i]+1 lt nplots then $
                 if plotInfo[changeid[i]+1].type eq -1 then begin
                   (*plotInfo[changeid[i]+1].zPtr)  =(*plotInfo[changeid[i]].zPtr)
                   plotInfo[changeid[i]+1].color    =newcolor
                   plotInfo[changeid[i]+1].thick    =plotInfo[changeid[i]].thick
                   plotInfo[changeid[i]+1].obj->getproperty,data=data
                   data[2,*]=-0.0001*(0.5+(*plotInfo[changeid[i]].zPtr))
                   plotInfo[changeid[i]+1].obj->setproperty,thick=plotInfo[changeid[i]+1].thick,color=newcolor,data=data
                 endif
              break
              end
          ;2d plot
          1:
          2:  begin
              plotInfo[changeid[i]].obj->getproperty,palette=drawPal
              drawPal=self->getpalette(plotInfo[changeid[i]].coltable,oldpal=drawPal)
              plotInfo[changeid[i]].obj->setproperty,shading=plotInfo[changeid[i]].shading
              end
         endswitch
       endfor
       self.drawLegend->setproperty,item_color=item_color,item_linestyle=item_linestyle,item_name=item_name,item_thick=item_thick
       (*self.plotInfo)=temporary(plotInfo)
    endif
    if n_elements(iso) ne 0 then begin
       self.iso=iso
       redraw=2  ;need to reset view
    endif
    if n_elements(hideLegend) ne 0 then begin
       self.hideLegend=0b>(fix(hideLegend[0],type=1))<1b
       self.drawLegend->setproperty,hide=self.hideLegend
       id    =  widget_info(self.tlb,find_by_uname='cs_hidelegend')
       dm_set_button,id,self.hideLegend,onstring='Show Legend',offstring='Hide Legend'
    endif
    if n_elements(hideintbar) ne 0 then begin
       redraw=2  ;need to reset view
       self.hideIntbar=0b>(fix(hideintbar[0],type=1))<1b
       id    =  widget_info(self.tlb,find_by_uname='cs_hidecbar')
       dm_set_button,id,self.hideIntbar,onstring='Show Colorbar',offstring='Hide Colorbar'
       if obj_valid(self.colBar) then begin
         self.drawxAxis[2]->setproperty,hide=self.hideIntbar
         self.drawxAxis[3]->setproperty,hide=self.hideIntbar
         self.drawyAxis[2]->setproperty,hide=self.hideIntbar
         self.drawyAxis[3]->setproperty,hide=self.hideIntbar
         self.colBar->setproperty,hide=self.hideIntbar
       end
    endif
    if n_elements(xdat) ne 0 then begin
       tmp_xmax=max(xdat,min=tmp_xmin,/nan)
       self.xran[*,0]=[temporary(tmp_xmin),temporary(tmp_xmax)]
       if self.xran[0,0] eq self.xran[1,0] then begin
         tmp  =    (self.xran[0,0] eq 0)? 0.5:abs(self.xran[0,0]/2.)
         self.xran[0,0]=self.xran[0,0]-tmp
         self.xran[1,0]=self.xran[1,0]+tmp
       endif
       poselts=where(xdat gt 0,count)
       if count gt 0 then begin
         tmp_xmax=max(xdat[temporary(poselts)],min=tmp_xmin)
       endif else begin
         tmp_xmin=1.0 & tmp_xmax=10.0
       endelse
       self.xran[0,2]=(tmp_xmin gt self.xran[0,0])?((self.xran[0,0] gt 0)?(self.xran[0,0]):(tmp_xmin)):(self.xran[0,0])
       self.xran[1,2]=(tmp_xmax gt self.xran[1,0])?((self.xran[1,0] gt 0)?(self.xran[1,0]):(tmp_xmax)):(self.xran[1,0])
       if self.xran[1,2] le self.xran[0,2] then self.xran[1,2]=self.xran[0,2]*10.0
    endif
    if n_elements(ydat) ne 0 then begin
       if n_elements(yerr) eq n_elements(ydat) then begin
         tmp_ymin=min(ydat-yerr,/nan)
         tmp_ymax=max(ydat+yerr,/nan)
       endif else $
         tmp_ymax=max(ydat,min=tmp_ymin,/nan)
       self.yran[*,0]=[temporary(tmp_ymin),temporary(tmp_ymax)]
       if self.yran[0,0] eq self.yran[1,0] then begin
         tmp  =    (self.yran[0,0] eq 0)? 0.5:abs(self.yran[0,0]/2.)
         self.yran[0,0]=self.yran[0,0]-tmp
         self.yran[1,0]=self.yran[1,0]+tmp
       endif
       poselts=where(ydat gt 0,count)
       if count gt 0 then begin
         ymax=max(ydat[temporary(poselts)],min=ymin)
       endif else begin
         ymin=1.0 & ymax=10.0
       endelse
       self.yran[0,2]=(ymin gt self.yran[0,0])?((self.yran[0,0] gt 0)?(self.yran[0,0]):(ymin)):(self.yran[0,0])
       self.yran[1,2]=(ymax gt self.yran[1,0])?((self.yran[1,0] gt 0)?(self.yran[1,0]):(ymax)):(self.yran[1,0])
       if self.yran[1,2] le self.yran[0,2] then self.yran[1,2]=self.yran[0,2]*10.0
    endif
    if n_elements(zdat) ne 0 then begin
       tmp_zmax=max(zdat,min=tmp_zmin,/nan)
       self.zran[*,0]=[temporary(tmp_zmin),temporary(tmp_zmax)]
       if self.zran[0,0] eq self.zran[1,0] then begin
         tmp  =    (self.zran[0,0] eq 0)? 0.5:abs(self.zran[0,0]/2.)
         self.zran[0,0]=self.zran[0,0]-tmp
         self.zran[1,0]=self.zran[1,0]+tmp
       endif
       finelts=where(finite(zdat),count)
       if count gt 0 then poselts=where(zdat[finelts] gt 0,count)
       if count gt 0 then begin
         zmax=max(zdat[finelts[poselts]],min=zmin)
       endif else begin
         zmin=1 & zmax=10.0
       endelse
       finelts=-1 & poselts=-1
       self.zran[0,2]=(zmin gt self.zran[0,0])?((self.zran[0,0] gt 0)?(self.zran[0,0]):(zmin)):(self.zran[0,0])
       self.zran[1,2]=(zmax gt self.zran[1,0])?((self.zran[1,0] gt 0)?(self.zran[1,0]):(zmax)):(self.zran[1,0])
       if self.zran[1,2] le self.zran[0,2] then self.zran[1,2]=self.zran[0,2]*10.0
       self.zmin=self.zran[0,2*self.zlog]
       self.zmax=self.zran[1,2*self.zlog]
    endif
    if n_elements(xtit) ne 0 then self.xtit->setproperty,strings=xtit[0]
    if n_elements(ytit) ne 0 then self.ytit->setproperty,strings=ytit[0]
    if n_elements(ztit) ne 0 then self.ztit->setproperty,strings=ztit[0]
    if n_elements(title) ne 0 then begin
       self.title->getproperty,strings=old
       self.title->setproperty,strings=title[0]
       if strlen(old) eq 0 or strlen(title[0]) eq 0 then redraw=2
    endif
    if n_elements(xran) eq 2 then $
      if (total(finite(xran)) eq 2) then $
       if  xran[1] gt xran[0] then begin
         redraw=2
         if total(finite(self.xran[*,2*self.xlog+1])) ne 2 then self.xran[*,2*self.xlog+1]=self.xran[*,2*self.xlog]
         if total(finite(self.yran[*,2*self.ylog+1])) ne 2 then self.yran[*,2*self.ylog+1]=self.yran[*,2*self.ylog]
         if total(finite(self.zran[*,2*self.zlog+1])) ne 2 then self.zran[*,2*self.zlog+1]=self.zran[*,2*self.zlog]
         if self.xlog eq 1 then begin
          if xran[0] gt 0 then self.xran[0,2]=xran[0]
          if xran[1] gt 0 then self.xran[1,2]=xran[1]
         endif else self.xran[*,0]=xran
       endif
    if n_elements(yran) eq 2 then $
      if (total(finite(yran)) eq 2) then $
       if  yran[1] gt yran[0] then begin
         redraw=2
         if total(finite(self.xran[*,2*self.xlog+1])) ne 2 then self.xran[*,2*self.xlog+1]=self.xran[*,2*self.xlog]
         if total(finite(self.yran[*,2*self.ylog+1])) ne 2 then self.yran[*,2*self.ylog+1]=self.yran[*,2*self.ylog]
         if total(finite(self.zran[*,2*self.zlog+1])) ne 2 then self.zran[*,2*self.zlog+1]=self.zran[*,2*self.zlog]
         if self.ylog eq 1 then begin
          if yran[0] gt 0 then self.yran[0,2]=yran[0]
          if yran[1] gt 0 then self.yran[1,2]=yran[1]
         endif else self.yran[*,0]=yran
       endif
    if n_elements(zran) eq 2 then $
      if (total(finite(zran)) eq 2) then $
       if  zran[1] gt zran[0] then begin
         redraw=2
         if n_elements(*self.plotInfo) gt 0 then begin
          if max((*self.plotInfo).type) eq 2 then begin
              if total(finite(self.xran[*,2*self.xlog+1])) ne 2 then self.xran[*,2*self.xlog+1]=self.xran[*,2*self.xlog]
              if total(finite(self.yran[*,2*self.ylog+1])) ne 2 then self.yran[*,2*self.ylog+1]=self.yran[*,2*self.ylog]
          endif
         endif
         if total(finite(self.zran[*,2*self.zlog+1])) ne 2 then self.zran[*,2*self.zlog+1]=self.zran[*,2*self.zlog]
         if self.zlog eq 1 then begin
          if zran[0] gt 0 then self.zran[0,2]=zran[0]
          if zran[1] gt 0 then self.zran[1,2]=zran[1]
         endif else self.zran[*,0]=zran
       endif
    if n_elements(zmin) ne 0 then self.zmin=zmin[0]
    if n_elements(zmax) ne 0 then self.zmax=zmax[0]
    if n_elements(axisthick) ne 0 then begin
       self.axisthick=1.0>(axisthick)<10.0
       for i=0,3 do begin
         if obj_valid(self.drawxAxis[i]) then self.drawxAxis[i]->setproperty,thick=self.axisthick
         if obj_valid(self.drawyAxis[i]) then self.drawyAxis[i]->setproperty,thick=self.axisthick
       endfor
       if obj_valid(self.drawzAxis) then self.drawzAxis->setproperty,thick=self.axisthick
    endif
    if (n_elements(xdat) ne 0) and (n_elements(ydat) ne 0) then begin
       redraw=2  ;need to reset view
       self.xran[*,[1,3]]    =  !values.f_nan
       self.yran[*,[1,3]]    =  !values.f_nan
       self.zran[*,[1,3]]    =  !values.f_nan
       oChildArr = self.drawModel->IDL_Container::Get(/ALL, COUNT=nKids)
       for i=0,nKids-1 do begin
         if (not obj_isa(oChildArr[i],'IDLgrModel')) and (not obj_isa(oChildArr[i],'IDLgrLegend')) $
          and (not obj_isa(oChildArr[i],'IDLgrText')) then $
                 obj_destroy,oChildArr[i]
       endfor
       oChildArr = self.drawGroup->IDL_Container::Get(/ALL, COUNT=nKids)
       for i=0,nKids-1 do begin
         if obj_isa(oChildArr[i],'IDLgrPlot') then begin
          oChildArr[i]->getproperty,symbol=tmp1
          if obj_valid(tmp1) then begin
              tmp1->getproperty,data=tmp2
              if obj_valid(tmp2) then obj_destroy,tmp2
              obj_destroy,tmp1
          endif
         endif else if obj_isa(oChildArr[i],'IDLgrPolygon') or obj_isa(oChildArr[i],'IDLgrContour') or $
          obj_isa(oChildArr[i],'IDLgrSurface') then begin
          oChildArr[i]->getproperty,palette=tmp
          if obj_valid(tmp) then obj_destroy,tmp
         endif
         obj_destroy,oChildArr[i]
       endfor
       if n_elements(*self.plotInfo) ne 0 then ptr_free,(*self.plotInfo)[*].xPtr,(*self.plotInfo)[*].yPtr,(*self.plotInfo)[*].zPtr
       ptr_free,self.plotInfo & self.plotInfo=ptr_new(/allocate_heap)
       self.drawGroup->reset
       self.drawModel->reset
       self.drawLegend->getproperty,item_object=item_object
       obj_destroy,item_object
       self.drawLegend->setproperty,item_object=[obj_new()]
       if n_elements(zdat) ne 0 then $
         self->add_plot,xdat,ydat,zdat,no_copy=no_copy,surfplot=surfplot,coltable=coltable,shading=shading,usepolygon=usepolygon $
       else $
         self->add_plot,xdat,ydat,yerr=yerr,no_copy=no_copy,legend=legend,thick=thick,color=color,psym=psym,linestyle=linestyle,$
          symsize=symsize,layer=layer
    endif
    if keyword_set(nodraw) then return
    if redraw eq 1 then self.drawWin->draw,self.drawView $
    else if redraw eq 2 then self->draw
end


;setup correct viewing plane rectangle
;and suppress xaxis[1]'s tickname
;keyword
;   save: if set, change white color to black color for saving purpose
pro dm_plot::set_view,save=save
    if keyword_set(save) then bgcolor=[255b,255b,255b] else bgcolor=[0b,0b,0b]
    if obj_valid(self.drawxAxis[1]) then begin
       self.drawxAxis[1]->getproperty,ticktext=xtick
       xtick->getproperty,strings=tmp
       for i=0,n_elements(tmp)-1 do tmp[i]=''
       xtick->setproperty,strings=tmp
       self.drawxAxis[1]->setproperty,ticktext=xtick
    endif

    self.drawView->getproperty,dimensions=dim                 ;viewport size
    xsize  =    dim[0]
    ysize  =    dim[1]
    fsize  =    fix((1>min(dim/self.default[5:6]))*self.fsize[*])
    for i=0,5 do $
       self.font[i]->setproperty,size=fsize[i]


    xoffset=[0.0,0.0]
    yoffset=[(fsize[0]*self.fscale[0]*2+fsize[1]*self.fscale[1]*2.5+20),self.fscale[0]*fsize[0]*0.5+20]
    ;get the axis tickmarks to figure out the offset of the viewplane
    self.drawxAxis[0]->getproperty,ticktext=xtick
    xtick->getproperty,strings=xticktext
    xoffset[1]=(dm_strlen(xticktext[n_elements(xticktext)-1])/2.+1)*fsize[0]*self.fscale[0]
    self.drawyAxis[0]->getproperty,ticktext=ytick
    ytick->getproperty,strings=yticktext
    xoffset[0]=(max(dm_strlen(yticktext))+0.75)*fsize[0]*self.fscale[0]+fsize[2]*self.fscale[2]*2.5+20
    self.title->getproperty,strings=title,color=color
    if total(abs(color-bgcolor)) eq 0 then self.title->setproperty,color=[255b,255b,255b]-color
    title  =    strlowcase(strtrim(title,2))
    if dm_strlen(title) ne 0 then $
       yoffset[1]=yoffset[1]+1.75*fsize[4]*self.fscale[4]

    twodplot=where((*self.plotInfo)[*].type gt 0,count1)
    surfplot=where((*self.plotInfo)[*].type eq 2,count2)
    if (self.hideIntbar eq 0b) and (count1 ne 0) then begin
       xran  =    [0.,1.2]
       self.drawyAxis[3]->getproperty,ticktext=cytick
       cytick->getproperty,strings=yticktext
       xoffset[1]=(0.5+max(dm_strlen(yticktext)))*fsize[0]*self.fscale[0]+20
    endif else $
       xran  =    [0.,1.]
    yran   = [0.,1.0]
    if count2 ne 0 then begin
       xran[0]=-0.7
       yran[0]=-0.3
       xran[1]=xran[1]-0.25
       self.drawzAxis->getproperty,ticktext=ztick
       ztick->getproperty,strings=zticktext
       xoffset[0]=(0.5+max(dm_strlen(zticktext)))*fsize[0]*self.fscale[0]+fsize[3]*self.fscale[3]*2.0+20
       yoffset[0]=fsize[0]*self.fscale[0]*0.9+fsize[1]*self.fscale[1]*1.0+20
    endif

    viewplane_rect =[xran[0],yran[0],xran[1]-xran[0],yran[1]-yran[0]]  ;viewing volume

    if self.iso then begin ;isotropic
       ratio =   float(ysize-yoffset[1]-yoffset[0])/(xsize-xoffset[1]-xoffset[0])
       target    =  (yran[1]-yran[0])/(xran[1]-xran[0])
       if ratio ge target then begin
         yoff =   (ratio-target)*(xran[1]-xran[0])/2.
         viewplane_rect[1]=yran[0]-yoff
         viewplane_rect[3]=ratio*(xran[1]-xran[0])
       endif else begin
         xoff =   (1./ratio-1./target)*(yran[1]-yran[0])/2.
         viewplane_rect[0]=xran[0]-xoff
         viewplane_rect[2]=1./ratio*(yran[1]-yran[0])
       endelse
    endif

    xoffset    =  xoffset/(xsize-total(xoffset))*viewplane_rect[2]
    yoffset    =  yoffset/(ysize-total(yoffset))*viewplane_rect[3]
    viewplane_rect[0]  =    viewplane_rect[0]-xoffset[0]
    viewplane_rect[1]  =    viewplane_rect[1]-yoffset[0]
    viewplane_rect[2]  =    viewplane_rect[2]+xoffset[0]+xoffset[1]
    viewplane_rect[3]  =    viewplane_rect[3]+yoffset[0]+yoffset[1]
    self.drawView->setproperty,viewplane_rect=viewplane_rect,color=bgcolor
    self.tranView->setproperty,viewplane_rect=viewplane_rect
    self.viewplane     =  viewplane_rect
    ;reverse black and white color of the axis if necessary
    for i=0,3 do begin
       if obj_valid(self.drawxAxis[i]) then begin
         self.drawxAxis[i]->getproperty,color=color
         if total(abs(color-bgcolor)) eq 0 then self.drawxAxis[i]->setproperty,color=[255b,255b,255b]-color
       endif
       if obj_valid(self.drawyAxis[i]) then begin
         self.drawyAxis[i]->getproperty,color=color
         if total(abs(color-bgcolor)) eq 0 then self.drawyAxis[i]->setproperty,color=[255b,255b,255b]-color
       endif
    endfor
    if obj_valid(self.drawzAxis) then begin
       self.drawzAxis->getproperty,color=color
       if total(abs(color-bgcolor)) eq 0 then self.drawzAxis->setproperty,color=[255b,255b,255b]-color
    endif
    ;update symbol size and color
    onedplot=where((*self.plotInfo)[*].type le 0,onedcount)
    for i=0,onedcount-1 do begin
       (*self.plotInfo)[onedplot[i]].obj->getproperty,symbol=drawPsym,color=color
       if total(abs(color-bgcolor)) eq 0 then (*self.plotInfo)[onedplot[i]].obj->setproperty,color=[255b,255b,255b]-color
       if obj_valid(drawPsym) then begin
         drawPsym=self->getpsym((*self.plotInfo)[onedplot[i]].psym,(*self.plotInfo)[onedplot[i]].symsize,$
          (*self.plotInfo)[onedplot[i]].thick,(*self.plotInfo)[onedplot[i]].color,old=drawPsym)
         drawPsym->getproperty,data=tmp
         if obj_valid(tmp) then begin
          tmp->getproperty,color=color
          if total(abs(color-bgcolor)) eq 0 then  tmp->setproperty,color=[255b,255b,255b]-color
         endif
       endif
    endfor
    if onedcount ne 0 then begin;reverse black and white color if necessary
       self.drawLegend->getproperty,item_color=item_color
       for i=0,n_elements(item_color[0,*])-1 do $
         if total(abs(item_color[*,i]-bgcolor)) eq 0 then item_color[*,i]=[255b,255b,255b]-bgcolor
       self.drawLegend->setproperty,item_color=item_color,text_color=[255b,255b,255b]-bgcolor,outline_color=[255b,255b,255b]-bgcolor
    endif
    if keyword_set(save) then bgcolor=[255b,255b,255b]*self.misscol
    twodplot=where((*self.plotInfo)[*].type ge 1,twodcount)
    if twodcount gt 0 then $
       for i=0,twodcount-1 do begin
         (*self.plotInfo)[twodplot[i]].obj->getproperty,palette=drawPal
         drawPal->getproperty,red_values=r,green_values=g,blue_values=b
         r[0]=bgcolor[0] & g[0]=bgcolor[1] & b[0]=bgcolor[2]
         drawPal->setproperty,red_values=temporary(r),green_values=temporary(g),blue_values=temporary(b)
       endfor
end

;mouse events in the plotwin
pro dm_plot::zoom,event
    if self.xlog eq 1 then xran=alog10(self.xran[*,2]) else xran=self.xran[*,0]
    if self.ylog eq 1 then yran=alog10(self.yran[*,2]) else yran=self.yran[*,0]
    case event.type of
       0:    begin  ;mouse button down
         case event.press of
          1:  begin    ;left button down
                 widget_control,self.plotWin,draw_motion_events=1
                 ;initialize the zoom box
                 self.drawView->getproperty,dimension=dim
                 xstart    =  self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
                 ystart    =  self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
                 tmp=self.drawWin->select(self.drawView,[event.x,event.y])
                 if obj_valid(tmp[0]) then $
                   if obj_isa(tmp[0],'IDLgrLegend') then begin
                    self.moveLegend=1b
                    self.drawLegend->getproperty,xcoord_conv=x,ycoord_conv=y
                    self.legdOffset=[x[0]-xstart,y[0]-ystart]
                    break
                   endif
                 xstart    =  xran[0]+(0.0>(xstart)<1.0)*(xran[1]-xran[0])
                 ystart    =  yran[0]+(0.0>(ystart)<1.0)*(yran[1]-yran[0])
                 self.zoomBox=obj_new('IDLgrPolyline',replicate(xstart,5),replicate(ystart,5),replicate(0.02,5),$
                   color=self->getcolor('green'),xcoord_conv=norm_coord(xran),ycoord_conv=norm_coord(yran))
                 self.tranModel=obj_new('IDLgrModel')
                 self.tranModel->add,self.zoomBox
                 ;take a snap short of current image
                 self.drawWin->draw,self.drawView,create_instance=2
                 self.tranView->add,self.tranModel
              end
          else:
         endcase
         end
       1:    begin  ;mouse button up
          widget_control,self.plotWin,draw_motion_events=0    ;disable motion event
          widget_control,self.plotWin,/clear_events
          self.moveLegend=0b
          if obj_valid(self.zoomBox) then begin
              self.zoomBox->getproperty,data=data
              xran   = [min(data[0,[0,1]]),max(data[0,[0,1]])]
              yran   = [min(data[1,[1,2]]),max(data[1,[1,2]])]
              obj_destroy,self.zoomBox
              obj_destroy,self.tranModel
              self.drawWin->draw,self.tranView,/draw_instance
              if (xran[1]-xran[0] ne 0) and (yran[1]-yran[0] ne 0) then $
                 self->setproperty,yran=(self.ylog eq 1)?(10.0^yran):yran,xran=(self.xlog eq 1)?(10.0^xran):xran
          end
         end
       2:    begin ;mouse motion
          self.drawView->getproperty,dimension=dim
          xend    =  self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
          yend    =  self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
          if self.moveLegend eq 1b then begin
              self.drawLegend->setproperty,xcoord_conv=[xend+self.legdOffset[0],1],ycoord_conv=[yend+self.legdOffset[1],1]
              self.drawWin->draw,self.drawView
          endif else begin
              xend   = xran[0]+(0.0>(xend)<1.0)*(xran[1]-xran[0])
              yend   = yran[0]+(0.0>(yend)<1.0)*(yran[1]-yran[0])
              self.zoomBox->getproperty,data=data
              data[0,[1,2]]=[xend,xend]
              data[1,[2,3]]=[yend,yend]
              self.zoomBox->setproperty,data=data
              self.drawWin->draw,self.tranView,/draw_instance
          endelse
         end
       else:
    endcase
end

;cleanup, life cycle method
pro dm_plot::Cleanup
    if widget_info(self.tlb,/valid_id) then widget_control,self.tlb,/destroy
    ;destroy all the plot symbols
    oChildArr = self.drawGroup->IDL_Container::Get(/ALL, COUNT=nKids)
    for i=0,nKids-1 do begin
       if obj_isa(oChildArr[i],'IDLgrPlot') then begin
         oChildArr[i]->getproperty,symbol=tmp1
         if obj_valid(tmp1) then begin
          tmp1->getproperty,data=tmp2
          if obj_valid(tmp2) then obj_destroy,tmp2
          obj_destroy,tmp1
         endif
       endif
    endfor
    if n_elements(*self.plotInfo) ne 0 then begin
        ;destroy the draw palettes
        twodplot=where((*self.plotInfo)[*].type gt 0,count1)
        for i=0,count1-1 do begin
            (*self.plotInfo)[twodplot[i]].obj->getproperty,   palette=tmp
            if obj_valid(tmp) then obj_destroy,tmp
        endfor
        ptr_free,(*self.plotInfo)[*].xPtr,(*self.plotInfo)[*].yPtr,(*self.plotInfo)[*].zPtr
    endif
    ptr_free,self.plotInfo
    if float(!version.release) lt 5.6 then begin
       self.title->getproperty,font=tmp
       if obj_valid(tmp) then obj_destroy,tmp
    endif
    obj_destroy,self.drawLegend
    obj_destroy,self.drawView
    obj_destroy,self.tranView
    obj_destroy,self.font
    obj_destroy,self.xtit
    obj_destroy,self.ytit
    obj_destroy,self.ztit
    obj_destroy,self.title
    obj_destroy,self.printer
    obj_destroy,self.trackBall
end

;initialization, life cycle method
;usuage:
;   newplot=obj_new('dm_plot')
;parameters:
;   NONE
;keyword:
;   see the definition
function dm_plot::Init,xdat,ydat,zdat,yerr=yerr,axisthick=axisthick,coltable=coltable,color=color,font=font,fsize=fsize,$
    group_leader=group_leader,hideintbar=hideintbar,hidelegend=hidelegend,iso=iso,legend=legend,linestyle=linestyle,misscol=misscol,$
    no_copy=no_copy,parentobj=parentobj,parenthandler=parenthandler,psym=psym,shading=shading,surfplot=surfplot,symsize=symsize,$
    thick=thick,title=title,usepolygon=usepolygon,xlog=xlog,xran=xran,xtit=xtit,ylog=ylog,yran=yran,ytit=ytit,zlog=zlog,zmax=zmax,$
    zmin=zmin,zran=zran,ztit=ztit,layer=layer

    if float(!version.release) ge 6.0 then v6p0_extra= {register_properties:1}  ;register_properties keyword for IDL6.0 or later
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(!error_state.msg,/error)
       return,0  ;indicating that an error occurs
    end

    self.time  =    systime(/sec)                 ;for debugging
    ;default values for [ticklength,symsize,thick,shading,coltable,xsize,ysize,legend gap]
    self.default=  [0.02,0.008,1.5,0.0,13.0,600,400,0.2]
    self.xran[*]=  !values.f_nan
    self.yran[*]=  !values.f_nan
    self.zran[*]=  !values.f_nan
    self.zmin   =  !values.f_nan & self.zmax=!values.f_nan
    ;check keywords
    if n_elements(parentobj) ne 0 then $
        if obj_valid(parentobj) then begin
            if n_elements(group_leader) eq 0 then parentobj->getproperty,tlb=group_leader
            if n_elements(parenthandler) eq 0 then parenthandler='event'
            self.parentobj      =parentobj
            self.parenthandler  =parenthandler
        endif
    if n_elements(group_leader) ne 0 then self.group_leader=group_leader else self.group_leader=0
    if n_elements(xtit) ne 0 then xtit=strtrim(xtit[0],2) else xtit='X'
    if n_elements(ytit) ne 0 then ytit=strtrim(ytit[0],2) else ytit='Y'
    if n_elements(ztit) ne 0 then ztit=strtrim(ztit[0],2) else ztit='Z'
    if n_elements(xlog) ne 0 then self.xlog=(0b>(byte(xlog))<1b) else self.xlog=0b
    if n_elements(ylog) ne 0 then self.ylog=(0b>(byte(ylog))<1b) else self.ylog=0b
    if n_elements(zlog) ne 0 then self.zlog=(0b>(byte(zlog))<1b) else self.zlog=0b
    if n_elements(zmin) ne 0 then self.zmin=zmin[0]
    if n_elements(zmax) ne 0 then self.zmax=zmax[0]
    if n_elements(title) ne 0 then title=strtrim(title[0],2) else title=''
    if n_elements(iso) ne 0 then self.iso=0>(iso)<1
    if n_elements(fsize) ne 0 then self.fsize[*]=(8>(fix(fsize[0]))) else self.fsize[*]=12
    if n_elements(axisthick) ne 0 then self.axisthick=1.0>(axisthick[0])<10.0 else self.axisthick=self.default[2]
    if keyword_set(hideintbar) then self.hideIntbar=1b else self.hideIntbar=0b
    if keyword_set(hidelegend) then self.hideLegend=1b else self.hideLegend=0b
    if n_elements(misscol) ne 0 then self.misscol=0>(misscol[0])<1 else self.misscol=0b
    if not keyword_set(font) then font='Helvetica'
    for i=0,5 do $
       self.font[i]  =    obj_new('IDLgrFont',font,size=self.fsize[i],substitute='Hershey')
    self.font[0]->getproperty,name=tmp
    if strmid(strlowcase(tmp),0,7) eq 'hershey' then self.fscale[*]=1.2 else self.fscale[*]=1.0

    self.xtit  =  obj_new('IDLgrText',xtit,recompute_dimensions=2,font=self.font[1],/enable,_extra=v6p0_extra,name='xtitle')
    self.ytit  =  obj_new('IDLgrText',ytit,recompute_dimensions=2,font=self.font[2],/enable,_extra=v6p0_extra,name='ytitle')
    self.ztit  =  obj_new('IDLgrText',ztit,recompute_dimensions=2,font=self.font[3],/enable,_extra=v6p0_extra,name='ztitle')
    self.title =  obj_new('IDLgrText',title,recompute_dimensions=2,font=self.font[4],/enable,color=self->getcolor('white'),$
                  name='titl',_extra=v6p0_extra)

    ;create the widgets
    self.tlb   = widget_base(uname='tlb',mbar=bar,group_leader=self.group_leader,/tlb_size_events,/col,$
       ypad=0,xpad=0,/align_center,map=0,space=0)
    filemenu   = widget_button(bar,value='File',/menu)
    editmenu   = widget_button(bar,value='Edit',/menu)
    if self.group_leader ne 0 then begin
       keepmenu  = widget_button(bar,value='Keep',/menu,uname='keepmenu')
       mkcrmenu  = widget_button(bar,value='Make Current',/menu,uname='mkcrmenu',sensitive=0)
       ;keep menu
       keepBut   = widget_button(keepmenu,value='Keep',uname='keepBut')
       mkcrBut   = widget_button(mkcrmenu,value='Make Current',uname='mkcrBut',sensitive=0)
    endif

    ;file menu
    saveBut    =  widget_button(filemenu,value='Save',uname='save')
    savemenu   =  widget_button(filemenu,value='Save As...',uname='savemenu',/menu)
    bgclmenu   =  widget_button(filemenu,value='Color of Missing Data',/menu)
    readBut    =  widget_button(filemenu,value='Read 2 or 3-Column 1D Data File',uname='readBut',/separator)
    printBut   =  widget_button(filemenu,value='Print',uname='printBut',/separator)
    pnsetBut   =  widget_button(filemenu,value='Printer Setup...',uname='pnsetBut')
    exitBut    =  widget_button(filemenu,value='Exit',uname='exitBut',/separator)
    saveps     =  widget_button(savemenu,value='ps file',uname='saveps')
    savejpeg   =  widget_button(savemenu,value='jpeg file',uname='savejpg')
    savebmp    =  widget_button(savemenu,value='bmp file',uname='savebmp')
    savepng    =  widget_button(savemenu,value='png file',uname='savepng')
    saveppm    =  widget_button(savemenu,value='ppm file',uname='saveppm')
    savetiff   =  widget_button(savemenu,value='tiff file',uname='savetif')
    saveascii  =  widget_button(savemenu,value='ascii file',uname='saveasc')
    blackbg    =  dm_widget_button(bgclmenu,value='Black',uname='blackbg')
    whitebg    =  dm_widget_button(bgclmenu,value='White',uname='whitebg')
    if self.misscol eq 0 then dm_toggle_menubut,blackbg,whitebg $
    else dm_toggle_menubut,whitebg,blackbg
    ;edit menu
    titlBut    =  widget_button(editmenu,value='Title',uname='titlBut')
    xtitBut    =  widget_button(editmenu,value='X Title',uname='xtitBut')
    xranBut    =  widget_button(editmenu,value='X Range',uname='xranBut')
    ylogBut    =  dm_widget_button(editmenu,value='X Log Scale',uname='xlogBut',set_button=self.xlog,$
          onstring='X Normal Scale',offstring='X Log Scale')
    ytitBut    =  widget_button(editmenu,value='Y Title',uname='ytitBut')
    yranBut    =  widget_button(editmenu,value='Y Range',uname='yranBut')
    ylogBut    =  dm_widget_button(editmenu,value='Y Log Scale',uname='ylogBut',set_button=self.ylog,$
          onstring='Y Normal Scale',offstring='Y Log Scale')
    ztitBut    =  widget_button(editmenu,value='Z Title',uname='ztitBut',sensitive=0)
    zranBut    =  widget_button(editmenu,value='Z Range',uname='zranBut',sensitive=0)
    zlogBut    =  dm_widget_button(editmenu,value='Z Log Scale',uname='zlogBut',sensitive=0,set_button=self.zlog,$
          onstring='Z Normal Scale',offstring='Z Log Scale')
    psymBut    =  widget_button(editmenu,value='Plot Symbol',uname='psymBut',sensitive=0,/separator)
    lineBut    =  widget_button(editmenu,value='Plot Linestyle',uname='lineBut',sensitive=0)
    pcolBut    =  widget_button(editmenu,value='Plot Color',uname='pcolBut',sensitive=0)
    lgndBut    =  widget_button(editmenu,value='Legend Label',uname='lgndBut',sensitive=0)
    hideBut    =  dm_widget_button(editmenu,value='Hide Legend',uname='cs_hidelegend',sensitive=0,set_button=self.hideLegend,$
          onstring='Show Legend',offstring='Hide Legend')
    ctabBut    =  widget_button(editmenu,value='Color Table',uname='cs_coltable',sensitive=0,/separator)
    shadBut    =  widget_button(editmenu,value='Shading',uname='cs_shade',sensitive=0)
    hideBut    =  dm_widget_button(editmenu,value='Hide Colorbar',uname='cs_hidecbar',sensitive=0,set_button=self.hideIntbar,$
          onstring='Show Colorbar',offstring='Hide Colorbar')
    if float(!version.release) ge 6.0 then $
    vallBut     = widget_button(editmenu,value='Show All Plots',/separator,uname='show_all',sensitive=0)
    ;menu bar separator for Windows operating system
    if !version.os_family eq 'Windows' then $
       self.mbarSep  =    widget_label(self.tlb,sensitive=0,/dynamic_resize,xsize=1)
    ;draw widget
    self.plotWin = widget_draw(self.tlb,/button_events,uname='plotWin',graphics_level=2,scr_xsize=self.default[5],$
       scr_ysize=self.default[6],renderer=1,retain=0,/no_copy)
       ;/no_copy,retain=0)
       ;renderer=0,/no_copy,expose_events=0)

    ;context senstive menu
    cs_axisMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_axisMenu')  ;axis object
       titlbut   = widget_button(cs_axisMenu,value='Title',uname='cs_title')
       rangebut  = widget_button(cs_axisMenu,value='Range',uname='cs_range')
       thickbut  = widget_button(cs_axisMenu,value='Thick',uname='cs_axisthick')
       fontbut   = widget_button(cs_axisMenu,value='Font', uname='cs_font')
       logbut    = dm_widget_button(cs_axisMenu,value='Log Scale',uname='cs_logscale')
    cs_titlMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_titlMenu')  ;title text object
       titlbut   = widget_button(cs_titlMenu,value='Title',uname='cs_title')
       fontbut   = widget_button(cs_titlMenu,value='Font', uname='cs_font')
       if float(!version.release) ge 6.0 then $
       propsheet = widget_button(cs_titlMenu,value='Property Sheet',uname='cs_propsheet',/separator)
    cs_plotMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_plotMenu')  ;1-d plot object
       symbbut   = widget_button(cs_plotMenu,value='Symbol',uname='cs_symbol')
       colbut    = widget_button(cs_plotMenu,value='Color', uname='cs_color')
       linebut   = widget_button(cs_plotMenu,value='Linestyle',uname='cs_linestyle')
       thickbut  = widget_button(cs_plotMenu,value='Thick', uname='cs_linethick')
       legndbut  = widget_button(cs_plotMenu,value='Legend',uname='cs_legend')
       layerbut  = widget_button(cs_plotMenu,value='Layer',uname='cs_layer',/separator)
       if float(!version.release) ge 6.0 then $
       propsheet = widget_button(cs_plotMenu,value='Property Sheet',uname='cs_propsheet',/separator)
    cs_twodMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_twodMenu')  ;2-d plot object
       coltbut   = widget_button(cs_twodMenu,value='Color table',uname='cs_coltable')
       shadbut   = widget_button(cs_twodMenu,value='Shading',uname='cs_shade')
    cs_legendMenu= widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_legendMenu')  ;legend object
       lablbut   = widget_button(cs_legendMenu,value='Labels',uname='lgndBut')
       fontbut   = widget_button(cs_legendMenu,value='Font',uname='cs_font')
       lgapbut   = widget_button(cs_legendMenu,value='Gap',uname='cs_lgndgap')
       gwidbut   = widget_button(cs_legendMenu,value='Glyph Width',uname='cs_lgndwidth')
       oulnbut   = widget_button(cs_legendMenu,value='Show Outline',uname='cs_outline')
       hidebut   = widget_button(cs_legendMenu,value='Hide',uname='cs_hidelegend',/separator)
       if float(!version.release) ge 6.0 then $
       propsheet = widget_button(cs_legendMenu,value='Property Sheet',uname='cs_propsheet',/separator)
    cs_cbarMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_cbarMenu')  ;color bar object
       zranbut   = widget_button(cs_cbarMenu,value='Range',uname='cs_range')
       coltbut   = widget_button(cs_cbarMenu,value='Color table',uname='cs_coltable')
       shadbut   = widget_button(cs_cbarMenu,value='Shading',uname='cs_shade')
       fontbut   = widget_button(cs_cbarMenu,value='Font', uname='cs_font')
       logbut    = dm_widget_button(cs_cbarMenu,value='Log Scale',uname='cs_logscale')
       hidebut   = widget_button(cs_cbarMenu,value='Hide',uname='cs_hidecbar',/separator)

    dm_center_kid,self.tlb
    widget_control,self.tlb,/realize
    tlb    =  self.tlb
    widget_control,self.plotWin,get_value=oWindow
    self.drawWin   =  oWindow
    self.drawView  =  obj_new('IDLgrView',location=[0,0],dimensions=self.default[5:6],viewplane_rect=[0,0,1,1],$
       color=self->getcolor('black'),zclip=[1.1,-1.1])
    self.tranView  =  obj_new('IDLgrView',/transparent,location=[0,0],dimensions=self.default[5:6])
    self.drawModel =  obj_new('IDLgrModel',lighting=0)
    self.drawGroup =  obj_new('IDLgrModel',lighting=0)
    self.drawModel->add,self.drawGroup
    self.drawView->add,self.drawModel
    self.drawLegend=  obj_new('IDLgrLegend',text_color=self->getcolor('white'),xcoord_conv=[0.65,1],ycoord_conv=[0.75,1],$
       zcoord_conv=[1.0,1.0],font=self.font[5],name='legend',gap=self.default[7],border_gap=2*self.default[7],$
       outline_color=self->getcolor('white'),glyph_width=1.5,_extra=v6p0_extra)
    if float(!version.release) ge 6.0 then begin    ;register some properties
       self.drawLegend->registerproperty,'border_gap',3,    name='border gap'
       self.drawLegend->registerproperty,'fill_color',5,    name='fill color'
       self.drawLegend->registerproperty,'gap',3
       self.drawLegend->registerproperty,'glyph_width',3,   name='glyph width'
       self.drawLegend->registerproperty,'outline_color',5, name='outline color'
       self.drawLegend->registerproperty,'outline_thick',3, name='outline thick'
       self.drawLegend->registerproperty,'show_fill',1,     name='show fill'
       self.drawLegend->registerproperty,'show_outline',1,  name='show outline'
       self.drawLegend->registerproperty,'text_color',5,    name='text color'
    endif
    self.drawModel->add,self.drawLegend
    self.trackBall =  obj_new('Trackball',(self.default[5:6])/2,self.default[6]/2)
    self.printer   =  obj_new('IDLgrPrinter',units=1,print_quality=2)
    self.plotInfo  =  ptr_new(/allocate_heap)

    if n_params() eq 2  then $
       self->setproperty,xdat=xdat,ydat=ydat,yerr=yerr,no_copy=no_copy,legend=legend,symsize=symsize,linestyle=linestyle,$
         thick=thick,psym=psym,color=color,xran=xran,yran=yran,layer=layer,/nodraw $
    else if n_params() eq 3 then $
       self->setproperty,xdat=xdat,ydat=ydat,zdat=zdat,no_copy=no_copy,surfplot=surfplot,shading=shading,coltable=coltable,$
         usepolygon=usepolygon,xran=xran,yran=yran,zran=zran,/nodraw
    ;show the horizontal bar style menu bar separator
    if widget_info(self.mbarsep,/valid_id) then begin
       sep=bytarr(1000) & sep[*]=byte(175) & sep=string(sep)
       widget_control,self.mbarSep,set_value=temporary(sep),xsize=self.default[5]
    endif

    widget_control,self.tlb,set_uvalue=self,/no_copy
    xmanager,'dm_plot',tlb,cleanup='dm_plot_Exit',/no_block
    return,1
end

;class definition
pro dm_plot__define
    struct={dm_plot,       $    ;class name
      group_leader:0L,     $    ;group leader of this object
      parentobj:obj_new(), $    ;parent object
      parenthandler:'',    $    ;event handler of parent object,default is 'event'
      tlb:0L,              $    ;identifier of top level base
      mbarSep:0L,          $    ;menu bar separator
                           $    ;ranges: [*,0]-current range [*,1]-original range  [*,2]-current log range [*,2]-original log range
      xran:fltarr(2,4),    $    ;xrange for the axis
      yran:fltarr(2,4),    $    ;yrange
      zran:fltarr(2,4),    $    ;zrange
      zmin:0e,             $    ;zmin for contour plot
      zmax:0e,             $    ;zmax
      plotWin:0L,          $    ;draw widget
      font:objarr(6),      $    ;font  0:for axis ticks 1:for xtitle 2:for ytitle 3:ztitle 4:title 5:legend
      fsize:intarr(6),     $    ;font size
      fscale:fltarr(6),    $    ;scaling factor
      axisthick:1.5e,      $    ;axis thickness
      title:obj_new(),     $    ;title
      xtit:obj_new(),      $    ;xtitle
      ytit:obj_new(),      $    ;ytitle
      ztit:obj_new(),      $    ;ztitle
      xlog:0b,             $    ;log10 scale x flag
      ylog:0b,             $    ;log10 scale y flag
      zlog:0b,             $    ;log10 scale z flag
      drawWin:obj_new(),   $    ;draw window object
      drawView:obj_new(),  $    ;draw view object
      drawModel:obj_new(), $    ;draw model object
      drawGroup:obj_new(), $    ;contains all moveable graphic atoms, for surface rotation
      drawLegend:obj_new(),$    ;legends
      moveLegend:0b,       $    ;flag for legend movement
      hideLegend:0b,       $    ;hide legend flag
      legdOffset:fltarr(2),$    ;offset for legend movement
      hideIntbar:0b,       $    ;flag for showing the intensity bar in 2d contour plot
      plotInfo:ptr_new(),  $    ;plot informations
      tranView:obj_new(),  $    ;transparent view for zoom box
      tranModel:obj_new(), $    ;model object for zoom box
      drawxAxis:objarr(4), $    ;x axis
      drawyAxis:objarr(4), $    ;y axis
      drawzAxis:obj_new(), $    ;z axis
      colBar:obj_new(),    $    ;color bar
      zoomBox:obj_new(),   $    ;zoom box, a polyline object
      trackBall:obj_new(), $    ;trackall object for rotation
      iso:0b,              $    ;1-isotropic 0-aspect
      keep:0b,             $    ;1: keep this plot  0:open to new plots
      viewplane:fltarr(4), $    ;view plane rectangle
      printer:obj_new(),   $    ;printer
      select:obj_new(),    $    ;selected object
      default:fltarr(8),   $    ;default values for [ticklength,symsize,thick,shading,coltable,xsize,ysize,legend gap]
      misscol:0b,          $    ;flag for missing data color in saved file, default is black:0
      time:0d              $    ;debug
    }
end


;examples: test,/oned & test,/use & test,/surf
pro test,oned=oned,_extra=_extra
    nx     =  101
    x=indgen(nx)
    y=indgen(nx)
    z   =   (dist(nx)+1.0)
    zr =[min(z),max(z)]
    xr =[min(x),max(x)]
    yr =[min(y),max(y)]

    current    =  systime(/sec)

    if keyword_set(oned)then begin
       nx    =  1e2
       tmp   = obj_new('dm_plot',sin(findgen(nx))*10,cos(findgen(nx))*10,title='1d plot',$
         color='white',linestyle='solid',psym='circle',_extra=_extra)
       tmp->draw
       tmp->add_plot,[-5,5,0,5.5],[0,0,6,5],color='green',psym='triangle',legend='hi'
       tmp->draw
       tmp->add_plot,[-5,5],[5,5],yerr=[0.8,0.8],color='white',psym=6,legend='     '    ;symbol 6='square'
       ;tmp  =    obj_new('dm_plot',[1,10],[10,10],yerr=[1,1],color='white',psym='circle',legend='',xr=[0.1,11],yr=[5,15])
    endif else begin
       z[0:100,70:75]=!values.f_nan  ;generate some missing data
       tmp   = obj_new('dm_plot',x-nx/2.,y-nx/2.,z,title='2d plot',zr=zr,xr=[-nx/5*6,nx/5*6],_extra=_extra)
       ;tmp   = obj_new('dm_plot')
       ;tmp->setproperty,xdat=x-nx/2.,ydat=y-nx/2.,zdat=z,title='2d plot',zr=zr,xr=[-nx/5*6,nx/5*6],_extra=_extra
       ;tmp->draw
       ;wait,1
       ;tmp->setproperty,xdat=[0,20],ydat=[10,10],linestyle='solid',psym='circle',color='red'
       ;tmp->draw
       ;wait,1
       ;tmp->setproperty,xdat=x-nx/2.,ydat=y-nx/2,zdat=z,title='2d plot',zr=zr,/surfplot
       ;tmp->draw
       ;wait,1
       ;tmp->setproperty,xdat=x-nx/2.,ydat=y-nx/2,zdat=z,title='2d plot',zr=zr,usepolygon=usepolygon,surfplot=surfplot,/no_copy
       ;tmp->draw
       ;wait,1
       ;tmp->add_plot,[0,20],[10,10],yerr=[0.8,0.8],linestyle='solid',psym='circle',color='red'
    endelse
    tmp->draw
    print,systime(/sec)-current,'seconds'
end
