; $Id: dm_stepgrid_bin.pro,v 1.8 2003/09/12 19:16:29 qiuym Exp $
; bin data according to give xstep and ystep
; parameters:
;   in_xstep:   xstep
;   in_ystep:   ystep
;   xdat:       input xdat[ne,ndet]    output xdat[xsize]
;   ydat:       input ydat[ne,ndet]    output ydat[ysize]
;   zdat:       input zdat[ne,ndet] output zdat[xsize,ysize]
; keyword:
;   nogrid:     0: grid according to given xstep and ystep
;               1: don't grid x
;               2: don't grid y
;   uniq_val:   if set, then use the given uniq values instead of the steps to determine the bins
;   avgsum:     0: average 1:sum
pro dm_stepgrid_bin,in_xstep,in_ystep,xdat,ydat,zdat,nogrid=nogrid,uniq_val=uniq_val,debug=debug,avgsum=avgsum
    if n_elements(avgsum) eq 0 then avgsum=0   ;default using average in binning
    if keyword_set(debug) then current=systime(/sec)
    if n_elements(nogrid) eq 0 then nogrid=0
    hstep  =    abs(in_xstep)
    vstep  =    abs(in_ystep)
    if hstep eq 0 then hstep=1.0
    if vstep eq 0 then vstep=1.0
    xmax   = max(xdat,min=xmin)
    ymax   = max(ydat,min=ymin)
    xmax   = xmax+hstep/2.
    xmin   = xmin-hstep/2.
    ymax   = ymax+vstep/2.
    ymin   = ymin-vstep/2.
    xsize  =    (xmax-xmin)/hstep
    ysize  =    (ymax-ymin)/vstep
    if xsize gt fix(xsize,type=3) then xsize=(2.>(xsize+1.))
    if ysize gt fix(ysize,type=3) then ysize=(2.>(ysize+1.))
    xsize  =    fix(xsize,type=3) & ysize=fix(ysize,type=3)
    xlookup    =  xmin+findgen(xsize+1)*hstep
    ylookup    =  ymin+findgen(ysize+1)*vstep
    if nogrid eq 1 then begin
       ;xlookup=xdat[uniq(xdat,sort(xdat))]
       xlookup=uniq_val[uniq(uniq_val,sort(uniq_val))]
       xlookup=[xlookup,xmax]
    endif
    if nogrid eq 2 then begin
       ;ylookup=ydat[uniq(ydat,sort(ydat))]
       ylookup=uniq_val[uniq(uniq_val,sort(uniq_val))]
       ylookup=[ylookup,ymax]
    endif
    xsize=n_elements(xlookup)
    ysize=n_elements(ylookup)
    tmp_x  =    fltarr(xsize)
    tmp_y  =    fltarr(ysize)
    tmp_z  =    fltarr(xsize,ysize)
    num_z  =    lonarr(xsize,ysize)
    num_x  =    lonarr(xsize) & num_y=lonarr(ysize)
    for i=0L,n_elements(xdat)-1L do begin
       ix    =  value_locate(xlookup , xdat[i])
       iy    =  value_locate(ylookup , ydat[i])
       tmp_x[ix] =   tmp_x[ix]+xdat[i]      & num_x[ix]   = num_x[ix]+1
       tmp_y[iy] =   tmp_y[iy]+ydat[i]      & num_y[iy]   = num_y[iy]+1
       tmp_z[ix,iy]= tmp_z[ix,iy]+zdat[i] & num_z[ix,iy] =   num_z[ix,iy]+1
    endfor
    indx   = where(num_x ne 0,countx)
    indy   = where(num_y ne 0,county)
    if countx ne 0 then begin
       tmp_x =   tmp_x[indx]   & num_x=num_x[indx]
       tmp_z =   tmp_z[indx,*] & num_z=num_z[indx,*]
       indx  =    -1
    endif
    if county ne 0 then begin
       tmp_y =   tmp_y[indy]   & num_y=num_y[indy]
       tmp_z =   tmp_z[*,indy] & num_z=num_z[*,indy]
       indy  =    -1
    endif
    xdat   = tmp_x/num_x
    ydat   = tmp_y/num_y
    zdat   = tmp_z
    index  =    where(num_z ne 0,countz)
    if (countz ne 0) and (avgsum eq 0) then zdat[index]    =  tmp_z[index]/num_z[index]    ;average
    index  =    -1
    index  =    where(num_z eq 0,countz)
    if countz ne 0 then zdat[index]    =  !values.f_nan
    index  =    -1
    if keyword_set(debug) then print,'stepgrid_bin finished in ',systime(/sec)-current,' secs.'
end