; $Id: hfbs_rebinonedimdata.pro,v 1.3 2002/05/30 17:12:35 john Exp $
;==========================================
;+
; NAME:
;       hfbs_RebinOneDimData
;
; PURPOSE:
;       Fast rebinning algorithm for one-dimensional data
;
; CATEGORY:
;       Mathematical routine
;
; CALLING SEQUENCE:
;       Result = hfbs_RebinOneDimData(xobins,nbins,yo,yoerr)
;
; INPUTS:  xobins:    original bins
;          nbins:     new number of bins
;          yo:        unbinned data
;          yoerr:     error bars for unbinned data
;
;
; KEYWORD PARAMETERS:
;       None
;
; OUTPUTS:
;       This function returns a three dimensional array consisting of the
;       bins, data, and new error bars
;
; COMMON BLOCKS:
;       None.
;
; PROCEDURE:
;
;
; SIDE EFFECTS:
;       None known
;
; EXAMPLE:
;
;
; MODIFICATION HISTORY:
;       Written by:
;       Robert Dimeo
;       robert.dimeo@nist.gov
;       November 17, 1999
;
function hfbs_RebinOneDimData,xobins,nbins,yo,yoerr
oldpts = n_elements(xobins)
newpts = nbins
if nbins ge oldpts then begin
  print,'error:  number of bins,',nbins,', cannot exceed number of existing channels,',oldpts
  yout = fltarr(newpts,3)
endif else begin
  binlo = xobins[0] & binhi = xobins[oldpts-1]
  dbin = (binhi-binlo)/(newpts-1)
  newbins = fltarr(newpts)
  newbins = binlo+dbin*findgen(newpts)
  yn = fltarr(newpts)
  ynerr = fltarr(newpts)
  for i = 0,newpts-1 do begin
    wherevalid = where(abs(xobins-newbins[i]) le 0.5*dbin,count)
    if count eq 0 then begin
      yn[i] = 0.0 & ynerr[i] = 0.0
    endif else begin
      yn[i] = total(yo[wherevalid])/count
      ynerr[i] = sqrt(total((yoerr[wherevalid])^2))/count
    endelse
  endfor
  yout = fltarr(newpts,3)
  yout[0:newpts-1,0] = newbins[0:nbins-1]
  yout[0:newpts-1,1] = yn[0:nbins-1]
  yout[0:newpts-1,2] = ynerr[0:nbins-1]
endelse
return,yout
end