;$Id: split_scan.pro,v 1.1 2003/08/22 14:20:15 dimeo Exp $
;+
; NAME:
;       SPLIT_SCAN
;
; PURPOSE:
;
;		Simple function wrapper for the "SPLIT" functionality of in the object
;		class HSCN_DATA.  Suitable for invocation at the command line.  The
;		function returns an array of new object references containing the data
;		for the split datasets.
;
; AUTHOR:
;
;       Robert M. Dimeo, Ph.D.
;		NIST Center for Neutron Research
;       100 Bureau Drive
;		Gaithersburg, MD 20899
;       Phone: (301) 975-8135
;       E-mail: robert.dimeo@nist.gov
;       http://www.ncnr.nist.gov/staff/dimeo
;
; CATEGORY:
;
;       Data reduction, DAVE project, HFBS
;
; CALLING SEQUENCE:
;
;		DNEW = SPLIT_SCAN(DATA,TARRAY)
;
; PARAMETERS:
;
;		DATA:		Object reference for the data objects to be split.  If N splits
;					are defined then there will be N+1 object references in DATA.  The
;					DATA and DNEW are of the HSCN_DATA class.
;
;		TARRAY:		Vector of temperatures (or times) at which points data will be split.
;					e.g. TARRAY = [T1,T2,T3]
;
; KEYWORDS:
;
;		None
;
; COMMON BLOCKS:
;
;		None
;
; REQUIREMENTS:
;
;		HSCN_DATA__DEFINE.PRO
;
; DISCLAIMER
;
;		This software is provided as is without any warranty whatsoever.
;		Permission to use, copy, modify, and distribute modified or
;		unmodified copies is granted, provided this disclaimer
;		is included unchanged.
;
; MODIFICATION HISTORY:
;
;       Written by Rob Dimeo, July 14, 2003
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function split_scan,data,tarray
; Check if correct number of parameters were passed in
if n_params() lt 2 then return,0
dnew = data[0]->split(tarray,msg = msg)
return,dnew
end