; $Id: createobjectgrerrors.pro,v 1.3 2003/08/20 20:02:29 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST Jan, 2003
;
; NAME:_______________________
;   createObjectGrErrors
; 
; PURPOSE:
;   Create a IDLgrPolyline object to represent errors for a given
;   (x,y,err) dataset.
;
; INPUT PARAMETERS:___________
;   PARAMETERS:
;
;     xData: a vector specifying the independent (x-axis) values.
;
;     yData: a vector specifying the dependent (y-axis) values.
;
;     yDErr: a vector specifying the corresponding y-axis errors
;
;   KEYWORDS:
;
; OUTPUTS:____________________
;
;     return value: null object if unsuccessful,  an IDLgrPoliline
;     object if successful.
;
;


function createObjectGrErrors, log=log,xdata_in,ydata,yderr,xrange=xrange,yrange=yrange,_EXTRA=etc

if (n_params() ne 3) then begin
    !error_state.code = -1
    !error_state.msg = "createObjectGrErrors: Three parameters are required!"    
    return, obj_new()
endif

; Chop off data that fall outside the display range; the display
; range is defined by xrange and yrange keywords
if (n_elements(xrange) gt 0) then begin
    xmin = xrange[0]
    xmax = xrange[1]
    ;; deal with lower x-axis limit
    index = where(xData_in ge xmin, count)
    if (count gt 0) then begin
        xData = xData_in[index]
        yEhi = yData[index] + yDerr[index]
        yElo = yData[index] - yDerr[index]
    endif else begin
        xData = xData_in
        yEhi = yData + yDerr
        yElo = yData - yDerr
    endelse
    ;; deal with upper x-axis limit
    index = where(xData le xmax, count)
    if (count gt 0) then begin
        xData = xData[index]
        yEhi = yEhi[index]
        yElo = yElo[index]
    endif 
endif else begin
    xData = xData_in
    yEhi = yData + yDerr
    yElo = yData - yDerr
endelse
if (n_elements(yrange) gt 0) then begin
    ymin = yrange[0]
    ymax = yrange[1]

    ;; if data is falls entirely outside specified range then exit
    if ((max(yEhi) le ymin) and (max(yElo) le ymin)) then return, obj_new()
    if ((min(yEhi) ge ymax) and (min(yElo) ge ymax)) then return, obj_new()
 
    ;; deal with lower y-axis limit
    index = where(yEhi gt ymin, count)    
    if (count gt 0) then begin
        xData = xData[index]
        yEhi = yEhi[index]
        yElo = yElo[index]
    endif
    index = where(yElo lt ymin, count)    
    if (count gt 0) then begin
        yElo[index] = ymin
    endif
    ;; deal with upper y-axis limit
    index = where(yElo lt ymax, count)    
    if (count gt 0) then begin
        xData = xData[index]
        yEhi = yEhi[index]
        yElo = yElo[index]
    endif
    index = where(yEhi gt ymax, count)    
    if (count gt 0) then begin
        yEhi[index] = ymax
    endif
endif

nd = n_elements(xData) 
nvert = 2*nd                    ; nos of vertices required
ind = lindgen(nd)

;; Define vertices which specify the end points of the lines
;; representing the errorbars
vert = fltarr(2,nvert)          ; array to hold vertices of error lines
vert[0,2*ind] = xData
vert[0,2*ind+1] = xData
vert[1,2*ind] = yElo
vert[1,2*ind+1] = yEhi
;vert[1,2*ind] = (keyword_set(log))? alog10(yElo) : yElo
;vert[1,2*ind+1] = (keyword_set(log))? alog10(yEhi) : yEhi

;; Each errorbar is a polyline with two vertices. 
;; Define an array describibg all the lines
polyline = intarr(3*nd)         ; array to hold polyline descriptions
polyline[ind*3] = 2             ; number of vertices per line
polyline[ind*3+1] = 2*ind       ; the indices which specify vertex 1 of lines
polyline[ind*3+2] = 2*ind+1     ; the indices which specify vertex 2 of lines

;;
oPoly = obj_new('IDLgrPolyline',vert,polylines=polyline,_EXTRA=etc)

return, (obj_valid(oPoly))? oPoly : obj_new()
end
