; $Id: davedat1d__clone.pro,v 1.2 2003/01/14 15:04:18 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat1D::clone
;
; PURPOSE:____________________
;   This function is used to 
;   create an identical copy of the current object
;
; INPUTS:_____________________
;   NONE
;
; OUTPUTS:____________________
;     return value: a DaveDat1D object if successful 
;                   a NULL object otherwise.
;
function DaveDat1D::clone

newObj = obj_new('davedat1d')   ; create a new (empty) DaveDat1D object

; if newObj was not created, exit now
if (not obj_valid(newObj)) then return, obj_new() ; return a NULL object.

; copy all current object's variables into the new object's
if (self->get(davePtr=davePtr, $
              descriPtr=descriPtr, $
              offset=offset, $
              filename=filename, $
              scalefactor=scalefactor, $
              instrument=instrument, $
              treatment=treatment, $
              xval=xval, $
              xlabel=xlabel, $
              xtype=xtype, $
              xunits=xunits, $
              zval=zval, $
              zerr=zerr, $
              zlabel=zlabel, $
              zunits=zunits)) then begin ; get was successful, so copy vars to newObj
    if (newObj->set(davePtr=davePtr, $
                    descriPtr=descriPtr, $
                    offset=offset, $
                    filename=filename, $
                    scalefactor=scalefactor, $
                    instrument=instrument, $
                    treatment=treatment, $
                    xval=xval, $
                    xlabel=xlabel, $
                    xtype=xtype, $
                    xunits=xunits, $
                    zval=zval, $
                    zerr=zerr, $
                    zlabel=zlabel, $
                    zunits=zunits)) then begin ; set successful, so return newObj
        if (ptr_valid(davePtr)) then heap_free, davePtr
        if (ptr_valid(descriPtr)) then ptr_free, descriPtr
        return, newObj
    endif else begin            ; set unsuccessful, so destroy newObj and return a NULL
        obj_destroy, newObj
        if (ptr_valid(davePtr)) then heap_free,davePtr
        if (ptr_valid(descriPtr)) then ptr_free, descriPtr
        return, obj_new()
    endelse
endif else begin                ; get unsuccessful, so destroy newObj and return a NULL
    obj_destroy, newObj
    if (ptr_valid(davePtr)) then heap_free,davePtr
    if (ptr_valid(descriPtr)) then ptr_free, descriPtr
    return, obj_new()
endelse

end
