; $Id: davedat1d__set.pro,v 1.2 2003/01/14 15:04:18 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat1D::set
; 
; PURPOSE:
;   Main set method for class. Used to set/reset class variables (all
;   class variables are 'protected' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     offset: assign value (expects float) to the object's offset
;     variable
;
;     filename: assign value (expects string) to object's filename variable
;
;     davePtr: assign value (expects a DAVE structure pointer) to object's davePtr variable
;
;     scalefactor: assign value (expects a number) to object's scalefactor variable.
;
;     descriPtr: assign value (expects a descriPtr structure pointer) to
;     object's descriPtr variable
;
;     instrument: assign value (expects scaler string) to object's
;     intrument variable
;
;     xVal: assign value (expects float array) to objects xVal (x-axis
;     values) variable 
;
;     xLabel: assign value (expects scaler string) to object's xLabel
;     (x-axis label) variable
;
;     xType: assign value (expects scaler string) to object's xType
;     (x-axis type) variable.
;
;     xUnits: assign value (expects scaler string) to object's xUnits
;     (x-axis units) variable.
;
;     zVal: assign value (expects float array) to objects zVal (z-axis
;     values) variable
;
;     zErr: assign value (expects float array) to objects zErr (z-axis
;     values) variable.
;
;     zLabel: assign value (expects scaler string) to object's zLabel
;     (z-axis label) variable
;
;     zUnits: assign value (expects scaler string) to object's zUnits
;     (z-axis units) variable.
;
;     treatment: assign value (expects string array) to object's
;     treatPtr (treatment records) variable.
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;
;
; NOTE: DaveDat1D inherits from DaveDat and most of the functionality
; of DaveDat1D::set() is handled directly by DaveDat::set()
;

function DaveDat1D::set, _EXTRA=etc, offset=offset

; reset !error_state structure, ie clear all errors
message,/reset

retVal = 1

; offset
if (n_elements(offset) gt 0) then begin
    ;; make sure value is a number
    offsetType = vartype(offset)
    if ( (offsetType eq 'FLOAT') or $
         (offsetType eq 'DOUBLE') or $
         (offsetType eq 'INT') or $
         (offsetType eq 'LONG') ) then begin
        self.offset = float(offset) 
    endif else begin
        !error_state.msg = 'DaveDat1D::set: offset keyword value is not a float/double'
        !error_state.code = -1
        retVal = 0
    endelse
endif


; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = self->DaveDat::set(_EXTRA=etc)
endif


return, retVal
end
;_______________________________________________________________________________
