; $Id: davegr1d__define.pro,v 1.9 2003/08/14 20:51:18 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr1D::cleanup
;
; PURPOSE:____________________
;   Cleanup method for the DaveGr1D class
;
; INPUTS:_____________________
;   None.
;
pro DaveGr1D::cleanup

if (ptr_valid(self.maskPtr)) then ptr_free, self.maskPtr

; call base class cleanup method
self->DaveGr::cleanup

end


;===============================================================================
; NAME:_______________________
;   DaveGr1D::init
; 
; PURPOSE:
;   Initialization method for the DaveGr1D class
;
; INPUTS:_____________________
;   KEYWORDS:
;
;     dataObj: A DaveDat* object which contains the data
;
;     label: A variable (an ANNOTATE structure) specifying a text
;     label to be associated with the DaveGr object created.
;
; OUTPUTS:____________________
;
;     return value: 0 signifies failure and 1 success.
;
function DaveGr1D::init, _EXTRA=etc 

; reset !error_state structure, ie clear all errors
message,/reset

; if a dataObj keyword was passed make sure that it's value
; is a DaveDat1D object
; then call base class initialization after
if (n_elements(etc) gt 0) then begin
    res = where(tag_names(etc) eq 'DATAREF', dataRef_isSet)
    if (dataRef_isSet gt 0) then begin
        if (vartype((etc.dataRef)[0]) ne 'DAVEDAT1D') then begin
            !error_state.msg = 'DaveGr1D::init: invalid object reference (dataRef) passed'
            !error_state.code = -1
            return, 0
        endif
    endif 
endif
if (self->DaveGr::init(_EXTRA=etc) ne 1) then return, 0 ; base class initialization

; Set default values for class variables. Use the x-axis type
; (points/histogram) to determine default values.
if (self->count() gt 0) then begin
    if (self->get(dataRef=dataRef)) then begin
        for i=0,n_elements(dataRef)-1 do begin
            if (dataRef[i]->get(xtype=xtype)) then begin
                case strmid(strupcase(xtype),0,4) of
                    'HIST': self.lineSymFlag[i] = 0
                    
                    'POIN': self.lineSymFlag[i] = 1
                    
                    else: self.lineSymFlag[i] = 1
                endcase
            endif else begin
                self.lineSymFlag[*] = 1 ; set to POINTS
            endelse
        endfor 
    endif else begin
        self.lineSymFlag[*] = 1 ; set to POINTS
    endelse
endif
self.lineType = 0               ; solid line
self.lineSize = 1.0             ;
self.symType = 6                ; Square
self.symSize = 1.5              ;
self.color = 255b   ; 'WHITE'


; Initialize miscellaneous axes graph attributes
self->resetMiscGrAttribs

; Initialize legend properties
self->resetLegend

return, 1

end


;===============================================================================
; NAME:_______________________
;   DaveGr1D__define
;
; PURPOSE:____________________
;   Definition of the class structure for the DaveGr1D class
;
; INPUTS:_____________________
;   None.
;
pro DaveGr1D__define

nDataMax = 50

legend = {LegendStr, $
          bkgdColor:[0b,0b,0b], $ ; bkgd color 
          font:'', $            ; item font
          fontsize:0.0, $       ; item fontsize
          tFont:'', $           ; legend title font
          tFontsize:0.0, $      ; legend title fontsize
          title:'', $           ; legend title
          tColor:[0b,0b,0b], $  ; legend title color
          gap:0.0, $            ; gap between items
          gWidth:0.0, $         ; width of glyphs
          hideFlag:0b, $        ; hide legend flag: 0-yes; 1-no
          fillFlag:1b, $        ; fill bkgd flag: 0-no, 1-yes
          outlFlag:1b, $        ; outline (border) flag: 0-no, 1-yes
          outlColor:[0b,0b,0b], $ ; outline (border) color
          outlThick:0.0, $      ; thickness of outline
          textColor:[0b,0b,0b], $ ; item text color
          location:[0.0,0.0] , $ ; legend location in normalised units
          itemName:strarr(nDataMax), $ ; item names
          itemThick:fltarr(nDataMax) $ ; item thicknesses
         }

void = {DaveGr1D, $
        lineSymFlag:bytarr(nDataMax), $ ; specifies display type: 0 - lines; 1 - symbols; 2 - lines+symbols
        lineType:bytarr(nDataMax), $ ; line style eg solid, dash etc
        lineSize:fltarr(nDataMax)+1.0, $ ; thickness of line
        symType:bytarr(nDataMax)+6b, $ ; symbol type - default is square
        symSize:fltarr(nDataMax)+1.0, $ ; symbol size
        color:bytarr(nDataMax,3)+255b, $ ; line/symbol display color - uses a custom (16) color table; 1 --> WHITE
        maskPtr:ptr_new(), $    ; ptr to an array of masked x-axis indices
        legend:{legendStr}, $   ; legend
        inherits DaveGr $       ; 
       }

end
