; $Id: davegr1d__get.pro,v 1.6 2003/10/21 15:27:03 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr1D::get
; 
; PURPOSE:
;   Main get method for class. Used to retrieve class variables (all
;   class variables are 'private' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     label: Return filename as string; an empty string is return
;     if the objects filename is undefined.
;
;     inUse: Return inUse as integer.
;
;     dataRef: Return a reference to the object's dataObj. Any
;     subsequent modifications to this reference will also alter the object.
;
;     dataObj: Return a _copy_ of the object's dataObj
;
;     lineType: Returns the linetype index (integer)
;
;     lineSize: Reurns the line size (float)
;
;     symType: Returns the symbol index (integer)
;
;     symSize: Returns the symbol size (float)
;
;     lineSymFlag: Returns the lineSymFlag which specifies whether
;     lines, symbols or symbols+lines are to be plotted 
;
;     myPlotType: Returns the plot type (string): 'POINT' or
;     'HISTOGRAM'. Normally, the plot type is determined by the
;     independent axis type. However, it is possible to overide the
;     default type using this keyword in combination with the
;     myPlotTypeFlag keyword. 
;
;     myPlotTypeFlag: Returns (integer) setting for myPlotTypeFlag
;     variable. If set to 1, then the myPlotType setting will be used
;     overiding the independent variable type. 
;
;     color: Returns the line/symbol display color (string).
;
;     maskIndex: Returns a list of independent axis (integer) indices to be masked
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;

function DaveGr1D::get, lineType=lineType, lineSize=lineSize, symType=symType, symSize=symSize, all=all, $
  lineSymFlag=lsFlag, color=color, maskIndex=maskIndex, position=pos, legend=legend, $
  xmin=xmin,xmax=xmax,zmin=zmin,zmax=zmax,_REF_EXTRA=etc

retVal = 1

pos_IsSet = 0b
if (n_elements(pos) gt 0) then pos_IsSet = 1b

if (keyword_set(all)) then begin
    pos_IsSet = 0b
    etc = (n_elements(etc) gt 0)? create_struct(etc,'all',1) : {all:1}
endif

; Return lineType
if (arg_present(lineType)) then begin
    lineType = (pos_IsSet)? (self.lineType)[pos] : self.lineType
endif

; Return lineSize
if (arg_present(lineSize)) then begin
    lineSize = (pos_IsSet)? (self.lineSize)[pos] : self.lineSize
endif

; Return symType
if (arg_present(symType)) then begin
    symType =(pos_IsSet)? (self.symType)[pos] : self.symType
endif

; Return symSize
if (arg_present(symSize)) then begin
    symSize =(pos_IsSet)? (self.symSize)[pos] : self.symSize
endif

; Return lineSymFlag
if (arg_present(lsFlag)) then begin
    lsFlag =(pos_IsSet)? (self.lineSymFlag)[pos] : self.lineSymFlag
endif

; Return myPlotType
;if (arg_present(mpType)) then begin
;    mpType = self.myPlotType
;endif

; Return myPlotTypeFlag
;if (arg_present(mpTypeFlag)) then begin
;    mpTypeFlag = self.myPlotTypeFlag
;endif

; Return color
if (arg_present(color)) then begin
    color =(pos_IsSet)? (self.color)[pos,*] : self.color
endif

; Return maskIndex
if (arg_present(maskIndex)) then begin
    if (ptr_valid(self.maskPtr)) then maskIndex = (*self.maskPtr)
endif


; global xmin,xmax,zmin,zmax for all datasets
if (arg_present(xmin) or arg_present(xmax) or arg_present(zmin) or arg_present(zmax)) then begin
    ;; get all data
    dataRef = self->IDL_CONTAINER::get(count=nData,/all)

    ;; determine minima/maxima for combine datasets
    for i = 0,nData-1 do begin
        if (not (dataRef[i])->get(xval=xval,zval=zval,offset=offset,scalefactor=scalefactor)) then $
          return, 1
        if (i eq 0) then begin
            xmin = min(xval, max=xmax)
            ;zmin = min((zval*scalefactor+offset), max=zmax)
            zmin = min(zval, max=zmax)
        endif else begin
            xmini = min(xval, max=xmaxi) ; get xmin, xmax for current data
            ;zmini = min((zval*scalefactor+offset), max=zmaxi) ; get zmin, zmax for current data
            zmini = min(zval,max=zmaxi); get zmin, zmax for current data
            xmin = (xmini < xmin) ; running xmin
            xmax = (xmaxi > xmax) ; running xmax
            zmin = (zmini < zmin) ;
            zmax = (zmaxi > zmax) ;
        endelse
    endfor
endif
         

; Return legend
if (arg_present(legend)) then begin
    legend = self.legend
endif


; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = (pos_IsSet)? self->DaveGr::get(position=pos, _EXTRA=etc) : self->DaveGr::get(_EXTRA=etc)
endif

return, retVal

end
