; $Id: davegr2d__resetmiscgrattribs.pro,v 1.2 2003/10/20 16:28:45 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr2D::resetMiscGrAttribs
;
; PURPOSE:____________________
;   Private method to initialize/update various axes graph attributes
;   so they are consistent with the data.
;
; INPUTS:_____________________
;   KEYWORDS
;     limits: update only the limits (minimum, maximum and major tick interval)
;   None.
;
pro DaveGr2D::resetMiscGrAttribs, limits=limits, index=index

message,/reset

nData = self->count()
if (nData le 0) then begin ; no data present
    xmin = 0.0
    xmax = 1.0
    ymin = 0.0
    ymax = 1.0
    zmin = 0.0
    zmax = 1.0
endif else begin
    if (not self->get(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax,zmin=zmin,zmax=zmax)) then begin
        if (!debug) then print, !error_state.msg
        return
    endif
endelse

if (keyword_set(limits)) then begin
    self.xMiscAttribs.useCustomFlag = 0
    self.yMiscAttribs.useCustomFlag = 0
    self.zMiscAttribs.useCustomFlag = 0

    xmin = xmin*self.xMiscAttribs.scaleFactor
    xmax = xmax*self.xMiscAttribs.scaleFactor
    ymin = ymin*self.yMiscAttribs.scaleFactor
    ymax = ymax*self.yMiscAttribs.scaleFactor
    zmin = zmin*self.xMiscAttribs.scaleFactor
    zmax = zmax*self.xMiscAttribs.scaleFactor
    ;; X
    self.xMiscAttribs.minimum = xmin
    self.xMiscAttribs.maximum = xmax
    ;; Y
    self.yMiscAttribs.minimum = ymin
    self.yMiscAttribs.maximum = ymax
    ;; Z
    self.zMiscAttribs.minimum = zmin
    self.zMiscAttribs.maximum = zmax   
    ;; tick interval - set nos of major ticks to 5 (including end points) by default
    self.xMiscAttribs.tickInterval = (xmax-xmin)*0.25 ; 5 major ticks ie end points + 3 middle ones
    self.yMiscAttribs.tickInterval = (ymax-ymin)*0.25 ; 5 major ticks ie end points + 3 middle ones
    self.zMiscAttribs.tickInterval = (zmax-zmin)*0.25 ; 5 major ticks ie end points + 3 middle ones
    return
endif

if (not n_elements(index)) then index=-1


if (index ne 2) then begin ; do x-axis
    ;; scalefactor
    self.xMiscAttribs.scaleFactor = 1.0

    ;; min & max
    self.xMiscAttribs.minimum = xmin
    self.xMiscAttribs.maximum = xmax

    ;; tick interval - set nos of major ticks to 5 (including end points) by default
    self.xMiscAttribs.tickInterval = (xmax-xmin)*0.25 ; 5 major ticks ie end points + 3 middle ones

    ;; flag to indicate whether custom user settings (min, max, tick
    ;; interval) should be used 
    self.xMiscAttribs.useCustomFlag = 0 ; don't use custom

    ;; log/linear axis
    self.xMiscAttribs.logFlag = 0 ; linear

    ;; Nos of decimals to display - 2 by default
    self.xMiscAttribs.decimalPlaces = 2

    ;; tick direction for primary axis- inwards/outwards
    self.xMiscAttribs.tickDir = 0 ; inwards

    ;; tick length (as percentage of axis length)
    self.xMiscAttribs.tickLen = 2.5

    ;; Nos of minor ticks between major ticks
    self.xMiscAttribs.minorTicks = 4
endif

if (index ne 2) then begin ; do y-axis
    ;; scalefactor
    self.yMiscAttribs.scaleFactor = 1.0

    ;; min & max
    self.yMiscAttribs.minimum = ymin
    self.yMiscAttribs.maximum = ymax

    ;; tick interval - set nos of major ticks to 5 (including end points) by default
    self.yMiscAttribs.tickInterval = (ymax-ymin)*0.25 ; 5 major ticks ie end points + 3 middle ones

    ;; flag to indicate whether custom user settings (min, max, tick
    ;; interval) should be used 
    self.yMiscAttribs.useCustomFlag = 0 ; don't use custom

    ;; log/linear axis
    self.yMiscAttribs.logFlag = 0 ; linear

    ;; Nos of decimals to display - 2 by default
    self.yMiscAttribs.decimalPlaces = 2

    ;; tick direction for primary axis- inwards/outwards
    self.yMiscAttribs.tickDir = 0 ; inwards

    ;; tick length (as percentage of axis length)
    self.yMiscAttribs.tickLen = 2.5

    ;; Nos of minor ticks between major ticks
    self.yMiscAttribs.minorTicks = 4
endif


if (index ne 0) then begin ; do z-axis 
    ;; scalefactor
    self.zMiscAttribs.scaleFactor = 1.0

    ;; min and max
    self.zMiscAttribs.minimum = zmin
    self.zMiscAttribs.maximum = zmax

    ;; tick interval - set nos of major ticks to 5 (including end points) by default
    self.zMiscAttribs.tickInterval = (zmax-zmin)*0.25 ; 5 major ticks ie end points + 3 middle ones

    ;; flag to indicate whether custom user settings (min, max, tick
    ;; interval) should be used 
    self.zMiscAttribs.useCustomFlag = 0 ; don't use custom

    ;; log/linear axis
    self.zMiscAttribs.logFlag = 0 ; linear

    ;; Nos of decimals to display - 2 by default
    self.zMiscAttribs.decimalPlaces = 2

    ;; tick direction for primary axis- inwards/outwards
    self.zMiscAttribs.tickDir = 0 ; inwards

    ;; tick length (as percentage of axis length)
    self.zMiscAttribs.tickLen = 2.5

    ;; Nos of minor ticks between major ticks
    self.zMiscAttribs.minorTicks = 4
endif

end
