; $Id: getformatcode.pro,v 1.2 2003/07/11 20:02:12 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
; July, 2003
;
; NAME:_______________________
;   getFormatCode
;
; PURPOSE:____________________
;   Determine appropriate format code to display data to a specified
;   precision (number of decimal places).
;
; INPUTS:_____________________
;   PARAMETERS
;     minimum: smallest number to be displayed
;     maximum: largest number to be displayed
;     decimalPlaces: number of decimal places to be displayed
; 
; OUTPUT:_____________________
;    returns a string containing the format code
;
;
function getFormatCode, minimum, maximum, decimalPlaces

fCode = '(G10.4)'               ; generic format string

if (n_params() ne 3) then return, fCode

absmax = abs(minimum) > abs(maximum) ; determine absolute maximum to be displayed

size = floor(alog10(absmax))  ; determine size of maximum data

case 1 of
    ((size ge 4) or (size le -4)): begin
        fieldWidth = strtrim(string(fix(7+decimalPlaces)),2)
        precision = strtrim(string(fix(1+decimalPlaces)),2)
        fCode = '(G'+fieldWidth+'.'+precision+')'
        return, fcode
    end

    ((size lt 1) and (size gt -4)): begin
        fieldWidth = strtrim(string(fix(3+decimalPlaces)),2)
        precision = strtrim(string(fix(decimalPlaces)),2)
        fCode = '(F'+fieldWidth+'.'+precision+')'
        return, fcode
    end

    else: begin
        fieldWidth = strtrim(string(fix(size+3+decimalPlaces)),2)
        precision = strtrim(string(fix(decimalPlaces)),2)
        fCode = '(F'+fieldWidth+'.'+precision+')'
        return, fcode
    end
endcase


end
