;$Id: db_display2dc__define.pro,v 1.16.2.1 2003/12/11 22:18:54 azuah Exp $
;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Cleanup procedure for DB_Display2DC
;
pro DB_Display2DC::cleanup

; store/save current GUI/widget settings in the DaveGr1D object
void = (self.oDaveGr)->get(guiAttribsPtr=guiAttribsPtr)
(*guiAttribsPtr) = {winDim:self.winDim, $
                    xrangeFlag:self.xrangeFlag, $
                    xrange:self.xrange, $
                    yrangeFlag:self.yrangeFlag, $
                    yrange:self.yrange, $
                    zrangeFlag:self.zrangeFlag, $
                    zrange:self.zrange, $
                    dispTitle:self.dispTitle $
                   }

obj_destroy, self.oScene
obj_destroy, self.oDest
(self.o_registry)->remove,self  ; unregister self from Object Registry
(self.oActive2DCont)->remove, self.oDaveGr ; unregister DaveGr object from the active view list
ptr_free,self.levelIDsPtr,self.ctableIDsPtr
device, decomposed=self.old_dec

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
;  Refresh (display data) method
pro DB_Display2DC::refresh, _EXTRA=etc

; require color decomposition (for using rgb colors directly)
; return to previous setting at end of this procedure
device, get_decomposed = old_dec
device, decompose=1

switch self.renewScene of

    0: begin
        self->renew  ; reset to default value of 1
        objArr = (self.oScene)->get(/all, count=nObj)
        ;; if Scene is empty, proceed to case '1' by not 'breaking'
        if (nObj gt 0) then begin
            (self.oDest)->draw, self.oScene
            break
        endif
    end

    1: begin
        ;; Modify tlb widget title so it includes the current name tag
        ;; for the DaveGr object being viewed
        void = (self.oDaveGr)->get(nameTag=title)
        widget_control, self.tlb,tlb_set_title='DATABROWSER - '+title

        ;; create etc with dummy variable
        etc = {dummyRefresh:0}

        ;; Set display mode
        ;case self.dispType of
        ;    0: etc = create_struct(etc,'dispType','POINTS')
        ;    1: etc = create_struct(etc,'dispType','HISTOGRAMS')
        ;endcase

        ;; Title
        etc = create_struct(etc,'dispTitle',self.dispTitle)

        ;; x-axis limits
        if (self.xrangeFlag) then begin
            etc = create_struct(etc,'xrange',self.xrange)
        endif
        ;; y-axis limits
        if (self.yrangeFlag) then begin
            etc = create_struct(etc,'yrange',self.yrange)
        endif
        ;; z-axis limits
        if (self.zrangeFlag) then begin
            etc = create_struct(etc,'zrange',self.zrange)
        endif

        ; display
        (self.oDaveGr)->display, self.oDest, self.oScene, _EXTRA=etc

        ; update data-range widgets
        if (self.zrangeFlag eq 0) then begin
            if (not (self.oDaveGr)->get(zmin=zmin,zmax=zmax)) then return
            self.zrange = [zmin,zmax]
            wid = widget_info(self.tlb, find_by_uname='ZMIN')
            widget_control, wid, set_value=self.zrange[0]
            wid = widget_info(self.tlb, find_by_uname='ZMAX')
            widget_control, wid, set_value=self.zrange[1]
        endif
    end

endswitch

; return color decomposition to previous setting
device, decompose=old_dec

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Method for dealing with events generated by Context
; Sensitive Menus
;
pro DB_Display2DC::handleConSenMenuEvent, event

;widget_control, event.top, get_uvalue = sPtr

; Get the selected CSM's name and value
uname = widget_info(event.id,/uname)
widget_control, event.id, get_uvalue=newValue
;newValue = newValueStr.val
;newIndex = newValueStr.ind

; Get the atom/model for which the CSM should act on
objRef = self.csmObjRef

switch uname of
    'FILL': begin
        validAtomNames = ['CONTOUR'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    objArr[nObj]->setProperty,fill=newValue
                    if (not daveGrRef->set(fillFlag=newValue)) then return
                    ;; Put a check mark against the selected value
                    fillIDs = self.fillIDs
                    case newValue of
                        0: begin
                            widget_control, fillIDs[0], set_button=1
                            widget_control, fillIDs[1], set_button=0
                        end
                        1: begin
                            widget_control, fillIDs[0], set_button=0
                            widget_control, fillIDs[1], set_button=1
                        end
                    endcase
                    self->noRenew
                endif
            endwhile
        endif
        break
    end

    'SHADE': begin
        validAtomNames = ['CONTOUR'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    objArr[nObj]->setProperty,shading=newValue
                    if (not daveGrRef->set(shadeFlag=newValue)) then return
                    ;; Put a check mark against the selected value
                    case newValue of
                        0: begin
                            widget_control, self.shadeIDs[0], set_button=1
                            widget_control, self.shadeIDs[1], set_button=0
                        end
                        1: begin
                            widget_control, self.shadeIDs[0], set_button=0
                            widget_control, self.shadeIDs[1], set_button=1
                        end
                    endcase
                    self->noRenew
                endif
            endwhile
        endif
        break
    end


    'CTABLE': begin
        validAtomNames = ['CONTOUR'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    if (not daveGrRef->set(colorIndex=newValue)) then return
                    if (not daveGrRef->get(palette=palette,nlevels=nlevels)) then return
                    palette->getProperty,n_colors=ncolors ;red=red,green=green,blue=blue,n_colors=ncolors
                    stepc = float((ncolors-1)/(nlevels-1))
                    ccolors = bindgen(nlevels)
                    for i=0,nlevels-1 do ccolors[i]=fix(i*stepc);[red[i*stepc],green[i*stepc],blue[i*stepc]]
                    objArr[nObj]->setProperty,c_color=ccolors
                    ;; Put a check mark against the selected value
                    ctableIDs = *(self.ctableIDsPtr)
                    for i=0,n_elements(ctableIDs)-1 do begin
                        if (ctableIDs[i] eq event.id) then $
                          widget_control, ctableIDs[i], set_button=1 else $
                          widget_control, ctableIDs[i], set_button=0
                    endfor
                    self->noRenew
                endif
            endwhile
        endif
        break
    end


    'NLEVEL': begin
        validAtomNames = ['CONTOUR'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    ;objArr[nObj]->setProperty,linestyle=newValue
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    if (not daveGrRef->get(palette=palette)) then return
                    nlevels = newValue
                    palette->getProperty, n_colors=ncolors ;,red=red,green=green,blue=blue
                    stepc = float((ncolors-1)/(nlevels-1))
                    ccolors = bindgen(nlevels)
                    for i=0,nlevels-1 do ccolors[i] = fix(i*stepc)
                    objArr[nObj]->setProperty,c_color=ccolors
                    if (not daveGrRef->set(nlevels=nlevels)) then return
                    ;; Put a check mark against the selected value
                    levelIDs = *(self.levelIDsPtr)
                    for i=0,n_elements(levelIDs)-1 do begin
                        if (levelIDs[i] eq event.id) then $
                          widget_control, levelIDs[i], set_button=1 else $
                          widget_control, levelIDs[i], set_button=0
                    endfor
                    ;self->renew
                endif
            endwhile
        endif
        break
    end


    'THICK': begin
        validAtomNames = ['AXIS'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->setProperty,thick=newValue
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    case datInd of
                        0: kw_val = {xLineSize:newValue} ; x-axis
                        1: kw_val = {yLineSize:newValue} ; y-axis
                        else: kw_val = {dummy:0}
                    endcase
                    if (not daveGrRef->set(_EXTRA=kw_val)) then return
                    self->noRenew
                endif
            endwhile
        endif 
        break
    end

    'COLOR': begin
        validAtomNames = ['AXIS','LABEL'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->setProperty,color=newValue
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    case atomName of
                        'AXIS': begin
                            case datInd of
                                0: kw_val = {xColor:newValue} ; x-axis
                                1: kw_val = {yColor:newValue} ; y-axis
                            endcase
                            if (not daveGrRef->set(_EXTRA=kw_val)) then return
                        end

                        'LABEL': begin
                            case datInd of
                                -1: begin ; title
                                    if (not daveGrRef->get(title=title)) then return
                                    title.color = newValue
                                    if (not daveGrRef->set(title=title)) then return
                                end
                                else: begin ; misc label
                                    if (not daveGrRef->get(label=label)) then return
                                    label[datInd].color = newValue
                                    if (not daveGrRef->set(label=label)) then return
                                end
                            endcase
                        end
                    endcase
                    self->noRenew
                endif
            endwhile
        endif
        break
    end

    'AFONT': begin
        validAtomNames = ['AXIS','LABEL'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    case atomName of 
                        'AXIS': begin
                            objArr[nObj]->getProperty, title=titleRef, uvalue=atomUval, ticktext=ttRef
                            datInd = atomUval.datInd
                            case datInd of
                                0: kw_val = {xFontSize:newValue} ; x-axis
                                1: kw_val = {yFontSize:newValue} ; y-axis
                                else: kw_val = {dummy:0}
                            endcase
                            if (obj_valid(titleRef)) then begin
                                titleRef->getProperty, font=fontRef
                                fontRef->setProperty, size=newValue
                                ;ttRef->setProperty, size=newValue
                                daveGrRef = atomUval.objRef
                                if (not daveGrRef->set(_EXTRA=kw_val)) then return
                            endif
                        end
                        'LABEL': begin
                            objArr[nObj]->getProperty, font=fontRef, uvalue=atomUval
                            datInd = atomUval.datInd
                            daveGrRef = atomUval.objRef
                            fontRef->setProperty, size=newValue
                            case datInd of
                                -1: begin ; title
                                    if (not daveGrRef->get(title=title)) then return
                                    title.fontsize = newValue
                                    if (not daveGrRef->set(title=title)) then return
                                end
                                else: begin ; misc label
                                    if (not daveGrRef->get(label=label)) then return
                                    label[datInd].fontsize = newValue
                                    if (not daveGrRef->set(label=label)) then return
                                end
                            endcase
                        end
                    endcase
                    self->noRenew
                endif
            endwhile
        endif
        break
    end

    'EDITLABLE' : begin
        validAtomNames = ['LABEL'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    DB_TxtLabel, datInd,objArr[nObj],daveGrRef,self,group_leader=event.top
                    self->noRenew
                endif
            endwhile
        endif
        break
    end

    'EDITAXIS' : begin
        validAtomNames = ['AXIS'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval, notext=emptyTitle
                    if (not emptyTitle) then begin
                        daveGrRef = atomUval.objRef
                        datInd = atomUval.datInd
                        ;datInd = (atomUval.datInd eq 0)? atomUval.datInd : 2 ; for 1D, zaxis == yaxis!!!
                        DB_AxisProp, datInd,objArr[nObj],daveGrRef,self,group_leader=event.top
                        self->noRenew
                    endif
                endif
            endwhile
        endif
        break
    end
    
    else: begin
        break
    end
    
endswitch

self->refresh      ; refresh (NOT through registry in db_container!)

return

end




;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Display Context Sensitive Menus
pro DB_Display2DC::dispConSenMenu, event

objArr = (self.oScene)->get(/all, count=nObj) ; get nos of objects in scene
while (nObj gt 0) do begin      ; loop through and
    nObj = nObj - 1
    if (vartype(objArr[nObj]) ne 'IDLGRVIEW') then continue
    oView = objArr[nObj]
    dims = [10,10]
    compArr = (self.oDest)->Select(oView,[event.x, event.y],dimensions = dims) ; grab model/atoms in view at mouse loc
    if (obj_valid(compArr[0]) ne 1) then continue
    count = n_elements(compArr)
    while (count gt 0) do begin
        count = count - 1
        ;; Analyse components
        (compArr[count])->getProperty, name=name
        self.csmObjRef = compArr[count]
        case name of
            'CONTMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBCONT')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
            end
            'LABMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBLABEL')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
            end
            'AMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBAXES')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
            end
            'COLORBAR':
        endcase
    endwhile
endwhile


end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
;  Print method
; - Print current display
; - Export to various file formats
pro DB_Display2DC::print, event, _EXTRA=etc
uname = widget_info(event.id,/uname)

switch uname of
    'TIFF':
    'BMP':
    'PNG':
    'JPEG': begin
        ;; force a window refresh before capturing image
        self->noRenew
        self->refresh
        ;; capture destination window contents
        oImage = (self.oDest)->read()
        oImage->getProperty, data=image
        obj_destroy, oImage
        title = 'DATABROWSER - Export '+uname+' Image'
        defFile = 'output2d.'+strlowcase(uname)
        ;print,self.workDir
        result = dialog_write_image(image,dialog_parent=event.top,path=self.workDir, $
                                    title=title,/warn_exist,type=uname,file=defFile)
        break
    end
    'POST': begin
        defFile = 'output2d.ps'
        title = 'DATABROWSER - Export Postscript'
        oPSC = obj_new("FSC_PSCONFIG",directory=self.workDir,filename=defFile)
        options = PSConfig(oPSC,cancel=cancelled,group_leader=event.top)
        pageDimensions = oPSC->pagedimensions()
        if (not cancelled) then begin
            if (options.landscape eq 1) then begin
                yoff = options.xoffset
                xoff = options.yoffset - options.xsize
                pageDimensions=[pageDimensions[1],pageDimensions[0]]
            endif else begin
                xoff = options.xoffset
                yoff = options.yoffset
            endelse
            units = (options.inches eq 1)? 1 : 2
            
            ;; Create a clipboard
            oClip = obj_new('IDLgrClipboard',units=units,dimensions=pageDimensions);[paperxsize,paperysize])

            ;; Get the scene/view that is being displayed.
            oView = (self.oScene)->get(/all,isa='IDLgrView')
            oView->getProperty, location=winLoc,dimensions=winDim,units=winUnits
            oView->setProperty, units=units, $
                                dimensions=[options.xsize,options.ysize], $
                                location=[xoff,yoff]

            ;; Render the scene to the clipboard
            oClip->draw, self.oScene, file=options.filename,/postscript,/vector
            
            ;; Cleapup and revert view to previous settings
            obj_destroy, [oClip,oPSC]
            oView->setProperty, location=winLoc,dimensions=winDim,units=winUnits
        endif

        break
    end
    'PRINT': begin
        oPrint = obj_new('IDLgrPrinter')
        title = 'DATABROWSER - Printer Setup'
        valid = dialog_printersetup(oPrint,dialog_parent=event.top,title=title)
        if (valid) then begin
            oPrint->draw, (self.oScene)
            oPrint->newDocument ; forces current document to be sent to printer
        endif
        obj_destroy, oPrint
        break
    end
endswitch


end




;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Handler for draw widget events
pro DB_Display2DC::drawEvent, event

uname = widget_info(event.id,/uname)

possibleEventTypes = ['DOWN','UP','MOTION','SCROLL','EXPOSE']
eventType = possibleEventTypes[event.type]

case eventType of
    'DOWN': begin               ; Button press event
        self->noRenew     ; ==> don't re-construct Scene
        if (event.press eq 4) then self->dispConSenMenu, event ; Right Mouse button was pressed; display Con Sen Menu
        if (event.press eq 1) then begin ;Left mouse button was pressed; zoomin functionality is required
           
            self.rbStart = [event.x,event.y] ; register the mouse location
            widget_control, event.id, draw_motion_events=1 ; switch on motion events
            ;self.LMBisDown = 1  ; LMB is down

            ;; get properties of view which contains current plot
            objArr = (self.oScene)->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                if (vartype(objArr[nObj]) ne 'IDLGRVIEW') then continue
                (self.oDest)->getProperty, dimensions = winDims ; dimensions of destination window
                (objArr[nObj])->getProperty, all=allViewProps ; properties of view gr object
                ;; create a rubberband obj gr atom
                rbandColor = [255,0,0];255 - allViewProps.color ; set rub band color to reverse of view color
                vprect = allViewProps.viewplane_rect ; dimensions of viewplane
                vprect[2] = vprect[2]/float(winDims[0])
                vprect[3] = vprect[3]/float(winDims[1])
                self.rbScale = vprect
                rbandModel = obj_new('IDLgrModel',name='RBANDMODEL')
                self.oRband = obj_new('IDLgrPolyline',name='RBAND',color=rbandColor)
                (objArr[nObj])->add, rbandModel
                rbandModel->add, self.oRband
                nObj = 0
            endwhile
        endif
    end
    'UP': begin                 ; Button release event
        if (event.release eq 1) then begin ; Left mouse button released
            ;; cleanup rubberband obj gr model and atom
            if (obj_valid(self.oRband)) then begin
                (self.oRband)->getProperty, parent=rbandModel
                rbandModel->getProperty, parent=oView
                oView->remove, rbandModel
                obj_destroy, rbandModel ; should destroy rub band atom as well
            endif

            widget_control, event.id, draw_motion_events=0 ; switch off motion events
            widget_Control, event.id, /clear_events ; clear any motion events that may have occurred.
            ;self.LMBisDown = 0

            if (not self.mouseMoved) then break ; Exit now if releasing from same spot that mouse was pressed

            ;; rubberband box is new zoom region
            self.xrangeFlag = 1
            xmin = self.rbStart[0]*self.rbScale[2] + self.rbScale[0]
            xmax = event.x*self.rbScale[2] + self.rbScale[0]
            self.xrange = (xmax gt xmin)? [xmin,xmax] : [xmax,xmin]
            self.yrangeFlag = 1
            ymin = self.rbStart[1]*self.rbScale[3] + self.rbScale[1]
            ymax = event.y*self.rbScale[3] + self.rbScale[1]
            self.yrange = (ymax gt ymin)? [ymin,ymax] : [ymax,ymin]

            ;self->noRenew ; ==> don't re-construct Scene
            self->refresh ; refresh plot scene via self
            
        endif
    end
    'MOTION': begin             ; Motion event
        self.mouseMoved = 1     ; register mouse move

        ;; determine rubber band box dimensions and modify the rub
        ;; band obj gr atom 
        start = self.rbStart
        boxVertices = fltarr(2,5)
        boxVertices[0,*] = [start[0],event.x,event.x,start[0],start[0]]*self.rbScale[2] + self.rbScale[0]
        boxVertices[1,*] = [start[1],start[1],event.y,event.y,start[1]]*self.rbScale[3] + self.rbScale[1]
        (self.oRband)->setProperty, data=boxVertices
        
        ;(self.oRband)->getProperty, parent=rbandModel
        ;rbandModel->getProperty, parent=rbandView
        self->noRenew     ; ==> don't re-construct Scene
        self->refresh ; refresh plot scene via self
        ;(self.oDest)->draw, rbandView ; draw rubber band
    end
    'SCROLL':
    'EXPOSE': begin             ; Expose event, so repaint window
        ;print,'Exposed Event generated...'
        self->noRenew     ; ==> don't re-construct Scene
        self->refresh ; refresh via self
    end
endcase

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Reset to default axis limits (ie x,y and data limits)
pro DB_Display2DC::defXYLimEvent, event

self.xrangeFlag = 0
self.yrangeFlag = 0

self->refresh      ; refresh via self

end

;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Reset to default data range
pro DB_Display2DC::defZLimEvent, event

self.zrangeFlag = 0

if (not (self.oDaveGr)->get(zmin=zmin,zmax=zmax)) then return
self.zrange=[zmin,zmax]
wid = widget_info(event.top, find_by_uname='ZMIN')
widget_control, wid, set_value=zmin
wid = widget_info(event.top, find_by_uname='ZMAX')
widget_control, wid, set_value=zmax

self->refresh      ; refresh via self

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Set minimum data to be displayed
pro DB_Display2DC::setZrange, event

uname = widget_info(event.id,/uname)

case uname of
    'ZMIN': begin
        if (event.value lt self.zrange[1]) then begin
            self.zrange[0] = event.value 
            self.zrangeFlag = 1
        endif else widget_control, event.id, set_value=self.zrange[0]
    end
    'ZMAX': begin
        if (event.value gt self.zrange[0]) then begin
            self.zrange[1] = event.value 
            self.zrangeFlag = 1
        endif else widget_control, event.id, set_value=self.zrange[1]
    end
endcase

self->refresh      ; refresh via self

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Resize
pro DB_Display2DC::tlbEvent, event

res = widget_info(event.top,/geometry)
new_x = event.x - self.dialog_x - 2*res.xpad
new_y = event.y - 2*res.ypad
new_x = new_x > self.min_draw_x
new_y = new_y > self.min_draw_y
(self.oDest)->setProperty, dimensions=[new_x,new_y]
(self.oDest)->getProperty, dimensions=dim
self.winDim = dim

self->noRenew
;print,'Refreshing b/c of resize event...'
self->refresh      ; refresh via self

end

;===============================================================================
; Switch main title on/off
pro DB_Display2DC::dispTitleEvent, event

self.dispTitle = 1 - self.dispTitle ; swap setting (0 <--> 1)

self->refresh

end


;===============================================================================
; Exit (quit)
pro DB_Display2DC::exitEvent, event

widget_control,event.top,/destroy

end


;===============================================================================
; Refresh graphics window
pro DB_Display2DC::displayEvent, event

self->refresh      ; refresh via self

end

;===============================================================================
; Switch off scene re-construction
pro DB_Display2DC::noRenew, event

self.renewScene = 0

end

;===============================================================================
; Switch on scene re-construction
pro DB_Display2DC::renew, event

self.renewScene = 1

end



;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Main (conventional) Event Handler for DB_Display2DC
;
pro DB_Display2DC_event, event

; Retrieve the command structure embedded in the uvalue of the
; widget. The command structure is stored in the uvalue of either
; event.id or evend.handler
widget_control, event.id, get_uvalue = cmd

childID = event.id
while (vartype(cmd) ne 'DB_DISPLAY2D_CMD') do begin
    parentID = widget_info(childID,/parent)
    if (parentID eq 0) then break
    widget_control, parentID, get_uvalue = cmd
    childID = parentID
endwhile


if (vartype(cmd) eq 'DB_DISPLAY2D_CMD') then $
   call_method, cmd.method, cmd.object, event

end



;===============================================================================
; Widget Cleanup procedure for DB_Display2DC that would be called by
; XMANAGER when DB_Display2DC's tlb is destroyed
;
pro DB_Display2DC_cleanup, tlb
widget_control, tlb, get_uvalue = cmd
; get the Widget Object's reference and destroy it
selfObject = cmd.object
obj_destroy, selfObject

end


;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST May, 2003
;
; NAME:_______________________
;   DB_Display2DC::INIT
;
; PURPOSE:
;   Widget dialog program which handles the display DaveGr1D object(s).
;
; INPUT PARAMETERS:___________
;   PARAMETERS:
;
;
;   KEYWORDS:
;
;
; OUTPUTS:____________________
;
;     none
;
function DB_Display2DC::init, group_leader=group_leader, objGrRef=oDaveGr, workDir=workDir

; 
device, get_decomposed = old_dec
self.old_dec = old_dec
;device, decomposed=1

; create a structure to specify methods that should act as event
; handlers for widgets defined in DB_Display2DC
cmdStr = {DB_DISPLAY2D_CMD, $
          object:self, $ ; reference to the widget object generating events
          method:'' $         ; the method that should handle the event
          ;parPtr:ptr_new() $  ; extra parameters to be passed to the method
         }



; Various tooltips
deTooltip = 'Togle on/off errorbars.'
pmTooltip = 'Switch plot mode. View data as points or histograms.'
refTooltip = 'Refresh the plot window'
defZTooltip = 'Reset intensity range to default values from data.'
defXYTooltip = 'Reset x,y-axis display limits to full range.'
exitTooltip = 'Close display and return to main DATABROWSER.'
ptTooltip = 'Set/modify plot type (as lines and/or symbols).'
lsTooltip = 'Set/modify plot linestyle.'
pthTooltip = 'Set/modify plot line thickness.'
symTooltip = 'Set/modify plot symbols.'
stTooltip = 'Set/modify symbol type.'
szTooltip = 'Set/modify symbol size.'
pcolTooltip = 'Set/modify plot color.'
acolTooltip = 'Set/modify axes color.'
atTooltip = 'Set/modify axis line thickness.'


; define the top-level-base
cmdStr.method = 'tlbEvent'
if (n_elements(group_leader) eq 0) then begin
    group_leader=0L
    modal=0
    self.dispType = 0b
    self.tlb = widget_base(/row,title=title,/align_right,/tlb_size_events,uname='TLB' $
                           ,uvalue=cmdStr,group_leader=group_leader,mbar=menubar)
endif else begin
    modal=1
    widget_control, group_leader, get_uvalue=sPtr

    ;; Get the DaveGr object to be viewed
    if (n_elements(oDaveGr) gt 0) then begin
        self.oDaveGr = oDaveGr
    endif else begin
        oSelCont = (*sPtr).sel_cont
        self.oDaveGr = oSelCont->get(position=0)
    endelse

    ;; Can't allow multiple views of the same DaveGr2D object so make
    ;; some checks now
    self.oActive2DCont = (*sPtr).o_active_gr2DC_cont
    if ((self.oActive2DCont)->count() gt 0) then begin
        ;; Exit immediately if DaveGr object is already being viewed
        if ((self.oActive2DCont)->isContained(self.oDaveGr)) then return, 0
    endif
    ;; register DaveGr object
    (self.oActive2DCont)->add, self.oDaveGr

    ; Retrieve ptr to previous GUI/widget settings
    if (not (self.oDaveGr)->get(zmin=zmin,zmax=zmax)) then return, 0
    if (not (self.oDaveGr)->get(guiAttribsPtr=guiAttribsPtr)) then return, 0
    usingPrevGuiSettings = (n_elements(*guiAttribsPtr) gt 0)? 1b : 0b
    if (usingPrevGuiSettings) then begin
        self.xrangeFlag = (*guiAttribsPtr).xrangeFlag
        self.yrangeFlag = (*guiAttribsPtr).yrangeFlag
        self.zrangeFlag = (*guiAttribsPtr).zrangeFlag
        self.xrange = (*guiAttribsPtr).xrange
        self.yrange = (*guiAttribsPtr).yrange
        case self.zrangeFlag of
            0: self.zrange = [zmin,zmax]
            1: self.zrange = (*guiAttribsPtr).zrange
        endcase
    endif else self.zrange = [zmin,zmax]


    void = (self.oDaveGr)->get(nameTag=title)
    title = 'DATABROWSER - '+title
    self.o_registry = (*sPtr).o_registry ; get the object registry
    (self.o_registry)->add, self    ; and register
    ;self.tlb = widget_base(group_leader=group_leader,/row,title=title,modal=modal $
    ;                 ,/align_right,/tlb_size_events,uname='TLB',uvalue=cmdStr)
    self.tlb = widget_base(group_leader=group_leader,/row,title=title $
                     ,/align_right,/tlb_size_events,uname='TLB',uvalue=cmdStr,mbar=menubar)
endelse

; Get working directory setting
if (n_elements(workDir) le 0) then $
  cd, current=workDir
self.workDir = workDir

; File Menu
filem = widget_button(menubar,value='File',/menu)
cmdStr.method='print'
expm = widget_button(filem,value='Export image as',/menu)
void = widget_button(expm,value='Postscript',uname='POST',uvalue=cmdStr)
void = widget_button(expm,value='Jpeg',uname='JPEG',uvalue=cmdStr,/separator)
void = widget_button(expm,value='Png',uname='PNG',uvalue=cmdStr)
void = widget_button(expm,value='Bmp',uname='BMP',uvalue=cmdStr)
void = widget_button(expm,value='Tiff',uname='TIFF',uvalue=cmdStr)
void = widget_button(filem,value='Print',uname='PRINT',uvalue=cmdStr)
cmdStr.method = 'exitEvent'
void = widget_button(filem,value='Close Window',/separator,uname='EXIT',uvalue=cmdStr)

; split TLB into:
;  1) an left pane for placing dialog buttons
;  2) a right pane for placing the draw window
cb1 = widget_base(self.tlb,/col)
cb2 = widget_base(self.tlb,/frame)

; Split the dialog base into several sections for grouping similar controls
cb11 = widget_base(cb1,/col,/align_center,/frame)
cb11a = widget_base(cb1,/col,/align_center,/frame)
cb12 = widget_base(cb1,/col,/align_center,/frame)
;cb13 = widget_base(cb1,/col,/align_center,/frame)
cb14 = widget_base(cb1,/col,/align_center)

; Now the buttons

xsize = 115

self.dispTitle = (usingPrevGuiSettings)? (*guiAttribsPtr).dispTitle : 1
cmdStr.method = 'dispTitleEvent'
void = cw_bgroup(cb11,['Display Title'],/col,/nonexclusive,frame=0,/return_index,uname='DISPTITLE' $
                 ,set_value=self.dispTitle,uvalue=cmdStr)

cmdStr.method = 'todoEvent'
void = widget_button(cb11,value='Edit X-axis',uname='EXAXIS',xsize=xsize,uvalue=cmdStr,sensitive=0)
void = widget_button(cb11,value='Edit Y-axis',uname='EYAXIS',uvalue=cmdstr,sensitive=0)
void = widget_button(cb11,value='Edit Title',uname='ETITLE',uvalue=cmdStr,sensitive=0)
void = widget_button(cb11,value='Add Text',uname='EDTEXT',uvalue=cmdStr,sensitive=0)

; Data limits
void = widget_label(cb11a,value='Intensity Range',xsize=xsize)
cmdStr.method = 'setZrange'
void = cw_field(cb11a,title='',/row,value=self.zrange[0],/float,uname='ZMIN' $
                ,/return_events,xsize=12,uvalue=cmdStr)
void = cw_field(cb11a,title='',/row,value=self.zrange[1],/float,uname='ZMAX' $
                ,/return_events,xsize=12,uvalue=cmdStr)
cmdStr.method = 'defZLimEvent'
void = widget_button(cb11a,value='Reset to Default',uname='DEFZLIM',xsize=xsize,uvalue=cmdStr,tooltip=defZTooltip)

cmdStr.method = 'displayEvent'
void = widget_button(cb12,value='Refresh',uname='DISPLAY',xsize=xsize,uvalue=cmdStr,tooltip=refTooltip)
cmdStr.method = 'defXYLimEvent'
void = widget_button(cb12,value='Reset Axes Limits',uname='DEFXYLIM',xsize=xsize,uvalue=cmdStr,tooltip=defXYTooltip)
cmdStr.method = 'exitEvent'
void = widget_button(cb12,value='Close Window',uname='EXIT',uvalue=cmdstr,tooltip=exitTooltip)

; store useful widget dimensions in self (widget object)
res = widget_info(cb1,/geometry)
self.dialog_x = res.xsize
dialog_y = res.ysize
;self.min_draw_y = fix(dialog_y*1.10) > 450
;self.min_draw_x = fix(self.min_draw_y*1.333) > 600
;self.winDim = (usingPrevGuiSettings)? (*guiAttribsPtr).winDim : [self.min_draw_x,self.min_draw_y]
self.min_draw_y = fix(dialog_y*1.05)
self.min_draw_x = fix(self.min_draw_y*1.333)
self.winDim = (usingPrevGuiSettings)? (*guiAttribsPtr).winDim : $
              [self.min_draw_x > 600,self.min_draw_y > 450]
cmdStr.method = 'drawEvent'
drawID = widget_draw(cb2,xsize=self.winDim[0],ysize=self.winDim[1],graphics_level=2,uname='DRAW' $
                     ,/button_events,/expose_events,uvalue=cmdStr)


;----------------------------------------------------------------------------
; Create Context Sensitive menus for Contour object.
cmdStr.method = 'handleConSenMenuEvent'
csb1 = widget_base(self.tlb, /context_menu,uname='CSBCONT',uvalue=cmdStr)
fill = widget_button(csb1,value='Filling',/menu,tooltip=fillTip)
shade = widget_button(csb1,value='Shading',/menu,tooltip=shadeTip)
level = widget_button(csb1,value='Levels',/menu,tooltip=levelTip)
color = widget_button(csb1,value='Color Table',/menu,tooltip=colorTip)

; Get various DaveGr object settings
void = (self.oDaveGr)->get(nlevels=nlevels,fillFlag=fillFlag, $
                           shadeFlag=shadeFlag,colorIndex=colorIndex)

; Filling
self.fillIDs[0] = widget_button(fill,value='Off',uname='FILL',uvalue=0,/checked_menu)
self.fillIDs[1] = widget_button(fill,value='On',uname='FILL',uvalue=1,/checked_menu)
case fillFlag of
    0: widget_control, self.fillIDs[0], set_button=1
    1: widget_control, self.fillIDs[1], set_button=1
endcase

; Shading
self.shadeIDs[0] = widget_button(shade,value='Flat',uname='SHADE',uvalue=0,/checked_menu)
self.shadeIDs[1] = widget_button(shade,value='Gouraud',uname='SHADE',uvalue=1,/checked_menu)
case shadeFlag of
    0: widget_control, self.shadeIDs[0], set_button=1
    1: widget_control, self.shadeIDs[1], set_button=1
endcase

; Nos of contour levels
levelVals=[5,10,15,20,25,30,35,40,45,50,55,60]
count = n_elements(levelVals)
levelIDs = lindgen(count)
for i=0,count-1 do begin
    val = levelVals[i]
    valStr = strtrim(string(val),2)
    uname = 'LEVEL'+valStr
    levelIDs[i] = widget_button(level,value=valStr,uname='NLEVEL',uvalue=val,/checked_menu)
    if (val eq nlevels) then widget_control, levelIDs[i], set_button=1
endfor
self.levelIDsPtr = ptr_new(levelIDs)

; Color Tables
ctable=['B-W LINEAR','BLUE/WHITE','GRN-RED-BLU-WHTt','RED TEMPERATURE','BLUE/GREEN/RED/YELLOW' $
        ,'STD GAMMA-II','PRISM','RED-PURPLE','GREEN/WHITE LINEAR','GRN/WHT EXPONENTIAL','GREEN-PINK' $
        ,'BLUE-RED','16 LEVEL','RAINBOW','STEPS','STERN SPECIAL','Haze','Blue-Pastel-Red','Pastels' $
        ,'Hue Sat Lightness 1','Hue Sat Lightness 2','Hue Sat Value 1','Hue Sat Value 2' $
        ,'Purple-Red+Stripes','Beach','Mac Style','Eos A','Eos B','Hardcandy','Nature','Ocean' $
        ,'Peppermint','plasma','Blue-Red','Rainbow','Blue Waves','Volcano','Waves','Rainbow18' $
        ,'Rainbow+white','Rainbow+black']
count=n_elements(ctable)
ctableIDs = lindgen(count)
for i=0,count-1 do begin
    ctableIDs[i] = widget_button(color,value=ctable[i],uname='CTABLE',uvalue=i,/checked_menu)
    if (i eq colorIndex) then widget_control, ctableIDs[i], set_button=1
endfor
self.ctableIDsPtr = ptr_new(ctableIDs)

;----------------------------------------------------------------------------
; Create Context Sensitive menus for 'Axes'
csb2 = widget_base(self.tlb, /context_menu,uname='CSBAXES',uvalue=cmdStr)
thick = widget_button(csb2,value='Line Thickness',/menu)
color = widget_button(csb2,value='Axes Color',/menu)
afont = widget_button(csb2,value='Axes Font Size',/menu)
void = widget_button(csb2,value='Properties...',uname='EDITAXIS')

void = widget_button(thick,value='1.00',uname='THICK',uvalue=1.00)
void = widget_button(thick,value='1.25',uname='THICK',uvalue=1.25)
void = widget_button(thick,value='1.50',uname='THICK',uvalue=1.50)
void = widget_button(thick,value='1.75',uname='THICK',uvalue=1.75)
void = widget_button(thick,value='2.00',uname='THICK',uvalue=2.00)
void = widget_button(thick,value='2.25',uname='THICK',uvalue=2.25)
void = widget_button(thick,value='2.50',uname='THICK',uvalue=2.50)
void = widget_button(thick,value='2.75',uname='THICK',uvalue=2.75)
void = widget_button(thick,value='3.00',uname='THICK',uvalue=3.00)
void = widget_button(thick,value='3.25',uname='THICK',uvalue=3.25)
void = widget_button(thick,value='3.50',uname='THICK',uvalue=3.50)
void = widget_button(thick,value='3.75',uname='THICK',uvalue=3.75)
void = widget_button(thick,value='4.00',uname='THICK',uvalue=4.00)
void = widget_button(thick,value='4.25',uname='THICK',uvalue=4.25)
void = widget_button(thick,value='4.50',uname='THICK',uvalue=4.50)
void = widget_button(thick,value='4.75',uname='THICK',uvalue=4.75)
void = widget_button(thick,value='5.00',uname='THICK',uvalue=5.00)

void = widget_button(color,value='Aqua',uname='COLOR',uvalue=[0,255,255])
void = widget_button(color,value='Black',uname='COLOR',uvalue=[0,0,0])
void = widget_button(color,value='Blue',uname='COLOR',uvalue=[0,0,255])
void = widget_button(color,value='Fuchsia',uname='COLOR',uvalue=[255,0,255])
void = widget_button(color,value='Gray',uname='COLOR',uvalue=[128,128,128])
void = widget_button(color,value='Green',uname='COLOR',uvalue=[0,128,0])
void = widget_button(color,value='Lime',uname='COLOR',uvalue=[0,255,0])
void = widget_button(color,value='Maroon',uname='COLOR',uvalue=[128,0,0])
void = widget_button(color,value='Navy',uname='COLOR',uvalue=[0,0,128])
void = widget_button(color,value='Olive',uname='COLOR',uvalue=[128,128,0])
void = widget_button(color,value='Purple',uname='COLOR',uvalue=[128,0,128])
void = widget_button(color,value='Red',uname='COLOR',uvalue=[255,0,0])
void = widget_button(color,value='Silver',uname='COLOR',uvalue=[192,192,192])
void = widget_button(color,value='Teal',uname='COLOR',uvalue=[0,128,128])
void = widget_button(color,value='White',uname='COLOR',uvalue=[255,255,255])
void = widget_button(color,value='Yellow',uname='COLOR',uvalue=[255,255,0])

void = widget_button(afont,value='8',uname='AFONT',uvalue=8)
void = widget_button(afont,value='10',uname='AFONT',uvalue=10)
void = widget_button(afont,value='12',uname='AFONT',uvalue=12)
void = widget_button(afont,value='14',uname='AFONT',uvalue=14)
void = widget_button(afont,value='16',uname='AFONT',uvalue=16)
void = widget_button(afont,value='18',uname='AFONT',uvalue=18)
void = widget_button(afont,value='20',uname='AFONT',uvalue=20)
void = widget_button(afont,value='22',uname='AFONT',uvalue=22)
void = widget_button(afont,value='24',uname='AFONT',uvalue=24)
void = widget_button(afont,value='26',uname='AFONT',uvalue=26)
void = widget_button(afont,value='28',uname='AFONT',uvalue=28)

;----------------------------------------------------------------------------
; Create Context Sensitive menus for Main Title
csb2 = widget_base(self.tlb, /context_menu,uname='CSBLABEL',uvalue=cmdStr)
color = widget_button(csb2,value='Text Color',/menu)
afont = widget_button(csb2,value='Font Size',/menu)
void = widget_button(csb2,value='Properties...',uname='EDITLABLE')

void = widget_button(color,value='Aqua',uname='COLOR',uvalue=[0,255,255])
void = widget_button(color,value='Black',uname='COLOR',uvalue=[0,0,0])
void = widget_button(color,value='Blue',uname='COLOR',uvalue=[0,0,255])
void = widget_button(color,value='Fuchsia',uname='COLOR',uvalue=[255,0,255])
void = widget_button(color,value='Gray',uname='COLOR',uvalue=[128,128,128])
void = widget_button(color,value='Green',uname='COLOR',uvalue=[0,128,0])
void = widget_button(color,value='Lime',uname='COLOR',uvalue=[0,255,0])
void = widget_button(color,value='Maroon',uname='COLOR',uvalue=[128,0,0])
void = widget_button(color,value='Navy',uname='COLOR',uvalue=[0,0,128])
void = widget_button(color,value='Olive',uname='COLOR',uvalue=[128,128,0])
void = widget_button(color,value='Purple',uname='COLOR',uvalue=[128,0,128])
void = widget_button(color,value='Red',uname='COLOR',uvalue=[255,0,0])
void = widget_button(color,value='Silver',uname='COLOR',uvalue=[192,192,192])
void = widget_button(color,value='Teal',uname='COLOR',uvalue=[0,128,128])
void = widget_button(color,value='White',uname='COLOR',uvalue=[255,255,255])
void = widget_button(color,value='Yellow',uname='COLOR',uvalue=[255,255,0])

void = widget_button(afont,value='8',uname='AFONT',uvalue=8)
void = widget_button(afont,value='10',uname='AFONT',uvalue=10)
void = widget_button(afont,value='12',uname='AFONT',uvalue=12)
void = widget_button(afont,value='14',uname='AFONT',uvalue=14)
void = widget_button(afont,value='16',uname='AFONT',uvalue=16)
void = widget_button(afont,value='18',uname='AFONT',uvalue=18)
void = widget_button(afont,value='20',uname='AFONT',uvalue=20)
void = widget_button(afont,value='22',uname='AFONT',uvalue=22)
void = widget_button(afont,value='24',uname='AFONT',uvalue=24)
void = widget_button(afont,value='26',uname='AFONT',uvalue=26)
void = widget_button(afont,value='28',uname='AFONT',uvalue=28)

;----------------------------------------------------------------------------
; realize widgets
widget_control, self.tlb,/realize

; add the destination object to self (widget object)
widget_control, drawID, get_value=oDest
self.oDest = oDest
oDest->getProperty, dimensions=dim
self.winDim = dim

; create and add a scene to self (widget object)
self.oScene = obj_new('IDLgrScene')

; assign self to uvalue of tlb
;widget_control, self.tlb, set_uvalue=self

; the graphics scene should be always reconstructed by default
self->renew
registerName = 'db_display2dc'
xmanager,registerName,self.tlb,/no_block,event_handler='db_display2dc_event',cleanup='db_display2dc_cleanup'

return,1
end


;===============================================================================
;
pro DB_Display2DC__define

void = {DB_Display2DC, $
        tlb:0L, $
        old_dec:0, $
        o_registry: obj_new(), $ ; object registry which handles communication with other programs
        oDest:obj_new(), $      ; the Destination object
        oScene:obj_new(), $     ; Scene to be drawn in the destination
        oDaveGr:obj_new(), $    ; the DaveGr object to be displayed
        oRband:obj_new(), $      ; a rubber band obj gr atom used for zooming
        oActive2DCont:obj_new(), $ ; contains a list of all DaveGr2D objects being actively viewed
        workDir:'', $           ; user's working directory
        csmObjRef:obj_new(), $  ; will contain the obj ref for object which triggers a con. sen menu
        renewScene:1b, $        ; flag to indicate whether the scene is to be completely reconstructed
        rbStart:[0,0], $        ; rubberband start point (for zooming)
        rbScale:[0.,0.,1.,1.], $ ; x,y scale factor to convert from device to data coordinates (for zooming)
        mouseMoved:0b, $        ; flag to indicate if mouse was moved while LMB is depressed
        xrangeFlag:0b, $        ; flag to indicate that user specified x-axis range should be used
        xrange:[0.,0.], $       ; user specified x-axis limits
        yrangeFlag:0b, $        ; flag to indicate that user specified y-axis range should be used
        yrange:[0.,0.], $       ; user specified y-axis limits
        zrangeFlag:0b, $        ; flag to indicate that user specified z-axis range should be used
        zrange:[0.,0.], $       ; user specified z-axis limits
        levelIDsPtr:ptr_new(), $ ; widget ID's for the different level items in con. sen. base menu
        ctableIDsPtr:ptr_new(), $ ; widget ID's for the different color table items in con. sen. base menu
        fillIDs:[0L,0L], $      ; widget ID's for type of filling (contour,surface)
        shadeIDs:[0L,0L], $     ; widget ID's for type of shading (contour,surface)
        min_draw_x:0, $         ; min dimensions for draw widget
        min_draw_y:0, $         ; min dimensions for draw widget
        dialog_x:0, $            ; min width for dialog portion of tlb
        winDim:[0,0], $         ; current draw window dimensions
        dispTitle:1b $          ; flag to indicate if main title is to be displayed
       }

end
