;+
; NAME:
;       DB_SCALEFACTOR
;
; PURPOSE:
;
;		Modal widget dialog that allows user to change the scaling factor
;		for a DAVE data set.
;
; AUTHOR:
;
;       Robert M. Dimeo, Ph.D.
;		NIST Center for Neutron Research
;       100 Bureau Drive
;		Gaithersburg, MD 20899
;       Phone: (301) 975-8135
;       E-mail: robert.dimeo@nist.gov
;       http://www.ncnr.nist.gov/staff/dimeo
;
; CATEGORY:
;
;       DAVE DATABROWSER, widget modal dialog
;
; CALLING SEQUENCE:
;
;       RESULT = DB_SCALEFACTOR(GROUP_LEADER = group_leader, OCONTAINER = ocontainer)
;
; INPUT PARAMETERS:
;
;		OCONTAINER:  A container object containing objects of
;		class DAVEGR2D or DAVEGR1D
;
; OUTPUT
; 		The call above will return a structure with the following fields:
;
;			CANCEL:		0 if false, 1 if true
;			SCALE:		array of scalefactors (1 for each file)
;
; REQUIREMENTS:
;
;		IDL 5.6 or higher
;
;
; COMMON BLOCKS:
;
;		None
;
; DISCLAIMER
;
;		This software is provided as is without any warranty whatsoever.
;		Permission to use, copy, modify, and distribute modified or
;		unmodified copies is granted, provided this disclaimer
;		is included unchanged.
;
; MODIFICATION HISTORY:
;
;       Written by Rob Dimeo, December 17, 2002.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro db_sf_update,event
widget_control,event.top,get_uvalue = pState
id_old = widget_info(event.top,find_by_uname = 'CURRENT')
widget_control,id_old,set_value= $
	strtrim(string((*pState).oldSF[(*pState).fileIndex]),2)
id_new = widget_info(event.top,find_by_uname = 'NEW')
widget_control,id_new,set_value=(*pState).newSF[(*pState).fileIndex]
               ;;strtrim(string((*pState).newSF[(*pState).fileIndex]),2)
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro db_sf_init,event
widget_control,event.top,get_uvalue = pState
oContainer = (*pState).oContainer
oall = oContainer->get(/all)
nFiles = oContainer->count()
for i = 0,nFiles-1 do begin
    ;;void = oall[i]->get(dataref = o)
    o = oall[i]
    r = o->get(SCALEFACTOR = sf)
    (*pState).oldSF[i] = sf
    (*pState).newSF[i] = sf
endfor
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro db_sf_changeFiles,event
widget_control,event.top,get_uvalue = pState
id_new = widget_info(event.top,find_by_uname = 'NEW')
widget_control,id_new,get_value = thisVal
thisVal = float(thisVal[0])
(*pState).newSF[(*pState).oldIndex] = thisVal
(*pState).fileIndex = event.index[0]
(*pState).oldIndex = (*pState).fileIndex
db_sf_update,event
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro db_sf_events,event
widget_control,event.top,get_uvalue = pState
uname = widget_info(event.id,/uname)

case uname of
'NEW': begin
    widget_control,event.id,get_value = myValue
    ;myValue = float(myValue[0])
    (*pState).newSF[(*pState).fileIndex] = myValue
end

'DATASELECTION': db_sf_changeFiles,event

'ACCEPT': widget_control,event.top,/destroy

'CANCEL': begin
    (*pState).cancel = 1
    widget_control,event.top,/destroy
end

else:

endcase

end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function db_scaleFactor, group_leader=group_leader, oContainer
; Pull out all of the "available" files
oall = oContainer->get(/all)
n_objects = oContainer->count()
validClasses = ['DAVEDAT1D','DAVEDAT2D']
for i = 0,n_objects-1 do begin
    o = oall[i]
    class = obj_class(o)
    res = where(validClasses eq class,okay)
    if (not okay) then return,{cancel:1}
    r = o->get(nameTag = nameTag)
    if i eq 0 then begin
        nameTags = nameTag
    endif else begin
        nameTags = [nameTags,nameTag]
    endelse
endfor

; Widget definition module
title='DATADROWSER: Scalefactor'
if n_elements(group_leader) eq 0 then begin
	group_leader = 0L
	modal = 0
        tlb = widget_base(/col, title=title)
endif else begin
	modal = 1
        tlb = widget_base(group_leader=group_leader,/col,title=title,modal=modal)
endelse

;selections = ['select 2D Data',filenames]
cb = widget_base(tlb,/col,/align_center,/frame)
drop = widget_droplist(cb,value=nameTags,uname='DATASELECTION',/align_center)
current = cw_field(cb,title='Current scalefactor',/row,value='',/string,uname='CURRENT', $
                   /noedit,xsize=25)
new     = cw_field(cb,title='New scalefactor    ',/row,value='',/float,uname='NEW', $
                   /return_events,xsize=25)
rowBase = widget_base(tlb,/row,/align_center)
void = widget_button(rowBase,value = 'Accept',uname = 'ACCEPT')
void = widget_button(rowBase,value = 'Cancel',uname = 'CANCEL')

widget_control,tlb,/realize
nfiles = n_elements(nameTags)
oldSf = replicate(1.0,nfiles)
newSf = oldSf

state = {cancel:0, $
         filenames:nameTags, $
         oldSf:oldSf, $
         newSf:newSf, $
         fileIndex:0, $
         oldIndex:0, $
         oContainer:oContainer $
        }

pState = ptr_new(state,/no_copy)
widget_control,tlb,set_uvalue = pState

ev = {peEvent,id:void,top:tlb,handler:0L}
db_sf_init,ev
db_sf_update,ev

registerName = 'db_sf'
xmanager,registerName,tlb,no_block=modal,event_handler='db_sf_events'

db_sf_structure = {cancel:(*pState).cancel,scaleFactor:(*pState).newSf}

ptr_free,pState
return,db_sf_structure
end
