; $Id: ref_isotropic_rotational_diffusion.pro,v 1.1 2003/05/20 16:57:06 dimeo Exp $
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function ref_isotropic_rotational_diffusion,xvals,parms,		$
                    	parmnames = parmnames,					$
                    	init_guess = init_guess,				$
                    	qty = qty,								$
                    	step = step,							$
                    	single_parmnames = single_parmnames,	$
                    	multi_parmnames = multi_parmnames,		$
                    	res_ptr = res_ptr,						$
                    	all_y = all_y,							$
                    	xunits = xunits,						$
                    	_Extra = extra
; all_y keyword is relevant for this model
if n_elements(all_y) eq 0 then all_y = 1
nx = extra.nx & ny = extra.ny
x = xvals[0:nx-1] & y = xvals[nx:nx+ny-1]
ux = 1+bytarr(nx) & uy = 1+bytarr(ny)

parmnames =	['Radius','Diffusion constant', $
			'Intensity:'+strtrim(string(indgen(ny)),2), $
			'Center:'+strtrim(string(indgen(ny)),2)]

single_parmnames = parmnames[0:1]
multi_parmnames = ['Intensity','Center']
if arg_present(step) then begin
	step = dblarr(2+2*extra.ny)
endif
if (n_params() eq 0) or (n_params() eq 1) then begin
	if arg_present(init_guess) then begin
		intensity = max(qty,dimension = 1,index)
		qty_size = size(qty)
		for i = 0,qty_size[2]-1 do begin
			qty_max = max(qty[*,i],index)
			if i eq 0 then index_array = index else $
				index_array = [index_array,index]
		endfor
		center = x[index_array]
		; Guess the 1/2 width at the highest y-value
		; Get the low value first
		amp = max(qty[*,qty_size[2]-2])
		ht = 0.5*amp
		wh_lo = where(abs(amp-ht) le min(abs(amp-ht)) and (x lt 0.0),count_lo)
		x_lo_lim = x[wh_lo[0]]
 		wh_hi = where(abs(amp-ht) le min(abs(amp-ht)) and (x gt 0.0),count_hi)
		x_hi_lim = x[wh_hi[0]]
		fwhm = x_hi_lim - x_lo_lim
		d = 0.25*fwhm/0.6528
		radius = 1.0
		init_guess = [radius,d,intensity,center]
	endif
  return,-1
endif

radius = parms[0]
D = parms[1]

intensity = ux#(parms[2:2+ny-1])
center = ux#(parms[2+ny:2+2*ny-1])

ao = ux#((sph_bessel(y*radius,0))^2)
hbar = 0.6528	; ueV.ns or meV.ps
nterms = 40
sum = dblarr(nx,ny)
for i = 1,nterms-1 do begin
	ai = ux#((2.0*i+1.0)*(sph_bessel(y*radius,i))^2)
	taul = 1.0/(i*(i+1.0)*D)
	lor = (ux#hbar#uy/taul)/((ux#hbar#uy/taul)^2+(x#uy-center)^2)
	sum = sum + ai*lor
endfor

xhi = max(x,min = xlo)
dx = x[1]-x[0] & dy = y[1]-y[0]
nshift = fix(0.5*((abs(xhi)-abs(xlo)))/dx)
resolution = *res_ptr
lineshape = ao*ref_delta(xvals,center,resolution,nx = nx,ny = ny)+$
		(1.0/!dpi)*scat_convolve(x,y,sum,resolution)
yout = intensity*lineshape

return,yout
end