;$Id: trifenestra_cwindowbase.pro,v 1.1 2002/07/05 17:59:50 amunter Exp $
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
; pro oimageDraw, event
;	widget_control,event.top,get_uvalue=self
;	self -> updateImage
;	return
;end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
pro trifenestra::updateImageEvent, event

	self -> updateImage

end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
pro trifenestra::updateImage

	; Spit out an error message if there is no data.
	if not self.oDave -> valid() then begin
		errarray = ['Error caught: No data available']
		void = dialog_message(errarray,/error)
		return
	endif

	; Restore the graphics setting.
	idl_state_restore, !trifenestra.mainWindowState


	; Get information from the widgets, e.g., titles, zmax, zmin, etc.

	zminid = 'zmin'
	zmaxid = 'zmax'


	self -> widget_control,zminid,get_value=zmin
	self -> widget_control,zmaxid,get_value=zmax


	; Strings Q and hw are converted into more beautiful forms.
	;titleid =  'title'
	;xtitleid = 'xtitle'
	;ytitleid = 'ytitle'
	self.oDave -> get_label $
		, histlabel = title, xlabel = xtitle, ylabel = ytitle
	;self -> widget_control,titleid,get_value=title
	;self -> widget_control,xtitleid,get_value=xtitle
	;self -> widget_control,ytitleid,get_value=ytitle
 	xtitle = trifenestra_beautify_string( xtitle )
 	ytitle = trifenestra_beautify_string( ytitle )
 	title = trifenestra_beautify_string( title )

	; Find the data range.
	self.oDave -> get_range, frange = zrange
	!z.range = [ zmin, zmax ]

	; The following condition should never met.
	if !z.range[0] ge !z.range[1] then begin
		errarray = ['Error caught: zmin > zmax']
		void = dialog_message(errarray,/error)
		return
	endif



	; Set values in the main image base.
	wXmin =  'image_xmin'
	self -> widget_control, wXmin, set_value = !x.range[0]
	wXmax = 'image_xmax'
	self -> widget_control, wXmax, set_value = !x.range[1]
	wYmin =  'image_ymin'
	self -> widget_control, wYmin, set_value = !y.range[0]
	wYmax ='image_ymax'
	self -> widget_control, wYmax, set_value = !y.range[1]



	; Draw on the PIXMAP window.
	wset,self.pixmapid
	; wset, self.cwinid

	message, 'Image Updated.', /continue

	self.oDave -> view $
		,pos=[.23,.2,.88,.9] $
		,xrange=!x.range, yrange=!y.range, zrange=!z.range $
		, title = title $
		, xtitle = xtitle $
		, ytitle = ytitle


	; Copy the PIXMAP into the main window.
	wset, self.cwinid
	device, copy=[0,0,self.xsize,self.ysize,0,0,self.pixMapid]
	; 0, 0 : Lower Left Coordinate of the FROM window
	; self.xsize, self.ysize : Size in columns & rows
	; 0, 0 : Lower Left Coordinate of the TO window
	; self.pixMapid : Window ID of the FROM window.

	; Save the graphics setting.
	!trifenestra.mainWindowState  = idl_state_save()

	return

	; We need to draw on the other Windows, too.

end



;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; Main Panel Event Handler
;
pro opDraw,event ; event-pro can't be a method

widget_control,event.top,get_uvalue=self
self -> opDrawEvent,event
return
end
;*******************************************************************************
pro trifenestra::opDrawEvent, event

	; If there is no data, do nothing.
	if not self.oDave -> valid() then return


	; If the left mouse button is pressed, give control to 'opZoom'.
	if event.press eq 1 then begin

		widget_control,event.id, event_pro='opZoom'
		self.xstart = event.x
		self.ystart = event.y

	endif

	self.oDave -> get, xrange = xrange, yrange = yrange, frange = frange $
		, xdata = q, ydata = hw, qty = intensity, err = error

	; If the right mouse button is pressed, un-Zoom.
	if event.press eq 4 then begin

		!trifenestra.mainWindowState.bang_x.range = xrange
		!trifenestra.mainWindowState.bang_y.range = yrange

		!trifenestra.bottomPanelState.bang_x.range = xrange
		!trifenestra.leftPanelState.bang_y.range = yrange

		self -> updateImage

	endif

	; Draw cross lines during the mouse motion.
	if event.type eq 2 then begin

		; Restore the system variables.
		idl_state = idl_state_save()

		idl_state_restore, !trifenestra.mainWindowState

		; Set the graphics window.
		wset,self.cwinid

		; Copy a clean picture from the PixMap window.
		device,copy=[0,0,self.xsize,self.ysize,0,0,self.pixMapid]

		; Draw the vertical and horizontal lines.
		plots,[event.x,event.x],[0,self.ysize],/device
		plots,[0,self.xsize],[event.y,event.y],/device

		cursorxy = convert_coord(event.x,event.y,/device,/to_data)
			; from device to data coordinate system

		; When the cursor moves to the edge of the window,
		; restore the system variables, and return.
		if (event.x lt 20 or event.x gt self.xsize-20)	$
			or (event.y lt 20 or event.y gt self.ysize-20) then begin
			idl_state_restore, idl_state
			return
		endif

		qsize = size(q)
		hwsize = size(hw)

		; If the cursor is within the shown data range,
 		if (cursorxy[0] ge !x.range[0] and cursorxy[0] le !x.range[1]) $
   			and (cursorxy[1] ge !y.range[0] and cursorxy[1] le !y.range[1]) $
   			then begin

	  		if hwsize[0] eq 1 then begin
	    		esoy = sort(abs(cursorxy[1]-(hw)))
	    		ypos =  (hw)[esoy[0]]
	    		if qsize[0] eq 1 then begin
	      			esox = sort(abs(cursorxy[0]-(q)))
	      			xpos =  (q)[esox[0]]
	      			intyproj = (intensity)[esox[0],*]
	      			dintyproj = (error)[esox[0],*]
	      			xproj = (q)
	    			yproj = (hw)


	    		endif else begin
	    		  	esox = sort(abs(cursorxy[0]-(q)[*,esoy[0]]))
	    		  	xpos =  (q)[esox[0],esoy[0]]
	    		  	intyproj = (intensity)[esox[0],*]
	    		  	dintyproj = (error)[esox[0],*]
	    		  	xproj = (q)[* , esoy[0]]
	    			yproj = (hw)


	    		endelse
	    		intxproj = (intensity)[*,esoy[0]]
	    		dintxproj = (error)[*,esoy[0]]



	  		endif else begin

	    		esoy = sort(abs(cursorxy[1]-(hw)[0,*]))
	    		ypos =  (hw)[0,esoy[0]]
   	    		esox = sort(abs(cursorxy[0]-(q)[*,esoy[0]]))
   	    		xpos =  (q)[esox[0],esoy[0]]
	    		esoy = sort(abs(cursorxy[1]-(hw)[esox[0],*]))
	    		ypos =  (hw)[esox[0],esoy[0]]
	    		xproj = (q)[*,esoy[0]]
	    		yproj = (hw)[esox[0],*]
	    		intxproj = (intensity)[*,esoy[0]]
	    		intyproj = (intensity)[esox[0],*]
	    		dintxproj = (error)[*,esoy[0]]
	    		dintyproj = (error)[esox[0],*]


	  		endelse

			; Set xmin, xmax, ymin, and ymax values of
			; the corresponding widgets.
			self -> widget_control, 'xmin',set_value=cursorxy[0]
			self -> widget_control, 'xmax',set_value=cursorxy[0]
			self -> widget_control, 'ymin',set_value=cursorxy[1]
			self -> widget_control, 'ymax',set_value=cursorxy[1]

			intpos = (intensity)[esox[0],esoy[0]]
			cursor = string(xpos,form='(g8.3)')+','+string(ypos,form='(g8.3)') $
				+', '+string(intpos,form='(g11.3)')
			cursor = strcompress(cursor)
			outputxy = convert_coord(10,10,/device,/to_data)
			xyouts,outputxy[0],outputxy[1],cursor,chars=1

			lisx = where(intxproj gt -1e30)
			*self.x_bp = xproj[lisx]
			*self.y_bp = intxproj[lisx]
			*self.dy_bp = dintxproj[lisx]

			if not self.togirange_bp then begin
				!trifenestra.bottomPanelState.bang_y.range = $
						[min(*self.y_bp),max(*self.y_bp)]
			endif
			self -> updateivsx

			lisy = where(intyproj gt -1e30)
			*self.x_lp = yproj[lisy]
			*self.y_lp = intyproj[lisy]
			*self.dy_lp = dintyproj[lisy]
			if not self.togirange_lp then begin
				!trifenestra.leftPanelState.bang_x.range = $
						[min(*self.y_lp),max(*self.y_lp)]


			endif
				self -> updateivsy



  		endif else begin

			; If the cursor is out of the range,
			; print 'no data'.
			cursor = string(cursorxy[0],form='(g8.3)')	$
				+','+string(cursorxy[1],form='(g8.3)') $
				+', no data'
			cursor = strcompress(cursor)
			outputxy = convert_coord(10,10,/device,/to_data)
			xyouts,outputxy[0],outputxy[1],cursor,chars=1

  		endelse

		; Restore the system variables.
	   idl_state_restore, idl_state

	endif
	return
end

;*******************************************************************************
; Zooming Event Handler for the Main Panel.
pro opzoom, event

	widget_control, event.top, get_uvalue=self
	self -> opzoomevent, event
	return

end
;*******************************************************************************
pro trifenestra::opzoomEvent,event


	idl_state_restore, !trifenestra.mainWindowState
	;!p = *self.bangp
	;!x = *self.bangx
	;!y = *self.bangy

	wset,self.cwinid

device,copy=[0,0,self.xsize,self.ysize,0,0,self.pixMapid] ; clean picture
plots,[self.xstart,event.x,event.x,self.xstart,self.xstart]	$
	,[self.ystart,self.ystart,event.y,event.y,self.ystart],/device
if event.release eq 1 then begin
	widget_control,event.id,event_pro='opDraw'

	if abs(self.xstart[0]-event.x[0]) gt 10 	$
		or abs(self.ystart[0]-event.y[0]) gt 10 then begin

		; If the mouse if moved, a zoom box is drawn.
	zoombox = convert_coord( [self.xstart,event.x],[self.ystart,event.y]	$
				,/device,/to_data)
	!x.range = [min(zoombox[0,*]),max(zoombox[0,*])]
	!y.range = [min(zoombox[1,*]),max(zoombox[1,*])]

	!trifenestra.mainWindowState = idl_state_save()

	!trifenestra.bottomPanelState.bang_x.range = !x.range
	!trifenestra.leftPanelState.bang_y.range = !y.range

	self -> updateImage
	endif else begin

		; One click freezes the selected point.
		widget_control, event.id, event_pro='opNoDraw'

		idl_state_restore, !trifenestra.mainWindowState

		wset,self.cwinid
		device,copy=[0,0,self.xsize,self.ysize,0,0,self.pixMapid] ; clean picture
		plots,[event.x,event.x],[0,self.ysize],/device ; vertical
		plots,[0,self.xsize],[event.y,event.y],/device

	endelse


endif
;

return
end
;*******************************************************************************
pro opNoDraw, event

	widget_control, event.top, get_uvalue=self
	self->opNoDraw, event
	return

end
;*******************************************************************************
pro trifenestra::opNoDraw, event

	if event.press eq 1 then begin
		; left button of the mouse

		widget_control,event.id, event_pro='opZoom'
		self.xstart = event.x
		self.ystart = event.y

	endif else if event.press eq 4 then begin
		; right button of the mouse

		widget_control, event.id, event_pro='opDraw'

;		self -> updateImage

endif

end


function trifenestra_cWindowBase, base, cxsize, cysize

	return, widget_draw( base $
		, xsize = cxsize $
		, ysize=cysize $
		, /motion_event $
		, event_pro='opDraw' $
		, /button_events)


end
