; $Id: AAA_readme.pro,v 1.2 2003/09/12 15:26:20 tanya Exp $
;+
; NAME:
;        AAA_Readme
;
; PURPOSE:
;
;	Documentation of general COMMON information and methods about
;	classes and routines used with the specObj class.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Documentation and utility "HELP", "PRINT" and "SHOW" methods
;       and the utility procedure "CHECK_HEAP".
;
; OBJECT CLASSES:
;   specobj__define.pro
;	 <A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;   focusObj__define.pro
;	 <A HREF="#FOCUSOBJ__DEFINE_arguments">FOCUSOBJ__DEFINE_arguments</A>
;   DAVE_tagContainer__define.pro
;	 <A HREF="#DAVE_TAGCONTAINER__DEFINE_arguments">DAVE_TAGCONTAINER__DEFINE_arguments</A>
;   qtyobj__define.pro
;	 <A HREF="#QTYOBJ__DEFINE_arguments">QTYOBJ__DEFINE_arguments</A>
;   tagobj__define.pro
;	 <A HREF="#TAGOBJ__DEFINE_arguments">TAGOBJ__DEFINE_arguments</A>
;
; HOW TO DOCUMENT A DIRECTORY:
;	Required mark-up of the source code.
;	To create a HTML file containing information marked in your 
;	files starting and ending with ";+", "";-" and ";*", ";/" use:
;	IDL> MK_HTML_HELPARG, 'yourPath', 'yourDoc.html', /verbose
;
; HOW TO DOCUMENT A FILE:
;	Required mark-up of the source code.
;	To create a HTML file containing information marked in your selected
;	file starting and ending with ";+", "";-" and ";*", ";/" use:
;	IDL> MK_HTML_HELPARG, 'yourFile', 'yourDoc.html', /verbose
;
; GENERIC "HELP" METHODS:
;  "Generic" methods require a wrapper for each class (like 
;  <LI><A HREF="#QTYOBJ::HELP_OBJ">QTYOBJ::HELP_OBJ</A> which defines 
;  the keywords and parameters). The only code in the wrapper is a
;  call to the generic code fragment using "@".
;
;  <LI><A HREF="#GENERIC__HELP_OBJ_FRAG.PRO">GENERIC__HELP_OBJ_FRAG.PRO</A>
;	Prints to screen or file unit number the TYPE of the object's contents
;	and lists all methods avaiable (with keywords in UPPERCASE 
;	and parameters in lowercase). Preceding the methods listed is a number,
;	with zero (0) indicating that the methods reside in the top most class
;	definition. The higher the number the "deeper" the inherentance 
;	depth is.
;
;   <LI><A HREF="#GENERIC__PRINT_FRAG.PRO">GENERIC__PRINT_FRAG.PRO</A>
;	Prints to the screen or file unit number the contents of the object
;	as either heap variable ID's or the actual contents if the /alsoPtrs
;	keyword is used. One field in the class structure may be requested.
;	The contents of pointers can be shown using the keyword /alsoPtrs.
;
;	Note: In IDL, classes using inheritance basically construct
; one huge structure including all the data. The inherited methods
; have access to ALL of the data in the structure, not just the parts that
; are relevent to the inherited class. Therefore it is best to use 
; generic code fragements for methods ::print_obj and ::help_obj
; ONLY FOR THE TOP MOST CLASS which has inherited classes.
; No inherited classes are currently used in the specObj class.
;
; EXAMPLES: GENERIC "HELP" METHODS:
;
; IDL> myObject = obj_new('qtyObj', name='qName', qty=[1,2,3], units='counts')
; IDL> myObject -> help_obj
; Class QTYOBJ has 10 tags of type: 
; self.NAME STRING
; self.UNITS STRING
; self.TNAME STRING
; self.STYLEERR STRING
; self.RANK INT
; self.DIM INT[4] 
; self.PTR_LEGEND POINTER UNDEF
; self.PTR_QTY POINTER INT[3] 
; self.PTR_ERR POINTER UNDEF
; self.PTR_NERR POINTER UNDEF
; QTYOBJ using the methods: (lowercase:arguments UPPERCASE:keywords)
; 0   Method Procedures:
; 0   QTYOBJ::CLEANUP          self 
; 0   QTYOBJ::HELPCLASS        self CLASS HELPFUNCSTR HELPPROSTR LEVEL LUN 
; 0   QTYOBJ::HELP_OBJ         self CLASS HELPFUNCSTR HELPPROSTR LEVEL LUN METHODSONLY 
; 0   QTYOBJ::PRINT_OBJ        self thisfield ALSOPTRS LUN 
; 0   QTYOBJ::SHOW             self HEAPNUM LEVEL LUN NOTRAILINGBLANK PREFIX RECURSIVE 
; 0                            STRARRAY 
;  
; 0   Method Functions:
; 0   QTYOBJ::APPEND           self ERR LEGEND NERR OBJ PTR_ERR PTR_LEGEND PTR_NERR PTR_QTY 
; 0                            QTY 
; 0   QTYOBJ::BIGGESTDIM       self 
; 0   QTYOBJ::CLONE            self 
; 0   QTYOBJ::COUNT            self 
; 0   QTYOBJ::GET              self ALL ARGPRESENT AXISTITLE DIM ERR EVERYTHING LEGEND NAME 
; 0                            NERR PTR_ERR PTR_LEGEND PTR_NERR PTR_QTY QTY RANK SIZESTRING 
; 0                            SOME STYLEERR TNAME UNITS 
; 0   QTYOBJ::GETDIMENSION     self ARGPRESENT DIM RANK TNAME 
; 0   QTYOBJ::GETONEVALUE      self ARGPRESENT DESCRIPTITLE FORLEGEND FORPOS ONEERR 
; 0                            ONELEGEND ONENERR ONEQTY 
; 0   QTYOBJ::GETSTRING        self ALL ARGPRESENT AXISTITLE EVERYTHING NAME SIZESTRING 
; 0                            SOME STYLEERR UNITS 
; 0   QTYOBJ::GETVALUE         self ARGPRESENT ERR LEGEND NERR PTR_ERR PTR_LEGEND PTR_NERR 
; 0                            PTR_QTY QTY 
; 0   QTYOBJ::HASERROR         self 
; 0   QTYOBJ::HASSTEP          self 
; 0   QTYOBJ::INIT             self DIM ERR INITVALUE LEGEND NAME NERR PTR_ERR PTR_LEGEND 
; 0                            PTR_NERR PTR_QTY QTY RANK STYLEERR TNAME UNITS 
; 0   QTYOBJ::ISCONSTANT       self 
; 0   QTYOBJ::ISNUMERIC        self 
; 0   QTYOBJ::ISSCALAR         self 
; 0   QTYOBJ::POSITIONLEGEND   self thislegend thispos 
; 0   QTYOBJ::REMOVE           self FORLEGEND FORPOS 
; 0   QTYOBJ::REMOVEALL        self 
; 0   QTYOBJ::SET              self DIM ERR INITVALUE LEGEND NAME NERR PTR_ERR PTR_LEGEND 
; 0                            PTR_NERR PTR_QTY QTY RANK STYLEERR TNAME UNITS 
; 0   QTYOBJ::SETDIMENSION     self DIM INITVALUE RANK TNAME 
; 0   QTYOBJ::SETONEVALUE      self FORLEGEND FORPOS ONEERR ONELEGEND ONENERR ONEQTY 
; 0   QTYOBJ::SETSTRING        self NAME STYLEERR UNITS 
; 0   QTYOBJ::SETVALUE         self ERR LEGEND NERR PTR_ERR PTR_LEGEND PTR_NERR PTR_QTY QTY 
;  
;
; IDL> myObject = obj_new('qtyObj', name='qName', qty=[1,2,3], units='counts')
; IDL> myObject -> print_obj
; self.(0) =	self.NAME=	qName
; self.(1) =	self.UNITS=	counts
; self.(2) =	self.TNAME=	INT
; self.(3) =	self.STYLEERR=	
; self.(4) =	self.RANK=	       1
; self.(5) =	self.DIM=	       3       0       0       0
; self.(6) =	self.PTR_LEGEND=	<NullPointer>
; self.(7) =	self.PTR_QTY=	<PtrHeapVar2921>
; self.(8) =	self.PTR_ERR=	<NullPointer>
; self.(9) =	self.PTR_NERR=	<NullPointer>
;
; IDL> myObject -> print_obj, 'ptr_qty', /alsoPtrs
; self.(7) =	*(self.PTR_QTY) =	       1       2       3
;
; NON-GENERIC HELP METHODS:
;	All classes in this directory have a similar SHOW method.
; 	Keyword options include "/recursive" for traveling down the  
;	limbs of a container class and an optional keyword for
;	writing out to a file unit number. Classnames appear in curley
;	brackets.
;
; EXAMPLES: NON-GENERIC HELP METHODS:
;
; IDL> myObject = obj_new('specObj', label='empty')
; IDL> myObject -> show
; 
; .container = [2] 
;  
; FOCUSOBJ.header = [1] DVheader empty specObj::init
;  
; FOCUSOBJ.data = [0] DVdata empty specObj::init
;  
; All contents:
; self.STORE = {DAVE_TAGCONTAINER}[2]
;   .[DVheader,empty,specObj::init]= {DAVE_TAGCONTAINER}[1]
;   .[DVdata,empty,specObj::init]= {DAVE_TAGCONTAINER}[0]
;
;
; DEBUGGING UTILITY PROCEDURES:
; <LI><A HREF="#CHECK_HEAP">CHECK_HEAP</A>
;	This routine prints the contents of all variables (recursively!) and 
; 	file unit numbers left on the heap.
; IDL> CHECK_HEAP
;
; OTHER FILES AND ROUTINES USED: 
;
;	HELP_OBJ methods require the HELPCLASS method.
;
;	SHOW methods require that objects contained within the 
; class have a SHOW method, with keywords lun, heapNum, and possibly
; recusive (if appropriate).
;
;	CHECK_HEAP routine requires COUNT method and GET method, with the keyword
; everything which returns a summary string. 
;
; INCLUDE FILES:
;	<A HREF="#GENERIC__HELCLASS_FRAG.PRO">GENERIC__HELCLASS_FRAG.PRO</A>
;	<A HREF="#GENERIC__HELP_OBJ_FRAG.PRO">GENERIC__HELP_OBJ_FRAG.PRO</A>
;	<A HREF="#GENERIC__PRINT_FRAG.PRO">GENERIC__PRINT_FRAG.PRO</A>
;
; MODIFICATION HISTORY:
;	Tanya Riseman, July, 2003. 
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################

; No real routine here!
