; $Id: cw_ow_oper.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	CW_OW_OPER
;
; PURPOSE:
;	This is a compound object widget which allows an operator or function
; to be selected, either from a pre-defined list or entered.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_OPER__DEFINE_arguments">CW_OW_OPER__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	FSC_FIELD
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;-
; Note: in widget definitions, using
;	uvalue = {wobject:self}, $
; so that in the event handler 
; 	 widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.
; 	 self = uObj.wObject		; alias
; This way, if we add a method command, we use 
;	uvalue = {wobject:self, method:'quit'}, 
;

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_oper::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_oper::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_oper::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_oper::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_oper::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_oper::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_oper::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
;*
pro cw_ow_oper_event, $
	event
;/


  widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.

  message, /info, string("Entered.  uObj.method = ", uObj.method)


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_oper_event, event


;==================================================================
pro cw_ow_oper::eventQuit, event=event

  message, /info, "Entered"
  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro cw_ow_oper::eventWhere, event=event

  message, /info, "Entered"

  debug = 0
  
  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

   where_select = widget_info( event.id, /droplist_select)
   item_select = (self.fromMenu)[where_Select]
   if (debug) then message, /info, string(wname, item_select, " Item selected = ", $
		item_select)

   if (self.fromMenu[where_Select] EQ 'input') then begin
     ; Keep these methods
     self -> setOperatorSensitive, 0	;, no				;;0
     self -> setValueSensitive, 1	;	yes			;;1
   endif else begin		; 'data' or 'header'
     self -> setOperatorSensitive, 1	;  yes 		;;1
     self -> setvalueSensitive, 0	;no		;; 0
   endelse
   
   if (debug) then begin
     message, /info, string("sensitive operator=", self.operatorSensitive, $
   	" value=", self.ValueSensitive)
     message, /info, string("IDs operator=", self.operatorID, $
   	" value=", self.ValueID)
   endif
    
end

;==================================================================
pro cw_ow_oper::eventValue, event=event
      
  message, /info, "Entered"

      widget_control, event.id, get_value= tempValue
      self.Value = tempValue
;      self -> setValue, tempValue
  

end

;==================================================================
pro cw_ow_oper::eventOperatorMenu, event=event

  message, /info, "Entered"

      menu_select = widget_info( event.id, /droplist_select)
      self.ioperator = menu_select
  
end


;==================================================================
;*
pro cw_ow_oper::event, $
	event
;/

  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Should not be here. uname =", uname, " wname=", wname)
 
end


;==================================================================
pro cw_ow_oper::setDataMenu, menu
  if (ptr_valid(self.opMenuPtr)) then ptr_free, self.opMenuPtr
  self.opMenuPtr = ptr_new(Menu)
  self.nOpMenu = n_elements(Menu)
end

;==================================================================
pro cw_ow_oper::setOperatorSensitive, yesNo

  self.operatorSensitive = yesNo

  if (yesNo) then begin
    widget_control,  self.operatorID, sensitive = 1
;;;    self.valueObj ->setProperty, nonsensitive= 1
  endif else begin
    widget_control,  self.operatorID, sensitive = 0
 ;;;   self.valueObj ->setProperty, nonsensitive= 0
  endelse
end

;==================================================================
pro cw_ow_oper::setValueSensitive, yesNo
  self.ValueSensitive = yesNo

  if (yesNo) then begin
    self.valueObj ->setProperty, nonsensitive= 0
;;;    widget_control,  self.operatorID, sensitive = 0
  endif else begin
    self.valueObj ->setProperty, nonsensitive= 1
;;;    widget_control,  self.operatorID, sensitive = 1
  endelse

end

;=======================================================================
pro cw_ow_oper::cleanup
 
;  message, /info, "Entered"

;  if (self.destroy) then begin 
;    message, /info, "Destroying self.specObj"
;;    (self.specObj) -> print_obj
;    obj_destroy, self.specObj
;  endif else begin
;    message, /info, "Dangling self.specObj"
;;    (self.specObj) -> print_obj
;    self.specObj = obj_new()	; Let the old reference dangle.
;  endelse
  
  obj_destroy, self.valueObj
  
  ptr_free, self.opMenuPtr

end


;==================================================================
pro cw_ow_oper_cleanup, tlb

    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_oper::cleanup

end

;==================================================================
pro cw_ow_oper::createWidgets

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_oper_TLB", $
	title = 'cw_ow_oper title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_oper_event',$
	/column)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_oper_TLB", $
	title = 'cw_ow_oper title.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_oper_event',$
	/column)
 endelse
 
 self.tlbID = tlbID

 ;---------------------------------------------------
   whereID = widget_droplist(tlbID, $
   	value = self.fromMenu, $
	uname = 'where', $
	uvalue = {wobject:self, method:'eventWhere'})
   self.whereID = whereID
   widget_control, whereID, set_droplist_select=self.ifrom


  ;---------------------------------------------------------
  if (strlen(self.operator) GT 0) then begin
     self.ioperator = 0
     self.ioperator = where( (self.operator EQ (*(self.opMenuPtr)) ),count)
     if (count GT 0) then begin
       self.ioperator = 0
     endif

  endif 

  ;---------------------------------------------------------
  
  operatorID = widget_droplist(tlbID, $
  	value= (*(self.opMenuPtr)), $
	uname = 'operatorMenu', $
	uvalue = {wobject:self, method:'eventOperatorMenu'}, $
	sensitive = self.operatorSensitive)
   self.operatorID = operatorID
   widget_control, operatorID, set_droplist_select=self.ioperator

  ;-------------------------------------------------------------

   valueID = fsc_field(tlbID, $		; allows sensitive
        object=valueObj, $
	uname = 'value', $
   	title = "", $
   	value = self.value, $
	uvalue = {wobject:self, method:'eventValue'}, $
	/string, $
	/cr_only, event_pro = 'cw_ow_oper_event', $
	sensitive=self.valueSensitive)	
   self.valueID = valueID
   self.valueObj = valueObj

   ;-------------------------------------------------------------

  if (self.ParentID EQ 0L) then begin
    void = widget_button(self.tlbID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_oper',  tlbID
  endif
   
end		;;  cw_ow_oper::createWidgets

;==================================================================
;*
function cw_ow_oper::count
;/

  return, 1

end


;;==================================================================
;;*
;function cw_ow_oper::show
;;/
;
;  self -> print_obj
;
;end

;==================================================================
;*
pro cw_ow_oper::setProperty, $
	from = thisFrom, $	; Changes whcih is sensitized
	oper = thisOper, $	; Goes to the droplist
	value = thisValue	; Goes to the text entry box
;/

  if (n_elements(thisFrom) GT 0) then begin
    iFrom = where( (thisFrom EQ self.fromMenu),count)
    if (iFrom NE -1) then begin
      self.ifrom = iFrom
      widget_control, self.whereID, set_droplist_select=self.ifrom
    endif
    ; Ignore mistakes
  endif

  if (n_elements(thisOper) GT 0) then begin
    ioper = where( (thisOper EQ (*(self.opMenuPtr)) ),count)
    if (ioper NE -1) then begin
      self.ioperator = ioper
      self.operator = thisOper
      widget_control, self.operID, set_droplist_select=self.ioperator
    endif
    ; Ignore mistakes
  endif

  if (n_elements(thisValue) GT 0) then begin
    self.value = thisValue
    widget_control, self.valueID, set_value = thisValue
  endif

end

;==================================================================
;*
pro cw_ow_oper::getProperty, $
	oper = thisOper			; return the operator selected 
;/
;	from = thisFrom, $
;	value = thisValue

  
  if (arg_present(thisOper)) then begin
    thisValue = 0.0
    if (self.operatorSensitive) then begin
      if (arg_present(thisOper)) then begin
        thisOper = (*(self.opMenuPtr))[self.ioperator]
      endif
    endif else begin
      if (self.valueSensitive) then begin
         widget_control, self.valueID, get_value = thisOper
	 if (thisOper NE self.value) then begin
;	   message, /info, string("Mismatch in value storage: '", $
;	   	 thisOper, "', '",  self.value, "'")
	   self.value = thisOper	
	 endif
      endif
    endelse
  endif
  
;  if (arg_present(thisFrom)) then begin
;    thisFrom = (self.fromMenu)[self.ifrom]
;  endif
;
;  if (arg_present(thisOper)) then begin
;    thisOper = (*(self.opMenuPtr))[self.ioperator]
;  endif
;
;  if (arg_present(thisValue)) then begin
;    widget_control, self.valueID, get_value = thisValue
;  endif

end


;==================================================================
;*
function cw_ow_oper::init, $
;	thisSpecObj, $		; required
	thisParentID, $		; optional
	operators = thisOperators, $	; List of operators. String array.
	from = thisFrom, $	; optional. defaults to 'input'
				; Acceptible values: 'input', 'operator',
	field = thisField, $	; Optional pre-selection. 
	destroy = thisDestroy	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
;/
	
  debug = 1

;   message, /info, 'Does self have a valid heap address at the begining of INIT?'
;   help, self
;  if (NOT obj_valid(thisSpecObj)) then begin
;    message, /info, "Invalid object specObj."
;    return, -1
;  endif

;  self.specObj = thisSpecObj
;  self.destroy = keyword_set(thisDestroy)
 

  if (n_elements(thisOperators) gt 0) then begin
    ptr_free, self.opMenuPtr
    self.opMenuPtr = ptr_new(thisOperators)
    self.nOpMenu = n_elements(thisOperators)
  endif
  
  ;-------------------------------------------------------------
  self.fromMenu = ['input', 'operator' ]	; could also be function
  showlen = n_elements(self.fromMenu)

  iFrom = 0
  if (n_elements(thisFrom) GT 0) then begin
    iFrom = where( (thisFrom EQ self.fromMenu),count)
    if (count EQ 0) then begin
      iFrom = 0
      message, /info, string("Unrecognized value for keyword from=", thisFrom)
    endif ;else begin
  endif
  self.iFrom = ifrom
  
  if (self.fromMenu[iFrom] EQ 'input') then begin	; 'input'
    self.valueSensitive = 1
  endif else begin	; 'operator'
    self.operatorSensitive = 1
  endelse
  ;-------------------------------------------------------------
  
  if (n_elements(thisField) EQ 0) then begin
    thisField = ''
  endif
  if (strlen(thisField) GT 0) then begin
     if (self.iFrom NE 0) then begin	; 'operator'
        self.operator = thisField
     endif else begin			; 'input'
        self.Value = thisField
     endelse 
  endif 
  
  ;-------------------------------------------------------------

  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_oper__define
  define = {cw_ow_oper, $
;  	specObj:obj_new(), $	; don't destroy
;	destroy: 0, $
	parentID:0l, $
	tlbID:0l, $
	fromMenu:['', ''], $	;['input', 'operator' ]
	ifrom: 0, $
	whereID:0L, $
	opMenuPtr: ptr_new(), $; string array, like ['+', '-', '*', '/']
	nOpMenu: 0, $
	ioperator: 0, $			;integer position. Doesn't work
	operator:'', $			; 1 string. Doesn't work
	operatorID:0L, $
	operatorSensitive: 0, $
	value: ' ', $			; eventValue not triggered. So useless.
	valueID:0L, $
	valueObj: obj_new(), $		; returned from fsc_field.pro
	valueSensitive:0 $
	}
;/

end
