; $Id: cw_ow_result.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_result
;
; PURPOSE:
;	This is a compound object widget which allows the label, name,
; and units to be created for a new field in an object of class specObj. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_RESULT__DEFINE_arguments">CW_OW_RESULT__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	FSC_FIELD
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;-
;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_result::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_result::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_result::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_result::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_result::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_result::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_result::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
;*
pro cw_ow_result_event, $
	event
;/


  widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.

;  message, /info, string("Entered.  uObj.method = ", uObj.method)


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_result_event, event


;==================================================================
pro cw_ow_result::eventQuit, event=event

;  message, /info, "Entered"
  widget_control, self.tlbID, /destroy
  
end


;==================================================================
;*
pro cw_ow_result::event, $
	event
;/

  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Nothing done here. uname =", uname, " wname=", wname)
 
end



;=======================================================================
pro cw_ow_result::cleanup
 
;  message, /info, "Entered"

;  obj_destroy, [self.labelObj, self.nameObj, self.unitsObj]	; is this needed?

  obj_destroy, self.labelObj
  obj_destroy, self.nameObj
  obj_destroy, self.unitsObj
  
end


;==================================================================
pro cw_ow_result_cleanup, tlb

;    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_result::cleanup

end

;==================================================================
pro cw_ow_result::createWidgets

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_result_TLB", $
	title = 'cw_ow_result title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_result_event',$
	/column)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_result_TLB", $
	title = 'cw_ow_result title. ' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_result_event',$
	/column)
 endelse
 
 self.tlbID = tlbID

  ;-------------------------------------------------------------

   labelID = fsc_field(tlbID, $		; allows sensitive
        object=labelObj, $
	uname = 'label', $
   	title = "label:", $
   	value = self.label, $
	uvalue = {wobject:self, method:'event'}, $
	/cr_only, event_pro = 'cw_ow_result_event', $
	/string)	
   self.labelID = labelID
   self.labelObj = labelObj

   ;-------------------------------------------------------------
   nameID = fsc_field(tlbID, $		; allows sensitive
        object=nameObj, $
	uname = 'name', $
   	title = "name:", $
   	value = self.name, $
	uvalue = {wobject:self, method:'event'}, $
	/cr_only, event_pro = 'cw_ow_result_event', $
	/string)	
   self.nameID = nameID
   self.nameObj = nameObj

   ;-------------------------------------------------------------
   unitsID = fsc_field(tlbID, $		; allows sensitive
        object=unitsObj, $
	uname = 'units', $
   	title = "units:", $
   	value = self.units, $
	uvalue = {wobject:self, method:'event'}, $
	/cr_only, event_pro = 'cw_ow_result_event', $
	/string)	
   self.unitsID = unitsID
   self.unitsObj = unitsObj

   ;-------------------------------------------------------------

  if (self.ParentID EQ 0L) then begin
    void = widget_button(self.tlbID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_result',  tlbID
  endif
   
end		;;  cw_ow_result::createWidgets

;==================================================================
;*
function cw_ow_result::count
;/

  return, 1

end

;==================================================================
;*
pro cw_ow_result::setProperty, $
	label = thisLabel, $ 
	name = thisName, $
	units = thisUnits
;/

  if (n_elements(thislabel) GT 0) then begin
    self.Label = thisLabel
    widget_control, self.labelID, set_value = thisLabel
  endif

  if (n_elements(thisName) GT 0) then begin
    self.name = thisName
    widget_control, self.nameID, set_value = thisName
  endif

  if (n_elements(thisUnits) GT 0) then begin
    self.units = thisUnits
    widget_control, self.unitsID, set_value = thisUnits
  endif
	
end

;==================================================================
;*
pro cw_ow_result::getProperty, $
	label = thisLabel, $ 
	name = thisName, $
	units = thisUnits
;/

  if (arg_present(thislabel)) then begin
    widget_control, self.labelID, get_value = thisLabel	; refresh
    self.Label = thisLabel
;    thisLabel = self.Label 
  endif

  if (arg_present(thisName)) then begin
    widget_control, self.nameID, get_value = thisName	; refresh
    self.Name = thisName
;    thisName = self.name 
  endif

  if (arg_present(thisUnits)) then begin
    widget_control, self.unitsID, get_value = thisUnits	; refresh
    self.units = thisUnits
;    thisUnits = self.units 
  endif

end

;==================================================================
;*
function cw_ow_result::init, $
	thisParentID		; optional
;/
; Note: in widget definitions, using
;	uvalue = {wobject:self}, $
; so that in the event handler 
; 	 widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.
; 	 self = uObj.wObject		; alias
; This way, if we add a method command, we use 
;	uvalue = {wobject:self, method:'quit'}, 
;

	
  debug = 1

  ;-------------------------------------------------------------

  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_result__define
  define = {cw_ow_result, $
;  	specObj:obj_new(), $	; don't destroy
;	destroy: 0, $
	parentID:0l, $
	tlbID:0l, $
	label: '', $	
	labelID :0L, $,
	labelObj: obj_new(), $	; object returned from fsc_field.pro
	name: '', $	
	nameID :0L, $,
	nameObj: obj_new(), $	; object returned from fsc_field.pro
	units: '', $	
	unitsID :0L, $
	unitsObj: obj_new() $	; object returned from fsc_field.pro
	}
;/

end
