; $Id: cw_ow_scalebymonitor.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_scalebymonitor
;
; PURPOSE:
;	This is a compound object widget which is ICP/triple axis specific.
; It performs the allows the quantity, errors, 
; and legend to be created for a new field in an object of class specObj. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_SCALEBYMONITOR__DEFINE_arguments">CW_OW_SCALEBYMONITOR___DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
; <LI><A HREF="#CW_OW_RESULT__DEFINE_arguments">CW_OW_RESULT__DEFINE_arguments</A>
; <LI><A HREF="#CW_OW_WHERE__DEFINE_arguments">CW_OW_WHERE__DEFINE_arguments</A>
; <LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;-

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_scalebymonitor::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_scalebymonitor::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_scalebymonitor::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_scalebymonitor::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_scalebymonitor::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_scalebymonitor::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_scalebymonitor::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
pro cw_ow_scalebymonitor::FreshenWhere, $ 
	inHeader = thisInHeader, $
	inData = thisInData

  self.where1WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  
end
;==================================================================
;*
pro cw_ow_scalebymonitor_event, $
	event
;/

  widget_control, event.id, get_uvalue=uObj 

  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_scalebymonitor_event, event

;==================================================================
pro cw_ow_scalebymonitor::eventQuit, event=event

  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro cw_ow_scalebymonitor::eventFreshen, event=event

  self -> freshenResults
  
end

;==================================================================
;*
pro cw_ow_scalebymonitor::eventProcess, event=event
;/

	infolabel = "cw_ow_scalebymonitor::eventProcess >> "
;   (self.result1WObj) -> getProperty, $
;   	label = res1Label		;, $ 
;;	name = res1Name, $
;;	units =res1Units

;   (self.result2WObj) -> getProperty, $
;   	label = res2Label		;, $ 
;;	name = res2Name, $
;;	units =res2Units

   (self.where1WObj) -> getProperty, $
	from = From1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1, $	; Goes to the droplist
	value = value1	; Goes to the text entry box

   inHeader = 0
   inData = 0
   if (from1 eq 'input') then begin
     tags1 = value1
   endif else begin
     tags1 = ['*', label1, '*'] 
     if (from1 EQ 'header') then begin
       inHeader = 1
     endif
     if (from1 EQ 'data') then begin
       inData = 1
     endif      
   endelse
   
   theErrMsg = ''   
   OK = (self.SpecObj) -> scalebymonitor(tags1, $
   	errmsg = theErrMsg)
;	newIntensity =  ['*', res1Label,  'cw_ow_scalebymonitor'], $
;	newTime =  ['*', res2Label,  'cw_ow_scalebymonitor'])
	
;   OK = (self.SpecObj) -> cloneField($
;   	tags1, $
;     	inHeader=inHeader, inData=inData, $
;	label =  resLabel, $
;;	resTags =  ['*', resLabel,  'cw_ow_scalebymonitor'], $
;	name = resName, $
;	units = resUnits)

   statusMsg = 'scalebymonitor successful'		; or an empty string
   if (NOT OK) then begin
      statusMsg = [theErrMsg, 'scalebymonitor FAILED']
   endif 
   
   print, infolabel, OK, "=OK. errmsg = ",  theErrMsg
   print, infolabel, OK, "=OK. statusmsg = ",  statusMsg
   
   self.parentWobj -> freshen, status=statusMsg
    
 if (OK) then begin 
   ;-----------------------------------------------------------------------
  treatString = "OK = specObj -> scalebymonitor(  $"
    if (size(tags1, /tname) EQ 'STRING') then begin
    treatString = [treatString, $
      string(", tags1 = ['", tags1[0], "', '", tags1[1], "', '", tags1[2], "'] $")]
    endif else begin
    treatString = [treatString, $
      string(", tags1 = ", strcompress(string(tags1)), " $")]
    endelse
;    treatString = [treatString, $
;	string(", newIntensity= ['*', ", res1Label, $
;		", 'cw_ow_scalebymonitor'] $")]
;    treatString = [treatString, $
;	string(", newTime= ['*', ", res2Label, $
;		", 'cw_ow_scalebymonitor'] $")]
    treatString = [treatString, ")"]

  ;-----------------------------------------------------------------------
  OK = (self.SpecObj) -> appendField($
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'], /inHeader,$
	qty= treatString ) 
  ;-----------------------------------------------------------------------

   self.parentWobj -> freshen, /Where, inHeader=inHeader, inData=inData
 endif	  
	  
end

;==================================================================
;*
pro cw_ow_scalebymonitor::event, $
	event
;/

  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Should not be here. uname =", uname, " wname=", wname)

end

;=======================================================================
pro cw_ow_scalebymonitor::cleanup
 
  if (self.destroy) then begin 
;    obj_destroy, self.specObj
  endif else begin
;    message, /info, "Dangling self.specObj"
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse

  self.parentWObj = obj_new()	; Let old reference dangle.

  obj_destroy, self.where1WObj
;  obj_destroy, [self.resultWObj, self.result2WObj, self.where1WObj]

  message, /info, "..."
  
end


;==================================================================
pro cw_ow_scalebymonitor_cleanup, tlb

    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_scalebymonitordialog_ow_modify::
end

;==================================================================
pro cw_ow_scalebymonitor::freshenResults

;  (self.where1WObj) -> getProperty, $
;	from = From1, $	; which is sensitized 'input', 'header' or 'data'
;	label = label1, $ ; Goes to the droplist
;	value = value1	; Goes to the text entry box
;
;   inHeader = 0
;   inData = 0
;   if (from1 EQ 'header') then begin
;     inHeader = 1
;   endif
;   if (from1 EQ 'data') then begin
;     inData = 1
;   endif 
;
;;   units1 = ''
;;   name1 = ''   
;;   OK = (self.specObj) -> getField(tags1, units=units1, name=name1,$
;;     	inHeader = inHeader, inData=inData)
;   
;   if (from1 eq 'input') then begin
;     tags1 = value1
;     units1 = strtrim( string(value1), 2)
;     name1 = units1
;     label1 = units1
;;clone     resName = Name1
;;clone     reslabel = Label1
;   endif else begin
;     tags1 = ['*', label1, '*'] 
;     OK = (self.specObj) -> getField(tags1, units=units1, name=name1,$
;    		   inHeader = inHeader, inData=inData)	;,  some=somejunk)
;     if (NOT OK) then message, /info, string($
;     	"Failed to get name and units for label=", label1) 
;;     resName = string(name1, '_scalebymonitor')
;;     reslabel = string( Label1, '_scalebymonitor')
;   endelse
;        	
;;   resUnits = units1
 ;  
;;   (self.resultWObj) -> setProperty, $
;;   	label = resLabel, $ 
;;	name = resName, $
;;	units =resUnits   

end

;==================================================================
pro cw_ow_scalebymonitor::createWidgets

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_scalebymonitor_TLB", $
	title = 'cw_ow_scalebymonitor title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_scalebymonitor_event',$
;	/frame, $
	/row)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_scalebymonitor_TLB", $
	title = 'cw_ow_scalebymonitor title. ' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_scalebymonitor_event',$
;	/frame, $
	/row)
 endelse
 
 self.tlbID = tlbID

  ;-------------------------------------------------------------

  buttonClusterID = widget_base(tlbID, /column, /frame)

; Not needed currently
;  voidID = widget_button(buttonClusterID, value='FRESHEN', $
;    		uvalue={wobject:self, method:'eventFreshen'})

  voidID = widget_button(buttonClusterID, $
  	value='scalebymonitor', $
;  	value='PROCESS', $
    	uvalue={wobject:self, method:'eventProcess'})

  if (self.ParentID EQ 0L) then begin
    void = widget_button(buttonClusterID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
  endif

  ;-------------------------------------------------------------
  doItID = widget_base(tlbID, /row, /frame)

  ;-------------------------------------------------------------
;  self.resultWObj = obj_new('cw_ow_result', doItID)
;  voidID = Widget_label(doItID, value="=")
 
  ;-------------------------------------------------------------
  self.where1WObj = obj_new('cw_ow_where', self.specObj, doItID,$
  	from = 'input', $
	field = 1.0, $	; default to value 1.0, not 0.0.
	/noData)
	; Allow user input or value from header.

  ;-------------------------------------------------------------
  self -> freshenResults

  ;-------------------------------------------------------------
  if (self.ParentID EQ 0L) then begin
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_scalebymonitor',  tlbID
  endif
   
end		;;  cw_ow_scalebymonitor::createWidgets


;==================================================================
;*
function cw_ow_scalebymonitor::init, $
	thisSpecObj, $		; required
	thisParentID, $		; optional
	destroy = thisDestroy, $	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
	parentWobj = thisParentWobj ; parent object
;/
	
  debug = 1
  
  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)
 
  ;-------------------------------------------------------------
  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  ;-------------------------------------------------------------
  if (obj_valid(thisParentWObj)) then begin
    self.parentWObj = thisParentWobj
  endif
 
  ;-------------------------------------------------------------
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_scalebymonitor__define
  define = {cw_ow_scalebymonitor, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	parentID:0l, $
	parentWObj: obj_new(), $
	tlbID:0l, $
;	resultWObj: obj_new(), $
	where1WObj: obj_new() $
	}
;/

end
