; $Id: cw_ow_setvalue.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_setValue
;
; PURPOSE:
;	This is a compound object widget which modified the mostly numeric
; portions (quantity, errors, and legend) of a field in an object of class 
; specObj. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_SETVALUE__DEFINE_arguments">CW_OW_SETVALUE__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
; <LI><A HREF="#CW_OW_RESULTVALUE__DEFINE_arguments">CW_OW_RESULTVALUE__DEFINE_arguments</A>
; <LI><A HREF="#CW_OW_WHERE__DEFINE_arguments">CW_OW_WHERE__DEFINE_arguments</A>
; <LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;-

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_setValue::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_setValue::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_setValue::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_setValue::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_setValue::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_setValue::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_setValue::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
pro cw_ow_setValue::FreshenWhere, $ 
	inHeader = thisInHeader, $
	inData = thisInData

  self.where1WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  
end
;==================================================================
;*
pro cw_ow_setValue_event, $
	event
;/

  widget_control, event.id, get_uvalue=uObj 

  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_setValue_event, event

;==================================================================
pro cw_ow_setValue::eventQuit, event=event

  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro cw_ow_setValue::eventFreshen, event=event

  self -> freshenResultValues
;  self -> freshenResults
  
end

;==================================================================
;*
pro cw_ow_setValue::eventProcess, event=event
;/

   (self.resultValueWObj) -> getProperty, $
	Qty = thisQty, $ 
	Err = thisErr, $
	nErr = thisnErr, $
	legend = thisLegend

   (self.where1WObj) -> getProperty, $
	from = From1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1 	; Goes to the droplist
;	value = value1	; Goes to the text entry box

   inHeader = 0
   inData = 0
;   if (from1 eq 'input') then begin
;;     tags1 = value1
;   endif else begin
     tags1 = ['*', label1, '*'] 
     if (from1 EQ 'header') then begin
       inHeader = 1
     endif
     if (from1 EQ 'data') then begin
       inData = 1
     endif      
;   endelse
   
  ;------------------------------------------------------
    treatStr = string("['", tags1[0], "','", tags1[1], $
    	"','", tags1[2], "']")
    treatStr = treatStr +  $
    	string(",inHeader=", strtrim(string(inHeader),2))
    treatStr = treatStr +  $
    	string(",inData=", strtrim(string(inData),2))
  if (strlen(thisQty) GT 0) then begin
    treatStr = treatStr +  $
    	string(",qty=", thisQty)
  endif
  if (strlen(thisErr) GT 0) then begin
    treatStr = treatStr +  $
    	string(",err=", thisErr)
  endif
  if (strlen(thisNerr) GT 0) then begin
    treatStr = treatStr +  $
    	string(",nerr=", thisnErr)
  endif
  if (strlen(thisLegend) GT 0) then begin
    treatStr = treatStr +  $
    	string(",legend=", thisLegend)
  endif
  treatStr = treatStr +  ") )"

  treatString = "OK=( (self.specObj)->setField(" + treatStr

; OK=((self.specObj)->setField(['*','scatPlane_u','*'],inHeader=1,inData=0,qty=[1.,0.3, 0.],legend=['3h', '3k', '3l']) )
;; OKExecute("OK=((self.specObj)->setField(['*','scatPlane_u','*'],inHeader=1,inData=0,qty=[1.,0.3, 0.],legend=['3h', '3k', '3l']) )")
 ;  OKexecute = execute(" silly = 0.5 + 4")
;  print, "silly=", silly
  
      print, treatString
  ;------------------------------------------------------
  ; Note: thisQty, thisErr, thisnErr and thisLegend are 
  ; all strings, not scalars or arrays, so need execute().
  OKexecute = execute( treatString)	;;;, quietCompile=0)
  
;   OK = (self.SpecObj) -> setField($
;   	tags1, $
;     	inHeader=inHeader, inData=inData, $
;	Qty = thisQty, $ 
;	Err = thisErr, $
;	nErr = thisnErr, $
;	Legend = thisLegend)

;  print, "OKexecute = ", OKexecute
;  print, " OK = ", OK		; Why is this not defined by execute???
  
   statusMsg = 'setValue successful'		; or an empty string
   if (NOT OKexecute) then begin
     statusMsg = ['Execute Syntax error in the following command:', $
    	treatString]
     message, /info, 'Execute Syntax error in the following command:'
      print, treatString
     OK = 0		; otherwise not defined
   endif else begin
     if (NOT OK) then begin
     statusMsg = ['Error performing the method:', $
    	treatString]
       message, /info, 'Error performing the method:"
         print, treatString
     endif
   endelse
   self.parentWobj -> freshen, status=statusMsg
   

 if (OKexecute AND OK) then begin
;   resCodeSource = 'cw_ow_setValue'
;   OK = (self.SpecObj) -> setTagField($
;   	tags1, $
;    	inHeader=inHeader, inData=inData, $
;;	label =  Label1, $
;	codeSource = resCodeSource)

	
   ;-----------------------------------------------------------------------
    treatStr = string("['", tags1[0], "', '", tags1[1], $
    	"', '", tags1[2], "'] $")
    treatStr = [treatStr, $
    	string(", inHeader=", strtrim(string(inHeader),2), " $")]
    treatStr = [treatStr, $
    	string(", inData=", strtrim(string(inData),2), " $")]
  if (strlen(thisQty) GT 0) then begin
    treatStr = [treatStr, $
    	string(", qty = ", thisQty,  " $")]
  endif
  if (strlen(thisErr) GT 0) then begin
    treatStr = [treatStr, $
    	string(", err = ", thisErr,  " $")]
  endif
  if (strlen(thisNErr) GT 0) then begin
    treatStr = [treatStr, $
    	string(", nerr = ", thisnErr,  " $")]
  endif
  if (strlen(thisLegend) GT 0) then begin
    treatStr = [treatStr, $
    	string(", legend = ", thisLegend,  " $")]
  endif
  treatStr = [treatStr, ")"]
  treatString = ["OK = specObj -> setField(  $", treatStr]


   ;-----------------------------------------------------------------------
;  treatString = [treatString, "OK = specObj -> setTagField(  $"]
;    treatString = [treatString, $
;      string("tags1 = ['", tags1[0], "', '", tags1[1], "', '", tags1[2], "'] $")]
;    treatString = [treatString, $
;    	string(", inHeader=", strtrim(string(inHeader),2), " $")]
;    treatString = [treatString, $
;    	string(", inData=", strtrim(string(inData),2), " $")]
;  treatString = [treatString, $
;    	string(", label = '", Label1,  "' $")]
;  treatString = [treatString, $
;    	string(", codeSource = '", resCodeSource,  "' $")]
;  treatString = [treatString, ")"]

  ;-----------------------------------------------------------------------
  OK = (self.SpecObj) -> appendField($
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'], /inHeader,$
	qty= treatString ) 
  ;-----------------------------------------------------------------------

   ;;; self.parentWobj -> freshen, /Where, inHeader=inHeader, inData=inData
 endif
  
end

;==================================================================
;*
pro cw_ow_setValue::event, $
	event
;/

  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Should not be here. uname =", uname, " wname=", wname)

end

;=======================================================================
pro cw_ow_setValue::cleanup
 
  if (self.destroy) then begin 
;    obj_destroy, self.specObj
  endif else begin
;    message, /info, "Dangling self.specObj"
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse

  self.parentWObj = obj_new()	; Let old reference dangle.

  obj_destroy, [self.resultValueWObj, self.where1WObj]

  message, /info, "..."
  
end


;==================================================================
pro cw_ow_setValue_cleanup, tlb

    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_setValuedialog_ow_modify::
end
;==================================================================
function cw_ow_setValue_num2str, numArr
     result = ''
     nlen = n_elements(numArr)

     if (nlen EQ 0) then begin
       return, result
     endif
     
     if (nlen EQ 1) then begin
       result = strtrim(string(numArr), 2)
       return,  result
     endif

    ;--- More than 1 element in the array     
    result = string('[', strtrim(string(numArr[0]), 2) )
    for i = 1, (nlen-1) do begin
       result = string(result, ', ', strtrim(string(numArr[i]), 2))
    endfor
    result = string(result, ']')
 
    return, result
end

;==================================================================
function cw_ow_setValue_strArr2str, numArr

     result = ''
     nlen = n_elements(numArr)

     if (nlen EQ 0) then begin
       return, result
     endif
     
     if (nlen EQ 1) then begin
       result = numArr
       return,  result
     endif

    ;--- More than 1 element in the array     
    result = string("['", strtrim(numArr[0], 2), "'" )
    for i = 1, (nlen-1) do begin
       result = string(result, ", '", strtrim(numArr[i], 2), "'")
    endfor
    result = string(result, ']')
 
    return, result
end

;==================================================================
pro cw_ow_setValue::freshenresultValues

  (self.where1WObj) -> getProperty, $
	from = From1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1, $ ; Goes to the droplist
	value = value1	; Goes to the text entry box

   inHeader = 0
   inData = 0
   if (from1 EQ 'header') then begin
     inHeader = 1
   endif
   if (from1 EQ 'data') then begin
     inData = 1
   endif 

;   units1 = ''
;   name1 = ''   
;   OK = (self.specObj) -> getField(tags1, units=units1, name=name1,$
;     	inHeader = inHeader, inData=inData)
   
   if (from1 eq 'input') then begin
     tags1 = value1
     units1 = strtrim( string(value1), 2)
     name1 = units1
     label1 = units1
     resName = Name1
     reslabel = Label1
   endif else begin
     tags1 = ['*', label1, '*'] 
     OK = (self.specObj) -> getField(tags1, $
	Qty = Qty1, $ 
	Err =  Err1, $
	nErr = nErr1, $
	Legend = Legend1, $
    		   inHeader = inHeader, inData=inData)	
;     OK = (self.specObj) -> getField(tags1, units=units1, name=name1,$
;    		   inHeader = inHeader, inData=inData)	;,  some=somejunk)

     if (NOT OK) then message, /info, string($
     	"Failed to get name and units for label=", label1) 
  ;     message, /info, somejunk

;     resName = name1 
;     reslabel = label1 

     resQty = cw_ow_setValue_num2str(qty1)
     resErr = cw_ow_setValue_num2str(err1)
     resNErr = cw_ow_setValue_num2str(nerr1)
     reslegend = cw_ow_setValue_strArr2str(legend1)

   endelse
        	
;   resUnits = units1
   
   (self.resultValueWObj) -> setProperty, $
	Qty = resQty, $ 
	Err = resErr, $
	nErr = resnErr, $
	Legend = resLegend

end

;==================================================================
pro cw_ow_setValue::createWidgets

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_setValue_TLB", $
;	title = 'cw_ow_setValue title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_setValue_event',$
;	/frame, $
	/row)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_setValue_TLB", $
;	title = 'cw_ow_setValue title. ' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_setValue_event',$
;	/frame, $
	/row)
 endelse
 
 self.tlbID = tlbID

  ;-------------------------------------------------------------

  buttonClusterID = widget_base(tlbID, /column, /frame)
  voidID = widget_button(buttonClusterID, value='FRESHEN', $
    		uvalue={wobject:self, method:'eventFreshen'})

  voidID = widget_button(buttonClusterID, $
  	value='SET VALUES', $
;  	value='PROCESS', $
    	uvalue={wobject:self, method:'eventProcess'})

  if (self.ParentID EQ 0L) then begin
    void = widget_button(buttonClusterID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
  endif

  ;-------------------------------------------------------------
  doItID = widget_base(tlbID, /row, /frame)

  ;-------------------------------------------------------------
  self.resultValueWObj = obj_new('cw_ow_resultValue', doItID)
  voidID = Widget_label(doItID, value="in")
;  voidID = Widget_label(doItID, value="=")
 
  ;-------------------------------------------------------------
  self.where1WObj = obj_new('cw_ow_where', self.specObj, doItID,$
  	from = 'header', /noInput, /noData)

;  self -> print_obj
  ;-------------------------------------------------------------
  self -> freshenresultValues

  ;-------------------------------------------------------------
  if (self.ParentID EQ 0L) then begin
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_setValue',  tlbID
  endif
   
  
end		;;  cw_ow_setValue::createWidgets


;==================================================================
;*
function cw_ow_setValue::init, $
	thisSpecObj, $		; required
	thisParentID, $		; optional
	destroy = thisDestroy, $	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
	parentWobj = thisParentWobj ; parent object
;/
	
  debug = 1

  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)
 
  ;-------------------------------------------------------------
  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  ;-------------------------------------------------------------
  if (obj_valid(thisParentWObj)) then begin
    self.parentWObj = thisParentWobj
  endif
 
  ;-------------------------------------------------------------
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_setValue__define
  define = {cw_ow_setValue, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	parentID:0l, $
	parentWObj: obj_new(), $
	tlbID:0l, $
	resultValueWObj: obj_new(), $
	where1WObj: obj_new() $
	}
;/

end
