; $Id: cw_ow_where.pro,v 1.3 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_where
;
; PURPOSE:
;	This is a compound object widget which selects a field from an 
; object of class specObj. The field can come from either the "data"
; or "header" sections of the object. A user input value (scalar) are allowed
; in lieu of a field.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_WHERE__DEFINE_arguments">CW_OW_WHERE__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	FSC_FIELD
; <LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;-
; Note: in widget definitions, using
;	uvalue = {wobject:self}, $
; so that in the event handler 
; 	 widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.
; 	 self = uObj.wObject		; alias
; This way, if we add a method command, we use 
;	uvalue = {wobject:self, method:'quit'}, 
;

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_where::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_where::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_where::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_where::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_where::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_where::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_where::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
;*
pro cw_ow_where_event, $
	event
;/

  widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.

;  message, /info, string("Entered.  uObj.method = ", uObj.method)

;;;  self = uObj.wObject		; alias

;  help, uObj
;  structure_show, uObj, /full

;  print, "obj_valid(uObj.wObject) =", obj_valid(uObj.wObject) 
;  if (obj_valid(uObj.wObject)) then begin
;;    help, uObj.wObject 
;;    (uObj.wObject) -> print_obj
;
;    (uObj.wObject) -> event, event
;;    call_method, 'event', (uObj.wObject), event
;  endif else begin
;    message, /info, "Error. Invalid object reference"
;  endelse


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_where_event, event


;==================================================================
pro cw_ow_where::eventQuit, event=event

;  message, /info, "Entered"
;   help, self

  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro cw_ow_where::eventWhere, event=event

  debug = 0
  
  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

;   message, /info, 'Entered'
    
;     no = 0
;     yes = 1
  
   where_select = widget_info( event.id, /droplist_select)
;   item_select = self -> fromMenu(where_Select)
   item_select = (self.fromMenu)[where_Select]
   self.ifrom = where_Select 
   
   if (debug) then message, /info, string(wname, item_select, " Item selected = ", $
		item_select)
;   if (self -> fromMenu(where_Select) EQ 'input') then begin
   if (self.fromMenu[where_Select] EQ 'input') then begin
     ; Keep these methods
     self -> setLabelSensitive, 0	;, no				;;0
     self -> setValueSensitive, 1	;	yes			;;1
   endif else begin		; 'data' or 'header'
     self -> setlabelSensitive, 1	;  yes 		;;1
     self -> setvalueSensitive, 0	;no		;; 0
   endelse
   
   if (debug) then begin
     message, /info, string("sensitive label=", self.LabelSensitive, $
   	" value=", self.ValueSensitive)
     message, /info, string("IDs label=", self.labelID, $
   	" value=", self.ValueID)
;     message, /info, string("sensitive label=", self -> getLabelSensitive(), $
;   	" value=", self -> getValueSensitive())
;     message, /info, string("IDs label=", self -> getlabelID(), $
;   	" value=", self -> getValueID())
   endif
   
   ;--------------------------------------------------------
   ; Refresh the menu
   if ( self.fromMenu[where_Select] EQ 'header') then begin
;   if ( self -> fromMenu(where_Select) EQ 'header') then begin
     self.InHeader = 1
     self.InData = 0
;     self -> setInHeader,  1
;     self -> setInData,  0
     nLabels = 0
   endif
   
   if ( self.fromMenu[where_Select] EQ 'data') then begin
;   if ( self -> fromMenu(where_Select) EQ 'data') then begin
     self.InHeader = 0
     self.InData = 1
;     self -> setInHeader, 0
;     self -> setInData, 1
     nLabels = 0
   endif
    
   if (debug) then message, /info, string(" focus inHeader=", $
   	self.InHeader, " inData=", self.InData)
   
;   message, /info, string(" focus inHeader=", $
;   	self -> getInHeader(), " inData=", self -> getInData())

   if (self.fromMenu[where_Select] NE 'input') then begin
      InHeader = self.InHeader
      InData = self.InData
      (self.SpecObj)  -> getLabel, $
	inHeader= InHeader, inData=InData, $
	label=LabelMenu			;, count=nLabels
     if (InHeader) then  begin
       self -> setHeaderMenu,  Labelmenu
     endif
     if (InData) then  begin
       self -> setDataMenu,  Labelmenu
     endif
     widget_control,  self.labelID, set_value=LabelMenu
   endif 
 
;   if (self.fromMenu[where_Select] EQ 'header') then begin
;     self -> freshenWhere, /inHeader
;   endif
;   if (self.fromMenu[where_Select] EQ 'Data') then begin
;     self -> freshenWhere, /inData
;   endif
   
;;   if (self -> fromMenu(where_Select) NE 'input') then begin
;   if (self.fromMenu[where_Select] NE 'input') then begin
;     tempObj = self.SpecObj
;     tempInHeader = self.InHeader
;     tempInData = self.InData
;;     tempObj = (self -> getSpecObj())
;;     tempInHeader = self -> getInHeader() 
;;     tempInData = self -> getInData()
;      tempObj  -> getLabel, $
;	inHeader=(tempInHeader), inData=(tempInData), $
;	label=LabelMenu			;, count=nLabels
;     if (tempInHeader) then  begin
;       self -> setHeaderMenu,  Labelmenu
;     endif
;     if (tempInData) then  begin
;       self -> setDataMenu,  Labelmenu
;     endif
;;      message, /info, string(" focus menu=") 
;;      print, labelMenu
;
;     widget_control,  self.labelID, set_value=LabelMenu
;;     widget_control,  self -> getlabelID(), set_value=LabelMenu
;      
;   endif 
 
end

;==================================================================
pro cw_ow_where::eventValue, event=event
      
;   message, /info, 'Entered'

      widget_control, event.id, get_value= tempValue
      self.Value = tempValue
;      self -> setValue, tempValue
  

end

;==================================================================
pro cw_ow_where::eventlabelMenu, event=event


;   message, /info, 'Entered'

      menu_select = widget_info( event.id, /droplist_select)
      self.ilabel = menu_select
;      self -> setIlabel, menu_select
  
end


;==================================================================
;*
pro cw_ow_where::event, $
	event
;/

  debug = 0
   
  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Should not be here. uname =", uname, " wname=", wname)
 

;  if ((uname EQ  'cw_ow_where_TLB') OR (uname EQ  'QUIT')) then begin 
;    message, /info, "Should not be here. uname =", uname
;    return
;  endif
;
;;Humm  if ((uname EQ  'cw_ow_where_TLB') OR (uname EQ  'QUIT')) then begin 
;;    widget_control, event.id, /destroy		; is also top.
;;;    widget_control, self ->get_tlbID(), /destroy
;;    obj_destroy, self
;;    return
;;  endif
;  ;------------------------------------------
; 
;
; if (debug) then begin
;  message, /info, string("Event from uname ", uname, " of type ", wname)
;  print, "obj_valid(self) =", obj_valid(self) 
;;  print, "obj_valid(uObj.wObject) =", obj_valid(uObj.wObject) 
;;  help, uObj
;;  structure_show, uObj
;;  help, uObj.wObject 
;;  (uObj.wObject) -> print_obj
; endif
;
; if (uname EQ 'where') then begin
;    self -> eventWhere, event=event
; endif else begin
;
;   if (uname EQ 'value') then begin
;     self -> eventValue, event=event
;   endif
;
;   if (uname EQ 'labelMenu') then begin
;     self -> eventlabelMenu, event=event
;   endif
;
; endelse 

end

;==================================================================
;pro cw_ow_where_process, event
;
;;  message, /info, "Entered"
;  
;  widget_control, event.id, get_uvalue=uObj
;
;;  help, uObj
;;  help, uObj.wObject 
;
;  call_method, uObj.method, uObj.wObject
;
;end

;;==================================================================
;function cw_ow_where::fromMenu, $
;	where_select
;
;;  message, /info, "Entered"
;;   help, self
;
;  return,  (self.fromMenu)[where_Select]
;
;end


;==================================================================
pro cw_ow_where::setHeaderMenu, menu
  if (ptr_valid(self.headerMenuPtr)) then ptr_free, self.headerMenuPtr
  self.headerMenuPtr = ptr_new(Menu)
  self.nHeaderMenu = n_elements(Menu)
end

;==================================================================
pro cw_ow_where::setDataMenu, menu
  if (ptr_valid(self.dataMenuPtr)) then ptr_free, self.dataMenuPtr
  self.DataMenuPtr = ptr_new(Menu)
  self.nDataMenu = n_elements(Menu)
end

;==================================================================
pro cw_ow_where::setLabelSensitive, yesNo
;   message, /info, string("yesNo=", yesNo, $
;   	 "self.labelSensitive=", self.labelSensitive)

  self.labelSensitive = yesNo

  if (yesNo) then begin
    widget_control,  self.labelID, sensitive = 1
;    widget_control,  self -> getlabelID(), sensitive = 1
  endif else begin
    widget_control,  self.labelID, sensitive = 0
;    widget_control,  self -> getlabelID(), sensitive = 0
  endelse
end

;==================================================================
pro cw_ow_where::setValueSensitive, yesNo

  if (NOT self.noInput) then begin

    self.ValueSensitive = yesNo

    if (yesNo) then begin
      self.valueObj ->setProperty, nonsensitive= 0
    endif else begin
      self.valueObj ->setProperty, nonsensitive= 1
    endelse
  endif
  
end

;=======================================================================
pro cw_ow_where::cleanup
 
;  message, /info, "Entered"
;   help, self

;Later  self.specObj = obj_new(), $	; don't destroy
  if (self.destroy) then begin 
;    message, /info, "Destroying self.specObj"
;    (self.specObj) -> print_obj
    obj_destroy, self.specObj
  endif else begin
;    message, /info, "Dangling self.specObj"
;    (self.specObj) -> print_obj
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse
  
  ptr_free, self.headerMenuPtr
  ptr_free, self.dataMenuPtr
  obj_destroy, self.valueObj
end


;==================================================================
pro cw_ow_where_cleanup, tlb

    message, /info, "Entered'
;    help, tlb
;    print, tlb
    
;    widget_control, tlb, get_uvalue=self
;
;    help, self
;;    structure_show, self
;    
;    obj_destroy, self		; calls cw_ow_where::cleanup

    widget_control, tlb, get_uvalue=uObj

    obj_destroy, (uObj.wObject)		; calls cw_ow_where::cleanup
    
    ; Problem. Get an error if called from ::init

end

;==================================================================
;*
pro cw_ow_where::freshenWhere, $
	inHeader = thisInHeader, $
	inData = thisInData
;/

;  debug = 0
  
;  if (debug) then print, "Entered", self.labelID, self.ilabel, self.label  

  item_select = (self.fromMenu)[self.ifrom]
  if (item_select EQ 'header') then begin
    label_select = (*self.headerMenuPtr)[self.iLabel]
  endif
  if (item_select EQ 'data') then begin
    label_select = (*self.dataMenuPtr)[self.iLabel]
  endif

;  if (debug) then print, "select=", item_select, label_select
  
  ;---------------------------------------------------------
  if (keyword_set(thisInHeader)) then begin
    nLabels = 0
    (self.specObj)  -> getLabel, $
	/inHeader,  label=LabelMenu, count=nLabels	; all labels
    if (ptr_valid(self.headerMenuPtr)) then ptr_free, self.headerMenuPtr
    self.headerMenuPtr = ptr_new(Labelmenu)
    self.nHeaderMenu = nLabels
    if (item_select EQ 'header') then begin
       ; Re-install the previous selected label (unless of course, it has disappeared.)
       iMenu = where( (label_select EQ (*(self.headerMenuPtr)) ),count)
       if (iMenu  LT 0) then iMenu = 0
       self.ilabel = imenu
       self.label = (*self.headerMenuPtr)[self.iLabel] 
       widget_control,  self.labelID, set_value=LabelMenu
    endif
  endif
  
  ;---------------------------------------------------------
  if (keyword_set(thisInData)) then begin
    nLabels = 0
    (self.specObj)  -> getLabel, $
	/inData, label=Labelmenu, count=nLabels		; all labels
    if (ptr_valid(self.dataMenuPtr)) then ptr_free, self.dataMenuPtr
    self.dataMenuPtr = ptr_new(Labelmenu)
    self.nDataMenu = nLabels
    if (item_select EQ 'data') then begin
	; Re-install the previous selected label (unless of course, it has disappeared.)
       iMenu = where( (label_select EQ (*(self.dataMenuPtr)) ),count)
       if (iMenu  LT 0) then iMenu = 0
       self.ilabel = imenu
       self.label = (*self.dataMenuPtr)[self.iLabel] 
       widget_control,  self.labelID, set_value=LabelMenu
    endif
  endif
  ;---------------------------------------------------------

;  if (debug) then print, "exit", self.labelID, self.ilabel,self.label  

  widget_control, self.labelID, set_droplist_select=self.ilabel

end 

;==================================================================
;*
pro cw_ow_where::getProperty, $
	from = thisFrom, $	; which is sensitized 'input', 'header' or 'data'
	label = thisLabel, $	; Goes to the droplist
	value = thisValue	; Goes to the text entry box
;/

  if (arg_present(thisFrom)) then begin
   self.ifrom = widget_info( self.whereID, /droplist_select)	; refresh
;   message, /info, string("Debug self.whereID=", self.whereID, $
;   		" self.ifrom=", self.ifrom)
   thisFrom = (self.fromMenu)[self.ifrom]
  endif

  if (arg_present(thisLabel)) then begin
;Broken    thisLabel = self.label
    thisLabel = ''
    if (self.inHeader) then begin
      thisLabel = (*(self.headerMenuPtr))[self.ilabel]
    endif
    if (self.inData) then begin
        thisLabel = (*(self.dataMenuPtr))[self.ilabel]
    endif
  endif

  if (arg_present(thisValue)) then begin
    if (NOT self.noINput) then begin
;    if ( self.valueID  NE 0L) then begin
      widget_control, self.valueID, get_value = thisValue	; refresh
      self.value = thisValue
    endif
  endif

end

;==================================================================
;*
pro cw_ow_where::setProperty, $
	from = thisFrom, $	; Changes which is sensitized
	label = thisLabel, $	; Goes to the droplist
	value = thisValue	; Goes to the text entry box
;/

  if (n_elements(thisFrom) GT 0) then begin
    iFrom = where( (thisFrom EQ self.fromMenu),count)
    if (iFrom NE -1) then begin
      self.ifrom = iFrom
      widget_control, self.whereID, set_droplist_select=self.ifrom
    endif else begin
      ifrom = 0    					; Ignore mistakes
    endelse
    if (self.fromMenu[ifrom] EQ 'header') then begin
      self.inHeader = 1
      self.inData = 0
    endif 
    if (self.fromMenu[ifrom] EQ 'data') then begin
      self.inHeader = 0
      self.inData = 1
    endif 
    if (debug) then message, /info, string("self.inHeader=", self.inHeader)
    if (debug) then message, /info, string("self.indata=", self.inData)

  endif

  iMenu = 0
  if (n_elements(thisLabel) GT 0) then begin
    if (self.inHeader) then begin
      iMenu = where( (thisLabel EQ (*(self.headerMenuPtr)) ),count)
    endif
    if (self.inData) then begin
      iMenu = where( (thisLabel EQ (*(self.dataMenuPtr)) ),count)
    endif
    if (debug) then message, /info, string("self.headerMenu", *(self.headerMenuPtr))
    if (debug) then message, /info, string("self.dataMenu", *(self.DataMenuPtr))
    if (debug) then message, /info, string("iMenu=", iMenu)
;    if (iMenu NE -1) then begin
;      self.label = thisLabel
;    endif
  endif 
  
    if (iMenu NE -1) then begin
      self.ilabel = iMenu
;      self.label = thisLabel
    endif else begin
      self.ilabel = 0
    endelse

      if (self.inHeader) then begin
        self.label = (*(self.headerMenuPtr))[self.ilabel]
      endif 
      if (self.inData) then begin
        self.label = (*(self.dataMenuPtr))[self.ilabel]
      endif 
     if (debug) then message, /info, string("self.label=", self.label)
     widget_control, self.labelID, set_droplist_select=self.ilabel
    ; Ignore mistakes


  if (n_elements(thisValue) GT 0) then begin
    if (NOT self.noInput) then begin
;    if (self.valueID GT 0L) then begin
      self.value = thisValue
      widget_control, self.valueID, set_value = thisValue
    endif
  endif

end

;==================================================================
pro cw_ow_where::createWidgets

  debug = 0
  
  infolabel = "cw_ow_where::createWidgets >>"
  if (debug) then message, /info, string("self.inHeader", self.inHeader)
  if (debug) then message, /info, string("self.inData", self.inData)
  if (strlen(self.label) GT 0) then begin
     self.iLabel = 0
     if (self.fromMenu[self.ifrom] NE 'input')  then begin ; 'header' or 'data'
;     if (self.iFrom NE 0) then begin		; 'header' or 'data'
 	if (self.inHeader) then self.label_header = self.label
	if (self.inData) then self.label_data = self.Label
        self.iLabel  = (self.specObj) -> getPosition( ['*', self.label, '*'],$
     	   inHeader=self.inHeader, inData=self.inData)
     endif 
  endif else begin
    if (self.fromMenu[self.ifrom] NE 'input')  then begin ; 'header' or 'data'
;    if (self.iFrom NE 0) then begin			; 'header' or 'data'
      ; Use the first item (zeroth) in the list.
      OK = self.SpecObj -> getTagField(0, label = self.Label, $
   		inHeader = self.inHeader, inData = self.inData)
      if (self.inHeader) then self.label_header = self.label
      if (self.inData) then self.label_data = self.label
      self.iLabel = 0
    endif 
  endelse
  if (debug) then message, /info, string("self.label=", self.label)
  if (debug) then message, /info, string("self.label_header=", self.label_header)
  if (debug) then message, /info, string("self.label_data=", self.label_data)
  if (debug) then message, /info, string("self.ilabel=", self.ilabel)
  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_where_TLB", $
	title = 'cw_ow_where title. No parent.' , $
;	uvalue=self,$			; needed for kill, cleanup
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_where_event',$
;	kill_notify='', $
	/column)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_where_TLB", $
	title = 'cw_ow_where title.' , $
;	uvalue=self,$			; needed for kill, cleanup
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_where_event',$
;	kill_notify='', $
	/column)
 endelse
 
 self.tlbID = tlbID
;Unneeded widget_control, tlbID, set_uvalue = uvalue=self		;; {wObject:self}

 ;---------------------------------------------------
   whereID = widget_droplist(tlbID, $
   	value = self.fromMenu, $
	uname = 'where', $
	uvalue = {wobject:self, method:'eventWhere'})
   self.whereID = whereID
   widget_control, whereID, set_droplist_select=self.ifrom

  ;---------------------------------------------------------
  
  ;---------------------------------------------------------
  labelMenu = [' ', ' ', ' ']	; junk
  if (self.inHeader) then begin
 ; inWhere = where( strmatch(self.fromMenu, 'header') EQ 1)
 ; if (inWhere NE -1) then begin
    nLabels = 0
    (self.specObj)  -> getLabel, $
	/inHeader,  label=LabelMenu, count=nLabels	; all labels
    if (ptr_valid(self.headerMenuPtr)) then ptr_free, self.headerMenuPtr
    self.headerMenuPtr = ptr_new(Labelmenu)
    self.nHeaderMenu = nLabels
    if (debug) then print, infolabel , "Header LabelMenu=",Labelmenu[0:1]
  endif 
  
  if (self.inData) then begin
;  inWhere = where( strmatch(self.fromMenu, 'data') EQ 1)
;  if (inWhere NE -1) then begin
    nLabels = 0
    (self.specObj)  -> getLabel, $
	/inData, label=Labelmenu, count=nLabels		; all labels
    if (ptr_valid(self.dataMenuPtr)) then ptr_free, self.dataMenuPtr
    self.dataMenuPtr = ptr_new(Labelmenu)
    self.nDataMenu = nLabels
    if (debug) then print, infolabel, "Data LabelMenu=",Labelmenu[0:1]
  endif
  ;---------------------------------------------------------
  
  if (debug) then print, infolabel, self.ilabel, " LabelMenu=",Labelmenu[0:1]
  labelID = widget_droplist(tlbID, $
  	value= labelMenu, $
	uname = 'labelMenu', $
	uvalue = {wobject:self, method:'eventLabelMenu'}, $
	sensitive = self.labelSensitive)
;	event_pro='cw_so_where_event', $
;	/context_events, sensitive=thisSensitive) 
   self.labelID = labelID
   widget_control, labelID, set_droplist_select=self.iLabel

  ;-------------------------------------------------------------

;    iFromInput = where( ('from'  EQ self.fromMenu),count)
;    if (count EQ 1) then begin
     if (NOT self.noInput) then begin
      ; Only make input window if 'from' is in the list self.fromMenu 
      valueID = fsc_field(tlbID, $		; allows sensitive
        object=valueObj, $
	uname = 'value', $
   	title = "", $
   	value = self.value, $
	uvalue = {wobject:self, method:'eventValue'}, $
	/float, $
	/cr_only, event_pro='cw_ow_where_event',$
	sensitive=self.valueSensitive)	
     self.valueID = valueID
     self.valueObj = valueObj
   endif
   
   ;-------------------------------------------------------------

  if (self.ParentID EQ 0L) then begin
;    message, /info, "Make a quit button"
    void = widget_button(self.tlbID, $
    	value='DISMISS', $
;    	value='QUIT', $
    	uvalue={wobject:self, method:'eventQuit'})
;		event_pro="cw_ow_where_event")
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_where',  tlbID
;;;;    , $ cleanup =  'cw_ow_where_cleanup'
    
  endif

;  if (debug) then message, /info, string("Exiting. self.ParentID=", self.ParentID)
;  self -> print_obj, /alsoPtrs

;   self -> help_obj
;   self -> print_obj
;   
;   tempStruct = {wobject:self, method:'QUIT'}
;   structure_show, tempStruct, /full
   
end		;;  cw_ow_where::createWidgets

;==================================================================
;*
function cw_ow_where::init, $
	thisSpecObj, $		; required
	thisParentID, $		; optional
	from = thisFrom, $	; optional. defaults to 'input'
				; Acceptible values: 'input', 'header', 'data'
	field = thisField, $	; Optional pre-selection. 
	destroy = thisDestroy, $ ; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
	noInput = thisNoInput, $   ; Don't have box to input a value
	noData = thisNoData     ; Don't allow info from Data section
;/ 
	
  debug = 0

;   message, /info, 'Does self have a valid heap address at the begining of INIT?'
;   help, self
  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)
 

;  if (debug) then message, /info, "Entered"
;  self -> print_obj, /alsoPtrs
  

  ;-------------------------------------------------------------
  if (keyword_set(thisNoInput))  then begin
    self.noInput = 1
    if (keyword_set(thisNoData))  then begin
      self.fromMenu = ['header' ]
    endif else begin
      self.fromMenu = ['header', 'data' ]
    endelse
  endif else begin
    self.noInput = 0
;    self.fromMenu = ['input', 'header', 'data' ]
    if (keyword_set(thisNoData))  then begin
      self.fromMenu = ['input', 'header' ]
    endif else begin
      self.fromMenu = ['input', 'header', 'data' ]
    endelse
  endelse  

  showlen = n_elements(self.fromMenu)
;;; fromMenu = ['input', 'header', 'data' ]

  iFrom = 0
  if (n_elements(thisFrom) GT 0) then begin
    iFrom = where( (thisFrom EQ self.fromMenu),count)
    if (count EQ 0) then begin
      iFrom = 0
      message, /info, string("Unrecognized value for keyword from=", thisFrom)
    endif ;else begin
;      self.from = thisFrom
;    endelse
  endif
  self.iFrom = ifrom
  
  if (self.fromMenu[iFrom] EQ 'input') then begin	; 'input'
    self.valueSensitive = 1
  endif else begin	; 'header' or 'data'
    self.labelSensitive = 1
    if (self.fromMenu[ifrom] EQ 'header') then begin
      self.inHeader = 1
    endif 
    if (self.fromMenu[ifrom] EQ 'data') then begin
      self.inData = 1
    endif 
  endelse
  ;-------------------------------------------------------------
  
  if (n_elements(thisField) EQ 0) then begin
    thisField = ''
  endif
  if (strlen(thisField) GT 0) then begin
     if (self.fromMenu[self.ifrom] NE 'input')  then begin ; 'header' or 'data'
;     if (self.iFrom NE 0) then begin	; 'header' or 'data'
        self.Label = thisField
	if (debug) then message, /info, string("self.label=", thisField)
     endif else begin			; 'input'
        self.Value = thisField
     endelse 
  endif 
  
  ;-------------------------------------------------------------

  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_where__define
  define = {cw_ow_where, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	parentID:0l, $
	tlbID:0l, $
	fromMenu:['', '', ''], $	;['input', 'header', 'data' ]
	ifrom: 0, $
	whereID:0L, $
;;;	whereSensitive: 0, $
	inHeader:0, $
	inData:0, $
	headerMenuPtr: ptr_new(), $
	nHeaderMenu: 0, $
	dataMenuPtr: ptr_new(), $
	nDataMenu: 0, $
	tags:['', '', ''], $
	ilabel: 0, $			;integer position. Doesn't work
	label_header: '', $		; Improvement in future
	label_data: '', $		; Improvement in future
	label:'', $			; 1 string. Doesn't work
	labelID:0L, $
	labelSensitive: 0, $
	noInput: 0, $
	value:0.0, $
	valueID:0L, $
	valueObj: obj_new(), $
	valueSensitive:0 $
	}
;/

end
