; $Id: davesave.pro,v 1.4 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
;       davesave
;
; PURPOSE:
;
;	This is the DAVE program button which does the functions
; OPEN/SAVE/REVIEW/CLOSE for DAVE data save files.
;
; AUTHOR:
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       DAVE application button.
;
; SEE ALSO:
; 	<LI><A HREF="#LAUNCH_DAVESAVE">LAUNCH_DAVESAVE</A>
; 	<LI><A HREF="#DAVEPTR_UTILS">DAVEPTR_UTILS</A>
;
; MODIFICATION HISTORY:
;	Tanya Riseman. Aug 6, 2003.
;	TMR. Aug 29, 2003. 
;		Update so_filename field when saving to disk in davesave_save.
;-



;===============================================================================
pro davesave_cleanup, tlbID

  message, /info, 'Cleaning up...'
  widget_control, tlbID, get_uvalue=sPtr

  ;---- BEGIN DAVE specific cleanup section
  icpInfo = {davesave_EVENT,$
          id:(*sPtr).notifyIDs[0],$
          top:(*sPtr).notifyIDs[1],$
          handler:0L}
  if widget_info((*sPtr).notifyIDs[0],/valid_id) then begin
;    message, /info, "Changing GUI focus back to main DAVE program"
    widget_control,(*sPtr).notifyIDs[0],send_event = icpInfo
  endif
  ;---- END DAVE specific section

  ; deal with state pointer cleanup
  ptr_free, sPtr

end



;;===============================================================================
;pro reduce_davesave_event,event
;
;uname = widget_info(event.id,/uname)
;
;case uname of
;    'DISMISS': widget_control,event.top,/destroy
;    else:
;endcase
;
;end



;===============================================================================
pro davesave_open, $
	davePtr, $
	notifyIds=notifyIds, $
	group_leader=group_leader

  compile_opt idl2

  ;----------------------------------------------- 
  ; file dialog for selecting files
  title = 'OPEN: DAVE save file selection'

  datDir = (*!dave_defaults).datdir         ; get current data directory from DAVE env var

  cd, datDir[0], cur=curDir       ; cd to fans data dir
  ;----------------------------------------------------
  ; state variable...
  state = { $
        davePtr:davePtr, $      ; the davePtr
        notifyIds:notifyIds, $	; event structure used to communicate with dave.pro through launchicp.pro
	action: 'davesave_open' $
        }
  sPtr = ptr_new(state)


  ;----------------------------------------------------
  allowed_extensions = ['*.dave']

  long_fileList = dialog_pickfile(title=title,$
  	get_path=newDir, $
        filter=allowed_extensions, $
	/must_exist,dialog_parent=group_leader)

  ;cd, curDir                      ; go back to previous dir

  nfiles = (strtrim(long_fileList[0],2) eq '') ? 0 : n_elements(long_fileList)

  if (nfiles le 0) then begin
      message, /info, "No file to open"
      ; return control to main DAVE
      widget_control, notifyIDs[0], set_uvalue=sPtr
      davesave_cleanup, notifyIDs[0]
    return     ; no need to carry on!
  endif

  message, /info, string("orig davePtr ID=", davePtr)	; debug
  ;----------------------------------------------------
  ; Assuming that (nfiles EQ 1)
  davePtr_restore, davePtr, (long_fileList[0]), ownerDavePtr = 'main'

  message, /info, string("OPENED file=", (long_fileList[0]))	; debug
;  davePtr_show, davePtr		; debug
  message, /info, string("new davePtr ID=", davePtr)	; debug
  ;----------------------------------------------------
  widget_control, notifyIDs[0], set_uvalue=sPtr
  davesave_cleanup, notifyIDs[0]

end

;===============================================================================
pro davesave_save, $
	davePtr, $
	notifyIds=notifyIds, $
	group_leader=group_leader

  compile_opt idl2

  ;----------------------------------------------- 
  ; file dialog for selecting files
  title = 'SAVE: DAVE save file selection'

  datDir = (*!dave_defaults).datdir         ; get current data directory from DAVE env var

  cd, datDir[0], cur=curDir       ; cd to fans data dir
  ;----------------------------------------------------
  ; state variable...
  state = { $
        davePtr:davePtr, $      ; the davePtr
        notifyIds:notifyIds, $	; event structure used to communicate with dave.pro through launchicp.pro
	action: 'davesave_save' $
        }
  sPtr = ptr_new(state)


  ;----------------------------------------------------
;  suggestedName = ''	; variable suggestedName is undefined by default
  specObjExists = DavePtr_containsSpecObj(DavePtr)
  if (specObjExists) then begin
    ; Up date the stored name with the save file name.
    tempSpecObj = davePtr_getSpecObj(thisDavePtr, from='davesave_save')
    OK = (tempSpecObj) -> getField(['*', 'so_filename', '*'], /inHeader, $
	qty = suggestedName)
    periodPos = strpos( suggestedName, '.')
    tempName = suggestedName
    ; Won't work for "hidden" files starting with "."
    if (periodPos GE 1) then begin
      tempname = strmid(suggestedName, 0, periodPos) + '_' $
      	+ strmid(suggestedName, (periodPos+1),  /reverse_offset)
;      tempname = strmid(suggestedName, 0, (periodPos-2)) + '_'  $
;      	+ strmid(suggestedName, (periodPos), strlen(suggestedName))
;Wrong      suggestedName[periodPos:periodPos] = '_'	
;      	; instrument string .bt2 now _bt2.
    endif
;    if (periodPos GE 0) then begin
;    suggestedName[periodPos:periodPos] = '_'
;    		; instrument string .bt2 now _bt2.
;    endif
;    suggestedName = suggestedName + '.dave'	; new suffix
    suggestedName = tempName + '.dave'	; new suffix


;    OK = OK AND (tempSpecObj) -> setField(['*', 'so_path', '*'], /inHeader, $
;	qty = (*!dave_defaults).workdir)
;    OK = OK AND (tempSpecObj) -> setField(['*', 'so_filename', '*'], /inHeader, $
;	qty = suggestedName)

    message, /info, string("suggestedpath = ", (*!dave_defaults).workdir)
    message, /info,  string("suggestedName = ", suggestedName )
    print, "suggestedName = ", suggestedName 
	
  endif
  

;  message, /info, "suggestedName = ", suggestedName 
;  print, "suggestedName = ", suggestedName 
  ;----------------------------------------------------
  allowed_extensions = ['*.dave']

  long_fileList = dialog_pickfile(title=title,$
  	get_path=newDir, $
        filter=allowed_extensions, $
	file = suggestedName, $
	dialog_parent=group_leader)

  ;cd, curDir                      ; go back to previous dir

  nfiles = (strtrim(long_fileList[0],2) eq '') ? 0 : n_elements(long_fileList)
  
  if (nfiles le 0) then begin
      message, /info, "Error saving file."
      ; return control to main DAVE
      widget_control, notifyIDs[0], set_uvalue=sPtr
      davesave_cleanup, notifyIDs[0]
    return     ; no need to carry on!
  endif

  ;----------------------------------------------------
  if (specObjExists) then begin
    ; Up date the stored name and path with the save file name.
    OK = (tempSpecObj) -> setField(['*', 'so_path', '*'], /inHeader, $
	qty = newdir)
    pos = strpos(long_fileList[0],!delimiter,/reverse_search) + 1
    length = strlen(long_fileList[0]) - pos
    short_file = strmid(long_fileList[0],pos,length)
    OK = (tempSpecObj) -> setField(['*', 'so_filename', '*'], /inHeader, $
	qty = short_fileList)
  endif
  
  ;----------------------------------------------------
  ; Assuming that (nfiles EQ 1)
  davePtr_save, davePtr, file=(long_fileList[0])

  ;----------------------------------------------------
  widget_control, notifyIDs[0], set_uvalue=sPtr
  davesave_cleanup, notifyIDs[0]

end

;===============================================================================
pro davesave_review, $
	davePtr, $
	notifyIds=notifyIds, $
	group_leader=group_leader

  compile_opt idl2

  state = { $
        davePtr:davePtr, $      ; the davePtr
        notifyIds:notifyIds, $	; event structure used to communicate with dave.pro through launchicp.pro
	action: 'davesave_review' $
        }
  sPtr = ptr_new(state)


  ;= similiar to =================================================================
  ; pro dialog_ow_menu::eventViewDavePtr, event=event

  newFile = 'temporary_davesave_review.txt'	; temporary file

  davePtr_show, DavePtr, file=newFile, $
  	/partial
;	/full

  ; Do we need to free tempDavePtr?

    
;  textGUI = obj_new('dialog_ow_textInfo', $
;	/read, $
;	text_ptr= thisText_ptr, $
;	filter = thisFilter, $
;	path= thisPath, $
;	filename=thisFilename, $
;	groupLeader = thisGroupLeader, $
;	destroy = thisDestroy	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.

 xdisplayfile, newFile, $
  	/block, $
  	group = group_Leader, $
	title = "Review Dave data contents"

  ;----------------------------------------------------
  widget_control, notifyIDs[0], set_uvalue=sPtr
  davesave_cleanup, notifyIDs[0]

end

;===============================================================================
pro davesave_empty, $		: empty out Dave data structure.
	davePtr, $
	notifyIds=notifyIds, $
	group_leader=group_leader

  compile_opt idl2

  state = { $
        davePtr:davePtr, $      ; the davePtr
        notifyIds:notifyIds, $	; event structure used to communicate with dave.pro through launchicp.pro
	action: 'davesave_empty' $
        }
  sPtr = ptr_new(state)


  DavePtr_empty, DavePtr
;  if (davePtr_containsSpecObj(DavePtr)) then begin
;    if (!debug) then begin 
;      message, /info, "obj_destroy, specObj	;, /skipDavePtr"
;    endif
;	  ;heap_free doesn't destroy the object OK. 
;	  tempObj = davePtr_getSpecObj(DavePtr, from='davesave_empty')
;	  if (!debug) then message, /info, string(tempObj, "destroying specObj")
;	  obj_destroy, tempObj	; , /skipDavePtr
;  endif
;  
;  ; Too brutal.
; ;  heap_free, (*DavePtr)  	; Empty out the contents! Will be replaced.;;
;
;  ; The official way from DAVE.
;  heap_free, (*DavePtr).descriPtr
;  heap_free, ((*((*DavePtr).dataStrPtr)).commonStr)
;  heap_free, ((*((*DavePtr).dataStrPtr)).specificPtr)
;    
  ;----------------------------------------------------
  widget_control, notifyIDs[0], set_uvalue=sPtr
  davesave_cleanup, notifyIDs[0]

end

