; $Id: dialog_ow_tabicp.pro,v 1.4 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
; 	dialog_ow_tabicp
;
; PURPOSE:
;	Menu for modifying a specObj in ways that are instrument
; 	specific, in particular ICP-based triple axis.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE GUI main dialog routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created June, 2003. Tanya Riseman
;-
; dialog_ow_tabICP.pro

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	dialog_ow_tabICP::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro dialog_ow_tabICP::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	dialog_ow_tabICP::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro dialog_ow_tabICP::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	dialog_ow_tabICP::helpclass
;
; PURPOSE:
;	Helper method for dialog_ow_tabICP::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro dialog_ow_tabICP::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass

;==================================================================
;+
; NAME:
; 	dialog_ow_tabICP::freshen
;
; PURPOSE:
;	The compound widgets cw_ow_*.pro used by dialog_ow_tabICP can create 
; new fields, or remove fields in the specific object self.specObj. 
; In order to update the droplists for all compound widgets, the 
; cw_ow* widgets call this method dialog_ow_tabICP::freshen, which then 
; calls the methods cw_ow*::freshenWhere in all the compound widgets used 
; in dialog_ow_tabICP. 
;-
;*
pro dialog_ow_tabICP::freshen, $
	where = thisWhere, $		; Update all calls to cw_ow_where
	inHeader = thisInHeader, $	; Used by where
	inData = thisInData, $		; used by where
	status = thisStatus		; Update the status line when "process"
;/
	if (keyword_set(thisWhere)) then begin
	  self.scaleByMonitorWObj -> freshenWhere, $
	  	inHeader=thisInHeader, inData =thisInData
	  self.constBkgdWObj -> freshenWhere, $
	  	inHeader=thisInHeader, inData =thisInData
	  self.fastBkgdWObj -> freshenWhere, $
	  	inHeader=thisInHeader, inData =thisInData
	  self.resVolWObj -> freshenWhere, $
	  	inHeader=thisInHeader, inData =thisInData
	endif

	if (n_elements(thisStatus) GT 0) then begin
 	  widget_control, self.statusID, set_value=thisStatus
	endif
end


;==================================================================
;*
pro dialog_ow_tabICP_event, $
	event
;/

;;  widget_control, event.id, get_uvalue=uObj   
  widget_control, event.top, get_uvalue=uObj 
;  help, uObj


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event_tlb') then begin
      ; Ignore base events.
 	 uname = widget_info(event.id, /uname)
 	 wname = widget_info(event.id, /name)
;  	 message, /info, string("uname =", uname, " wname=", wname)
	if (uname EQ 'eventQuit') then begin
	  call_method, uname,  (uObj.wObject), event = event
;          message, /info, 'call_method, uname,  (uObj.wObject), event = event'
	endif else begin
;          message, /info, 'Ignoring event_tlb'
	endelse
;;;;      (uObj.wObject) -> event, event
    endif else begin
        message, /info, 'Calling method uObj.method'    
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; dialog_ow_tabICP_event, event
;==================================================================
pro dialog_ow_tabICP::eventQuit, event=event
;  message, /info, '...'
;  print, event
  widget_control, self.tlbID, /destroy
  
end

;==================================================================
;;*
;pro dialog_ow_tabICP::event, $
;	event
;;/
;
;  uname = widget_info(event.id, /uname)
;  wname = widget_info(event.id, /name)
;
;  ;------------------------------------------
;   message, /info, string("Should not be here. uname =", uname, " wname=", wname)
;
;end

;=======================================================================
pro dialog_ow_tabICP::cleanup
 
  if (self.destroy) then begin 
;    message, /info, "Destroying self.specObj"
    obj_destroy, self.specObj
  endif else begin
;    message, /info, "Dangling self.specObj"
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse

  obj_destroy, [self.scaleByMonitorWObj, $
  	self.constBkgdWObj, self.fastBkgdWObj, self.resVolWObj]
  
end

;==================================================================
pro dialog_ow_tabICP_cleanup, tlb

  message, /info, "Entered'

  widget_control, tlb, get_uvalue=uObj
  if ((uObj.wObject) -> no_block()) then begin	
    obj_destroy, (uObj.wObject)		; calls dialog_ow_tabICP::cleanup
  endif
end


;==================================================================
function dialog_ow_tabICP::no_block	; return 0/1 for no/yes
  return, self.no_block
end

;==================================================================
function dialog_ow_tabICP::createWidgets	; return 0/1 for failure/success

 ;---------------------------------------------------  
  if (self.groupLeaderID EQ 0L) then begin
     floating = 0
  endif else begin
    floating = 1
  endelse

 ;-------------------------------------------------------------  
  OK = self.SpecObj -> getField(['*', 'so_filename', '*'], /inHeader, qty=dataFile)
  if (OK) then begin
    name = datafile
  endif else begin
    name = ''
  endelse

  aTitle = 'dialog_ow_tabICP' + ' ' + Name

;Problem with the event handler name with this fanciness.
;  if (strlen(Name) GT 0) then begin
;    registerName = 'dialog_ow_tabICP' + '_' + Name		;;  $
;;;;    		+ '_' + strtrim(string(self.groupleaderID), 2)
;  endif else begin
;    registerName = 'dialog_ow_tabICP' ;;+ '_' + strtrim(string(self.groupleaderID), 2)
;  endelse
;
;  if (xregistered(registerName)) then begin
;    message, /info, string(registerName,  $
;    	' Already registered. Only single instance allowed.')
;    return, 0
;  endif else begin
;    message, /info,  string(registerName,  ' just registered.')
;  endelse
;  self.registername = registerName

 ;---------------------------------------------------  
  if ((self.No_block) OR (self.groupLeaderID EQ 0L)) then begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
  	uname = "dialog_ow_tabICP", $
  	title = aTitle, $	
;  	title = self.registername, $	
;	event_pro='dialog_ow_tabICP_event',$
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endif else begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
    	/modal, $
  	uname = "dialog_ow_tabICP", $
  	title = aTitle, $	
;  	title = self.registername, $	
;	event_pro='dialog_ow_tabICP_event',$
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endelse
;  tlbID = widget_base(group_leader=group_leader, floating=floating, $
;  	uname = "dialog_ow_tabICP", $
;  	title = self.registername, $	
;	event_pro='dialog_ow_tabICP_event',$
;	uvalue={wobject:self, method:'event_tlb'}, $
;  	/column)
  self.tlbID = tlbID

  
;  wTab = widget_tab(tlbID, uname = 'tabs')
  holderHorizID0 =widget_base(tlbID, /row)
  wTab = widget_tab(holderHorizID0, uname = 'tabs')

  ;-------------------------------------------------------------
  scaleByMonitorID = widget_base(wTab, /row, title='scaleByMonitor',$
      	uname = 'scaleByMonitor')	;, $
  self.scaleByMonitorWObj = obj_new('cw_ow_scaleByMonitor', $
	self.specObj, $
	scaleByMonitorID, $
    	parentWObj = self)

  ;-------------------------------------------------------------
  constBkgdID = widget_base(wTab, /row, title='constBkgd',$
      	uname = 'constBkgd')	;, $
  self.constBkgdWObj = obj_new('cw_ow_constBkgd', $
	self.specObj, $
	constBkgdID, $
    	parentWObj = self)

  ;-------------------------------------------------------------
  fastBkgdID = widget_base(wTab, /row, title='fastBkgd',$
      	uname = 'fastBkgd')	;, $
  self.fastBkgdWObj = obj_new('cw_ow_fastBkgd', $
	self.specObj, $
	fastBkgdID, $
    	parentWObj = self)

  ;-------------------------------------------------------------
  resVolID = widget_base(wTab, /row, title='resVol',$
      	uname = 'fastBkgd')	;, $
  self.resVolWObj = obj_new('cw_ow_resVol', $
	self.specObj, $
	resVolID, $
    	parentWObj = self)

; ;-------------------------------------------------------------
;  binID = widget_base(wTab, /row, title='z=x op y',$
;      	uname = 'binaryOp') ;, $
;;  	uvalue={wobject:self, method:'event_modify'})
;  self.binaryOpWObj = obj_new('cw_ow_binaryOp', self.specObj, binID, $ ;tlbID, $
;    	parentWObj = self)
; 
;  ;-------------------------------------------------------------
;  unID = widget_base(wTab, /row, title='z=f(x)',$
;      	uname = 'unitaryOp')	;, $
;;  	uvalue={wobject:self, method:'event_modify'})
;  self.unitaryOpWObj = obj_new('cw_ow_unitaryOp', self.specObj, unID, $ ;tlbID, $
;    	parentWObj = self)
; 
;  ;-------------------------------------------------------------
;  cloneID = widget_base(wTab, /row, title='z=clone(x)',$
;      	uname = 'clone')	;, $
;;  	uvalue={wobject:self, method:'event_modify'})
;;   saveSpaceID = widget_base(tlbID, /row)
;    self.cloneWObj = obj_new('cw_ow_clone', self.specObj, cloneID, $	;saveSpaceID, $
;    	parentWObj = self)
; 
;  ;-------------------------------------------------------------
;  setStringID = widget_base(wTab, /row, title='setString(x)',$
;      	uname = 'setString')	;, $
;    self.setStringWObj = obj_new('cw_ow_setString', self.specObj, setStringID, $	;saveSpaceID, $
;    	parentWObj = self)
; 
;  ;-------------------------------------------------------------
;  setValueID = widget_base(wTab, /row, title='setValue(x)',$
;      	uname = 'setValue')	;, $
;    self.setValueWObj = obj_new('cw_ow_setValue', self.specObj, setValueID, $	;saveSpaceID, $
;    	parentWObj = self)
; 
;    ;-------------------------------------------------------------
;    removeID = widget_base(wTab, /row, title='remove(x)',$
;      	uname = 'remove')	;, $
;;  	uvalue={wobject:self, method:'event_modify'})
;    self.removeWObj = obj_new('cw_ow_remove', self.specObj, removeID, $;saveSpaceID, $
;    	parentWObj = self)
; 

  ;-------------------------------------------------------------
;;;  self -> freshenResults
  ;-------------------------------------------------------------

  ;-------------------------------------------------------------
;    voidID = widget_button(tlbID, value='QUIT', $
    voidID = widget_button(holderHorizID0, value='QUIT', $
    	uname = 'eventQuit', $
	event_pro='dialog_ow_tabICP_event',$
    	uvalue={wobject:self, method:'eventQuit'})

  ;-------------------------------------------------------------
    holderHorizID1 =widget_base(tlbID, /row)
;    holderHorizID1 = tlbID
    voidID = widget_label(holderHorizID1, value='Status:', $
	units=2, $	; cm
    	scr_xsize = 5)
    
    statusID = widget_text(holderHorizID1, value='', $
    	uname = 'eventStatus', $
	/scroll, $
    	xsize = 80, $		; characters
	ysize = 5, $		; lines
;	units=2, $	; cm
;	scr_xsize = 20, $
;	scr_ysize = 2, $
	event_pro='dialog_ow_tabICP_event',$
    	uvalue={wobject:self, method:'eventStatus'})
    self.statusID = statusID

  ;-------------------------------------------------------------
;    holderHorizID2 = tlbID
    holderHorizID2 =widget_base(tlbID, /row)
    voidID = widget_label(holderHorizID2, value='Specific Object Macro:', $
	units=2, $	; cm
    	scr_xsize = 5)

    macroID = widget_text(holderHorizID2, value='', $
    	uname = 'eventMacro', $
	/editable, $
	/scroll, $
   	xsize = 80, $		; characters
	ysize = 5, $		; lines
;	units=2, $	; cm
;	scr_xsize = 20, $
;	scr_ysize = 5, $
	event_pro='dialog_ow_tabICP_event',$
    	uvalue={wobject:self, method:'eventMacro'})
    self.macroID = macroID
 
  ;-------------------------------------------------------------
    widget_control,tlbID,/realize		; make room for all these things.
  ;-------------------------------------------------------------
    if (self.No_block) then begin
      xmanager, 'dialog_ow_tabICP', /no_block, tlbID, $
    	cleanup="dialog_ow_tabICP_cleanup"
    endif else begin
      xmanager, 'dialog_ow_tabICP',  tlbID, $
    	cleanup="dialog_ow_tabICP_cleanup"
    endelse

;   message, /info, "After xmanager"	; Don't see if it is a blocking widget

  return, 1  
end		;;  dialog_ow_tabICP::createWidgets


;==================================================================
;+
; NAME:
; 	dialog_ow_tabICP::init
;
; PURPOSE:
;	The compound widgets cw_ow_*.pro used by dialog_ow_tabICP can create 
; new fields, or remove fields in the specific object self.specObj. 
; In order to update the droplists for all compound widgets, the 
; cw_ow* widgets call this method dialog_ow_tabICP::freshen, which then 
; calls the methods cw_ow*::freshenWhere in all the compound widgets used 
; in dialog_ow_tabICP. 
;
; 	If the keyword no_block is set, the widget is non-blocking and
; automatically the object of class dialog_ow_tabICP is destroyed upon 
; closing the dialog box. If not set, the programmer must obj_destroy 
; the object upon return from init. Don't use /no_block
; simulataneously with dialog boxes which depend on the
; contents of the specific object.
;-
;*
function dialog_ow_tabICP::init, $
	thisSpecObj, $		; required
	groupLeader = thisGroupLeader, $
	destroy = thisDestroy	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
;/
;	parentWobj = thisParentWObj, $	; required for freshening where
	
  debug = 0	; 1

  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)

  ;-------------------------------------------------------------
  if (keyword_set(thisNo_block)) then begin
    self.no_block = thisNo_block
  endif
 
;  ;-------------------------------------------------------------
;  if (obj_valid(thisParentWobj)) then begin
;    self.parentWobj = thisParentWobj
;  endif 
  ;-------------------------------------------------------------
  if (n_elements(thisGroupLeader) NE 0) then begin
    self.groupLeaderID = thisGroupLeader
  endif else begin
     ; Must have a group_leader with blocking widget
     self.no_block = 0
  endelse 
  ;-------------------------------------------------------------

  OK = self -> createWidgets()
  
;Debug  self -> help_obj
  
;Debug  self -> print_obj
  
  return, OK
  
end

;==================================================================
;*
pro dialog_ow_tabICP__define
  define = {dialog_ow_tabICP, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	groupLeaderID : 0L, $
	no_block: 0, $
	tlbID:0l, $
	scaleByMonitorWObj: obj_new(), $
	constBkgdWObj: obj_new(), $
	fastBkgdWObj: obj_new(), $
	resVolWObj: obj_new(), $
	statusID: 0L, $
	macroID: 0L $
	}
;/
;	registername: '', $
;	parentID:0l, $
;	parentWObj: obj_new(), $

end
