; $Id: dialog_ow_where.pro,v 1.4 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
; 	dialog_ow_where
;
; PURPOSE:
;	Very simple dialog box that only calls cw_ow_where()
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE GUI main dialog routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created Sept, 2003. Tanya Riseman
;	TMR. Sept 15, 2003. Removed call to createWidgets method.
;	Must be done on exterior. 
;-
; dialog_ow_where.pro

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	dialog_ow_where::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro dialog_ow_where::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	dialog_ow_where::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro dialog_ow_where::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	dialog_ow_where::helpclass
;
; PURPOSE:
;	Helper method for dialog_ow_where::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro dialog_ow_where::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass
;==================================================================
;*
function dialog_ow_where::get, $	; return value of class field as selected by keyword
  selectedLabel = thisSelectedLabel, $
;  selectedField = thisSelectedField, $
  from = thisFrom, $
  groupLeader = boolGroupLeader, $
  no_block = boolNo_block
;/

  if (keyword_set(thisSelectedLabel)) then return, self.selectedLabel
;  if (keyword_set(thisSelectedField)) then return, self.selectedField
  if (keyword_set(thisFrom)) then return, self.from
  if (keyword_set(boolGroupLeader)) then return, self.GroupLeaderID
  if (keyword_set(boolNo_block)) then return, self.no_block
  
  return, 0		; error

end

;==================================================================
;+
; NAME:
; 	dialog_ow_where::freshen
;
; PURPOSE:
;	The compound widgets cw_ow_*.pro used by dialog_ow_where can create 
; new fields, or remove fields in the specific object self.specObj. 
; In order to update the droplists for all compound widgets, the 
; cw_ow* widgets call this method dialog_ow_where::freshen, which then 
; calls the methods cw_ow*::freshenWhere in all the compound widgets used 
; in dialog_ow_where. 
;-
;*
pro dialog_ow_where::freshen, $
	where = thisWhere, $		; Update all calls to cw_ow_where
	inHeader = thisInHeader, $	; Used by where
	inData = thisInData		; used by where
;/

	if (keyword_set(thisWhere)) then begin
	  self.whereWObj -> freshenWhere, $
	  	inHeader=thisInHeader, inData =thisInData
	endif

	
end


;==================================================================
;*
pro dialog_ow_where::eventProcess, event=event
;/

  message, /info, "Entered"
  
   (self.whereWObj) -> getProperty, $
	from = From1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1, $	; Goes to the droplist
	value = value1	; Goes to the text entry box

   inHeader = 0
   inData = 0
     tags1 = ['*', label1, '*'] 
     if (from1 EQ 'header') then begin
       inHeader = 1
     endif
     if (from1 EQ 'data') then begin
       inData = 1
     endif      
   
    message, /info, string("Tag = ", format_list(['qtyObj', Label1, '*']))
    
    OK = self.specObj -> getTagField(['qtyObj', Label1, '*'], $
   	inData = inData, inHeader = inHeader, $
	tags = newTags)
  
    if (OK) then begin
      if (self.xyzFocus EQ 'x') then begin
        self.specObj -> setFocus, xtags = newTags
      endif
      if (self.xyzFocus EQ 'y') then begin
        self.specObj -> setFocus, ytags = newTags
      endif
      if (self.xyzFocus EQ 'z') then begin
        self.specObj -> setFocus, ztags = newTags
      endif
      message, /info, string(OK, "= OK. Setting ", self.xyzFocus, $
      	"tags =", format_list( newTags))
    endif else begin
      message, /info, string(OK, "= OK. Failed.")
    endelse
    
     
;   OK = (self.SpecObj) -> setField($
;   	tags1, $
;     	inHeader=inHeader, inData=inData, $
;	name = resName, $
;	units = resUnits)
	 	  	  
end

;==================================================================
;*
pro dialog_ow_where_event, $
	event
;/

  widget_control, event.top, get_uvalue=uObj 


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event_tlb') then begin
      ; Ignore base events.
 	 uname = widget_info(event.id, /uname)
 	 wname = widget_info(event.id, /name)
;	if (uname EQ 'eventQuit') then begin
;	  call_method, uname,  (uObj.wObject), event = event
;	endif
	call_method, uname,  (uObj.wObject), event = event
    endif else begin
        message, /info, 'Calling method uObj.method'    
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; dialog_ow_where_event, event
;==================================================================
pro dialog_ow_where::eventQuit, event=event

  widget_control, self.tlbID, /destroy  
end

;=======================================================================
pro dialog_ow_where::cleanup
 
  if (self.destroy) then begin 
    obj_destroy, self.specObj
  endif else begin
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse

  obj_destroy, self.whereWObj
  
end

;==================================================================
pro dialog_ow_where_cleanup, tlb

  message, /info, "Entered'

  widget_control, tlb, get_uvalue=uObj

;  statePtr = (uObj.wObject)-> get(/statePtr)
;  groupLeaderID = (uObj.wObject) -> get(/groupLeader)	; identical to above for now.


  if ((uObj.wObject) -> no_block()) then begin	
    obj_destroy, (uObj.wObject)		; calls dialog_ow_where::cleanup
  endif
end


;==================================================================
function dialog_ow_where::no_block	; return 0/1 for no/yes
  return, self.no_block
end

;==================================================================
function dialog_ow_where::createWidgets	; return 0/1 for failure/success

 ;---------------------------------------------------  
  if (self.groupLeaderID EQ 0L) then begin
     floating = 0
  endif else begin
    floating = 1
  endelse

 ;-------------------------------------------------------------  
  OK = self.SpecObj -> getField(['*', 'so_filename', '*'], /inHeader, qty=dataFile)
  if (OK) then begin
    name = datafile
  endif else begin
    name = ''
  endelse

  aTitle = 'dialog_ow_where' + ' ' + Name

 ;---------------------------------------------------  
  if ((self.No_block) OR (self.groupLeaderID EQ 0L)) then begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
  	uname = "dialog_ow_where", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endif else begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
    	/modal, $
  	uname = "dialog_ow_where", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endelse
  self.tlbID = tlbID

  ;-------------------------------------------------------------
  voidID = widget_label( tlbID, value = self.labelString)

  ;-------------------------------------------------------------
  self.whereWObj = obj_new('cw_ow_where', self.specObj, tlbID, $	
;    	parentWObj = self, $
	from = self.from, $
	field = self.selectedLabel, $ 
;	field = self.selectedField, $ 
	/noInput)
  
  ;-------------------------------------------------------------
;;;  self -> freshenResults
  ;-------------------------------------------------------------

  ;-------------------------------------------------------------
  buttonClusterID = widget_base(tlbID, /column, /frame)

  voidID = widget_button(buttonClusterID, $
  	value='PROCESS', $
    	uname = 'eventProcess', $
	event_pro='dialog_ow_where_event',$
    	uvalue={wobject:self, method:'eventProcess'})
	
  voidID = widget_button(buttonClusterID, value='DISMISS', $
;  voidID = widget_button(buttonClusterID, value='QUIT', $
    	uname = 'eventQuit', $
	event_pro='dialog_ow_where_event',$
    	uvalue={wobject:self, method:'eventQuit'})

  ;-------------------------------------------------------------
    widget_control,tlbID,/realize		; make room for all these things.
  ;-------------------------------------------------------------
    if (self.No_block) then begin
      xmanager, 'dialog_ow_where', /no_block, tlbID, $
    	cleanup="dialog_ow_where_cleanup"
    endif else begin
      xmanager, 'dialog_ow_where',  tlbID, $
    	cleanup="dialog_ow_where_cleanup"
    endelse

;   message, /info, "After xmanager"	; Don't see if it is a blocking widget

  return, 1  
end		;;  dialog_ow_where::createWidgets


;==================================================================
;+
; NAME:
; 	dialog_ow_where::init
;
; PURPOSE:
;	The compound widgets cw_ow_*.pro used by dialog_ow_where can 
; select a field in the specific object self.specObj. 
; In order to update the droplists for all compound widgets, the 
; cw_ow* widgets call this method dialog_ow_where::freshen, which then 
; calls the methods cw_ow*::freshenWhere in all the compound widgets used 
; in dialog_ow_where. 
;
; 	If the keyword no_block is set, the widget is non-blocking and
; automatically the object of class dialog_ow_where is destroyed upon 
; closing the dialog box. If not set, the programmer must obj_destroy 
; the object upon return from init. Don't use /no_block
; simulataneously with dialog boxes which depend on the
; contents of the specific object.
;-
;*
function dialog_ow_where::init, $
	thisSpecObj, $		; required
	groupLeader = thisGroupLeader, $
	label = thisLabel, $	; Default selection
;	field = thisField, $	; Default selection
	from = thisFrom, $	; Header or data section for thisField
	xyzfocus = thisxyzFocus, $	; Either 'x', 'y', or 'z'
				; Stash the selected field tags in either
				; the x, y, or z focus. Defaults to z focus.
	title = thisLabelString, $ ; Optional string placed on GUI
	destroy = thisDestroy	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
;/
;	parentWobj = thisParentWObj, $	; required for freshening where
	
  debug = 0	;1

  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)

  ;-------------------------------------------------------------
  if (keyword_set(thisNo_block)) then begin
    self.no_block = thisNo_block
  endif
 
;  ;-------------------------------------------------------------
;  if (obj_valid(thisParentWobj)) then begin
;    self.parentWobj = thisParentWobj
;  endif 
  ;-------------------------------------------------------------
  if (n_elements(thisGroupLeader) NE 0) then begin
    self.groupLeaderID = thisGroupLeader
  endif else begin
     ; Must have a group_leader with blocking widget
     self.no_block = 0
  endelse 
  ;-------------------------------------------------------------

  if (n_elements(thislabel) GT 0) then begin
    self.selectedLabel = thisLabel
;    self.selectedField = thisField
;    message, /info, string("Intializing  self.selectedField = ", $
;	    format_list(thisField))
    message, /info, string("Intializing  self.selectedLabel = ", $
	    thisLabel)
  endif
  ;-------------------------------------------------------------
  if (n_elements(thisFrom) GT 0) then begin
    self.from = thisFrom
    message, /info, string("Intializing  self.from = ", $
	    thisFrom)
  endif

  ;-------------------------------------------------------------
  self.xyzFocus = 'z'
  if (n_elements(thisxyzFocus) GT 0) then begin
    self.xyzFocus = thisxyzFocus
  endif
  
  ;-------------------------------------------------------------
  self.labelString = string("Select field for use as designated ", $
  	self.xyzFocus, " column in operation")
	
  if (n_elements(thisLabelString) GT 0) then begin
    self.labelString = thisLabelString
  endif
  
  ;-------------------------------------------------------------
  
;Sept152003  OK = self -> createWidgets() 
;Sept152003  return, OK

  return, 1
  
end

;==================================================================
;*
pro dialog_ow_where__define
  define = {dialog_ow_where, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	groupLeaderID : 0L, $
	no_block: 0, $
	tlbID:0l, $
	whereWObj: obj_new(), $
	selectedLabel: '',  $
	from :'', $
	labelString :'', $
	xyzFocus: '' $
	}
;/

;	selectedField: ['', '', ''], $

end
