; $Id: dialog_viewdata.pro,v 1.4 2003/09/12 15:25:37 tanya Exp $
;+
; NAME:
; 	dialog_viewdata
;
; PURPOSE:
;	Menu for viewing the "data" section of a specObj.
;	Also allows the "focus" to be modified.
;	Is a traditional GUI, not a object widget.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE GUI main dialog routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created April, 2003. Tanya Riseman
;	TMR. Aug 29, 2003. Added keyword to "OK=specObj->updateDavePtr(sort=sort)"
;-
; dialog_viewData.pro
; This compound widget makes a multi-column selction list and plots to 
; a predefined plot window elsewhere.
; Tanya Riseman, NIST Center for Neutron Research. April, 2003.

;===================================================================

;===================================================================
pro dialog_viewData_cleanup_event, event
; Almost no clean up required
;    infolabel = "dialog_viewData_cleanup_event >> "     
    widget_control, event, get_uvalue=pInfo

    obj_destroy,  (*pInfo).textDataReportObj
    obj_destroy,  (*pInfo).textFocusObj
    obj_destroy,  (*pInfo).textDataInfoObj
 
    ptr_free, pInfo

end

;===================================================================

pro dialog_viewData_dismiss_event, event
    infolabel = "dialog_viewData_dismiss_event >> "  

    widget_control, event.id, get_uvalue=pInfo

;    specObj =(*pinfo).specObj		; Just temporary alias
;    davePtr = specObj -> getDavePtr()	; Just temporary alias
;    if (ptr_valid(davePtr)) then begin
;      ptr_free, davePtr		; Just temporary alias;
;
;    endif else begin
;      print, infolabel, "Object has no valid davePtr available for updating."
;    endelse

;    ptr_free, pInfo		; done in dialog_viewdata_cleanup_event
    widget_control, event.top, /destroy
end

;===================================================================

pro dialog_viewData_update_event, event
    infolabel = "dialog_viewData_update_event >> "  
 
    widget_control, event.id, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias

    OK = specObj -> getFocus(xTags = theXtag)
    OK = specObj -> getFocus(yTags = theYtag)
    OK = specObj -> getFocus(zTags = theZtag)

    xrank = -1		; garbage value
    yrank = -2
    zrank = -3
    xdim = [-1,-1,-1,-1]; garbage value
    ydim = [-2,-2,-2,-2]; garbage value
    zdim = [-3,-3,-3,-3]; garbage value
    	    
    yEmpty= 1    
    for i = 0, 2 do begin
      if (strlen(theYTag[i]) NE 0) then yEmpty = 0
    endfor
    
    sort = 0
;    sortByX = 0
    sortHeader = 0
    if ( yEmpty) then begin
      OK = specObj -> getField(theXtag, rank=xrank, dim=xdim, qty=xqty, /inData)
      if (OK) then begin
        if (xrank EQ 1) then begin	; 1D array or scalar
;	  indexSort = sort(xqty)
;	  sortedIndex = make_array(/LONG, /index, xdim[0])
;	  different = int(norm(indexSort - sortedIndex))
	  ; This eqarray will work on differently dimensioned arrays.
	  if (NOT eqarrays(sort(xqty), make_array(/LONG, /index, xdim[0]) )) then begin
	    xlabel = theXtag[1]
            sort = dialog_message($
;            sortByX = dialog_message($
      	      string("The 'x'-axis ", xlabel, $
	    	" contains 1-D data. Want to sort z=histo by x?") , $
	      /question, $
	      title = "dialog_viewData_update_event")
;	     if (sortByX) then begin
;               sortHeader = dialog_message($
;      	          string("Want to sort header fields by x as well?" ), $
;	          /question, $
;	          title = "dialog_viewData_update_event")
;	     endif
	   endif else begin
	     if (!debug) then message, /info, $
	     		string("x= ", xlabel, " is already sorted.")
	   endelse
	endif
      endif
      	
    endif else begin			;; if ( yEmpty) then begin
      OK = specObj -> getField(theXtag, rank=xrank, dim=xdim, qty=xqty, /inData)
      OK = specObj -> getField(theYtag, rank=yrank, dim=ydim, qty=yqty, /inData)
    ; probably 2D z data.
      if (xrank EQ 1) then begin	; 1D array or scalar
     	needsSorting = (NOT eqarrays(sort(xqty), make_array(/LONG, /index, xdim[0]) ))
      endif	
      if (yrank EQ 1) then begin	; 1D array or scalar
     	needsSorting = needsSorting OR  $
	  (NOT eqarrays(sort(yqty), make_array(/LONG, /index, ydim[0]) ))
      endif
      if (needsSorting) then begin
            sort = dialog_message($
      	      string("The 'x' ", xlabel, " and/or 'y' ", ylabel, $
	    	" axes are currently unsorted. Want to sort z=histo by x,y?") , $
	      /question, $
	      title = "dialog_viewData_update_event")
      endif    
    endelse

    if (!debug) then message, string(" sort = ", sort, $
;    if (!debug) then message, string(" sortByX = ", sortByX, $
    	" sortHeader=", sortHeader)  

    OK = specObj -> updateDavePtr(sort=sort)
;    OK = specObj -> updateDavePtr(sortByX=sortByX)

    if (NOT OK) then print, infolabel, "failure updating specObj"
 
end

;===================================================================

pro dialog_viewData_header_event, event
    infolabel = "dialog_viewData_header_event >> "  
 
    widget_control, event.id, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias
    headerID = dialog_viewheader(specObj=specObj, group_Leader=event.top)
        
end

;===================================================================

pro dialog_viewdata_annotated_event, event
    infolabel = "dialog_viewdata_annotated_event >> "  

    widget_control, event.top, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias

    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''

    strArray = specObj -> textInfo(/inData, /getAll, /onlyAnnotations)

;Old way. Aug 13, 2003 TMR
;    textBaseID = dialog_message(strArray, /info, $
;	   	title=(datafile + ' annotated data'))


    textfile = 'dataReport_' + dataFile + '_txt'
    temp_ptr = ptr_new(strArray)
    textbaseObj = obj_new( 'dialog_ow_textInfo', $
	text_ptr= temp_ptr, $
	filename=textFile, $
	groupLeader = (*pinfo).group_leader)
    ptr_free, temp_ptr
    OK = textbaseObj -> createWidgets()

    obj_destroy, (*pinfo).textDataReportObj
    (*pinfo).textDataReportObj = textbaseObj

end


;===================================================================

pro dialog_viewData_focus_event, event
    infolabel = "dialog_viewData_focus_event >> "  

    widget_control, event.top, get_uvalue=pInfo


    specObj =(*pinfo).specObj		; Just temporary alias
    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''

    strArray = specObj -> textInfo(/inFocus, /getEverything)

;Old way. Aug 13, 2003 TMR
;    textBaseID = dialog_message(strArray, /info, title=(dataFile + ' Focus'))

    textfile = 'focus_' + dataFile + '_txt'
    temp_ptr = ptr_new(strArray)
    textbaseObj = obj_new( 'dialog_ow_textInfo', $
	text_ptr= temp_ptr, $
	filename=textFile, $
	groupLeader = (*pinfo).group_leader)
    ptr_free, temp_ptr
    OK = textbaseObj -> createWidgets()

    obj_destroy, (*pinfo).textFocusObj
    (*pinfo).textFocusObj = textbaseObj

end

; ;-----------------------------------------------------------------------------------------------------------------------------
;FUNCTION dialog_viewData_done_Event, event
;
;  
;;Widget_Control, event.ID, Get_UValue=theMessage
;;event = Call_Method(theMessage.method, theMessage.object, event)
;
;RETURN, event
;END
;-----------------------------------------------------------------------------------------------------------------------------

;===================================================================

pro dialog_viewData_tagxyz_event, event

    common cb_dialog_viewData_tagxyz_event, old_itag
    
    infolabel = "dialog_viewData_tagxyz_event >> "  
;debug    print, infolabel, "Entered..."


    widget_control, event.top, get_uvalue=pInfo
    specObj =(*pinfo).specObj		; Just temporary alias
    
    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''

    whichXYZ = widget_info(event.id, /uname)  ; 'tag_x', etc.

;    ;-------------------------------------------------------------------    
;    if (whichxyz EQ 'DISMISS') then begin
;       widget_control, event.top, /destroy	; this calls cleanup routine.
;       return
;    endif
;    ;-------------------------------------------------------------------

    widget_control, event.id, get_value=itag_xyz_orig	; 0, 1, 2, 3, etc.
    if (whichXYZ eq 'tag_annotate') then begin
;      if (n_elements(old_itag) EQ 0) then begin
;        old_itag = make_array(n_elements(itag_xyz_orig), /int)	; zeros
;      endif
;      print, "old_itag     =", old_itag
;      print, "itag_xyz_orig=",itag_xyz_orig
      itag_xyz = where( old_itag NE itag_xyz_orig, count)
      if (count EQ 1) then begin 
        thistag_xyz = (*pinfo).menutxt[itag_xyz]
      endif else begin
        thistag_xyz = ''
      endelse
      old_itag = itag_xyz_orig
    endif else begin
;debug    print, infolabel, 'widget_control, event.id, get_value=itag_xyz=', itag_xyz
      itag_xyz = itag_xyz_orig
     endelse
     thistag_xyz = (*pinfo).menutxt[itag_xyz]
;    print, infolabel, "itag_xyz=", itag_xyz, "thistag_xyz = " , thistag_xyz
    
;Debug    print, infolabel, itag_xyz, ' gives thisTag_xyz= "', thisTag_xyz, '"'
;Debug    print, infolabel, 'whichxyz= "', whichxyz, '"'

    
    if (obj_valid(specObj)) then begin
;;;    if (ptr_valid(*(*(*DavePtr).dataStrPtr).SpecificPtr )) then begin
;;;      specPtr = (*(*(*DavePtr).dataStrPtr).SpecificPtr)
;      print, infolabel, "thisTag_XYZ =", thisTag_XYZ
;      print, infolabel, "event.select =", event.select
      if (thisTag_Xyz eq '<none>') then thisTag_xyz = ''
      OK = 1
      buttonOn = event.select
      case (whichXYZ) of 
        'tag_annotate': begin
           OK = specObj -> setTagField( ['*', thisTag_xyz, '*'], /inData, $
	   		annotate=buttonOn)		
	   end
        'tag_x': begin
	   specObj -> setFocus, xtags =['*', thisTag_xyz, '*']
;debug	   print, infolabel, " Event xtag =", thisTag_xyz
	   end
        'tag_y': begin
	   specObj -> setFocus, ytags =['*', thisTag_xyz, '*']
;debug	   print, infolabel, " Event ytag =", thisTag_xyz
	   end
        'tag_z': begin
	   specObj -> setFocus,  ztags =['*', thisTag_xyz, '*']
;debug	   print, infolabel, " Event ztag =", thisTag_xyz
	   end
        'tag_info': begin
	   if ((thisTag_XYZ EQ '') OR (thisTag_XYZ EQ '<NONE>')) then begin
	   	; Do nothing
	     strArray = specObj -> textInfo(/inData, /getSome, $
	     	/showPendent, /showTags)	; shortest version
;	     strArray = specObj -> textInfo(/inData, /getAll)	; short version
	   endif else begin
;	     strArray = specObj -> textInfo(/inData, position=itag_XYZ, $
;	     		/getAll)
	     strArray = specObj -> textInfo(/inData, position=itag_XYZ, $
	     		/getEverything, /showTags)
;debug	     print, infolabel, itag_XYZ, ' /getEverything. str=', strArray
	   endelse 

;Old way. Aug 13, 2003 TMR
;	   textBaseID = dialog_message(strArray, /info, $
;	   	title= (string(datafile, ' Data: ', thisTag_XYZ) ) )
;;	   ok=(*SpecPtr).specObj -> setFocus( ztags =['*', thisTag_xyz, '*'])

	    textfile = 'data_' + thisTag_XYZ  + '_' + dataFile + '_txt'
	    temp_ptr = ptr_new(strArray)
	    textbaseObj = obj_new( 'dialog_ow_textInfo', $
		text_ptr= temp_ptr, $
		filename=textFile, $
		groupLeader = (*pinfo).group_leader)
	    ptr_free, temp_ptr
	    OK = textbaseObj -> createWidgets()

	    obj_destroy, (*pinfo).textDataInfoObj
	    (*pinfo).textDataInfoObj = textbaseObj

	   end
	 else: begin
	   OK = 0
	   print, infolabel, '1st Case, whichXYZ= "', whichXYZ, '"'
	   end
      endcase

      if (not OK) then begin
        print, infolabel, 'Failed set(', whichXYZ,' = ', thisTag_XYZ, ')'
	return 
      endif

     if ( (whichXYZ eq 'tag_x') OR (whichXYZ eq 'tag_y') $
     	 OR (whichXYZ eq 'tag_z') ) then begin
       if ( ((*pInfo).DrawIndex GE 0) AND (NOT (*pInfo).SkipPlot)) then begin 
         wset, (*pInfo).DrawIndex
         specObj -> plot, indexNum=(*pInfo).indexNum
       endif
     endif
          
;Debug     print, infolabel, "DrawIndex=", (*pInfo).DrawIndex
;Debug     print, infolabel, "indexNum=", (*pInfo).indexNum
;;        specObj -> plot

    endif else begin
      print, infolabel, "NOT (obj_valid(specObj)"
    endelse

;    if (ptr_valid(Daveptr)) then begin
;      ; Now update the dave pointer
;      ptr_free, davePtr		; Just temporary alias
;    endif
    
end

;=========================================================================
;*
function dialog_viewData, $
	group_leader=group_leader, $
	name = thisName, $		; name, such as run number
	davePtr = thisdavePtr, $
	specObj = thisSpecObj, $
	skipPlot = thisSkipPlot, $
	drawIndex = thisDrawIndex, $		; window to direct plots to.
	indexNum = thisIndexNum		; For multiple plot, use plot with 0
					; use oplot with thisIndexNum > 0
;/

;;;	file=datafile, $
;	floating = thisFloating, $	; If floating, has a done button.

;Assume that the DavePtr is already filled up with ICP triple axis data
; Non-modal widget. Copying nmocurve.pro.

  common cb_dialog_viewData_tagxyz_event, old_itag
  
    
  infolabel = "dialog_viewData >> " 
;  debug = 1 

  if (n_elements(thisIndexNum) eq 0) then begin
    indexNum = 0
  endif else begin
    indexNum = thisIndexNum
  endelse
  
  if ((n_elements(thisDavePtr) EQ 0) AND $
  	(n_elements(thisSpecObj) EQ 0)) then begin
    print, infolabel, "Require one of the 2 keywords: davePtr, specObj"
    return, -1
  endif

  if (n_elements(thisDavePtr) EQ 0) then begin
    davePtr = thisSpecObj -> getDavePtr()
  endif else begin
    davePtr = thisDavePtr
  endelse

  if (n_elements(thisSpecObj) EQ 0) then begin
    specObj = (*(*(*DavePtr).dataStrPtr).SpecificPtr).specObj
  endif else begin
    specObj = thisSpecObj
  endelse

  if (NOT obj_valid(specObj)) then begin
    print, infolabel, "Invalid object specObj."
    return, -1
  endif
  
  if (NOT ptr_valid(davePtr)) then begin
    print, infolabel, "WARNING. Invalid pointer davePtr."
;    return, -1
  endif
  
;  if (keyword_set(thisfloating)) then begin
;    floating = 1
;  endif else begin
;    floating = 0
;  endelse
  
  if (n_elements(group_leader) EQ 0) then begin
    group_leader = 0L
    floating = 0
  endif else begin
    floating = 1
  endelse

  print, "group_leader=", group_leader, " floating=", floating
  inputOutputID = widget_base(group_leader=group_leader, floating=floating, /row)
  ;; Check if there is an instance of this program running
  if (n_elements(thisName) GT 0) then begin
    registerName = 'dialog_viewData' + thisName + strtrim(string(group_leader), 2)
  endif else begin
    registerName = 'dialog_viewData' + strtrim(string(group_leader), 2)
  endelse
  if (xregistered(registerName)) then begin
    print, infolabel, registerName,  $
    	' Already registered. Only single instance allowed.'
    return, -1
  endif else begin
    print, infolabel, registerName,  ' just registered.'
  endelse
 
;;; inputOutputID = widget_base( /row)

 child = widget_base(inputOutputID, /column)
; child = widget_base(parent, /column)
 datafile = ''

;Short if (not ptr_valid(DavePtr)) then DavePtr = ptr_new()
 tempObj =  specObj -> get(position=['*', 'SO_filename', '*'] , /inHeader)
 if (obj_valid(tempObj)) then begin
    OK = tempObj -> get(qty=datafile)
    message, /info, string("len(SO_filename)=", n_elements(datafile))
    if (n_elements(datafile) GT 1) then datafile = datafile[0]	; buggy
 endif else begin
   if (n_elements(thisName) GT 0) then begin
     datafile = thisName
   endif else begin
     datafile = 'UNKNOWN FILENAME'
   endelse
 endelse

 nLabels = 0
 specObj -> getLabel, /inData, $
 	label=dataLabels, $
	sizeString=sizeStringArray, $
	count=nLabels

;--------------------------------------------------------------------------
;NotNeeded  voidID = Widget_label(child, value=datafile)

 modal = 0
 rowID = widget_base(child, /row, modal=modal, $
 	event_pro='dialog_viewData_tagxyz_event', uvalue=specObj)
 
 max_nlabels  = nLabels
 xbutID = make_array(/long, (max_nlabels+1))
 ybutID = make_array(/long, (max_nlabels+1))
 zbutID = make_array(/long, (max_nlabels+1))

 reportLabels  = make_array(/string, (max_nlabels), value=' ')
 workingLabels  = make_array(/string, (max_nlabels+1))
 workingSize  = make_array(/string, (max_nlabels+1))

 itagx = max_nLabels	; <none> selected option
 itagy = max_nLabels	; <none> selected option
 itagz = max_nLabels	; <none> selected option

 OK = specObj -> getFocus(xtags=tagx)
 OK = specObj -> getFocus(ytags=tagy)
 OK = specObj -> getFocus(ztags=tagz)

 itagx = specObj -> getPosition(tagx, /inData)
 itagy = specObj -> getPosition(tagy, /inData)
 itagz = specObj -> getPosition(tagz, /inData)

 workingLabels[0:(nlabels-1)] = dataLabels
 workingSize[0:(nlabels-1)] = sizeStringArray
 workingLabels[nlabels] = '<none>'
 workingSize[nlabels] = '<all>'
; workingSize[nlabels] = '[]'
 
 if (itagx eq -1) then itagx = max_nLabels	; <none> selected option
 if (itagy eq -1) then itagy = max_nLabels	; <none> selected option
 if (itagz eq -1) then itagz = max_nLabels	; <none> selected option

  ;; Initlaize with zeros.
  old_itag = make_array(n_elements(workingLabels), /int)	; zeros

 annotateID = cw_bgroup(rowID, reportLabels, /col, /nonexclusive,  $
 	label_top='Report', frame=1,  $
	button_uvalue=workingLabels, $
	uname= 'tag_annotate')
 xID = cw_bgroup(rowID, workingLabels, /col, /exclusive,  $
 	label_top='Select X', /no_release, frame=1, set_value=itagx, $
	button_uvalue=workingLabels, $
	uname= 'tag_x')
 yID = cw_bgroup(rowID, workingLabels, /col,  /exclusive, $
 	label_top='Select Y', /no_release, frame=1, set_value=itagy, $
	button_uvalue=workingLabels, $
	uname= 'tag_y')
 zID = cw_bgroup(rowID, workingLabels, /col, /exclusive, $
 	label_top='Select Z', /no_release, frame=1, set_value=itagz, $
	button_uvalue=workingLabels, $
	uname= 'tag_z')

 infoID = cw_bgroup(rowID, workingSize, /col, /exclusive, $
 	label_top='View values', /no_release, frame=1, set_value=max_nLabels, $
	button_uvalue=workingSize, $
;???	event_funct = 'dialog_viewData_info_event', $
	uname= 'tag_info')
	
;--------------------------------------------------------------------------
;   wset, thisDrawIndex
;   specObj -> plot, indexNum=indexNum

 

  DrawIndex = -1
  info = {group_leader:group_leader, SpecObj:specObj, menuTxt:workingLabels, $
  	skipPlot:keyword_set(skipThisPlot), $
	drawIndex:drawIndex, $
	indexNum:indexNum, $
	textDataReportObj: obj_new(), $
	textFocusObj: obj_new(), $
	textDataInfoObj: obj_new() $
	}
  pInfo = ptr_new(info)


; ; Make the aspect ratio close to portrait-oriented paper.
;;; drawID = widget_draw(inputOutputID, xsize = 400, ysize=560, retain=2)
; ; Make the aspect ratio square so that the title will fit..

 ; have to wait intil widget_draw is realized to get it's number.
 column2ID = widget_base(inputOutputID,  /column)
 if ((n_elements(thisDrawIndex) EQ 0) AND (NOT keyword_set(thisSkipPlot))) then begin 
   drawID = widget_draw(column2ID, xsize = 400, ysize=400, retain=2)
;   drawID = widget_draw(inputOutputID, xsize = 400, ysize=400, retain=2)
 endif
  
; Rob will call this elsewhere
 voidID = Widget_label(column2ID, value=datafile)

 voidID = Widget_button(column2ID, $
; voidID = Widget_button(child, $
 	value='View Header and select focus', $
; 	value=string(datafile, ': View Header and select focus'), $
 	uname='View header', $
 	event_pro='dialog_viewData_header_event', uvalue=pInfo)

 voidID = Widget_button(column2ID, $
 	value='View data report', $
; 	value=string(datafile, ': View data report'), $
 	uname='View data report', $
 	event_pro='dialog_viewData_annotated_event', uvalue=pInfo)

 voidID = Widget_button(column2ID, $
 	value='View Focus', $
; 	value=string(datafile, ': View Focus'), $
 	uname='View header', $
 	event_pro='dialog_viewData_focus_event', uvalue=pInfo)

 voidID = Widget_button(column2ID, $
; voidID = Widget_button(child, $
 	value="Update DavePtr's common area with focus",$
 	uname='Update', $
 	event_pro='dialog_viewData_update_event', uvalue=pInfo)

 voidID = Widget_button(column2ID, $
; voidID = Widget_button(child, $
 	value="Dismiss",$
 	uname='Dismiss', $
 	event_pro='dialog_viewData_dismiss_event', uvalue=pInfo)

;  if ((n_elements(thisDrawIndex) GT 0) AND (NOT keyword_set(thisSkipPlot))) then begin 
;     wset, thisDrawIndex
;     specObj -> plot, indexNum=indexNum
;     drawIndex = thisDrawIndex
;  endif else begin
;    DrawIndex = -1
;  endelse
;  (*pInfo).drawIndex = drawIndex
;   print, infolabel, "(*pInfo).drawIndex = ", drawIndex
  
;  If (group_leader NE 0L) then begin
    Widget_Control, inputOutputID, /Realize
;  endif 
  
  widget_control,  inputOutputID, set_uvalue=pInfo
 
; if ((n_elements(thisDrawIndex) GT 0) AND (NOT keyword_set(thisSkipPlot))) then begin 
;   wset, thisDrawIndex
;   specObj -> plot, indexNum=indexNum
; endif

 if ((n_elements(thisDrawIndex) EQ 0) AND (NOT keyword_set(thisSkipPlot))) then begin 
   widget_control, drawID, get_value=thisDrawIndex
;debug   print, infolabel, "widget_control, ", drawID, ", get_value=", thisDrawIndex
 endif
  
  if ((n_elements(thisDrawIndex) GT 0) AND (NOT keyword_set(thisSkipPlot))) then begin 
     wset, thisDrawIndex
     specObj -> plot, indexNum=indexNum
     drawIndex = thisDrawIndex
  endif else begin
    DrawIndex = -1
  endelse
  (*pInfo).drawIndex = drawIndex
;Debug   print, infolabel, "(*pInfo).drawIndex = ", drawIndex


  xmanager, registerName,  inputOutputID, $
  	event_handler= 'dialog_viewData_tagxyz_event', $ 
;	/no_block, $
  	cleanup = 'dialog_viewData_cleanup_event'
  
;--------------------------------------------------------------------------
  
  return, inputOutputID
  
end			; function dialog_viewData
;=========================================================================
