; $Id: format_list.pro,v 1.2 2003/09/12 15:25:37 tanya Exp $
;+
; NAME:
;      format_list
;
; PURPOSE:
;	Returns a string of format "#", "[#,#]" , "[#,#,#]" , "[#,#,#,#]". 
; corresponding to the number of dimensions and the size of those
; dimensions for an array.
;
;	The keyword array is used with arrays (passed by value).
; Use this only for small arrays, to avoid efficiency problems.
;
;	The keyword ptr_array is used with pointers to arrays 
; (passed by reference). Use this for large arrays.
;
;	The keyword dim_array is used when one already knows the size of 
; array, from the IDL command size(). In this case, zeros on the end of the
; array are stripped off. 
;
;	Examples:
;      retString = 'OBJ' + format_list(array=thingy)
;      retString = '{PTR}'  +  format_list(ptr_array=thingy)
;      retString =   format_list(dim_array=[2,3,0,0])
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;	DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED: none
;
; EXAMPLE
;
; MODIFICATION HISTORY:
;	Created Spring, 2003. Tanya Riseman
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;*
function format_list, $		; returns string of form  * or [*,*,...]
  ; Use either  ptr_Array (pass by reference) or array (pass by value)
  anArray		; An input 1D array or scalar; String or numeric type
;/

     if (n_elements(anArray) EQ 0) then begin
       return, '[NULL]'
     endif 
 
   tname = size(anArray, /tname)
   aLen =n_elements(anArray) 
   
   if (aLen eq 1) then begin		; scalar
     if (tName eq 'STRING') then begin
       aString = "'" + anArray + "'"
;       aString = "['" +  anArray +  "']"
     endif else begin
       aString = strcompress(string(anArray),/remove_all)
;      aString = "[" + strcompress(string(anArray),/remove_all) + "]"
     endelse
       return, aString
   endif

   ; true array:
   astring = '['
   for i = 0, (aLen-1) do begin
     if (tname EQ 'STRING') then astring = astring + "'"
     astring = astring + strcompress(string(anArray[i]), /remove_all)
     if (tname EQ 'STRING') then astring = astring + "'"
     if (i NE (aLen-1)) then begin
       astring = astring + ','
     endif else begin
       astring = astring + ']'
     endelse
   endfor

   return, astring

end
