; $Id: func_icp_make_resvolume.pro,v 1.3 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
;       func_icp_make_resvolume
;
; PURPOSE:
;
;	Fake method for specObj class and associated functions.
;
;	Creates data items Ei and Ef from delta E and E_fixed.
;	Creates neutron wavelength lambda_i (monchrometer) 
;	and lambda_f (analyzer).
;	Creates the normalized resolution volume.  
;	Creates the normalized nuetron scattering intensity using the 
;	resolution volume.  
; 	Based on Fortran routines used by spurion program.
;	These routines are used by func_icp_header_init and func_icp_data_init.
;
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;       <LI><A HREF="#SPECOBJ::INIT,_arguments">SPECOBJ::INIT,_arguments</A>
;	
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Aug, 2003.
;-
; func_icp_make_resvolume.pro

;============================================================================
function func_icp_get_qty, $ 	; Returns 0/1 for failure/success
	self, $				; Instance of specObj class
	thisE_in, $			; Label for E_in or E_out
	qty=E_in, $			; Value stored
	err=Eerr, $			; Error stored
	units = eUnits, $
	inHeader = thisInHeader, $
	inData = thisInData, $
	from = thisFrom
;	errMsg = thisErrMsg		; Optional error message string.

  E_in = 0.0	
  thisInHeader = 0
  thisInData = 0
  
  thisFrom = 'input'
  OK_Ein = self -> getField( thisE_in, /inData, $
  	qty=E_in, err=Eerr, units = eUnits)
  if (NOT OK_Ein) then begin
    OK_Ein = self -> getField(thisE_in, /inHeader, $
    	qty=E_in, err=Eerr, units = eUnits)
    if (OK_Ein) then begin 
      thisInHeader = 1
      thisFrom = 'header'
    endif
  endif else begin
    thisInData = 1
    thisFrom = 'data'
  endelse
  
  return, OK_Ein
 
end

;============================================================================
function func_icp_make_Einout, $ 	; Returns 0/1 for failure/success
		; Assumes that the following ICP fields exists: 
		;	'which_fixed', 'eE_fixed', 'E' ...
	self, $				; Instance of specObj class	
	thisE_in, $			; Tags for E_in 
	thisE_out,$ 			; Tags for E_out
	errMsg=thisErrMsg
	

        ;--------------------------------------------
        if (n_elements(thisE_in) EQ 0) then begin
	  thisE_in = ['qtyObj', 'E_in', 'func_icp_make_resvolume']
	endif else begin
;	  thisE_in[2] = 'func_icp_make_resvolume'
	endelse

        ;--------------------------------------------
        if (n_elements(thisE_out) EQ 0) then begin
	  thisE_in = ['qtyObj', 'E_out', 'func_icp_make_resvolume']
	endif else begin
;	  thisE_out[2] = 'func_icp_make_resvolume'
	endelse

        ;--------------------------------------------
	OK = self -> getField (['qtyObj', 'which_fixed', '*'], $
		/inHeader, $
		qty = whichString)
	
	if (NOT OK) then begin
	  thisErrMsg = "Field 'which_fixed' not found"
	  return, 0
	endif
	
        ;--------------------------------------------
	 OK = OK and self -> getField (['qtyObj', 'E_fixed', '*'], $
;		/inHeader, $
		units = eUnits, $
		qty = qty_E_fixed, $
		isScalar = efixedScalar)
	if (NOT OK) then begin
	  thisErrMsg = "Field 'E_fixed' not found"
	  return, 0
	endif

	  E_fixed_inHeader = 0
	  E_fixed_inData = 0
  	  if (efixedScalar) then begin
	    E_fixed_From = 'header'
	    E_fixed_inHeader = 1
	  endif else begin
	    E_fixed_From = 'data'
	    E_fixed_inData = 1
	  endelse

        ;--------------------------------------------
	 OK = OK and self -> getField (['qtyObj', 'E', '*'], $
		/inData, $
		qty = qty_e, $
		isScalar = eIsScalar)
	if (NOT OK) then begin
	  thisErrMsg = "Field 'E' not found in data section"
	  return, 0
	endif


	; First, check if they already exist in data base.
	; Make them as vectors, if exist in the header.
	OK_Ein = self -> getField(thisE_in, /inData,  qty=E_in)
	OK_Eout = self -> getField(thisE_out, /inData,  qty=E_out)

;	OK_Ein = self -> getField(['qtyObj', 'E_in', '*'], /inData,  qty=E_in)
;;	if (NOT OK_Ein) then begin
;;	   OK_Ein = self -> getField(['qtyObj', 'E_in', '*'], /inHeader,  qty=E_in)
;;	endif
;	OK_Eout = self -> getField(['qtyObj', 'E_out', '*'], /inData,  qty=E_out)
;;	if (NOT OK_Eout) then begin
;;	   OK_Eout = self -> getField(['qtyObj', 'E_out', '*'], /inHeader,  qty=E_out)
;;	endif

       if ((NOT OK_Ein) OR (NOT OK_Eout)) then begin
	if (whichString EQ 'EA') then begin
	  if (NOT OK_Eout) then begin
	   E_out = qty_e - qty_e + qty_E_fixed	; Get dimensions to match E. 
	   self -> add, /inData, $
	    tags = ['qtyObj', thisE_out[1], 'func_icp_make_resvolume'], $
;	    tags = ['qtyObj', 'E_out', 'func_icp_make_resvolume'], $
	    errMsg = thisErrMsg, $
	    contents = obj_new('qtyObj', $
	   	name='Final Energy', $
	  	qty = E_out,$
		units=eUnits)
	   if (n_elements(thisErrMsg) NE 1) then begin
	    thisErrMsg = [thisErrMsg, "Field 'E_out' not copied from 'E_fixed'"]
	    return, 0
	   endif
	   if (strlen(thisErrMsg) NE 0) then begin
	    thisErrMsg = [thisErrMsg, "Field 'E_out' not copied from 'E_fixed'"]
	    return, 0
	   endif
          endif 	; if (NOT OK_Eout) 
	  
	  if (NOT OK_Ein) then begin

	    message, /info, string("E_fixed_From=", E_fixed_From)
            OK = OK AND self -> binaryOp('+', $
	   	tags1 = ['qtyObj', 'E', '*'], $
		from1= 'data', $
	   	tags2 =['qtyObj', thisE_out[1], '*'], $
;	   	tags2 =['qtyObj', 'E_out', '*'], $
		from2 = 'data', $
;		from2 = E_fixed_From, $
;		from2= 'header', $	; Might not work with zipped files.
		resTags =  ['*', thisE_in[1],  'func_icp_make_resvolume'], $
;		resTags =  ['*', 'E_in',  'func_icp_make_resvolume'], $
		name = 'Initial Energy', $
		units = eUnits, $
		errMsg = thisErrMsg)
	    if (NOT OK) then begin
	      thisErrMsg = [thisErrMsg, "Field 'E_in' not created from 'E' + 'E_out'"]
	      return, 0
	    endif
;	   initialIsVector = (NOT eIsScalar)
         endif ;if (NOT OK_Ein) then begin

	endif else begin

	  if (whichString EQ 'EM') then begin

	   if (NOT OK_Ein) then begin
	    E_in = qty_e - qty_e + qty_E_fixed	; Get dimensions to match E. 
	    self -> add, /inData, $
	   	tags = ['qtyObj', thisE_in[1], 'func_icp_make_resvolume'], $
;	   	tags = ['qtyObj', 'E_in', 'func_icp_make_resvolume'], $
		errMsg = thisErrMsg, $
	   	contents = obj_new('qtyObj', $
	   		name='Initial Energy', $
	  		qty = E_in,$
			units=eUnits)
	    if (strlen(thisErrMsg) NE 0) then begin
	    	thisErrMsg = [thisErrMsg, "Field 'E_in' not copied from 'E_fixed'"]
	    	return, 0
	    endif
	   endif 		; (NOT OK_Ein)
	   
	  
	   if (NOT OK_Eout) then begin
              OK = self -> binaryOp('-', $
	   	tags1 =['qtyObj', thisE_in[1], '*'], $
;	   	tags1 =['qtyObj', 'E_in', '*'], $
		from1 = 'data', $
;		from1 = E_fixed_From, $
;		from1= 'header', $ 	; Might not work with zipped files.
	   	tags2 = ['qtyObj', 'E', '*'], $
		from2= 'data', $
		resTags =  ['*', thisE_out[1],  'func_icp_make_resvolume'], $
;		resTags =  ['*', 'E_out',  'func_icp_make_resvolume'], $
		name = 'Final Energy', $
		units = eUnits, $
		errMsg = thisErrMsg)
	     if (NOT OK) then begin
	       thisErrMsg = [thisErrMsg, "Field 'E_in' not created from 'E_in' - 'E' "]
	       return, 0
	     endif
;	   finalIsVector = (NOT eIsScalar)
	   endif	; (NOT OK_Eout)	  
	  endif else begin
	    thisErrMsg = string("Field 'which_fixed' has invalid contents: ", $
	    	whichString)
    	    return, 0
	  endelse
	endelse
       endif		; if ((NOT OK_Ein) OR NOT OK_Eout))
       
	
end

;============================================================================
;+
; NAME:
; 	func_icp_make_resvolume
;
; PURPOSE:
;	Fake method for specObj class.
;
;	Creates data items Ei and Ef from delta E and E_fixed.
;	Creates neutron wavelength lambda_i (monchrometer) 
;	and lambda_f (analyzer).
;	Creates the normalized resolution volume.  
;	Creates the normalized nuetron scattering intensity using the 
;	resolution volume.  
;-
;*
function func_icp_make_resvolume, $	; Returns 0/1 for failure/success
				; Fake method for specObj class.
 	self, $			; instance of class specObj
;	noTreatmentHistory = thisNoTreatmentHistory, $
	;--------------------------------------------------------------	
	; The following are tags for the things needed in the calculation.
	; If not provided, they default to values appropriate for triple axis ICP data..
	rawIntensity = thisRawIntensity, $; defaults to ['*','INTENSITY', '*']
	E_in = thisE_in, $		; defaults to ['*','E_in', '*']
	E_out = thisE_Out, $		; defaults to ['*','E_out', '*']
	dspacing_mon = thisDspacing_mon, $; defaults to ['*','dspacing_mon ', '*']
	dspacing_ana = thisDspacing_ana, $; defaults to ['*','dspacing_ana ', '*']
	restags = thisResTags, $; defaults to ['*','INTENSITY_ResVol', '*']
	resName = thisResName, $
	resUnits = thisResUnits, $
	;--------------------------------------------------------------	
	errMsg = thisErrMsg		; Optional error message string.
;/

  ; make all new things be vectors in the data section, even if constant!
  
  infolabel = 'func_icp_make_resvolume >> '
  debug = 0	;1

  thisErrMsg = ''
  
  if (n_elements(thisRawIntensity) EQ 0) then begin
    thisRawIntensity = ['qtyObj', 'INTENSITY', '*']	
  endif

  if (n_elements(thisResTags) EQ 0) then begin
    thisResTags = ['qtyObj', string(thisRawIntensity[1], '_ResVol') ,$
    	 'func_icp_make_resvolume']	
  endif

  if (n_elements(thisResName) EQ 0) then begin
    thisResName ='Neutron Intensity normalized by resolution volume'
  endif

  if (n_elements(thisResUnits) EQ 0) then begin
    thisResUnits ='Arbritary'
  endif

  if (n_elements(thisE_in) EQ 0) then begin
    thisE_in = ['qtyObj', 'E_in', '*']	
  endif

  if (n_elements(thisE_out) EQ 0) then begin
    thisE_out = ['qtyObj', 'E_out', '*']	
  endif

  if (n_elements(thisDspacing_mon) EQ 0) then begin
    thisDspacing_mon = ['qtyObj', 'thisDspacing_mon', '*']	
  endif

  if (n_elements(thisDspacing_ana) EQ 0) then begin
    thisDspacing_ana = ['qtyObj', 'thisDspacing_ana', '*']	
  endif

  thisErrMsg = ''
  OK_Ein = func_icp_get_qty(self, thisE_in, qty=E_in, units=eUnits)
  OK_Eout = func_icp_get_qty(self, thisE_out, qty=E_out)

  if ((NOT OK_Ein) or (NOT OK_Eout)) then begin
    OK_Einout = func_icp_make_Einout(self, thisE_in, thisE_out, errMsg=thisErrMsg)
    if (NOT OK_Einout) then begin
	  return, 0
    endif
  endif

  OK_Ein = func_icp_get_qty(self, thisE_in, qty=E_in, units=eUnits, $
  	from = from_E_in)
;  	inHeader = E_in_InHEader, $
;	inData = E_in_inData)
  OK_Eout = func_icp_get_qty(self, thisE_out, qty=E_out, $
  	from = from_E_out)
;  	inHeader = E_out_InHEader, $
;	inData = E_out_inData)
 
;  from_E_in = 'input'
;  if (E_in_inheader) then from_E_in = 'header'
;  if (E_in_inData) then from_E_in = 'data'
;  
;  from_E_out = 'input'
;  if (E_out_inheader) then from_E_out = 'header'
;  if (E_out_inData) then from_E_out = 'data'
  
  
  if (debug) then begin
    print, infolabel, "E_in = ", E_in
    print, infolabel, "E_out = ", E_out
    print, infolabel, "thisE_in=", format_list(thisE_in)
    print, infolabel, "thisE_out=", format_list(thisE_out)
  endif
        ;--------------------------------------------
	; make neutron wavelength lambda_f (analyzer) and lambda_i (monochrometer)
	lam_units = '!dave_angstromSym'
	if (eUnits NE 'meV') then begin
	  lam_units = '!dave_angstromSym * sqrt(meV /' + eUnits + ')' 
	endif

        if (debug) then begin
	  print, infolabel, "from_E_in=", from_E_in
	  print, infolabel, "from_E_out=", from_E_out
        endif
	
        OK = self -> binaryOp('/', $
	   	tags1 = 81.80425, $
		from1= 'input', $
	   	tags2 = ['qtyObj', thisE_in[1], '*'], $
		from2= from_E_in, $
;	   	tags2 = ['qtyObj', 'E_in', '*'], $
;		from2= 'data', $	; Might not work with zipped files.
		resTags =  ['qtyObj', 'lambda2_i',  'func_icp_make_resvolume'], $
		name = 'Initial neutron wavelength^2', $
		units = string('(', lam_units, ')^2'), $
		errMsg = thisErrMsg)
	if (NOT OK) then begin
	  thisErrMsg = [thisErrMsg, $
		  "Field 'lambda2_i' not created from 81.80425/'E_in'"]
	  return, 0
	endif

       OK_lambda2_i = func_icp_get_qty(self, ['qtyObj', 'lambda2_i',  '*'], $
       		qty=lambda2_i, $
;  		inHeader = lambda2_i_InHEader, $
;		inData = lambda2_i_inData, $
		from = from_lambda2_i)
  
        if (debug) then print, OK_lambda2_i, " lambda2_i=", lambda2_i, $
		 " from_lambda2_i=",from_lambda2_i 

        OK = self -> binaryOp('^', $	; take the square root.
	   	tags1 = ['qtyObj', 'lambda2_i',  '*'], $
		from1= from_lambda2_i, $
;		from1= 'data', $
	   	tags2 = 0.5, $
		from2= 'input', $
		resTags =  ['qtyObj', 'lambda_i',  '*'], $
		name = 'Initial neutron wavelength', $
		units = lam_units, $
		errMsg = thisErrMsg)
	  
	if (NOT OK) then begin
	  thisErrMsg = [thisErrMsg, $
		  "Field 'lambda_i' not created from lambda2_i'^0.5"]
	  return, 0
	endif

 	self -> remove, /destroy, $
	     	position=['qtyObj', 'lambda2_i', '*']

        ;--------------------------------------------
        OK = self -> binaryOp('/', $
	   	tags1 = 81.80425, $
		from1= 'input', $
	   	tags2 = ['qtyObj', thisE_out[1], '*'], $
		from2= from_E_out, $
;	   	tags2 = ['qtyObj', 'E_out', '*'], $
;		from2= 'data', $	; Might not work with zipped files.
		resTags =  ['qtyObj', 'lambda2_f',  '*'], $
		name = 'Final neutron wavelength^2', $
		UNITS = string('(', lam_units, ')^2'), $
		errMsg = thisErrMsg)
	if (NOT OK) then begin
	  thisErrMsg = [thisErrMsg, $
	  	"Field 'lambda2_f' not created from 81.80425/'E_out'"]
	  return, 0
	endif
	  
        OK_lambda2_f = func_icp_get_qty(self, ['qtyObj', 'lambda2_f',  '*'], $
       		qty=lambda2_f, $
		from = from_lambda2_f)
        if (debug) then print, OK_lambda2_f, " lambda2_f=", lambda2_f, $
		 " from_lambda2_f=",from_lambda2_f 
        OK = self -> binaryOp('^', $	; take the square root.
	   	tags1 = ['qtyObj', 'lambda2_f',  '*'], $
		from1= from_lambda2_f, $
;;		from1= 'data', $
	   	tags2 = 0.5, $
		from2= 'input', $
		resTags =  ['qtyObj', 'lambda_f',  '*'], $
		name = 'Final neutron wavelength', $
		units = lam_units, $
		errMsg = thisErrMsg)
	if (NOT OK) then begin
	  thisErrMsg = [thisErrMsg, $
	  	"Field 'lambda_f' not created from lambda2_f'^0.5"]
	  return, 0
	endif
	  
	self -> remove, /destroy, $
	     	position=['qtyObj', 'lambda2_f', '*']

        ;--------------------------------------------
	; Calculate the angles: theta_f (analyzer) and theta_i (monochrometer)
	OK = self -> getField( /inData, $
		['qtyObj', 'lambda_i', '*'], $
;		['qtyObj', 'lambda_i', 'func_icp_make_resvolume'], $
		qty = lambda_i)		; Should not have errors,
	if (NOT OK) then begin
	  OK = self -> getField( /inHeader, $
		['qtyObj', 'lambda_i', '*'], $
;		['qtyObj', 'lambda_i', 'func_icp_make_resvolume'], $
		qty = lambda_i)		; Should not have errors,
	  if (NOT OK) then begin
	    thisErrMsg = "Failed self->getField(['qtyObj','lambda_i','*']...)"
	    return, 0
	  endif
	endif

	OK = self -> getField($ 
		thisDspacing_mon, $
;		['qtyObj', 'dspacing_mon', '*'], $
		qty = dspacing_i, $
		units = units_dspacing_i)	; Should not have errors,
	if (NOT OK) then begin
	  thisErrMsg = "Failed self->getField(" +format_list(thisDspacing_mon) + $
	  	 "...)"
;	  thisErrMsg = "Failed self->getField(['qtyObj','dspacing_mon','*']...)"
	  return, 0
	endif
	if (lam_units NE units_dspacing_i) then begin
	  thisErrMsg = string("Incompatible units. Initial lambda has units of ", $
	  	lam_units, " while d-spacing has ", units_dspacing_i)
	 return, 0
	endif
	theta_i = asin(lambda_i / (2.0 * dspacing_i))	; units are radians

	OK = self -> getField( /inData, $
		['qtyObj', 'lambda_f', '*'], $
;		['qtyObj', 'lambda_f', 'func_icp_make_resvolume'], $
		qty = lambda_f)		; Should not have errors,
	if (NOT OK) then begin
	  OK = self -> getField( /inHeader, $
		['qtyObj', 'lambda_f', '*'], $
;		['qtyObj', 'lambda_f', 'func_icp_make_resvolume'], $
		qty = lambda_f)		; Should not have errors,
	  if (NOT OK) then begin	
	    thisErrMsg = "Failed self->getField(['qtyObj','lambda_f','*']...)"
	    return, 0
	  endif
	endif

	OK = self -> getField($ 
		thisdspacing_ana, $
;		['qtyObj', 'dspacing_ana', '*'], $
		qty = dspacing_f, $
		units = units_dspacing_f)	; Should not have errors,
	if (NOT OK) then begin
	  thisErrMsg = "Failed self->getField(" + format_list(thisDspacing_ana) + $
	  	"...)"
;	  thisErrMsg = "Failed self->getField(['qtyObj','dspacing_ana','*']...)"
	  return, 0
	endif
	if (lam_units NE units_dspacing_f) then begin
	  thisErrMsg = string("Incompatible units. Final lambda has units of ", $
	  	lam_units, " while d-spacing has ", units_dspacing_f)
	 return, 0
	endif	
	theta_f = asin(lambda_f / (2.0 * dspacing_f))	; units are radians

        ;--------------------------------------------
        ;--------------------------------------------
	; Create the resolution volume correction. Might be upside down.

	E_out_obj = self -> get(position=thisE_out, /inData)
;Wrong	E_out_obj = self -> getField(thisE_out, /inData)
	if (debug) then help, E_out_obj 

	E_out_isConstant = 0
	if (NOT obj_valid(E_out_obj)) then begin
	   if (n_elements(E_out) GT 0) then begin
	     E_out_isConstant = 1
	   endif else begin
	     thisErrMsg = "Failed E_out_obj = self -> getField(" + $
	    	format_list(thisE_out) + ", /inData)"
	     return, 0
	   endelse
	endif else begin
	  if (E_out_obj -> isConstant()) then E_out_isConstant = 1
	endelse

	if (E_out_isConstant) then begin
	  ; The analyszer is fixed. 
	  ; Varying the angle on the monochrometer has no affect on the 
	  ; resolution volume. 
;;;;	  E_out_obj -> get(qty=E_out)
	  resVol = E_out - E_out + 1.0		; Defaults to the value of 1.0.
	  self -> add, /inData,  $
	   tags = ['qtyObj', 'resVol', 'func_icp_make_resvolume'], $
	   contents = obj_new('qtyObj', $
	   	name='Resolution Volume', $
	  	qty = resVol,$
		units='Unitless')
	  ; No need to create a new intensity data field.
	  OK = 1
	endif else begin


	  resVol = ((E_in/E_out)^1.5) * (tan(theta_f)/tan(theta_i))
	  self -> add, /inData,  $
	   tags = ['qtyObj', 'resVol', 'func_icp_make_resvolume'], $
	   contents = obj_new('qtyObj', $
	      name='Resolution Volume ((E_in/E_out)^1.5) * (tan(theta_f)/tan(theta_i))', $
	  	qty = resVol,$
		units='Unitless')

          ;--------------------------------------------
	  ; Normalize the intensity.
	  
;	   resLabel = thisRawIntensity[1] + '_ResVol'
           OK = self -> binaryOp('*', $
	   	tags1 = thisRawIntensity, $
		from1= 'data', $
	   	tags2 = ['qtyObj', 'resVol', 'func_icp_make_resvolume'], $
		from2= 'data', $
		resTags = thisResTags, $
;		resTags = ['qtyObj', resLabel, 'func_icp_make_resvolume'], $
;;		resTags = ['qtyObj', 'INTENSITY_RESVOL', 'func_icp_make_resvolume'], $
		name = thisResName, $
		units = thisResUnits, $
;		name = 'Neutron Intensity normalized by resolution volume', $
;		units = 'Arbritary', $
		errMsg = thisErrMsg)

	  if (NOT OK) then begin
	    thisErrMsg = [thisErrMsg, string("Failed 'INTENSITY_NORMED' =", $
	    	 thisRawIntensity[1], "* 'resVol'")]
	    return, 0
	  endif
	endelse
	
   if (OK) then begin
     treatString = ['; Calculated incoming and outgoing neutron energies, wavelengths.', $
   	'; Calculate the resolution volume and correct the neutron intensity.', $
;        string(OK, ' = func_icp_make_resvolume( SpecObj $'), $
        string('OK = func_icp_make_resvolume( SpecObj $'), $
   	string(', rawIntensity=', format_list(thisRawIntensity), ', $'), $
   	string(', E_in=', format_list(thisE_in), ', $'), $
   	string(', E_out=', format_list(thisE_out), ', $'), $
   	string(', dspacing_mon=', format_list(thisDspacing_mon), ', $'), $
   	string(', dspacing_ana=', format_list(thisDspacing_ana), ', $'), $
   	string(', resTags=', format_list(thisResTags), ', $'), $ 
   	string(', resName=', thisResName, ', $'), $ 
   	string(', resUnits=', thisResUnits, ', $'), $ 
	 ')']

    OK_treat = self -> appendField($
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'], /inHeader,$
	qty= treatString ) 
  endif

  return, OK
	
	
end		; func_icp_make_resVolume
