; $Id: mn_tname_2_tcode.pro,v 1.2 2003/09/12 15:25:09 tanya Exp $
;+
; NAME:
; 	mn_tname_2_tcode
;
; PURPOSE:
;
;	Converts the type name to the type code
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;	DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED:
;
; EXAMPLE
;	IDL> print, mn_tname_2_tcode('INT')
;		2
;
; MODIFICATION HISTORY:
;	Created Spring, 2003. Tanya Riseman
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;================================================================================
;*
function mn_tname_2_tcode, $	; returns integer containing the IDL type code
   tname			; string code for the IDL data type. E.g., 'INT'
;/
   
   tcode = -1
   tname_temp = strupcase(tname)
   
   case tname_temp of
    'UNDEF':	tcode= 0   ; IDL_TYP_UNDEF	Undefined	<None>	
    'BYTE':	tcode=1   ; IDL_TYP_BYTE	Unsigned byte	UCHAR	
    'INT':	tcode=2   ; IDL_TYP_INT	16-bit integer	short	
    'LONG':	tcode=3   ; IDL_TYP_LONG	32-bit integer	IDL_LONG	
    'FLOAT':	tcode=4   ; IDL_TYP_FLOAT	Single precision floating float
    'DOUBLE':	tcode=5   ; IDL_TYP_DOUBLE  Double precision floating double
    'COMPLEX':	tcode=6   ; IDL_TYP_COMPLEX Single precision complex IDL_COMPLEX	
    'STRING':	tcode=7   ; IDL_TYP_STRING String	IDL_STRING
    'STRUCT':	tcode=8   ; IDL_TYP_STRUCT Structure	See StructureVariables
    'DCOMPLEX':	tcode=9   ; IDL_TYP_DCOMPLEX Double precision complex IDL_DCOMPLEX
    'PTR':	tcode=10  ; IDL_TYP_PTR	32-bit integer	IDL_ULONG
    'OBJREF':	tcode=11  ; IDL_TYP_OBJREF 32-bit integer	IDL_ULONG
    'UNINT':	tcode=12  ; IDL_TYP_UINT	Unsigned 16-bit integer	IDL_UINT
    'ULONG':	tcode=13  ; IDL_TYP_ULONG	Unsigned 32-bit integer	IDL_ULONG
    'LONG64':	tcode=14  ; IDL_TYP_LONG64 64-bit integer	IDL_LONG64
    'ULONG64':	tcode=15  ; IDL_TYP_ULONG64 Unsigned 64-bit integer IDL_ULONG64

;	Only works under windows
;    'UNDEF':	tcode= IDL_TYP_UNDEF
;    		;0   ; IDL_TYP_UNDEF	Undefined	<None>	
;    'BYTE':	tcode=IDL_TYP_BYTE;
;    		;1   ; IDL_TYP_BYTE	Unsigned byte	UCHAR	
;    'INT':	tcode=IDL_TYP_INT
;    		;2   ; IDL_TYP_INT	16-bit integer	short	
;    'LONG':	tcode=IDL_TYP_LONG
;    		;3   ; IDL_TYP_LONG	32-bit integer	IDL_LONG	
;    'FLOAT':	tcode=IDL_TYP_FLOAT
;    		;4   ; IDL_TYP_FLOAT	Single precision floating float
;    'DOUBLE':	tcode=IDL_TYP_DOUBLE
;    		;5   ; IDL_TYP_DOUBLE  Double precision floating double
;    'COMPLEX':	tcode=IDL_TYP_COMPLEX
;    		;6   ; IDL_TYP_COMPLEX Single precision complex IDL_COMPLEX	
;    'STRING':	tcode=IDL_TYP_STRING
;    		;7   ; IDL_TYP_STRING String	IDL_STRING
;    'STRUCT':	tcode=IDL_TYP_STRUCT
;    		;8   ; IDL_TYP_STRUCT Structure	See StructureVariables
;    'DCOMPLEX':	tcode=IDL_TYP_DCOMPLEX
;    		;9   ; IDL_TYP_DCOMPLEX Double precision complex IDL_DCOMPLEX
;    'PTR':	tcode=IDL_TYP_PTR
;    		;10  ; IDL_TYP_PTR	32-bit integer	IDL_ULONG
;    'OBJREF':	tcode=IDL_TYP_OBJREF
;    		;11  ; IDL_TYP_OBJREF 32-bit integer	IDL_ULONG
;    'UNINT':	tcode=IDL_TYP_UINT
;    		;12  ; IDL_TYP_UINT	Unsigned 16-bit integer	IDL_UINT
;    'ULONG':	tcode=IDL_TYP_ULONG
;    		;13  ; IDL_TYP_ULONG	Unsigned 32-bit integer	IDL_ULONG
;    'LONG64':	tcode=IDL_TYP_LONG64
;    		;14  ; IDL_TYP_LONG64 64-bit integer	IDL_LONG64
;    'ULONG64':	tcode=IDL_TYP_ULONG64
;    		;15  ; IDL_TYP_ULONG64 Unsigned 64-bit integer IDL_ULONG64
   endcase

   return, tcode

end
