; $Id: qtyobj__define.pro,v 1.8 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
;       qtyObj
;
; PURPOSE:
;
;	This object qtyObj contains values, with associated units and errors.
; It is intended as a container class for holding scalar, 1D and 2D data
; of any IDL numeric, or string type. Of course, the errors
; only make sense for numeric types. It is intended to be the building
; block for database class "DAVE_tagContainer", which is used in term by
; the class "specObj" (containing the database plus its focus objects).
; 
;	This qtyObj is an object MINIMALLY containing the 
;	  1. self.name, a string
;	  2. self.units, a string
;	  3. self.ptr_qty which is a pointer to an array of quantities.
;	  4. self.rank gives the number of dimensions of the array
;	     that self.ptr_qty points to. 0, 1, 2, 3, 4 are OK. 
;		and self.n_rows which specify if
;	    the contents of self.ptr_qty are scalar, 1D or 2D.
;		self.dim  array of the sizes of the dimensions.
;	  5. self.dim is an array [#,#,#,#] containing the dimensions
;	     of the array that self.ptr_qty points to. It is only used up 
;	     to the length self.rank, the rest is zero-filled.
;	     For example, a 2D data set might have [2, 128, 0,0].
;	  6. selt.tname which is the type of the quantities that are 
;	     pointed to by self.ptr_qty.
;	     The type is allowed to be numeric (INT, FLOAT, etc.) or STRING. 
;	     Note: self.ptr_err and self.ptr_nerr need not be the
;	     the same type. For example, self.ptr_qty might point to UNSIGNED INT
;	     while self.ptr_err might point to float.
;
;	Optionally, qtyObj uses 
;	  1. self.ptr_err for symmteric errors or the positive portion of
;	     asymmetric errors. Its contents must have the same dimensions 
;            as self.ptr_qty or be NULL. Test using ptr_valid(self.ptr_err).
;	     It need not have the same type as self.ptr_qty.
;	  2. self.ptr_nerr for negative errors.
;	     Its contents must have the same dimesions as self.ptr_qty or be NULL.
;	     It need not have the same type as self.ptr_qty.
;	  3. self.ptr_legend, a list of STRINGs which is an alternative
;	     method of referring to the contents of the other pointers,
;	     the alternative being by column and row.
;	     Its contents must have the same dimensions are self.ptr_qty or be NULL.
;	     It is intended to be of type STRING, but other types seem to work.
;	  4. self.StyleErr has values of 
;	     '' 		Default. Assumed to be Gaussian errors.
;	     'poisson'		Poisson errors (e.g. counting statistics).
;	     'center_step' 	Using self.ptr_err as a step size. For ICP files.
;	     'start_step' 	Using self.ptr_err as a step size. For ICP files.
;	     'cov_matrix'	self.ptr_err has dimensions of dim^2 rather than dim
;;
;	 Quick example: self.ptr_qty might point to an array of unsigned longs
; which correspond to detected events while the self.ptr_err would containing
; FLOATs equal to sqrt(float(*(self.ptr_qty))). 
; self.ptr_err would not need any more accuracy than FLOAT provides.
; self.ptr_nerr=ptr_new() would just be a null pointer, as it is unneeded.
; 
;.........................................................................
; Note: array size temp = size(myarray) returns
; temp[1] = n_cols and temp[2] = n_rows (as per IDL documentation)
; But when you print it out, it looks like [rows, cols].
; To access internals of the array, in sepcObj, use the keywords 
; 	position = [icol, irow]	or
;	position = "thatLegend"
; 
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Objects. DAVE aaplication programming.
;
; OBJECT DEFINTION
;	 <LI><A HREF="#QTYOBJ__DEFINE_arguments">QTYOBJ__DEFINE_arguments</A>
;
;
; PRIVATE FUNCTION AND PROCEEDURES:
;
; 	function qtyObj_remove1d, iwhere, $	; Helper function for REMOVE method.
;
; OTHER FILES AND ROUTINES USED:
;
;	function mn_tname_2_tcode, tname   	; File mn_tname_2_tcode.pro
;	function format_size, ptr_Array = ptr_Array, Array = anArray
;						; File format_size.pro
; 	pro qtyObj_example, whichexample	; for testing
; 	pro test_IS, thisObj			; for testing
; 	pro check_heap				; for testing
; 	function whatsIt			
;
; 
; COMMON BLOCKS:
;
;   common QTYOBJ_DIMS, DIM_MAX, DIM_WARN 
;
; INCLUDE FILES:
;	<A HREF="#GENERIC__HELCLASS_FRAG.PRO">GENERIC__HELCLASS_FRAG.PRO</A>
;	<A HREF="#GENERIC__HELP_OBJ_FRAG.PRO">GENERIC__HELP_OBJ_FRAG.PRO</A>
;	<A HREF="#GENERIC__PRINT_FRAG.PRO">GENERIC__PRINT_FRAG.PRO</A>
;
; EXAMPLE
;	...
;
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Feb, 2003.
;  Changed from descripObj class (scalar only) to Descrip2DObj (0D, 1D, or 2D.) ;-
;
;	BUILD can not resolve this function whatsIt, due to it being late in 
;	the alphabet. Rename whatsIt to AAA_whatsIt. TMR. Aug 1st, 2003.
;
;	Changed name from AAA_whatsIt to whatits. TMR. Aug 6, 2003.
;
;	TMR Aug 6, 2003. Added ptr_valid's to scalar case for GETONEVALUE
;	TMR Sept 5, 2003. Added routines qtyObj__prepErrors and qtyObj__BinaryOp.
;	and qtyObj_UnitaryOp.
;
;-
;
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	qtyObj::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro qtyObj::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $		; string.
  lun=thisLun, $	; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs	 ; Optionally print contents of pointers.
;/
	  @generic__print_frag.pro
  
end		; ::print_obj

;======== HELP method common to all classes ===============================
;+
; NAME:
; 	qtyObj::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;-
;*
pro qtyObj::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;/
;  ; Note: The following keywords are only used by internal recrusion call.
;;;;  methodsOnly=methodsOnly,  helpStr=strArray, class=thisClass, level=level, $
;  methodsOnly=methodsOnly,  $
;  helpProStr=helpProStr, helpFuncStr=helpFuncStr, class=thisClass, level=level, $
; 	lun=thisLun 	; Optionally, write to a file. If absent, to screen

	  @generic__help_obj_frag.pro
end		; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	qtyObj::helpclass
;
; PURPOSE:
;	Helper method for qtyObj::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro qtyObj::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/
; ; Note: The following keywords are only used by internal recrusion call.
;   helpProStr=helpProStr, helpFuncStr=helpFuncStr, class=thisClass, level=level, $
;;;;   helpStr=helpStr, class=thisClass, level=level, $
; 	lun=thisLun 	; Optionally, write to a file. If absent, to screen

	  @generic__helpClass_frag.pro
end		; ::HelpClass

;========= End of methods common to all classes =============================
;=============================================================================
;*
function qtyObj::init, $	; Returns 0/1 for failure/success
  name = thisname, $		; String
  units = thisUnits, $		; String. Units
  rank = thisrank, $		; Integer. number of dimensions
  dim = thisDim, $		; actuals dimensions. Use # or [#,#], etc.
  tname = thisTname, $		; type 'INT', 'FLOAT', 'STRING', 'OBJREF', etc.
  initValue = thisInitValue, $  ; Initial value for array *(self.ptr_qty)
  StyleErr = thisStyleErr, $    ; String. '' is the default. 
  				; Options 'center_step', 'start_step'
	; qty and ptr_qty both get stored in self.ptr_qty.
	; keyword qty is pass by value.
	; keyword ptr_qty is pass by reference.
	; qty and *(ptr_qty) can be any numeric type or string. 
	; Size of *(ptr_qty) must agree with self.rank and self.dim.
  ptr_qty = thisptr_qty, $	; shares memory using the same pointer.
  qty = thisqty, $		; shares memory using new pointer.
	; err and ptr_err both get stored in self.ptr_err.
	; err and *(ptr_err) can be any numeric type,
	; or not initialized (NULL). If not NULL,
	; size of *(ptr_err) must agree with self.rank and self.dim.
  ptr_err = thisptr_err, $	; Optional error on the values
  err = thiserr, $		; Optional error on the values
	; nerr and ptr_nerr both get stored in self.ptr_nerr.
	; nerr and *(ptr_nerr) can be any numeric type, 
	; or not initialized (NULL). If not NULL,
	; size of *(ptr_nerr) must agree with self.rank and self.dim.
  ptr_nerr = thisptr_nerr, $	; Optional negative error on the values
  nerr = thisnerr, $		; Optional negative error on the values
	; legend and ptr_legend both get stored in self.ptr_legend.
	; legend and *(ptr_legend) must be STRING type, 
	; or not initialized (NULL). If not NULL,
	; size of *(ptr_legend) must agree with self.rank and self.dim.
  ptr_legend = thisptr_legend, $; shares memory using the same pointer.
  legend = thislegend		; shares memory using new pointer.
;/
;;;  obj=thisObj ; Copy contents of same kind of object into self

; Note: if rank = 0, a scalar or dim = [1], a scalar.
 
  infolabel = "qtyObj::init >> "  
  debug = 0	;1
  
  common QTYOBJ_DIMS, DIM_MAX, DIM_WARN 
  DIM_WARN = 3
  DIM_MAX = 4

  self.tname = 'UNDEF'	; default value
 
;-------------------------------------------------------------------------  
  success = 1
  if ((n_elements(thisname) gt 0) or (n_elements(thisUnits) gt 0) $
     or (n_elements(thisRank) gt 0) or (n_elements(thisDim) gt 0) $
     or (n_elements(thisStyleErr) gt 0) $
     or (n_elements(thisptr_qty) gt 0) or (n_elements(thisqty) gt 0) $
     or (n_elements(thisptr_legend) gt 0) or (n_elements(thislegend) gt 0) $
     or (n_elements(thisptr_err) gt 0) or (n_elements(thiserr) gt 0)  $
     or (n_elements(thisptr_nerr) gt 0) or (n_elements(thisnerr) gt 0)) $
     then begin
;Debug       print, infolabel, " success = self->set(...)"
    success = self->set(  name = thisname, $
         units = thisUnits, $
	 rank = thisrank, $
	 dim = thisDim, $
 	 tname = thisTname, $
         initValue = thisInitValue, $
         StyleErr = thisStyleErr, $   ' default''. Options 'center_step', 'start_step'
	 ptr_qty = thisptr_qty, $
	 qty = thisqty, $
	 ptr_err = thisptr_err, $
	 err = thiserr, $
	 ptr_nerr = thisptr_nerr, $
	 nerr = thisnerr, $
	 ptr_legend = thisptr_legend, $
	 legend = thislegend $
;;;	 obj=thisObj
	 )
   endif

; if (debug) then begin
;    print, infolabel, self.name, " has error style '", self.styleErr, "'"
;    if (n_elements(thisStyleErr) GT 0) then begin
;    	print, infolabel, "   ", self.name, " intended '",  thisStyleErr, "'"
;    endif
; endif  	

 return, success

end		; function qtyObj::init

;=============================================================================
;*
function qtyObj::clone		; Returns an object which is a copy of self
;/

   infolabel = "qtyObj::clone >> "  

    success = self->get(  name = thisname, $
         units = thisUnits, $
;Dupl	 rank = thisrank, $
;Dupl	 dim = thisDim, $
;Dupl 	 tname = thisTname, $
         StyleErr = thisStyleErr, $   ' default''. Options 'center_step', 'start_step'
;	 ptr_qty = thisptr_qty, $
	 qty = thisqty, $
;	 ptr_err = thisptr_err, $
	 err = thiserr, $
;	 ptr_nerr = thisptr_nerr, $
	 nerr = thisnerr, $
;	 ptr_legend = thisptr_legend $
	 legend = thislegend $
	 )
  
  clone = obj_new('qtyObj', $
 	 name = thisname, $
         units = thisUnits, $
;Dupl	 rank = thisrank, $
;Dupl	 dim = thisDim, $
;Dupl 	 tname = thisTname, $
         initValue = thisInitValue, $
         StyleErr = thisStyleErr, $   ' default''. Options 'center_step', 'start_step'
;	 qty = (*thisptr_qty), $
	 qty = thisqty, $
;	 err = (*thisptr_err), $
	 err = thiserr, $
;	 nerr = (*thisptr_nerr), $
	 nerr = thisnerr, $
;	 legend = (*thisptr_legend) $
	 legend = thislegend $
    	  )
	  
;Debug  if (n_elements(thisErr) GT 0) then print, infolabel, "thisErr=",  thisErr 

  return, clone
  
end		; function qtyObj::clone

;=============================================================================
;*
function qtyObj::biggestDim	
	; Returns integer with the largest dimension of multi-minensional data.
;/

   if (self.rank eq 0) then begin		; scalar
     return, self.dim
   endif else begin
     if (self.rank eq 1) then begin		; scalar
       return, self.dim
     endif   else begin
       maxdim = self.dim[0]
       for i = 1, (self.rank-1) do begin
         if (maxdim LT self.dim[i]) then maxdim = self.dim[i]
       endfor
       return, maxdim
     endelse
   endelse
end

;=============================================================================
;*
function qtyObj::hasError
	; Returns 1 (true) if (self.ptr_err) is not a null pointer. 0 otherwise.
;/
   hasIt = 0
   if (ptr_valid(self.ptr_err)) then begin
     if (self.StyleErr eq '') then  begin
       if (n_elements(*(self.ptr_err)) EQ 1) then begin
         if (*(self.ptr_err) NE 0.0) then begin
           hasIt = 1
         endif 
       endif else begin
        if (norm(*(self.ptr_err)) GT 0.0) then begin
           hasIt = 1
         endif          
       endelse
     endif
   endif
   
   return, hasIt
    
end

;=============================================================================
;*
function qtyObj::hasStep
	; Returns 1 (true) if (self.ptr_err) is not a null pointer. 0 otherwise.
;/
   hasIt = 0
   if (ptr_valid(self.ptr_err)) then begin
     if ((self.StyleErr eq 'center_step') OR $
     		(self.StyleErr eq 'start_step')) then  begin
       if (n_elements(*(self.ptr_err)) EQ 1) then begin
         if (*(self.ptr_err) NE 0.0) then begin
           hasIt = 1
         endif 
       endif else begin
        if (norm(*(self.ptr_err)) GT 0.0) then begin
           hasIt = 1
         endif          
       endelse
     endif
   endif
   
   return, hasIt
    
end

;=============================================================================
;*
function qtyObj::isScalar		; Returns 0/1 for TRUE/FALSE
;/
	; Returns 1 (true) if *(self.ptr_qty) is a scalar. 0 otherwise.
   if (self.rank eq 0) then begin		; scalar
     return, 1
   endif   else begin
     return, 0
   endelse

end

;=============================================================================
;*
function qtyObj::isNumeric	; Returns 0/1 for TRUE/FALSE
;/
	; Returns 1 (true) if *(self.ptr_qty) has a numeric type 
	; (as a scalar or array). 
	; Return 0 (false) if has type STRING or OBJREF.
; Probably won't have PTR  Pointer type. UNDEF is not properly
; initialized. (Perhaps a pruned item). Should not be object, either

   if ((self.tname  eq 'OBJREF') OR (self.tname  eq 'STRING') $
   	or (self.tname  eq 'PTR') or (self.tname  eq 'UNDEF'))  then begin
     return, 0
   endif   else begin
     return, 1
   endelse
   
end

;=============================================================================
;*
function qtyObj::isConstant	; returns 0/1 for TRUE/FALSE
;/
; Tests qty, err and nerr to see if all the elements are 
; the same. Legend is ignored.
; Returns -1 (true) if *(self.ptr_qty) is a scalar. Trivial case.
; Returns 1 if all members of the array(s) *(self.ptr_qty), *(self.ptr_err), 
; *(self.ptr_nerr) are constant. Err and NErr might not be arrays.
; Returns is the members are varying.

   ;................................................
   if (self.rank EQ 0)  then begin		; scalar
     return, -1
   endif      
   ;................................................

   qty_same = 1
   err_same = 1
   nerr_same = 1

   do_err = 0
   do_nerr = 0
   if (ptr_valid(self.ptr_err)) then begin
     if (n_elements(*(self.ptr_err)) GT 1) then begin
       do_err = 1
     endif
   endif
   if (ptr_valid(self.ptr_nerr)) then begin
     if (n_elements(*(self.ptr_nerr)) GT 1) then begin
       do_nerr = 1
     endif
   endif

      
   if (self.rank EQ 2) then begin		; 2D
;;;     print, infolabel, "n_cols=", n_cols, ' n_rows=n_row=", n_rows
     firstqty = (*(self.ptr_qty))[0, 0]
     if (do_err) then firstErr = (*(self.ptr_err))[0, 0]
     if (do_nerr) then firstNErr = (*(self.ptr_nerr))[0, 0]
     n_cols = self.dim[0]
     n_rows = self.dim[1]     
     for icol = 0, (n_cols-1) do begin
       for irow = 0, (n_rows-1) do begin
;     for icol = 0, (self.n_cols-1) do begin
;       for irow = 0, (self.n_cols-1) do begin
         qty_same = qty_same AND (firstQty EQ (*(self.ptr_qty))[icol, irow]) 
         if (do_err) then begin
	   err_same = err_same AND (firstErr EQ (*(self.ptr_err))[icol, irow]) 
	 endif
         if (do_nerr) then begin
           nerr_same = nerr_same AND (firstNErr EQ (*(self.ptr_nerr))[icol, irow]) 
	 endif
       endfor
     endfor
   endif		; 2D
   
   ;................................................
   if ((self.rank EQ 1)) then begin		; 1D       
     firstqty = (*(self.ptr_qty))[0]
     if (do_err) then firstErr = (*(self.ptr_err))[0]
     if (do_nerr) then firstNErr = (*(self.ptr_nerr))[0]
     n_cols = self.dim[0]
     for icol = 0, (n_cols-1) do begin
         qty_same = qty_same AND (firstQty EQ (*(self.ptr_qty))[icol]) 
         if (do_err) then begin
	   err_same = err_same AND (firstErr EQ (*(self.ptr_err))[icol]) 
	 endif
         if (do_nerr) then begin
           nerr_same = nerr_same AND (firstNErr EQ (*(self.ptr_nerr))[icol]) 
	 endif
     endfor
   endif 			; 1D
   
  all_same = (qty_same AND err_same AND nerr_same)
  return, all_same

end

;=============================================================================
;+
; NAME:
; 	qtyObj::setString
;
; PURPOSE:
;	Helper method for qtyObj::set.
;-
;*
function qtyObj::setString, $	; Returns 0/1 for failure/success.
  name = thisname, $		; String. self.name, suitable for plot
  units = thisunits, $		; String. self.units, units for qty, err, nerr.
  StyleErr = thisStyleErr	; String. for self.styleErr
  			; '' is the default. Options 'center_step', 'start_step'
;/
;  intendedTag = thisIntendedTag, $	; short name, suitable for GUI buttons

  infolabel = "qtyObj::setString >> "  
  debug = 0	;1
  nsuc = 0
  
  ;-------------------------------------------------------
  if (n_elements(thisname) GT 0) then begin
    self.name = thisname
    nsuc = nsuc + 1
  endif
  
;  ;-------------------------------------------------------
;  if (n_elements(thisIntendedTag) GT 0) then begin
;    self.IntendedTag = thiIntendedsTag
;    nsuc = nsuc + 1
;  endif
  
  ;-------------------------------------------------------
  if (n_elements(thisunits) GT 0) then begin
    self.units = thisunits
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (n_elements(thisStyleErr) GT 0) then begin
    self.StyleErr = thisStyleErr
    nsuc = nsuc + 1
  endif
  
  ;-----------------------------------------
  success = (nsuc gt 0)

;     if ((DEBUG) AND (n_elements(thisStyleErr) GT 0)) then begin
;       print, infolabel, self.name, " with this styleErr = '", thisStyleErr, "'"
;       print, infolabel, self.name, "   has self.styleErr='", self.styleErr, "'"
;     endif
  
  if (success EQ 0) then begin
     print, infolabel, self.name, ' No recognized keywords provided'
  endif

  return, success 

end		;; qtyObj::setString

;=============================================================================
;+
; NAME:
; 	qtyObj::getString
;
; PURPOSE:
;	Helper method for qtyObj::get.
;-
;*
function qtyObj::GetString, $		; Returns 0/1 for failure/success.
   argPresent = thisArgPresent, $	; anonymous structure.
   	; ARG_PRESENT() function does not work on propagated keywords 
	; from other calling routines. Needed for get* methods.
  ;-----------------------------------------
  name = thisname, $		; Returns string. self.name, suitable for plot
  units = thisunits, $ 		; Returns string. self.units, units for qty, err, nerr.
  StyleErr = thisStyleErr, $    ; Returns string. for self.styleErr
  			; '' is the default. Options 'center_step', 'start_step'
  axisTitle = thisAxisTitle, $ 	; Returns string for plot axis label. 
			  	; Includes name, and units.
  sizeString = thisSizeString, $; Returns string in format [#,#] 
  				; based on self.rank and self.dim
  some=thisSome, $	; 1 line summary of one qtyObj Object. No vectorvalues.
  all=thisAll, $  	; 1 line summary of one qtyObj Object
  everything=thisEverything	; String array containing all contents of one qtyObj Object
;/

;  intendedTag = thisIntendedTag, $	; short name, suitable for GUI buttons
;  noLimt = thisNoLimit		; Don't truncate output of large arrays.

  infolabel = "qtyObj::GetString >> " 
  debug = 0;	1 
  nsuc = 0
  
  
  numericmaxperline = 6	;8 characters per number. 
  stringmaxperline = 1	;8 characters per number. 
;  maxperline = 8	;8 characters per number. 

;  if (keyword_set(thisNoLimit)) then begin
;     max_shown = 2048	; Show up to medium sized arrays
;  endif else begin
     max_shown = 10	; Don't show really large arrays
;     max_shown = 24	; Don't show really large arrays
;  endelse  

;Debug  print, infolabel, self.name, " has error style '", self.styleErr, "'"

  ;-------------------------------------------------------
  ; "Inherit" passed parameters from calling method via thisArgPresent.
  ; Otherwise, this method might calculate more things than are required.
  if (NOT arg_present(thisArgPresent)) then begin   
   ArgPresent = {$			; make this be anonymous structure
	  name:		arg_present(thisName), $	
;	  intendedTag:	arg_present(thisIntendedTag), $	
	  units:	arg_present(thisUnits), $
	  StyleErr:	arg_present(thisStyleErr), $
	  axisTitle:	arg_present(thisAxisTitle), $
	  sizeString:	arg_present(thisSizeString), $
	  some:		arg_present(thisSome), $
	  all:		arg_present(thisAll), $
	  everything:	arg_present(thisEverything) $
	  }
  endif else begin
    ; ArgPresent had better include the tags "name", "intendedTag", and "units"
    ArgPresent = thisArgPresent
  endelse

;Debug
;  message, /info, string(self.name, " Entered with argPresent=")
;  printArgPresent, ArgPresent   
  

  ;-------------------------------------------------------
  if (argPresent.name) then begin
    thisname = self.name
    nsuc = nsuc + 1
  endif
  
;  ;-------------------------------------------------------
;  if (argPresent.intendedTag) then begin
;    thisintendedTag = self.intendedTag
;    nsuc = nsuc + 1
;  endif
  
  ;-------------------------------------------------------
  if (argPresent.units) then begin
    thisunits = self.units
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.StyleErr) then begin
    thisStyleErr = self.StyleErr
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.axisTitle) then begin
      if (self.units NE '') then begin
        str_units = ' (' + self.units + ')'
      endif else begin
       str_units = ''
      endelse
      thisAxisTitle =  self.name +  str_units      
      nsuc = nsuc + 1
   endif

  ;-------------------------------------------------------
  if (argPresent.sizeString) then begin
      thisSizeString =  format_size(self.dim)      
      nsuc = nsuc + 1
;      print, infolabel, 'thisSizeString = ',  thisSizeString 
   endif

 ;-------------------------------------------------------
  if (argPresent.Some or argPresent.All or $
  	argPresent.Everything) then begin

   maxperline = stringmaxperline
   if (self.tname NE 'STRING') then begin
     maxperline = numericmaxperline
   endif

   str_type = " '" + self.tname + "'"		;;  + ' '
   str_size = format_size(ptr_array=self.ptr_qty)
   str_name = "'" +self.name + "'" 		;;+ ' ' + str_size
   localClassSize = '{' + obj_class(self) +  '}' + $
	 format_size(ptr_array=self.ptr_qty)
   
   str_units = ''
   if (n_elements(self.units) gt 0) then begin
	  if (self.units EQ '') then begin
	    str_units = ''
	  endif else begin
	    str_units = string(' (', self.units, ') ')
	  endelse
   endif
	
   str_legend = ''
   if (ptr_valid(self.ptr_legend)) then begin
;;     if (n_elements(*(self.ptr_legend)) eq 1) then begin
	  str_legend = string("<", *(self.ptr_legend), "> ")
;;     endif
   endif 
   
   str_qty = ''
   if (ptr_valid(self.ptr_qty)) then begin
     if (n_elements(*(self.ptr_qty)) eq 1) then begin
       str_qty = whatsIt(*(self.ptr_qty))
     endif else begin
      temp_array = *(self.ptr_qty)
      nshow = n_elements(*(self.ptr_qty))
      if (nshow GT max_shown) then nshow = max_shown
       for i = 0, (nshow -1) do begin
;      if (n_elements(*(self.ptr_qty)) LT max_shown) then  begin
;       for i = 0, (n_elements(*(self.ptr_qty)) -1) do begin
        aWhat =  whatsIt( (*(self.ptr_qty))[i])
        str_qty = str_qty +  aWhat
;;;        str_qty = str_qty +  whatsIt( (*(self.ptr_qty))[i])
       if (i NE (n_elements(*(self.ptr_qty)) -1) ) then begin
	  str_qty = str_qty + ', '
       endif
      endfor
;     endif
      if (n_elements(*(self.ptr_qty)) GE max_shown) then  begin
 	  str_qty = str_qty + ' ... '
      endif
    endelse
   endif

   str_perr = ''
   if (ptr_valid(self.ptr_err)) then begin
     if (n_elements(*(self.ptr_err)) eq 1) then begin
        str_perr =  whatsIt(*(self.ptr_err))
     endif else begin
      nshow = n_elements(*(self.ptr_err))
      if (nshow GT max_shown) then nshow = max_shown
       for i = 0, (nshow -1) do begin
;      if (n_elements(*(self.ptr_err)) LT max_shown) then  begin
;       for i = 0, (n_elements(*(self.ptr_err)) -1) do begin
         str_perr =  str_perr + whatsIt( (*(self.ptr_err))[i])
       endfor
;      endif
      if (n_elements(*(self.ptr_err)) GE max_shown) then  begin
 	  str_perr = str_perr + ' ... '
      endif
     endelse

   endif

   str_nerr = ''
   if (ptr_valid(self.ptr_nerr)) then begin
     if (n_elements(*(self.ptr_nerr)) eq 1) then begin
        str_nerr = whatsIt(*(self.ptr_nerr))
     endif else begin
      nshow = n_elements(*(self.ptr_nerr))
      if (nshow GT max_shown) then nshow = max_shown
       for i = 0, (nshow -1) do begin
;      if (n_elements(*(self.ptr_nerr)) LT max_shown) then  begin
;       for i = 0, (n_elements(*(self.ptr_nerr)) -1) do begin
         str_nerr =  str_nerr + whatsIt( (*(self.ptr_nerr))[i] )
       endfor
      if (n_elements(*(self.ptr_nerr)) GE max_shown) then  begin
 	  str_nerr = str_nerr + ' ... '
      endif
;      endif
     endelse

   endif

   str_err = ''
   if (str_perr NE '') then begin
     if (str_nerr NE '') then begin
       str_err = ' + ' + str_perr + ' - ' + str_nerr
     endif else begin
       if (self.StyleErr EQ 'start_step') then begin
         str_err = ' start with step ' + str_perr 
       endif else begin
         if (self.StyleErr EQ 'center_step') then begin
           str_err = ' center with step ' + str_perr 
         endif else begin
           str_err = ' +- ' + str_perr 
         endelse 
       endelse 
;Simple       str_err = ' +- ' + str_perr 
     endelse
   endif

  endif		;; (argPresent.Some or argPresent.All or argPresent.Everything) 


 ;-------------------------------------------------------
  if (argPresent.Some) then begin
   thisSome = string(str_name,  str_type, str_size, ' ',  $
;   		str_qty, str_err, str_nerr,  $
   		str_units, str_legend)
      nsuc = nsuc + 1
  endif

 ;-------------------------------------------------------
  if (argPresent.All) then begin
   thisAll = string(str_name,  str_type, str_size, ' ', $
   	str_qty, str_err, str_nerr, $
   	str_units, str_legend)
      nsuc = nsuc + 1
  endif

 ;-------------------------------------------------------
 if (argPresent.Everything) then begin
   aline = string(localClassSize, ' ', str_name, str_type, ' ')
   if ptr_valid(self.ptr_qty) then begin
     if (n_elements(*(self.ptr_qty)) eq 1) then aline = aline + str_qty
   endif
   if ptr_valid(self.ptr_err) then begin 
     ; str_err combines both positive errors (self.ptr_err) 
     ; and negative errors (self.ptr_nerr) when the negative errors are present.
     ; Negative errors can not be present without positive errors.
     if (n_elements(*(self.ptr_err)) eq 1) then aline = aline + str_err
   endif
   aline = aline + str_units
   if ptr_valid(self.ptr_legend) then begin
     if (n_elements(*(self.ptr_legend)) eq 1) then aline = aline + str_legend
   endif
   eStyle = ''
   if (self.styleErr NE '') then eStyle = " '" + self.styleErr + "'"
   thisEverything = [string(aline, eStyle)] 
  
  noprefix = '     '		; 4 spaces = "'qty=' + '['"
  icat_qty = 0
  icat_err = 1
  icat_nerr = 2
  icat_legend = 3
  num_cols = self.dim[0]
  num_rows = self.dim[1]
  for icat = 0, 3 do begin
   if (icat eq icat_qty) then begin
     if (ptr_valid(self.ptr_qty) ) then begin
;       if (n_elements(*(self.ptr_qty)) GT max_shown) then  begin
;         goto, skip_this
;       endif
       where_ptr = self.ptr_qty
     endif else begin
       goto, skip_this
     endelse
     prefix = 'qty= '
   endif

   if (icat eq icat_err) then begin
     if (ptr_valid(self.ptr_err) ) then begin
;       if (n_elements(*(self.ptr_err)) GT max_shown) then  begin
;         goto, skip_this
;       endif
       where_ptr = self.ptr_err
     endif else begin
       goto, skip_this
     endelse
     prefix = 'perr='
   endif

   if (icat eq icat_nerr) then begin
     if (ptr_valid(self.ptr_nerr) ) then begin
;       if (n_elements(*(self.ptr_nerr)) GT max_shown) then  begin
;         goto, skip_this
;       endif
       where_ptr = self.ptr_nerr
     endif else begin
       goto, skip_this
     endelse
     prefix = 'nerr='
   endif

   if (icat eq icat_legend) then begin
     if (ptr_valid(self.ptr_legend) ) then begin
;       if (n_elements(*(self.ptr_legend)) GT max_shown) then  begin
;         goto, skip_this
;       endif
       if (n_elements(*(self.ptr_legend)) le 1) then begin
	 goto, skip_this
       endif
       where_ptr = self.ptr_legend
     endif else begin
       goto, skip_this
     endelse

;       num_cols = self.dim[0]
;      num_rows = self.dim[1]
     prefix = 'legd='
   endif

   ; Use instead of self.tname
   if (ptr_valid(where_ptr)) then begin
     temp_tname = size(*(where_ptr), /tname)
     NULL_PTR = 0
   endif else begin
     temp_tname = 'NULL'
     print, infolabel, 'Nothing stored in where_ptr'
     NULL_PTR = 1
   endelse
 
;   print, infolabel, icat, "=icat.  temp_tname = ", temp_tname,$
;   	 " NULL_PTR=", NULL_PTR
;   print, infolabel, "num_rows, num_cols = ", num_rows, num_cols
;   if (NOT NULL_PTR) then begin
;     print, infolabel, "name=", self.name , " *(where_ptr)=",  *(where_ptr)
;   endif
   
   if ((num_rows eq 0) and (num_cols eq 0)) then goto, skip_this
   if (self.dim[2] GT 0) then goto, skip_this ; 3D or 4D.
   
   if (num_rows gt 0) then begin
     ;---------------- 2D -----------------------------
    for irow = 0, (num_rows-1) do begin
     if (irow eq 0) then begin
       if (icat eq icat_legend) then begin
         thisRow = prefix + '<' 
       endif else begin
         thisRow = prefix + '[' 
       endelse
     endif else begin
       thisRow = noprefix
     endelse
     if (irow gt max_shown) then begin
       thisRow = thisRow + ' ... '
       goto, skip_2d_rows
     endif

     for icol = 0, (num_cols-1) do begin 
       if (icol EQ 0) then thisRow = thisRow + '['
       if (icol gt max_shown) then begin
         thisRow = thisRow + ' ...] '
 	 thisEverything = [thisEverything , thisRow]
 	 thisRow = noprefix + ' '
         goto, skip_2d_cols
       endif
       if (NULL_PTR) then begin
         thisRow = thisRow + '_'
       endif else begin
         thisRow = thisRow + whatsIt( (*(where_ptr))[icol,irow] )
       endelse
       if (icol EQ(num_cols-1) ) then begin
         thisRow = thisRow + ']'
       endif else begin
         thisRow = thisRow + ','
	 if ( (icol mod maxperline) EQ (maxperline-1)) then begin
;;	 if (icol eq (maxperline-1)) then begin
	   thisEverything = [thisEverything , thisRow]
	   thisRow = noprefix + ' '
	 endif
       endelse
     endfor
     skip_2d_cols:
     skip_2d_rows:
     if (irow EQ (num_rows-1)) then begin
       if (icat eq icat_legend) then begin
         thisRow = thisRow + '>' 
       endif else begin
         thisRow = thisRow + ']' 
       endelse
     endif else begin
       thisRow = thisRow + ','
     endelse
       thisEverything = [thisEverything , thisRow]
    endfor
   endif else begin

     ;---------------- 1D -----------------------------
;debug       print, infolabel, '1D', 'num_cols=', num_cols
       if (icat eq icat_legend) then begin
         thisRow = prefix + '<' 
       endif else begin
         thisRow = prefix + '[' 
       endelse
     for icol = 0, (num_cols-1) do begin 
       if (NULL_PTR) then begin
         thisRow = thisRow + '_'
       endif else begin
         thisRow = thisRow + whatsIt( (*(where_ptr))[icol] )
       endelse
       if (icol gt (max_shown*maxperline)) then begin
         thisRow = thisRow + ' ...] '
         goto, skip_1d_cols
       endif
       if (icol EQ(num_cols-1) ) then begin
          if (icat eq icat_legend) then begin
           thisRow = thisRow + '>' 
          endif else begin
            thisRow = thisRow + ']' 
          endelse
       endif else begin
         thisRow = thisRow + ','
	 if ( (icol mod maxperline) EQ (maxperline-1)) then begin
;debug	   print, infolabel, '1D ', icol, '"', thisRow, '"'
;	 if (icol eq (maxperline-1)) then begin
	   thisEverything = [thisEverything , thisRow]
	   thisRow = noprefix + ' '
	 endif
       endelse
     endfor
     skip_1d_cols:
     thisEverything = [thisEverything , thisRow]
   endelse
   
   skip_this:
  endfor		; for icat = 0, 3
   nsuc = nsuc + 1
 endif 		;; (argPresent.Everything)

   ;-----------------------------------------
  success = (nsuc gt 0)
  
  if (debug) then begin
    print, infolabel, 'nsuc = ', nsuc
    printArgPresent, ArgPresent, /showValues
    print
  endif

  if (success EQ 0) then begin
     print, infolabel, self.name, ' No recognized keywords provided'
     printArgPresent, ArgPresent  
  endif

  return, success 

END		;; qtyObj::GetString


;=============================================================================
;+
; NAME:
; 	qtyObj::setDimension
;
; PURPOSE:
;	Helper method for qtyObj::set.
;
; 	This function overrides the current size and type of *(self.ptr_qty).
; Useful when initializing data using init method, but don't have the
; data handy yet. Won't be generally used, otherwise.
;-
;*
function qtyObj::setDimension, $; Returns 0/1 for failure/success.
  rank=thisRank, $ 	; Integer. 0, 1, 2, 3, or 4-D
  dim=thisDim, $ 	; Integer. 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
  tname = thisTName, $	; String. Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
  initValue = thisInitValue	; Initial value for array *(self.ptr_qty)
;/

;  DIM_WARN = 3		; 
;  DIM_MAX = 4		; ???
;OLD   include 'leaf_size.pro'
   common QTYOBJ_DIMS, DIM_MAX, DIM_WARN 
 
  infolabel = "qtyObj::setDimension >> "  
  nsuc = 0
  nfail = 0
  str_fail = ''

  ;-------------------------------------------------------
  if (n_elements(thisRank) GT 0 ) then begin
    if (thisRank GT DIM_MAX) then begin
      print, infolabel, temp_rank, " Dimensions is > ",  DIM_MAX,  $
      	  ". Exceeds IDL's capability."
      nfail = nfail + 1
    endif else begin
      self.rank = thisRank
    endelse
  endif
   
  ;-------------------------------------------------------
  if (n_elements(thisDim) GT 0 ) then begin
    temp_rank = n_elements(thisDim)
    for i = 0, (n_elements(thisDim)-1) do begin
      ; Zero-filled  items don't count.
      if (thisDim[i] EQ 0) then temp_rank = temp_rank -1
    endfor
    if (temp_rank GT DIM_MAX) then begin
      print, infolabel, temp_rank, " Dimensions is > ",  DIM_MAX,  $
      	  ". Exceeds IDL's capability."
      nfail = nfail + 1
    endif else begin
      if (temp_rank GE DIM_WARN) then begin
          print, infolabel, temp_rank, " Dimensions is >=",  DIM_WARN,  $
        	". Program not be able to handle fully."
      endif
      self.rank = temp_rank
      if (temp_rank EQ 1) then begin
        if (thisDim[0] EQ 1) then begin
;Old        if (thisDim EQ 1) then begin
	  self.rank = 0		; scalar
	endif 
      endif
;;      ptr_free, self.ptr_dim
;;      self.ptr_dim = ptr_new(thisDim)
      rank = self.rank
;      print, infolabel, "thisDim=", thisDim, " rank=", rank; Aug 15, 2003 DEBUG
;      help, thisDim  				; Aug 15, 2003 DEBUG     
      if (rank EQ 0) then begin
        self.dim = [0,0,0,0]
      endif else begin
        self.dim[0:(rank-1)] = thisDim[0:(rank-1)]
      endelse
      nsuc = nsuc + 1
    endelse
  endif 
  
  ;-------------------------------------------------------
  if (n_elements(thisTName) GT 0 ) then begin
    self.tname = thisTName
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  success = (nsuc gt 0) and (nfail eq 0)

  ;-------------------------------------------------------
  if (success) then begin
    if (ptr_valid(self.ptr_qty)) then begin
      ptr_free, self.ptr_qty 
    endif
    temp_type = mn_tname_2_tcode(self.tname)
    temp_dim = self.dim
    rank = self.rank
;Debug    print, infolabel, "rank=", rank, " dim=",temp_dim
    if (n_elements(thisInitValue) GT 0) then begin
;Debug      print, infolabel, "thisInitValue=", thisInitValue
;Wrong      temp_dim = self.dim[0:(rank-1)]
      rank = size(thisInitValue, /n_dimensions)
      temp_dim = size( thisInitValue, /dimensions)
      print, infolabel, "rank=", rank, " dim=",temp_dim
       temp_array = make_array(temp_dim, $
		 type=temp_type, value=thisInitValue)
;;;      temp_array = make_array(self.dim. $
;;;		 type=mn_tname_2_tcode(self.tname), value=thisInitValue)
    endif else begin
      ; Zero fills by default.
     if (rank GT 0) then begin
      temp_dim = self.dim[0:(rank-1)]
;Debug      print, infolabel, "Zero fills for temp_array, len=", temp_dim
      temp_array = make_array(temp_dim, $
		 type=temp_type) 
;;;      temp_array = make_array(self.dim. $
;;;		 type=mn_tname_2_tcode(self.tname)) 
     endif 
    endelse
    if (n_elements(temp_array) GT 0) then begin
      self.ptr_qty = ptr_new(temp_array)
    endif
  endif

  ;-----------------------------------------
    if (success EQ 0) then begin
     print, infolabel, self.name, ' ',  $
    	strcompress(string(nfail), /remove_all), ' failure(s) and ', $
	strcompress(string(nsuc), /remove_all), '  success(es)'
    print, str_fail
  endif

  ;-----------------------------------------
  return, success                              ;;; Successful? SETting

END		;; qtyObj::setDimension


;=============================================================================
;+
; NAME:
; 	qtyObj::count
;
; PURPOSE:
;	 Stupider than qtyObj::GetDimension(rank=thisrank, dim=thisDim)
; as it returns the equivalent 1D number of elements for multi-dimensional arrays.
; Compatible with count method in IDL_container and DAVE_tagContainer.
;-
;*
function qtyObj::count	; Returns integer with n_elements(*(self.ptr_qty))
;/

   if (ptr_valid( self.ptr_qty)) then begin
     return, n_elements(*(self.ptr_qty))
   endif else begin
     return, 0
   endelse   
end		; qtyObj::count

;=============================================================================
;+
; NAME:
; 	qtyObj::getDimension
;
; PURPOSE:
;	Helper method for qtyObj::get.
;
;-
;*
function qtyObj::GetDimension, $ ; Returns 0/1 for failure/success.
   argPresent = thisArgPresent, $	; anonymous structure.
   	; ARG_PRESENT() function does not work on propagated keywords 
	; from other calling routines. Needed for get* methods.
  ;-----------------------------------------
  rank=thisrank, $ 	; Integer number of dimensions. 0 for scalar.
  			; 1 for 1-D, 2 for 2-D, etc.
  dim=thisDim, $ 	; Integer array. 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
  tname = thisTName 	; String. Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
;/

  infolabel = "qtyObj::GetDimension >> " 
  debug = 0
   
  nsuc = 0
  
  ;-------------------------------------------------------
  ; "Inherit" passed parameters from calling method via thisArgPresent.
  ; Otherwise, this method might calculate more things than are required.
  if (NOT arg_present(thisArgPresent)) then begin   
   ArgPresent = {$			; make this be anonymous structure
	  rank:		arg_present(thisrank), $	
	  Dim:		arg_present(thisDim), $	
	  tname:	arg_present(thisTName)}
  endif else begin
    ; ArgPresent had better include the tags "rank", "dim", and "tname"
    ArgPresent = thisArgPresent
  endelse

  ;-------------------------------------------------------
  if (argPresent.rank) then begin
    thisrank = self.rank
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.Dim) then begin
    localrank = self.rank
;Old    thisDim = self.dim[0:(localrank-1)]
    if (localRank GE 1) then begin
      thisDim = self.dim[0:(localrank-1)]
    endif else begin
      thisDim = 0
    endelse    
;    thisDim = *(self.ptr_dim)
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.tname) then begin
    thisTName = self.TName
    nsuc = nsuc + 1
  endif
  
   ;-----------------------------------------
  success = (nsuc gt 0)
  
  if (debug) then begin
    print, infolabel, 'nsuc = ', nsuc
    printArgPresent, ArgPresent, /showValues
    print
  endif

  if (success EQ 0) then begin
     print, infolabel, self.name, ' No recognized keywords provided'
     printArgPresent, ArgPresent  
  endif

  return, success 

END		;; qtyObj::GetDimension


;=============================================================================
;*
function qtyObj::setValue, $; Returns 0/1 for failure/success.
  ;----------------------- 
  ptr_qty = thisptr_qty, $
  qty = thisqty, $
  ptr_err = thisptr_err, $		; Optional. Error on the values
  err = thiserr, $			; Optional..Error on the values
  ptr_nerr = thisptr_nerr, $		; Optional. Negative error on the values
  nerr = thisnerr, $			; Optional. Negative error on the values
  ptr_legend = thisptr_legend, $	; Optional. Shares memory using the same pointer.
  legend = thislegend			; Optional. Shares memory using new pointer.
;/

  infolabel = "qtyObj::setValue >> "  

;  if (ptr_valid(thisPtr_err)) then begin
;    message, /info, string("ptr_valid(thisPtr_err)=", (*thisptr_err)[0], "..")
;  endif

  nsuc = 0
  nfail = 0
;;;  str_fail = self.name + ' dims=' + format_size(temp_rank) 		;;''
  
  tempPtr_qty = ptr_new()
  tempPtr_err = ptr_new()
  tempPtr_nerr = ptr_new()
  tempPtr_legend = ptr_new()
  
  temp_tname = self.tname
  temp_rank = self.rank
  temp_dim = self.Dim    
;;;  temp_dim = *(self.Dim)    

  str_fail = self.name + ' dims=' + format_size(temp_dim) 		;;''
  str_suc = self.name 
   ;-------------------------------------------------------
  if (n_elements(thisptr_qty) GT 0) then begin
    ; Maybe add error or warning if rank is too large?
    tempPtr_qty = ptr_new(*thisptr_qty)
    temp_tname = size( (*(thisptr_qty)), /tname)
    temp_rank = size( (*(thisptr_qty)), /n_dimensions)
    temp_dim = size( (*(thisptr_qty)), /dimensions)  
    nsuc = nsuc + 1   
    self.dim = temp_dim
    self.rank = temp_rank
    if (NOT eqArrays(self.dim, temp_dim)) then begin
      str_fail = str_fail + ' size(*(thisptr_qty)) ='$
      		 + format_size(temp_dim)
;;;     		 + format_size(temp_rank)
    endif else begin
      str_suc = self.name + ' size(*(thisptr_qty)) ='$
      		 + format_size(temp_dim)
    endelse
  endif

   ;-------------------------------------------------------
  if (n_elements(thisQty) GT 0) then begin
    ; Maybe add error or warning if rank is too large?
    tempPtr_qty = ptr_new(thisqty)
    temp_tname = size(thisqty, /tname)
    temp_rank = size(thisqty, /n_dimensions)
    temp_dim = size(thisqty, /dimensions)  
    nsuc = nsuc + 1   
    self.dim = temp_dim
    self.rank = temp_rank
    if (NOT eqArrays(self.dim, temp_dim)) then begin
      str_fail = str_fail + ' size(qty) ='$
      		 + format_size(temp_dim)
;;;      		 + format_size(temp_rank)
    endif else begin
      str_suc = self.name + ' size(qty) ='$
      		 + format_size(temp_dim)
    endelse

  endif

   ;-------------------------------------------------------
  if (NOT ptr_valid(tempPtr_qty)) then begin
    tempPtr_qty = ptr_new(*(self.ptr_qty))
  endif
  
   ;-------------------------------------------------------
   if (ptr_valid(thisPtr_err)) then begin
;   if (ptr_valid(tempPtr_err)) then begin
;;   if (n_elements(thisptr_err) GT 0) then begin
    
    if ( n_elements(*(tempPtr_qty)) EQ n_elements(*(thisptr_err))) then begin
      tempPtr_err = ptr_new(*(thisPtr_Err))
      nsuc = nsuc + 1   
      str_suc = self.name + ' size(thisptr_err) ='$
      		 + format_size(ptr_array=thisptr_err)

    endif else begin
      nfail = nfail + 1
      str_fail = str_fail + ' size(thisptr_err) ='$
      		 + format_size(ptr_array=thisptr_err)
;      print, undefinedVar		; Crash program here
    endelse
  endif

   ;-------------------------------------------------------
   if (n_elements(thisErr) GT 0) then begin
    
    if ( n_elements(*(tempPtr_qty)) EQ n_elements(thisErr)) then begin
        tempPtr_err = ptr_new(thisErr)
        nsuc = nsuc + 1 
        str_suc = self.name + ' size(Err) ='$
      		 + format_size(array=thisErr)
    endif else begin
      nfail = nfail + 1
      str_fail = str_fail + ' size(Err) ='$
      		 + format_size(array=thisErr)
;      help, thisErr
;      print, thisErr
;      print, infolabel, str_fail
;      print, undefinedVar		; Crash program here
    endelse
   endif

   ;-------------------------------------------------------
   if (ptr_valid(thisPtr_Nerr)) then begin
;   if (ptr_valid(tempPtr_Nerr)) then begin
;;   if (n_elements(thisptr_Nerr) GT 0) then begin
    
    if ( n_elements(*(tempPtr_qty)) EQ n_elements(*(thisptr_Nerr))) then begin
      tempPtr_nerr = ptr_new(*(thisPtr_NErr))
      nsuc = nsuc + 1   
      str_suc = self.name + ' size(thisptr_nerr) ='$
      		 + format_size(ptr_array=thisptr_nerr)
    endif else begin
      nfail = nfail + 1
      str_fail = str_fail + ' size(*(thisptr_Nerr)) ='$
      		 + format_size(ptr_array=thisptr_Nerr)
    endelse
   endif

   ;-------------------------------------------------------
  if (n_elements(thisNErr) GT 0) then begin
    
    if ( n_elements(*(tempPtr_qty)) EQ n_elements(thisNErr)) then begin
      tempPtr_Nerr = ptr_new(thisNErr)
      nsuc = nsuc + 1   
      str_suc = self.name + ' size(NErr) ='$
      		 + format_size(array=thisNErr)
    endif else begin
      nfail = nfail + 1
      str_fail = str_fail + ' size(NErr) ='$
      		 + format_size(array=thisNErr)
    endelse
  endif
  
   ;-------------------------------------------------------
  if (ptr_valid(thisPtr_legend)) then begin
;  if (ptr_valid(tempPtr_legend)) then begin
;;  if (n_elements(thisptr_legend) GT 0) then begin
    
    if ( n_elements(*(tempPtr_qty)) EQ n_elements(*(thisptr_legend))) then begin
      tempPtr_legend = ptr_new(*(thisPtr_legend))
      nsuc = nsuc + 1   
      str_suc = self.name + ' size(thisptr_legend) ='$
      		 + format_size(ptr_array=thisptr_legend)
    endif else begin
      nfail = nfail + 1
      str_fail = str_fail + ' size(*(thisptr_legend)) ='$
      		 + format_size(ptr_array=thisptr_legend)
    endelse
  endif

   ;-------------------------------------------------------
  if (n_elements(thislegend) GT 0) then begin
    
    if ( n_elements(*(tempPtr_qty)) EQ n_elements(thislegend)) then begin
      tempPtr_legend = ptr_new(thislegend)
      nsuc = nsuc + 1   
      str_suc = self.name + ' size(legend) ='$
      		 + format_size(array=thisLegend)
    endif else begin
      nfail = nfail + 1
      str_fail = str_fail + ' size(legend)) ='$
      		 + format_size(array=thislegend)
    endelse
  endif

  ;------------------------------------------
  success = (nsuc GT 0) AND (nfail EQ 0)
  ;------ End ------------------------------------
  if (success EQ 0) then begin
     print, infolabel, self.name, ' ',  $
    	strcompress(string(nfail), /remove_all), ' failure(s) and ', $
	strcompress(string(nsuc), /remove_all), '  success(es)'
    print, "Success:",  str_suc
    print, "Failed:",  str_fail
  endif

  if (success) then  begin
    self.tname = temp_tname
    self.rank = temp_rank
    self.dim = temp_dim

;    if (n_elements(thisQty) GT 0) then begin
;      temp_dim = size( thisQty, /dimensions)
;    endif else begin
;      if (n_elements(thisPtr_Qty) GT 0) then begin
;        temp_dim = size( thisPtr_Qty, /dimensions)
;      endif
;    endelse
;;;    ptr_free, self.dim 
;;;    self.Dim = ptr_new(temp_dim)
;    self.Dim = temp_dim

    if (ptr_valid(TempPtr_Qty)) then begin
      ptr_free, self.ptr_qty
      self.ptr_qty = ptr_new(*(TempPtr_Qty))
    endif
    ptr_free, TempPtr_Qty

    if (ptr_valid(TempPtr_Err)) then begin
      ptr_free, self.ptr_Err
      self.ptr_Err = ptr_new(*(TempPtr_Err))
    endif
    ptr_free, TempPtr_Err
    
    if (ptr_valid(TempPtr_Nerr)) then begin
      ptr_free, self.ptr_nerr
      self.ptr_nerr = ptr_new(*(TempPtr_nerr))
    endif
    ptr_free, TempPtr_nerr
    
    if (ptr_valid(TempPtr_Legend)) then begin
      ptr_free, self.ptr_legend
      self.ptr_legend = ptr_new(*(TempPtr_Legend))
    endif
    ptr_free, TempPtr_legend
 
  endif 

;;;  print, infolabel, "self.rank= ", self.rank 
;  if (n_elements(thisptr_qty) GT 0) then begin
;    print, infolabel, self.name, " loaded with ptr_qty[0] =", (*(self.ptr_qty))[0]
;    print, infolabel, self.name, "   rank = ", self.rank, " dim = ", self.dim
;  endif
;  if (n_elements(thisqty) GT 0) then begin
;    print, infolabel, self.name, " loaded with qty[0] =", (*(self.ptr_qty))[0]
;    print, infolabel, self.name, "   rank = ", self.rank, " dim = ", self.dim
;  endif


  return, success 

END		;; qtyObj::setValue

;=============================================================================
;*
function qtyObj::GetValue, $	; Returns 0/1 for failure/success.
   argPresent = thisArgPresent, $	; anonymous structure.
   	; ARG_PRESENT() function does not work on propagated keywords 
	; from other calling routines. Needed for get* methods.
  ;----------------------- 
  ptr_qty = thisptr_qty, $
  qty = thisqty, $
  ptr_err = thisptr_err, $		; Optional error on the values
  err = thiserr, $			; Optional error on the values
  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
  nerr = thisnerr, $			; Optional negative error on the values
  ptr_legend = thisptr_legend, $	; Optional. shares memory using the same pointer.
  legend = thislegend			; Optional. shares memory using new pointer.
;/

  infolabel = "qtyObj::GetValue >> "  
  debug = 0;	1
  
  nsuc = 0
  nfail = 0
  ;-------------------------------------------------------
  ; "Inherit" passed parameters from calling method via thisArgPresent.
  ; Otherwise, this method might calculate more things than are required.
  if (NOT arg_present(thisArgPresent)) then begin   
   ArgPresent = { $			; make this be anonymous structure
	  ptr_qty:	arg_present(thisptr_qty), $
	  qty:		arg_present(thisqty), $
	  ptr_err:	arg_present(thisptr_err), $
	  err:		arg_present(thiserr), $
	  ptr_nerr:	arg_present(thisptr_nerr), $
	  nerr:		arg_present(thisnerr), $
	  ptr_legend:	arg_present(thisptr_legend), $
	  legend:	arg_present(thislegend)$
	  }

  endif else begin
    ArgPresent = thisArgPresent
  endelse

   ;-------------------------------------------------------
   ; Place the requested values in the return parameters (arrays OK.)
   ;-------------------------------------------------------
  if (argPresent.ptr_qty) then begin
    if (ptr_valid(self.ptr_qty)) then begin
      thisptr_qty = ptr_new(*self.ptr_qty)
    endif else begin
      thisptr_qty = ptr_new()
    endelse
    nsuc = nsuc + 1
  endif

   ;-------------------------------------------------------
  if (argPresent.ptr_err) then begin
    if (ptr_valid(self.ptr_err)) then begin
      thisptr_err = ptr_new(*self.ptr_err)
    endif else begin
      thisptr_err = ptr_new()
    endelse
    nsuc = nsuc + 1
  endif

   ;-------------------------------------------------------
  if (argPresent.ptr_nerr) then begin
    if (ptr_valid(self.ptr_nerr)) then begin
      thisptr_nerr = ptr_new(*self.ptr_nerr)
    endif else begin
      thisptr_nerr = ptr_new()
    endelse
    nsuc = nsuc + 1
  endif

   ;-------------------------------------------------------
  if (argPresent.ptr_legend) then begin
    if (ptr_valid(self.ptr_legend)) then begin
      thisptr_legend = ptr_new(*self.ptr_legend)
    endif else begin
      thisptr_legend = ptr_new()
    endelse
    nsuc = nsuc + 1
  endif

   ;-------------------------------------------------------
  if (argPresent.qty) then begin
    if (ptr_valid(self.ptr_qty)) then begin
      thisqty = *(self.ptr_qty)
      nsuc = nsuc + 1
    endif 
  endif
  
   ;-------------------------------------------------------
  if (argPresent.err) then begin
    if (ptr_valid(self.ptr_err)) then begin
      thiserr = *(self.ptr_err)
      nsuc = nsuc + 1
    endif 
  endif
  
   ;-------------------------------------------------------
  if (argPresent.nerr) then begin
    if (ptr_valid(self.ptr_nerr)) then begin
      thisnerr = *(self.ptr_nerr)
      nsuc = nsuc + 1
    endif
  endif
  
   ;-------------------------------------------------------
  noLegend = 1 
  if (argPresent.legend) then begin
    if (ptr_valid(self.ptr_legend)) then begin
      noLegend = 0
      thislegend = *(self.ptr_legend)
      nsuc = nsuc + 1
    endif else begin
      noLegend = 1 
    endelse
  endif
  

 ;-------------------------------------------------------
  success = (nsuc gt 0) 
  
  if (debug) then begin
    print, infolabel, 'nsuc = ', nsuc
    printArgPresent, ArgPresent, /showValues
    print
  endif

  if (success EQ 0) then begin
     if (NOT noLegend) then begin
       print, infolabel, self.name,  ' ', $
	'"', strcompress(string(nsuc), /remove_all), '"  successes'
       printArgPresent, ArgPresent  
     endif else begin
       success = 1	; No problme if legend is missing
     endelse
  endif

  return, success                              ;;; Successful? GETvalue

END		;; qtyObj::GetValue


;=============================================================================
;*
function qtyObj::setOneValue, $  ; Returns 0, 1 for failure, success
	  ; Doesn't make a lot of sense for scalars.
  ;-------- Need to location position in storage array using forPos or forLegend.
  forPos=thisForPos, $		; Integer or integer array. E.g. # or [#,#]
  forLegend=thisForLegend, $	; String, alternative to forPos
  ;------ Use at least one of the following keywords: 
  oneQty = thisOneQty, $	; String or numeric, 
  				; matching current type of *(self.ptr_qty).
  oneErr = thisOneErr, $	; Numeric, matching current type of *(self.ptr_err).
  oneNErr = thisOneNErr, $	; Numeric, matching current type of *(self.ptr_nerr).
  oneLegend = thisOneLegend	; String.
;/

;OLD  include 'leaf_size.pro'
   common QTYOBJ_DIMS, DIM_MAX, DIM_WARN 

  infolabel = "qtyObj::setOneValue >> "  
  nsuc = 0
  nfail = 0
;;  str_fail = ''
  
   ;-----------------------------------------------------
  if (n_elements(thisforLegend) gt 0) then begin
    OK = self -> positionLegend(thisforLegend, specWhere )
    if (not OK) then begin
      nfail = nfail + 1
      print, infolabel, "Failed to find legend ", thisForLegend
      return, 0      
    endif else begin
      localforPos = specWhere
      nsuc = nsuc + 1
    endelse
  endif

   ;-----------------------------------------------------
  if (n_elements(thisforPos) eq 0) then begin
    if (self.rank EQ 0) then begin
     localForPos = 0		; silly. Make adjustment for a scalar.
    endif else begin
      ; nothing requested.
      print, infolabel, "Missing keyword forPos or forLegend. ", $
      	"self.name=", self.name
      return, 0
    endelse
  endif else begin

  localForPos = thisforPos

   ;-----------------------------------------------------
   ; Check that localForPos is sensible.
   if (self.rank gt 1) then begin
     if (size(localForPos, /dimensions) NE self.rank ) then begin
       print, infolabel, "Mismatch in number of dimensions" 
       return, 0
     endif
   endif
   
   if (self.rank le 1) then begin
     if (localForPos GT self.dim) then begin
       print, infolabel, "Requested position has too large a value", $
       	" in dimension #:", self.rank
	print, infolabel, "Requested position is ", $
		 localForPos , '>', self.dim
       return, 0
     endif
   endif else begin 	; 2-D and higher
     nBad = 0
     for idim = 0, (self.rank-1) do begin
      if ( localForPos[idim] GT (self.dim)[idim] ) then begin
        print, infolabel, "Requested position has too large a value", $
         	" in dimension #", dim 
	print, infolabel, "Requested position is ", $
		 localForPos[idim] , '>', (self.dim)[idim]
	nBad = nBad + 1
       endif
     endfor
     if (nBad GT 0) then return, 0
   endelse
  endelse

   ;-----------------------------------------------------
   if (self.rank eq 0) then begin		; scalar
     if (n_elements(thisOneQty) gt 0) then begin
       (*(self.ptr_qty)) = thisOneQty
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneErr) gt 0) then begin
       (*(self.ptr_err)) = thisOneErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneNErr) gt 0) then begin
       (*(self.ptr_nerr)) = thisOneNErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneLegend) gt 0) then begin
       (*(self.ptr_legend)) = thisOneLegend
       nsuc = nsuc + 1
     endif
   endif		; scalar

   ;-----------------------------------------------------
   if (self.rank eq 1) then begin		; 1D
     i0 = localForPos[0] 
;     i0 = localForWhere[0] 
     if (n_elements(thisOneQty) gt 0) then begin
       (*(self.ptr_qty))[i0] = thisOneQty
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneErr) gt 0) then begin
       (*(self.ptr_err))[i0] = thisOneErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneNErr) gt 0) then begin
       (*(self.ptr_nerr))[i0] = thisOneNErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneLegend) gt 0) then begin
       (*(self.ptr_legend))[i0] = thisOneLegend
       nsuc = nsuc + 1
     endif
   endif		; 1D

  ;-------------------------------------------------------------
   if (self.rank eq 2) then begin		; 2D
     i0 = localForPos[0] 
     i1 = localForPos[1] 
;     i0 = localForWhere[0] 
;      i1 = localForWhere[1] 
     if (n_elements(thisOneQty) gt 0) then begin
       (*(self.ptr_qty))[i0, i1] = thisOneQty
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneErr) gt 0) then begin
       (*(self.ptr_err))[i0, i1] = thisOneErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneNErr) gt 0) then begin
       (*(self.ptr_nerr))[i0, i1] = thisOneNErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneLegend) gt 0) then begin
       (*(self.ptr_legend))[i0, i1] = thisOneLegend
       nsuc = nsuc + 1
     endif
   endif		; 2D

  ;-------------------------------------------------------------
   if (self.rank eq 3) then begin		; 3D
     i0 = localForPos[0] 
     i1 = localForPos[1] 
     i2 = localForPos[2] 
;     i0 = localForWhere[0] 
;     i1 = localForWhere[1] 
;     i2 = localForWhere[2] 
     if (n_elements(thisOneQty) gt 0) then begin
       (*(self.ptr_qty))[i0, i1, i2] = thisOneQty
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneErr) gt 0) then begin
       (*(self.ptr_err))[i0, i1, i2] = thisOneErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneNErr) gt 0) then begin
       (*(self.ptr_nerr))[i0, i1, i2] = thisOneNErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneLegend) gt 0) then begin
       (*(self.ptr_legend))[i0, i1, i2] = thisOneLegend
       nsuc = nsuc + 1
     endif
   endif		; 3D

   
  ;-------------------------------------------------------------
   if (self.rank eq 4) then begin		; 4D
     i0 = localForPos[0] 
     i1 = localForPos[1] 
     i2 = localForPos[2] 
     i3 = localForPos[3] 
;     i0 = localForWhere[0] 
;     i1 = localForWhere[1] 
;     i2 = localForWhere[2] 
;     i3 = localForWhere[3] 
     if (n_elements(thisOneQty) gt 0) then begin
       (*(self.ptr_qty))[i0, i1, i2, i3] = thisOneQty
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneErr) gt 0) then begin
       (*(self.ptr_err))[i0, i1, i2, i3] = thisOneErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneNErr) gt 0) then begin
       (*(self.ptr_nerr))[i0, i1, i2, i3] = thisOneNErr
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneLegend) gt 0) then begin
       (*(self.ptr_legend))[i0, i1, i2, i3] = thisOneLegend
       nsuc = nsuc + 1
     endif
   endif		; 4D

   
  ;-----------------------------------------
  success = (nsuc gt 0) and (nfail eq 0)
  
  if (success EQ 0) then begin
     print, infolabel, self.name, ' ',  $
;;    	strcompress(string(nfail), /remove_all), ' failure(s) and ', $
	strcompress(string(nsuc), /remove_all), '  success(es)'
;;    print, str_fail
  endif

  return, success 

END		;; qtyObj::setOneValue

;=============================================================================
;*
function qtyObj::GetOneValue, $  ; Returns 0, 1 for failure, success
	  ; Doesn't make a lot of sense for scalars.
   argPresent = thisArgPresent, $	; Optional anonymous structure.
   	; ARG_PRESENT() function does not work on propagated keywords 
	; from other calling routines. Needed for get* methods.
  ;--------
  forPos=thisForPos, $		; Integer or integer array. E.g. # or [#,#]
  forLegend=thisForLegend, $	; String, alternative to forPos
  ;------ Use at least one of the following keywords:
  descripTitle = thisDescripTitle, $ ; Returns string for descriptive purposes. 
	  ; Includes name, legend, scalar qty, error, and units.
  oneQty = thisOneQty, $		; Optional. Returns string or numeric.
  oneErr = thisOneErr, $		; Optional. Returns numeric.
  oneNErr = thisOneNErr, $		; Optional. Returns numeric.
  oneLegend = thisOneLegend		; Optional. Returns string.
;/


  infolabel = "qtyObj::GetOneValue >> "  
  debug = 0		;	1
  nsuc = 0
  nfail = 0
;;  str_fail = ''
  
  ;-------------------------------------------------------
  ; "Inherit" passed parameters from calling method via thisArgPresent.
  ; Otherwise, this method might calculate more things than are required.
  if (NOT arg_present(thisArgPresent)) then begin   
   ArgPresent = {$		; make this be anonymous structure
;	  forPos:	arg_present(thisforPos), $ ; # or {#,#], etc.
;	  forLegend:	arg_present(thisforLegend), $; alternative to forPos
	  descripTitle:	arg_present(thisDescripTitle), $
	  oneQty:	arg_present(thisoneqty), $
	  oneErr:	arg_present(thisoneerr), $
	  oneNErr:	arg_present(thisonenerr), $
	  oneLegend:	arg_present(thisonelegend) $
	}
	
  endif else begin
    ArgPresent = thisArgPresent
  endelse

   ;-----------------------------------------------------
  if ( (n_elements(thisForLegend) eq 0) AND (n_elements(thisForPos) eq 0)) then begin
;;  if ( (NOT(ArgPresent.forLegend)) AND (NOT (ArgPresent.forPos))) then begin
      print, infolabel, "Missing keyword forPos or forLegend. ", $
      	"self.name=", self.name 
      printArgPresent, ArgPresent, /showValues
      print
      return, 0
  endif

   ;-----------------------------------------------------
  if (n_elements(thisForLegend) gt 0) then begin
;;;;  if (ArgPresent.forLegend) then begin
;Debug    print, infolabel, 'ArgPresent.forLegend = "', thisForLegend, '"'
;Debug    print, infolabel, 'thisForLegend = "', thisForLegend, '"'
    if (self.rank EQ 0) then begin
      localForPos = 0		; silly. Make adjustment for a scalar.
    endif else begin
      OK = self -> positionLegend(thisforLegend, specWhere )
      if (not OK) then begin
        nfail = nfail + 1
        print, infolabel, "Failed to find legend ", thisForLegend
	localForPos = -1
        return, 0      
      endif else begin
        localforPos = specWhere
        nsuc = nsuc + 1
      endelse
    endelse
  endif

   ;-----------------------------------------------------
  if (n_elements(thisForPos) gt 0) then begin
;;;;  if (ArgPresent.forPos) then begin
;    print, infolabel, 'ArgPresent.forPos = "', thisForPos, '"'
;Debug    print, infolabel, 'thisForPos = "', thisForPos, '"'
    if (self.rank EQ 0) then begin
     localForPos = 0		; silly. Make adjustment for a scalar.
    endif else begin
    localForPos = thisforPos
    endelse
  endif else begin
   ;-----------------------------------------------------
   ; Check that localForPos is sensible.
   if (self.rank gt 1) then begin
     if (size(localForPos, /n_dimensions) NE (self.rank)) then begin
;debug       print, 'self.rank = ', self.rank
;debug       print, 'size(localForPos, /dimensions) =', size(localForPos, /dimensions)
       print, infolabel, "Mismatch in number of dimensions" 
       return, 0
     endif
   endif
      
   if (self.rank le 1) then begin
     if (localForPos GT self.dim[0]) then begin
       print, infolabel, "Requested position has too large a value", $
       	" in dimension #:", self.rank
	print, infolabel, "Requested position is ", $
		 localForPos , '>', self.dim
       return, 0
     endif
   endif else begin 	; 2-D and higher
     nBad = 0
     for idim = 0, (self.rank-1) do begin
      if ( localForPos[idim] GT (self.dim)[idim] ) then begin
        print, infolabel, "Requested position has too large a value", $
         	" in dimension #", dim 
	print, infolabel, "Requested position is ", $
		 localForPos[idim] , '>', (self.dim)[idim]
	nBad = nBad + 1
       endif
     endfor
     if (nBad GT 0) then return, 0
   endelse
  endelse

   ;-----------------------------------------------------
   if (self.rank eq 0) then begin		; scalar
     if (ArgPresent.oneQty) then begin
       thisOneQty = (*(self.ptr_qty))
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneErr) then begin
       if (ptr_valid(self.ptr_err)) then begin		; TMR Aug 6, 2003
         thisOneErr = (*(self.ptr_err)) 
       endif
       nsuc = nsuc + 1
     endif
     if (n_elements(thisOneNErr) gt 0) then begin
       if (ptr_valid(self.ptr_nerr)) then begin	; TMR Aug 6, 2003
         thisOneNErr = (*(self.ptr_nerr)) 
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneLegend) then begin
       if (ptr_valid(self.ptr_legend)) then begin	; TMR Aug 6, 2003
         thisOneLegend = (*(self.ptr_legend)) 
       endif
       nsuc = nsuc + 1
     endif
   endif		; scalar

   ;-----------------------------------------------------
   if (self.rank eq 1) then begin		; 1D
     i0 = localForPos
;;;     i0 = localForWhere[0] 
     if (ArgPresent.oneQty) then begin
       thisOneQty = (*(self.ptr_qty))[i0] 
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneErr) then begin
       if (ptr_valid(self.ptr_err)) then begin
         thisOneErr = (*(self.ptr_err))[i0] 
       endif 
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneNErr) then begin
       if (ptr_valid(self.ptr_nerr)) then begin
         thisOneNErr = (*(self.ptr_nerr))[i0] 
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneLegend) then begin
       if (ptr_valid(self.ptr_legend)) then begin
         thisOneLegend = (*(self.ptr_legend))[i0] 
       endif
       nsuc = nsuc + 1
     endif
   endif		; 1D

  ;-------------------------------------------------------------
   if (self.rank eq 2) then begin		; 2D
     i0 = localForPos[0] 
     i1 = localForPos[1] 
;     i0 = localForWhere[0] 
;     i1 = localForWhere[1] 
     if (ArgPresent.oneQty) then begin
       thisOneQty = (*(self.ptr_qty))[i0, i1] 
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneErr) then begin
       if (ptr_valid(self.ptr_err)) then begin
         thisOneErr = (*(self.ptr_err))[i0, i1]
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneNErr) then begin
       if (ptr_valid(self.ptr_nerr)) then begin
         thisOneNErr = (*(self.ptr_nerr))[i0, i1] 
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneLegend) then begin
       if (ptr_valid(self.ptr_legend)) then begin
         thisOneLegend = (*(self.ptr_legend))[i0, i1]
       endif
       nsuc = nsuc + 1
     endif
   endif		; 2D

  ;-------------------------------------------------------------
   if (self.rank eq 3) then begin		; 3D
     i0 = localForPos[0] 
     i1 = localForPos[1] 
     i2 = localForPos[2] 
;     i0 = localForWhere[0] 
;     i1 = localForWhere[1] 
;     i2 = localForWhere[2] 
     if (ArgPresent.oneQty) then begin
       thisOneQty = (*(self.ptr_qty))[i0, i1, i2]
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneErr) then begin
       if (ptr_valid(self.ptr_err)) then begin
         thisOneErr = (*(self.ptr_err))[i0, i1, i2]
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneNErr) then begin
       if (ptr_valid(self.ptr_nerr)) then begin
         thisOneNErr = (*(self.ptr_nerr))[i0, i1, i2]
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneLegend) then begin
       if (ptr_valid(self.ptr_legend)) then begin
         thisOneLegend = (*(self.ptr_legend))[i0, i1, i2]
       endif
       nsuc = nsuc + 1
     endif
   endif		; 3D

   
  ;-------------------------------------------------------------
   if (self.rank eq 4) then begin		; 4D
     i0 = localForWhere[0] 
     i1 = localForWhere[1] 
     i2 = localForWhere[2] 
     i3 = localForWhere[3] 
     if (ArgPresent.oneQty) then begin
       thisOneQty = (*(self.ptr_qty))[i0, i1, i2, i3] 
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneErr) then begin
       if (ptr_valid(self.ptr_err)) then begin
         thisOneErr = (*(self.ptr_err))[i0, i1, i2, i3]
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneErr) then begin
       if (ptr_valid(self.ptr_nerr)) then begin
         thisOneNErr = (*(self.ptr_nerr))[i0, i1, i2, i3]
       endif
       nsuc = nsuc + 1
     endif
     if (ArgPresent.oneLegend) then begin
       if (ptr_valid(self.ptr_legend)) then begin
         thisOneLegend = (*(self.ptr_legend))[i0, i1, i2, i3]
       endif
       nsuc = nsuc + 1
     endif
   endif		; 4D

 
  ;-------------------------------------------------------
  if (debug) then printArgPresent, ArgPresent, /showValues
  if (debug) then print, "argPresent.descripTitle =", argPresent.descripTitle
  if (argPresent.descripTitle) then begin
    if (debug) then print, "argPresent.descripTitle =", argPresent.descripTitle
        dstring = self.name + ' '
 	legString = ''
    if (debug) then print, infolabel, "dstring=", dstring
        if (self.rank GE 1) then begin	   
;;;        if (n_elements(*(self.ptr_qty) GT 1)) then begin	   
           ; select 1 value from array
	   legString = '<' +  $
	       		strcompress(string(localForPos), /remove_all) + '> '
	   
	   if (ptr_valid(self.ptr_legend)) then begin
	     if (n_elements(*(self.ptr_legend)) NE 0) then begin
	   	legString = '<' + (*(self.ptr_legend))[localForPos] + '> '
	     endif
	   endif 
	   
 	   dstring = dstring + legString + $
	     	strcompress(string((*(self.ptr_qty))[localForPos]), /remove_all)
	   if (ptr_valid(self.ptr_err)) then begin
	       if (self.StyleErr EQ 'start_step') then begin
	         dstring = dstring + ' start with step ' + $
	   	   strcompress(string((*(self.ptr_err))[localForPos]), /remove_all)
	       endif else begin
	         if (self.StyleErr EQ 'center_step') then begin
	           dstring = dstring + ' center with step ' + $
	   	     strcompress(string((*(self.ptr_err))[localForPos]), /remove_all)
	     	  dstring = dstring + ' center with step ' + $
	   		strcompress(string((*self.ptr_err)), /remove_all)
	         endif else begin
		   dstring = dstring + ' +- ' + $
		     strcompress(string((*(self.ptr_err))[localForPos]), /remove_all)
	         endelse 
	       endelse 
;	       dstring = dstring + ' +- ' + $
;	   	strcompress(string((*(self.ptr_err))[localForPos]), /remove_all)
	     dstring = dstring  + " (" +  self.Units + ')'
	   endif
	    if (debug) then print, infolabel, "dstring=", dstring
	endif else begin

         ; Should be a scalar
	   dstring = dstring + strcompress(string((*self.ptr_qty)), /remove_all)
	   if (ptr_valid(self.ptr_err)) then begin
	       if (self.StyleErr EQ 'start_step') then begin
	         dstring = dstring + ' start with step ' + $
	   		strcompress(string((*self.ptr_err)), /remove_all)
	       endif else begin
	         if (self.StyleErr EQ 'center_step') then begin
	     	  dstring = dstring + ' center with step ' + $
	   		strcompress(string((*self.ptr_err)), /remove_all)
	         endif else begin
	     	  dstring = dstring + ' +- ' + $
	   		strcompress(string((*self.ptr_err)), /remove_all)
	         endelse 
	       endelse 
;	     dstring = dstring + ' +- ' + $
;	   	strcompress(string((*self.ptr_err)), /remove_all)
	   endif
	   dstring = dstring + ' (' +  self.Units + ')'
        endelse
	    if (debug) then print, infolabel, "dstring=", dstring
	
      thisDescripTitle =  dstring   
      if (debug) then begin
        print, infolabel, nsuc, "thisDescripTitle = ", thisDescripTitle 
      endif 
      nsuc = nsuc + 1
  endif
  

;-------------------------------------------------------------------------------------
   
  ;-----------------------------------------
  success = (nsuc gt 0) and (nfail eq 0)
  
  if (debug) then begin
    print, infolabel, 'nsuc = ', nsuc, ' nfail=', nfail
    printArgPresent, ArgPresent, /showValues
    print
  endif

  if (success EQ 0) then begin
     print, infolabel, self.name, ' ',  $
;;    	strcompress(string(nfail), /remove_all), ' failure(s) and ', $
	strcompress(string(nsuc), /remove_all), '  success(es)'
      printArgPresent, ArgPresent, /showValues
      if (n_elements(thisForPos) gt 0) then begin
        print, infolabel, 'thisForPos = "', thisForPos, '"'
      endif
      if (n_elements(thisForLegend) gt 0) then begin
        print, infolabel, 'thisForLegend = "', thisForLegend, '"'
      endif
;;    print, str_fail
  endif

  return, success 

END		;; qtyObj::GetOneValue


;=============================================================================
;*
function qtyObj::set, $		; Returns 0/1 for failure/success.
	  ; use at least one of the following keywords:
  ;--- setString( *** keywords below *** ) -------------------
  name = thisname, $		; String.
  units = thisunits, $		; String. Units of qty, err, nerr.
  StyleErr = thisStyleErr, $	; String. Indicates errors or step or covariance matrix
  ;--- setDimensions( *** keywords below *** ) -------------------
  rank = thisRank, $		; Integer.  0,1,2,3,4-D
  dim = thisDim, $		; Integer or integer array. E.g.  # or [#,#]	
  tname = thisTName, $		; String. Type of *(self.ptr_qty).
  initValue = thisInitValue, $	; Numeric or string.
  ;--- setValue( *** keywords below *** ) -------------------
  ptr_qty = thisptr_qty, $	; Pointer to numeric or string scalar or array.
  qty = thisqty, $		; Numeric or string scalar or array.
  ptr_err = thisptr_err, $	; Pointer to Numeric scalar or array.
  				; Optional error on the values
  err = thiserr, $		; Numeric scalar or array.
  				; Optional error on the values
  ptr_nerr = thisptr_nerr, $	; Pointer to Numeric scalar or array.
  				; Optional negative error on the values
  nerr = thisnerr, $		; Numeric scalar or array.
  				; Optional negative error on the values
  ptr_legend = thisptr_legend, $; Pointer to String scalar or array. Optional.
  				; shares memory using the same pointer.
  legend = thislegend		; String scalar or array. Optional.
  				; shares memory using new pointer.
;/

;  intendedTag = thisIntendedTag, $
;;;  obj=thisObj ; Copy contents of same kind of object into self
;

  infolabel = "qtyObj::set >> "  
  debug = 0

;  if (ptr_valid(thisPtr_err)) then begin
;    message, /info, string("ptr_valid(thisPtr_err)=", (*thisptr_err)[0], "..")
;  endif

  nsuc = 0
  nfail = 0
  str_fail = ''
  
  ;----------------------------------------------------------------
  if ((n_elements(thisname) GT 0) OR (n_elements(thisunits) GT 0)  OR $
        (n_elements(thisStyleErr) GT 0)  OR $
 	(n_elements(thisN_rows) GT 0 ) ) then begin
     nsuc = nsuc + self -> setString( name = thisname, $	; long name, suitable for plot
;  	intendedTag = thisIntendedTag, $	; short name, suitable for GUI buttons
  	units = thisunits, $
	StyleErr = thisStyleErr)
     if ((DEBUG) AND (n_elements(thisStyleErr) GT 0)) then begin
       print, infolabel, self.name, " with this styleErr = '", thisStyleErr, "'"
       print, infolabel, self.name, "   has self.styleErr='", self.styleErr, "'"
     endif
   endif
	
  ;----------------------------------------------------------------
  if ((n_elements(thisRank) GT 0) OR $
  	(n_elements(thisDim) GT 0) OR $
	(n_elements(thisTName) GT 0) OR  $
 	(n_elements(thisInitValue) GT 0 ) ) then begin
;Debug	print, infolabel, "Exists: tank, dim, tname, initvalue ", $
;		(n_elements(thisRank) GT 0),$
;	  	(n_elements(thisDim) GT 0), $
;		(n_elements(thisTName) GT 0),   $
; 		(n_elements(thisInitValue) GT 0 )
     nsuc = nsuc + self -> setDimension(  rank=thisRank, dim=thisDim, $ 		; 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
  	tname = thisTName, $	; Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
 	initValue = thisInitValue)
   endif
	
  ;----------------------------------------------------------------
  if ((n_elements(thisPtr_qty) GT 0) OR (n_elements(thisqty) GT 0)  $
	 OR (n_elements(thisptr_err) GT 0) OR (n_elements(thiserr) GT 0)  $
	 OR (n_elements(thisptr_nerr) GT 0) OR (n_elements(thisnerr) GT 0)  $
	 OR (n_elements(thisptr_legend) GT 0) OR (n_elements(thisLegend) GT 0) $
	 		 ) then begin
;     print, infolabel, 'self -> setValue.  n_elements(thisqty)=', n_elements(thisqty) 
     nsuc = nsuc + self -> setValue( $
     	  ptr_qty = thisptr_qty, $
	  qty = thisqty, $
	  ptr_err = thisptr_err, $		; Optional error on the values
	  err = thiserr, $			; Optional error on the values
	  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
	  nerr = thisnerr, $			; Optional negative error on the values
	  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
	  legend = thislegend)
   endif
	
;;;  nfail = 3 - nsuc
   ;-----------------------------------------
  success = (nsuc gt 0) 			;; and(nfail EQ 0)
  
  if (success EQ 0) then begin
     print, infolabel, self.name, ' ',  $
    	strcompress(string(nfail), /remove_all), ' failure(s) and ', $
	strcompress(string(nsuc), /remove_all), '  success(es)'
    print, str_fail
  endif

  return, success 

END		;; qtyObj::set

;=============================================================================
;*
function qtyObj::Get, $		; Returns 0/1 for failure/success.
   argPresent = thisArgPresent, $	; Optional anonymous structure.
   	; ARG_PRESENT() function does not work on propagated keywords 
	; from other calling routines. Needed for get* methods.
  ; use at least one of the following keywords:
  ;------------ qtyObj::GetString
  name = thisname, $		; String
  units = thisunits, $		; String
  StyleErr = thisStyleErr, $	; String
  axisTitle = thisAxisTitle, $ 	; Returns string for plot axis label. 
	  		; Includes name, legend? and units.
  sizeString = thisSizeString, $ ; Returns string for dimensions. 
  			; '[0]', '[4]', '[#,#]', etc.. 
  some=thisSome, $	; Returns string. 
  			; 1 line summary of one qtyObj Object. No vectorvalues.
  all=thisAll, $  	; Returns string.
  			; 1 line summary of one qtyObj Object
  everything=thisEverything, $	; Returns string.
  			; String array containing all contents of one qtyObjObObject
  ;------------ qtyObj::GetDimension
  rank=thisrank, $ 	; Integer. Number of dimensions. 0 for scalar.
  			; 1 for 1-D, 2 for 2-D, etc.
  dim=thisDim, $ 	; Integer or integer array. 
  			; 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
  tname = thisTName, $ 	; String. Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
  ;----- qtyObj::GetValue 
  ptr_qty = thisptr_qty, $	; Pointer to numeric or string scalar or array.	
  qty = thisqty, $		; Numeric or string scalar or array.
  ptr_err = thisptr_err, $	; Pointer to Numeric scalar or array.
  				; Optional error on the values
  err = thiserr, $		; Numeric scalar or array.
  				; Optional error on the values
  ptr_nerr = thisptr_nerr, $	; Pointer to Numeric scalar or array.
  				; Optional negative error on the values
  nerr = thisnerr, $		; Numeric scalar or array.
  				; Optional negative error on the values
  ptr_legend = thisptr_legend, $; Pointer to String scalar or array. Optional.
  				; shares memory using the same pointer.
  legend = thislegend		; String scalar or array. Optional.
  				; shares memory using new pointer.
;/

;Old  obj=thisObj, $ ; return contents in the same kind of object
;  ;-------- qtyObj::GetOneValue
;  forPos=thisForPos, $	; [#] or [#,#]
;  forLegend=thisForLegend, $	; alternative to forPos
;  oneQty = thisOneQty, $
;  oneErr = thisOneErr, $		; Optional error on the value
;  oneNErr = thisOneNErr, $		; Optional negative error on the value
;  oneLegend = thisOneLegend		; Optiojnal. 
;;;  oneObj = thisOneObj, $	; Scalar qtyObj
;;;  oneRec = thisOneRec, $	; descripPtrStr type

  infolabel = "qtyObj::Get >> "  
  debug = 0;	1
  
  nsuc = 0
  nfail = 0
  str_fail = ''
  ;-------------------------------------------------------
  ; "Inherit" passed parameters from calling method via thisArgPresent.
  ; Otherwise, this method might calculate more things than are required.
  
  if (debug) then print, infolabel, "arg_present(thisArgPresent)=", $
  	arg_present(thisArgPresent)
  
  if (NOT arg_present(thisArgPresent)) then begin   
   ArgPresent = {$		;; use anonymous structure 
      ;--- qtyObj::GetString
	  name:		arg_present(thisName), $	
	  units:	arg_present(thisUnits), $	
	  StyleErr:	arg_present(thisStyleErr), $	
	  axisTitle:	arg_present(thisAxisTitle), $
	  sizeString:	arg_present(thisSizeString), $
	  some:		arg_present(thisSome), $
	  all:		arg_present(thisAll), $
	  everything:	arg_present(thisEverything), $
      ;--- qtyObj::GetDimension
	  rank:		arg_present(thisrank), $	
	  Dim:		arg_present(thisDim), $	
	  tname:	arg_present(thisTName), $
      ;--- qtyObj::GetValue
	  ptr_qty:	arg_present(thisptr_qty), $
	  qty:		arg_present(thisqty), $
	  ptr_err:	arg_present(thisptr_err), $
	  err:		arg_present(thiserr), $
	  ptr_nerr:	arg_present(thisptr_nerr), $
	  nerr:		arg_present(thisnerr), $
	  ptr_legend:	arg_present(thisptr_legend), $
	  legend:	arg_present(thislegend) $
;     ;--- qtyObj::GetOneValue
;	  forPos:	arg_present(thisforPos), $ ; # or {#,#], etc.
;	  forLegend:	arg_present(thisforLegend), $; alternative to forPos
;	  oneQty:	arg_present(thisoneqty), $
;	  oneErr:	arg_present(thisoneerr), $
;	  oneNErr:	arg_present(thisonenerr), $
;	  oneLegend:	arg_present(thisonelegend) $
;;;	  oneRec:	arg_present(thisOneRec), $
;;;	  oneObj:	arg_present(thisOneObj), $
;;;	  obj:		arg_present(thisObj), $
	  }

  endif else begin
    ArgPresent = thisArgPresent
  endelse

;  if (debug) then begin
;    print, infolabel
;    printArgPresent, ArgPresent, /showValues
;  endif

  if ((ArgPresent.name) OR (ArgPresent.units) OR (ArgPresent.axisTitle) OR $
     (ArgPresent.sizeString) OR $
    (ArgPresent.some) OR (ArgPresent.all) OR (ArgPresent.everything)) then begin
    nsuc = nsuc + self -> getString($
	argPresent = argPresent, $
    	name = thisname, $	; long name, suitable for plot
;  	intendedTag = thisIntendedTag, $	; short name, suitable for GUI buttons
  	units = thisunits, $
  	StyleErr = thisStyleErr, $
 	axisTitle = thisAxisTitle, $ 	; Returns string for plot axis label. 
 	sizeString = thisSizeString, $ 
	some=thisSome, $
	all=thisAll, $
	everything=thisEverything)
  endif
  
  if ((ArgPresent.rank) OR (ArgPresent.dim) OR (ArgPresent.tname) ) then begin
     nsuc = nsuc + self -> getDimension($
	argPresent = argPresent, $
        rank = thisRank, $
	dim = thisDim, $ 
  	tname = thisTName)
  endif
 
  if ((ArgPresent.ptr_qty) OR (ArgPresent.qty) OR $
      (ArgPresent.ptr_err) OR (ArgPresent.err) OR $
      (ArgPresent.ptr_nerr) OR (ArgPresent.nerr) OR $
      (ArgPresent.ptr_legend) OR (ArgPresent.legend)) then begin
     nsuc = nsuc + self -> getValue($
	  argPresent = argPresent, $
     	  ptr_qty = thisptr_qty, $
	  qty = thisqty, $
	  ptr_err = thisptr_err, $		; Optional error on the values
	  err = thiserr, $			; Optional error on the values
	  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
	  nerr = thisnerr, $			; Optional negative error on the values
	  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
	  legend = thislegend)
  endif

;  if ((ArgPresent.forPos) OR $(ArgPresent.forLegend) OR $
;      (ArgPresent.oneQty) OR $
;      (ArgPresent.oneErr) OR $
;      (ArgPresent.oneNErr) OR $
;      (ArgPresent.oneLegend)) then begin
;  endif

  ;-----------------------------------------
  success = (nsuc gt 0)

  if (debug) then begin
    print, infolabel, 'nsuc = ', nsuc
    printArgPresent, ArgPresent, /showValues
    print
  endif

  if (success EQ 0) then begin
     print, infolabel, ' Either no recognized keywords provided, ', $
     	" or requesed object is null so return 0'
     printArgPresent, ArgPresent  
     return, success
  endif

  return, success 

end
			;; qtyObj::Get
;=============================================================================
;+
; NAME:
; 	qtyObj::append
;
; PURPOSE:
; 	Append to the end of arrays more things
; 	This MIGHT only work properly with 1-D arrays.
;-
;*
function qtyObj::Append, $	; Returns 0/1 for failure/success.
  ;----------------------- 
  ptr_qty = thisptr_qty, $
  qty = thisqty, $
  ptr_err = thisptr_err, $	; Optional error on the values
  err = thiserr, $		; Optional error on the values
  ptr_nerr = thisptr_nerr, $	; Optional negative error on the values
  nerr = thisnerr, $		; Optional negative error on the values
  ptr_legend = thisptr_legend, $; shares memory using the same pointer.
  legend = thislegend, $	; shares memory using new pointer.
  obj=thisObj 			; Copy contents of same kind of object into self
;/

;Old  include 'leaf_size.pro'
   common QTYOBJ_DIMS, DIM_MAX, DIM_WARN 
  
  infolabel = "qtyObj::Append >> "  
  nsuc = 0
  nfail = 0
  nfail_err = 0
  nfail_nerr = 0
  nfail_legend = 0
  str_fail = ''
  
;Debug  print, infolabel, "New stuff = "
;Debug  print, infolabel, "qty=", thisqty
;Debug  print, infolabel, "err=", thiserr

   str_fail = ''
;Debug  print, "self->count()=", self->count()
  str_orig = ''
  str_orig =  str_orig + '<' + self.name + '>' + format_size(self->count()) + '('
  if (ptr_valid(self.ptr_qty)) then begin
    str_orig =  str_orig + 'qty,'
  endif
  if (ptr_valid(self.ptr_err)) then begin
    str_orig =  str_orig + 'err,'
  endif
  if (ptr_valid(self.ptr_nerr)) then begin
    str_orig =  str_orig + 'nerr,'
  endif
  if (ptr_valid(self.ptr_legend)) then begin
    str_orig =  str_orig + 'leg'
  endif
  str_orig =  str_orig + ')'
  
  str_append = 'appending ('
  if (ptr_valid(thisptr_qty)) then begin
    str_append =  str_append + 'qty[' + $
    	strcompress(string(n_elements(*thisPtr_qty)), /remove_all) + '],'
  endif
  if (n_elements(thisqty) GT 0) then begin
 ;   print, infolabel, "thisQty =", thisQty
    str_append =  str_append + 'qty[' + $
    	strcompress(string(n_elements(thisqty)), /remove_all) + '],'
  endif
;  print, infolabel, "str_append=", str_append

  if (ptr_valid(thisptr_err)) then begin
    str_append =  str_append + 'err[' + $
    	strcompress(string(n_elements(*thisPtr_err)), /remove_all) + '],'
  endif
  if (n_elements(thiserr) GT 0) then begin
    str_append =  str_append + 'err[' + $
    	strcompress(string(n_elements(thiserr)), /remove_all) + '],'
  endif
;  print, infolabel, "str_append=", str_append

  if (ptr_valid(thisptr_nerr)) then begin
    str_append =  str_append + 'nerr[' + $
    	strcompress(string(n_elements(*thisPtr_nerr)), /remove_all) + '],'
  endif
  if (n_elements(thisnerr) GT 0) then begin
    str_append =  str_append + 'nerr[' + $
    	strcompress(string(n_elements(thisnerr)), /remove_all) + '],'
  endif

  if (ptr_valid(thisptr_legend)) then begin
    str_append =  str_append + 'legend[' + $
    	strcompress(string(n_elements(*thisPtr_legend)), /remove_all) + ']'
  endif
  if (n_elements(thisqty) GT 0) then begin
    str_append =  str_append + 'legend[' + $
    	strcompress(string(n_elements(thislegend)), /remove_all) + ']'
  endif

  if (obj_valid(thisObj)) then begin
    str_append =  str_append + 'Obj[' + $
    	strcompress(string(thisObj->count()), /remove_all) + ']'
  endif

  str_append =  str_append + ')'

;debug  print, infolabel, "str_append=", str_append
  
   ;--------------------------------------------------------------
    if ( n_elements(thisObj) gt 0) then begin
      ; Append from an object of the same kind. 
      thisType = size(thisObj, /tname)
      if (thisType EQ 'UNDEFINED') then begin
        print, infolabel, self.name, ' "Obj" is Undefined'        
    	nfail = nfail + 1
      endif else begin
	if (thisType NE 'OBJREF') then begin
    	  nfail = nfail + 1
	  print, infolabel, "The variable for 'obj' keyword is not an object."
	  print, infolabel, "It has type ", thisType 
	endif else begin
	  ; Now check that the object is the same class.
	  if (obj_class(thisObj) NE obj_class(self)) then begin
    	    nfail = nfail + 1
	    print, infolabel, "The variable for 'obj' keyword is in ", $
	    'the object class "', obj_class(thisObj), '". It should be "', $
	    obj_class(self), '"'
	  endif else begin
	    success = 1B
	    success = success and $
	    	thisObj -> getValue(ptr_legend=ptr_templegend,$ 
			ptr_qty=ptr_tempqty, $
			ptr_err=ptr_temperr, ptr_nerr=ptr_tempnerr)
	    if (NOT success) then begin
	      print, infolabel, "Failure with 'thisObj -> getValue(...)'"
	    endif
	    success = success and $
		 self -> Append(ptr_legend=ptr_templegend,$ 
			ptr_qty=ptr_tempqty, $
			ptr_err=ptr_temperr, ptr_nerr=ptr_tempnerr)
	    if (NOT success) then begin
	      print, infolabel, "Failure with 'self -> Append(...)'"
	    endif
	    ptr_free, ptr_templegend
	    ptr_free, ptr_tempqty
	    ptr_free, ptr_temperr
	    ptr_free, ptr_tempNerr
	    if (success) then begin
      	      nsuc = nsuc + 1
	    endif else begin
    	      nfail= nfail + 1
	    endelse
	  endelse
        endelse	
      endelse
      return, success
    endif

 

  ;------------- QTY ------------------------------------------
  if (ptr_valid(thisptr_qty)) OR (n_elements(thisqty) GT 0) then begin
;  if (n_elements(thisptr_qty) GT 0) OR (n_elements(thisqty) GT 0) then begin
    if (not ptr_valid(self.ptr_qty)) then begin
      if (ptr_valid(thisptr_qty)) then begin
;      if (n_elements(thisptr_qty) GT 0) then begin
        holder_ptr_qty = ptr_new( *thisptr_qty)
      endif
      if (n_elements(thisqty) GT 0) then begin
        holder_ptr_qty = ptr_new(thisqty)
      endif
    endif else begin
      if (ptr_valid(thisptr_qty)) then begin
;      if (n_elements(thisptr_qty) GT 0) then begin
        holder_ptr_qty = ptr_new([*(self.ptr_qty), *thisptr_qty])
      endif
      if (n_elements(thisqty) GT 0) then begin
        holder_ptr_qty = ptr_new([*(self.ptr_qty), thisqty])
      endif
    endelse
     
    temp_rank = size(*(holder_ptr_qty), /n_dimensions)
    temp_dim = size(*(holder_ptr_qty), /dimensions)
    ; Maybe add warning is dimension is too large.
    if (temp_rank GE DIM_WARN) then begin
      print, infolabel, "Program may have trouble with # of dimensions = ", $
      	temp_rank, " >= ", DIM_WARN
    endif
    if (temp_rank GT DIM_MAX) then begin
      print, infolabel, "Program WILL have trouble with # of dimensions = ", $
      	temp_rank, " > ", DIM_MAX
     str_fail =  str_fail + 'qty' + format_size(temp_dim)
      nfail = nfail + 1
    endif else begin
      nsuc = nsuc + 1
    endelse
    
;Debug    print, infolabel, "*holder_ptr_qty=", *holder_ptr_qty
   
  endif		;; n_elements(thisptr_qty) GT 0) OR (n_elements(thisqty) GT 0) 

  ;-------------- ERR -----------------------------------------
  if (ptr_valid(thisptr_err)) OR (n_elements(thiserr) GT 0) then begin
;  if (n_elements(thisptr_err) GT 0) OR (n_elements(thiserr) GT 0) then begin
    if (not ptr_valid(self.ptr_err)) then begin
      if (ptr_valid(thisptr_err)) then begin
;      if (n_elements(thisptr_err) GT 0) then begin
        holder_ptr_err = ptr_new( *thisptr_err)
      endif
      if (n_elements(thiserr) GT 0) then begin
        holder_ptr_err = ptr_new(thiserr)
      endif
    endif else begin
      if (ptr_valid(thisptr_err)) then begin
;      if (n_elements(thisptr_err) GT 0) then begin
        holder_ptr_err = ptr_new([*(self.ptr_err), *thisptr_err])
      endif
      if (n_elements(thiserr) GT 0) then begin
        holder_ptr_err = ptr_new([*(self.ptr_err), thiserr])
      endif
    endelse
    
    err_rank = size(*(holder_ptr_err), /n_dimensions)
    err_dim = size(*(holder_ptr_err), /dimensions)
    if (temp_rank NE err_rank) then begin
;      print, infolabel, "# of dimensions for self.err ", $
;      	err_rank, " does not match self.qty ", temp_rank
      nfail_err = nfail_err + 1
    endif
    if (nfail_err EQ 0) then begin
      if (temp_rank eq 1)  then begin
        if (temp_dim NE err_dim) then begin
;          print, infolabel, temp_rank, " dimension for self.err ", err_rank,  $
;		" does not match self.qty ", temp_dim
          nfail_err = nfail_err + 1
        endif
      endif else begin
        for idim = 0, (temp_rank-1) do begin
          if (temp_dim[idim] NE err_dim[idim]) then begin
;            print, infolabel, [idim], " dimension for self.err ", err_rank[idim],  $
;		" does not match self.qty ", temp_dim[idim]
            nfail_err = nfail_err + 1
          endif
	endfor  
      endelse
    endif
 
    if (nfail_err NE 0) then begin
      str_fail =  str_fail + ' err' + format_size(err_dim)
      nfail = nfail + nfail_err
    endif else begin
      nsuc = nsuc + 1
    endelse

;Debug    print, infolabel, "*holder_ptr_err=", *holder_ptr_err
    
  endif		;; n_elements(thisptr_err) GT 0) OR (n_elements(thiserr) GT 0) 
 
  ;-------------- NERR -----------------------------------------
  if (ptr_valid(thisptr_nerr)) OR (n_elements(thisnerr) GT 0) then begin
;  if (n_elements(thisptr_nerr) GT 0) OR (n_elements(thisnerr) GT 0) then begin
    if (not ptr_valid(self.ptr_nerr)) then begin
      if (ptr_valid(thisptr_nerr)) then begin
;      if (n_elements(thisptr_nerr) GT 0) then begin
        holder_ptr_nerr = ptr_new( *thisptr_nerr)
      endif
      if (n_elements(thisnerr) GT 0) then begin
        holder_ptr_nerr = ptr_new(thisnerr)
      endif
    endif else begin
      if (ptr_valid(thisptr_nerr)) then begin
;      if (n_elements(thisptr_nerr) GT 0) then begin
        holder_ptr_nerr = ptr_new([*(self.ptr_nerr), *thisptr_nerr])
      endif
      if (n_elements(thisnerr) GT 0) then begin
        holder_ptr_nerr = ptr_new([*(self.ptr_nerr), thisnerr])
      endif
    endelse
    
    nerr_rank = size(*(holder_ptr_nerr), /n_dimensions)
    nerr_dim = size(*(holder_ptr_nerr), /dimensions)
    if (temp_rank NE nerr_rank) then begin
;      print, infolabel, "# of dimensions for self.nerr ", $
;      	nerr_rank, " does not match self.qty ", temp_rank
      nfail_nerr = nfail_nerr + 1
    endif
    if (nfail_nerr EQ 0) then begin
      if (temp_rank eq 1)  then begin
        if (temp_dim NE nerr_dim) then begin
;          print, infolabel, temp_rank, " dimension for self.nerr ", nerr_rank,  $
;		" does not match self.qty ", temp_dim
          nfail_nerr = nfail_nerr + 1
        endif
      endif else begin
        for idim = 0, (temp_rank-1) do begin
          if (temp_dim[idim] NE nerr_dim[idim]) then begin
;            print, infolabel, [idim], " dimension for self.nerr ", nerr_rank[idim],  $
;		" does not match self.qty ", temp_dim[idim]
            nfail_nerr = nfail_nerr + 1
          endif
	endfor  
      endelse
    endif
 
    if (nfail_nerr NE 0) then begin
      str_fail =  str_fail + ' nerr' + format_size(nerr_dim)
      nfail = nfail + nfail_nerr
    endif else begin
      nsuc = nsuc + 1
    endelse
    
   endif	;; n_elements(thisptr_nerr) GT 0) OR (n_elements(thisnerr) GT 0) 

  ;-------------- LEGEND -----------------------------------------
  if (ptr_valid(thisptr_Legend)) OR (n_elements(thisLegend) GT 0) then begin
;  if (n_elements(thisptr_Legend) GT 0) OR (n_elements(thisLegend) GT 0) then begin
    if (not ptr_valid(self.ptr_Legend)) then begin
      if (ptr_valid(thisptr_Legend)) then begin
;      if (n_elements(thisptr_Legend) GT 0) then begin
        holder_ptr_Legend = ptr_new( *thisptr_Legend)
      endif
      if (n_elements(thisLegend) GT 0) then begin
        holder_ptr_Legend = ptr_new(thisLegend)
      endif
    endif else begin
      if (ptr_valid(thisptr_Legend)) then begin
;      if (n_elements(thisptr_Legend) GT 0) then begin
        holder_ptr_Legend = ptr_new([*(self.ptr_Legend), *thisptr_Legend])
      endif
      if (n_elements(thisLegend) GT 0) then begin
        holder_ptr_Legend = ptr_new([*(self.ptr_Legend), thisLegend])
      endif
    endelse
    
    Legend_rank = size(*(holder_ptr_Legend), /n_dimensions)
    Legend_dim = size(*(holder_ptr_Legend), /dimensions)
    if (temp_rank NE Legend_rank) then begin
;      print, infolabel, "# of dimensions for self.Legend ", $
;      	Legend_rank, " does not match self.qty ", temp_rank
      nfail_Legend = nfail_Legend + 1
    endif
    if (nfail_Legend EQ 0) then begin
      if (temp_rank eq 1)  then begin
        if (temp_dim NE Legend_dim) then begin
;          print, infolabel, temp_rank, " dimension for self.Legend ", Legend_rank,  $
;		" does not match self.qty ", temp_dim
          nfail_Legend = nfail_Legend + 1
        endif
      endif else begin
        for idim = 0, (temp_rank-1) do begin
          if (temp_dim[idim] NE Legend_dim[idim]) then begin
;            print, infolabel, [idim], " dimension for self.Legend ", Legend_rank[idim],  $
;		" does not match self.qty ", temp_dim[idim]
            nfail_Legend = nfail_Legend + 1
          endif
	endfor  
      endelse
    endif
 
    if (nfail_Legend NE 0) then begin
      str_fail =  str_fail + ' Legend' + format_size(Legend_dim)
      nfail = nfail + nfail_Legend
    endif else begin
      nsuc = nsuc + 1
    endelse
    
  endif ;; (n_elements(thisptr_Legend) GT 0) OR (n_elements(thisLegend) GT 0) 

  ;-----------------------------------------
  success = (nsuc gt 0) and (nfail eq 0)
  
;  if (success NE 0) then begin
;    ; Don't reasign the items stored in qtyObj until we've verified that 
;    ; the Appended arrays lengths match.
;    match = 1
;    if (ptr_valid(holder_ptr_err)) then begin
;      match = match and $
;      	((n_elements(*holder_ptr_qty)) EQ (n_elements(*holder_ptr_err)) )
;    endif
;    if (ptr_valid(holder_ptr_nerr)) then begin
;      match = match and $
;      	((n_elements(*holder_ptr_qty)) EQ (n_elements(*holder_ptr_nerr)) )
;    endif
;    if (ptr_valid(holder_ptr_legend)) then begin
;      if (n_elements(*holder_ptr_legend) GT 1) then begin
;        match = match and $
;      	  ((n_elements(*holder_ptr_qty)) EQ (n_elements(*holder_ptr_legend)) )
;      endif
;    endif
;
;    if (match) then begin
    if (success) then begin
        ptr_free, self.ptr_qty
	self.ptr_qty = ptr_new(*holder_ptr_qty)
	temp_tname = size(*holder_ptr_qty, /tname)
	if (temp_tname NE self.tname) then begin
	  print, infolabel, 'Automatic type conversion from ', self.tname, $
	  	' to ', temp_tname
	  self.tname = temp_tname
	endif
	self.rank = temp_rank
	self.dim =  temp_dim

	if (ptr_valid(holder_ptr_err)) then begin
          ptr_free, self.ptr_err
	  self.ptr_err = ptr_new(*holder_ptr_err)
	endif

	if (ptr_valid(holder_ptr_nerr)) then begin
          ptr_free, self.ptr_nerr
	  self.ptr_nerr = ptr_new(*holder_ptr_nerr)
	endif

 	if (ptr_valid(holder_ptr_legend)) then begin
          ptr_free, self.ptr_legend
	  self.ptr_legend = ptr_new(*holder_ptr_legend)
	endif
	
     endif else begin
;       print, infolabel, 'Mismatch in lengths of ', $
;         str_fail
       print, infolabel, "Original object contained: ", str_orig      
       print, infolabel, "Trying to append: ", str_append
       if (obj_valid(thisObj)) then begin
         thisObj -> show
       endif
       print, infolabel, "Perhaps mismatch in 'active' fields or ", $ 
       		"in lengths of new stuff."
       print, infolabel, "Error message = ",  str_fail
;       	'QTY', format_size(ptr_array=holder_ptr_qty), ' ', $
;	'ERR',  format_size(ptr_array=holder_ptr_err), ' ', $
;	'NERR',  format_size(ptr_array=holder_ptr_nerr), ' ', $
;	' and/or ', $
;	'LEGEND', format_size(ptr_array=holder_ptr_legend), $
;       		'. Object not AppendED to.'       
     endelse

;  endif

  ; Cleanup temporary holder pointers now that we are finished with them.
	if (ptr_valid(holder_ptr_qty)) then begin
	  ptr_free, holder_ptr_qty
	endif
	
	if (ptr_valid(holder_ptr_err)) then begin
	  ptr_free, holder_ptr_err
	endif
	if (ptr_valid(holder_ptr_nerr)) then begin
	  ptr_free, holder_ptr_nerr
	endif
	if (ptr_valid(holder_ptr_legend)) then begin
	  ptr_free, holder_ptr_legend
	endif


;   ;-----------------------------------------
;  success = (nsuc gt 0) and (nfail eq 0)
;  
;  if (success EQ 0) then begin
;     print, infolabel, self.name, ' ',  $
;    	strcompress(string(nfail), /remove_all), ' failure(s) and ', $
;	strcompress(string(nsuc), /remove_all), '  success(es)'
;    print, str_fail
;  endif

  return, success                              ;;; Successful? Appending

END		;; qtyObj::Append

;=============================================================================
;+
; NAME:
; 	qtyObj_remove1D
;
; PURPOSE:
; 	Helper function for REMOVE method.
; 	Removes 1 item from a 1D array.
;-
;*
function qtyObj_remove1D, $		; Returns 1 upon success, 0 for failure.
	iwhere, $			; Integer
	original_ptr, $
	new_ptr
;/
; Helper method for REMOVE method.
; returns 1 upon success, 0 for failure.
; removes 1 item from a 1D array.
	
  infolabel = "qtyObj_remove >> "  

    if (NOT ptr_valid(original_ptr)) then begin
      return, 0				; not worthy of a noisy message
    endif

    originalSize = n_elements(*(original_ptr))

    if ((iwhere LT 0) OR (iwhere GE originalSize)) then begin
        print, infolabel, 'Item # ', iwhere, ' is out of range 0-', originalSize
	return, 0
    endif

    if (originalSize eq 1) then begin
       new_ptr = ptr_new()
       return, 1
    endif
    
    if (iwhere EQ 0) then begin
       new_ptr = ptr_new( (*original_ptr)[1:*])
    endif else begin
       if (iwhere EQ (originalSize-1)) then begin
         new_ptr = ptr_new( (*original_ptr)[0:(iwhere-1)])
       endif else begin
;Debug         print, infolabel, 'Ranges 0:', (iwhere-1), ' and ', $
;Debug	 	(iwhere+1), ':', (originalSize-1)
;Debug	 print, infolabel, ' first half: ', (*original_ptr)[0:(iwhere-1)]
;Debug	 print, infolabel, ' second half: ', (*original_ptr)[(iwhere+1): *] 
         new_ptr = ptr_new([ (*original_ptr)[0:(iwhere-1)],$
	    	 (*original_ptr)[(iwhere+1): *] ])
       endelse
    endelse

    return, 1		; success
    
end

;=============================================================================
;*
function qtyObj::remove, $	 ; Returns 0/1 for failure/success.
 ;-------- use one of the two following keywords
  forPos=thisforPos, $	; [#] or [#,#], etc.
  forLegend=thisforLegend 	; alternative to forPos
  		; Give name of legend corresponding to item's position.
;/

;
;  icol=thisICol, $	; Only set one element from 1D and 2D arrays
;  irow=thisIRow, $	; Only set one element from 2D arrays
;  Only scalar and !d implimented so far.
; 1D: forPos = [icol]  destroys item at position icol and shortens array
; 2D: forPos = [-1, irow] destroys "row" irow
; 2D: forPos = [icol, -1] destroys "col" icol
; 2D: forPos = [icol, irow] is not defined.

  infolabel = "qtyObj::remove >> "  
  
  debug = 0;	1
  
  nsuc = 0
  nfail = 0
  str_fail = ''
  
  if (debug) then begin
    print, infolabel, 'Keywords passed are:'
;    if (n_elements(thisIcol) gt 0) then print, '	thisICol=', thisIcol 
;    if (n_elements(thisIrow) gt 0) then print, '	thisIrow=', thisIrow
    if (n_elements(thisforPos) gt 0) then $
  	print, '	thisforPos=', thisforPos
    if (n_elements(thisforLegend) gt 0) then $
  	 print, '	thisforLegend=', thisforLegend
    print, 'End of keywords'
    print
  endif 		; debug 
   ;-----------------------------------------------------
  if (n_elements(thisforLegend) gt 0) then begin
    OK = self -> positionLegend(thisforLegend, specWhere)
    if (debug) then begin 
      print, infolabel,  'specWhere = ', specWhere 
    endif
      if (not OK) then begin
        ; No match found
        nfail = nfail + 1
	print, infolabel, 'No match found for "', thisforLegend ,  '"'
        return, 0
      endif
    if (n_elements(specWhere) eq 1) then begin
      if (OK EQ -2) then begin
        ; positionLegend returned value -2
        ; Means trivial match because n_elements(qtyObj.ptr_legend) EQ 1
	print, infolabel, 'Trivial match found for "', thisforLegend ,  '"'
	print, infolabel, ' Use RemoveAll method instead.'
        nfail = nfail + 1
	return, 0
      endif
      if (n_elements(specWhere) EQ 1) then begin	; successful 1D array
        if (specWhere GE 0) then begin		; successful 1D array
          thisforPos = specWhere
	  print, infolabel, specWhere, '= 1D Match found for "', thisforLegend ,  '"'
          nsuc = nsuc + 1
	endif
      endif
      if (n_elements(specWhere) GT 1) then begin	; successful 2D array
        thisforPos = specWhere
	print, infolabel, specWhere, '= 2D Match found for "', thisforLegend ,  '"'
        nsuc = nsuc + 1
      endif

    endif else begin
      ; successful 2D array 
      thisforPos = specWhere
      nsuc = nsuc + 1
    endelse

    if (debug) then begin
      print, infolabel, $
      	OK, 'OK = self -> positionLegend(thisforLegend,', specWhere, ' )='
;      print, 'nfail, nsuc = ', nfail, nsuc
    endif		; debug
  endif

   ;-----------------------------------------------------
  if (n_elements(thisforPos) gt 0) then begin
    if (debug) then print, infolabel, 'thisforPos = ', thisforPos
    if (n_elements(thisforPos) eq 1) then begin
      thisICol = thisforPos 
    endif else begin
      if (n_elements(thisforPos) eq 2) then begin
        thisICol = thisforPos[0]
        thisIRow = thisforPos[1]
      endif else begin
        print, 'forPos must be an integer scalar or an array of length 2', $
		'(for 1D or 2D quantities).'
	nfail = nfail + 1 
      endelse
    endelse
  endif

  if (debug) then begin
    if (n_elements(thisIcol) GT 0) then $
	print, infolabel, ' thisICol = ', thisIcol
    if (n_elements(thisIrow) GT 0) then $
  	print, infolabel, ' thisIRow = ', thisIRow
  endif	; debug  
  
   ;-----------------------------------------------------
  if (self.rank eq 0) then begin 		; scalar
;;;  if ((self.n_rows EQ 0) AND (self.n_cols EQ 0)) then begin 		; scalar
    if (debug) then begin
      print, infolabel, 'Removing scalar by deleting most info about it.
    endif
    if (self.tname eq 'OBJREF') then begin
      obj_destroy, *(self.ptr_qty)
    endif
    ptr_free, self.ptr_qty
    ptr_free, self.ptr_err
    ptr_free, self.ptr_nerr
    ptr_free, self.ptr_legend
    
    return, 1
  endif								; scalar
  
   ;-----------------------------------------------------
  if (self.rank GT 1) then begin 		; 2D, 3D, or 4D
;  if ((self.n_rows GT 0) AND (self.n_cols GT 0)) then begin 		; 2D
    print, infolabel, strcompress(string(self.rank), /remove_all), $
    	'D remove is not implimented yet.'
    return, 0
  endif									; 2D

   ;-----------------------------------------------------
   if (self.rank EQ 1) then begin 		; 1D
;;   if ((self.n_rows eq 0) and (self.n_cols gt 0)) then begin		; 1D  
     nsuc = qtyObj_remove1D(thisicol, self.ptr_qty, holder_ptr_qty)
     nsuc = nsuc + qtyObj_remove1D(thisicol, self.ptr_err, holder_ptr_err)
     nsuc = nsuc + qtyObj_remove1D(thisicol, self.ptr_nerr, holder_ptr_nerr)
     nsuc = nsuc + qtyObj_remove1D(thisicol, self.ptr_legend, holder_ptr_legend)
     if (nsuc EQ 0) then begin
       print, infolabel, '1D Error removing item ', thisicol
       return, 0
     endif else begin
       if (ptr_valid(holder_ptr_qty)) then begin
         if (self.tname eq 'OBJREF') then begin
           obj_destroy, (*self.ptr_qty)[thisicol]
         endif
	 self.dim[0] = self.dim[0] -1
;	 self.n_cols = self.n_cols -1
         ptr_free, self.ptr_qty
	 self.ptr_qty = ptr_new(*(holder_ptr_qty))
	 ptr_free, holder_ptr_qty
       endif
       if (ptr_valid(holder_ptr_err)) then begin
         ptr_free, self.ptr_err
	 self.ptr_err = ptr_new(*(holder_ptr_err))
	 ptr_free, holder_ptr_err
       endif
       if (ptr_valid(holder_ptr_nerr)) then begin
         ptr_free, self.ptr_nerr
	 self.ptr_nerr = ptr_new(*(holder_ptr_nerr))
	 ptr_free, holder_ptr_nerr
       endif
       if (ptr_valid(holder_ptr_legend)) then begin
         ptr_free, self.ptr_legend
	 self.ptr_legend = ptr_new(*(holder_ptr_legend))
	 ptr_free, holder_ptr_legend
       endif
     endelse
  endif
    
  ;-----------------------------------------
  success = (nsuc gt 0)
  
  if (success EQ 0) then begin
     print, infolabel, self.name, ' ',  $
	strcompress(string(nsuc), /remove_all), '  success(es)'
  endif

  return, success                              ;;; Successful? REMOVing

END		;; qtyObj::remove

;=============================================================================
;*
function qtyObj::removeAll	; Returns 0/1 for failure/success.
		; Only leaves self.name, self.units, self.tname left.
;/


  infolabel = "qtyObj::removeAll >> "  

    if (self.tname eq 'OBJREF') then begin
      obj_destroy, *(self.ptr_qty)
    endif
    ptr_free, self.ptr_qty
    self.ptr_qty = ptr_new()
    ptr_free, self.ptr_err
    self.ptr_err = ptr_new()
    ptr_free, self.ptr_nerr
    self.ptr_nerr = ptr_new()
    ptr_free, self.ptr_legend
    self.ptr_legend = ptr_new()
    self.rank = 0
    self.dim = [0,0,0,0]
;    self.n_rows = 0
;    self.n_cols = 0
    
    return, 1

END		;; qtyObj::removeAll


;=============================================================================
;+
; NAME:
; 	qtyObj::positionLegend
;
; PURPOSE:
;	Returns position thisPos of thisLegend in array *(self.ptr_legend) 
; Function returns 0 for failure and 1 for success. 
; Returns -2 (false) for trivial matches, or if the requested legend is 
; not available. 
; Error if no match or an error (like self.ptr_legend is empty).
; Note: Only returns first item found with the legend thisLegend.
; Case insensitive.
;-
;*
function qtyObj::positionLegend, $	; returns 0/1 for failure/success
  thisLegend, $		; Input string
  thisPos		; Output Array. 0 for scalar, # for 1D, [#,#] for 2-D, etc. 
;/

  infolabel = "qtyObj::positionLegend >> "  
  debug = 0	;1

  if (NOT ptr_valid(self.ptr_legend)) then begin
    if (debug) then print, "NOT ptr_valid(self.ptr_legend)"
    return, 0		; Nothing to look through. return bad error.
  endif
  
  if (NOT arg_present(thisPos)) then begin
    print, infolabel, "Require 2nd parameter to be a variable 'pos'"
    return, 0		; Nothing to look through. return bad error.
  endif

;  if ( thisLegend EQ '*') then begin
;    thisPos = 0
;    return, 1		; First item in the list, by default.
;  endif

  if (self.rank eq 0) then begin		; scalar
    if (StrUpCase(*(self.ptr_legend)) EQ StrUpCase(thisLegend)) then begin
      thisPos = 0
      return, -2	; Trivial Match. True value
    endif else begin
      thisPos = -2
      return, 0	; Trivially no match found.
    endelse
  endif 
  
  if (self.rank EQ 1) then begin
    if ( thisLegend EQ '*') then begin
      thisPos = 0
      return, 1		; First item in the list, by default.
    endif
;    if (debug) then print, infolabel, $
;    	 "StrUpCase(*(self.ptr_legend))=", StrUpCase(*(self.ptr_legend))
;    if (debug) then print, infolabel, $
;    	"StrUpCase(thisLegend) =", StrUpCase(thisLegend) 
;    if (debug) then print, infolabel, $
;    	"StrUpCase(thisLegend[0]) =", StrUpCase(thisLegend[0]) 
;    if (debug) then print, infolabel, $
;    	"StrUpCase((*(self.ptr_legend))) EQ StrUpCase(thisLegend)=", $
;	StrUpCase((*(self.ptr_legend))) EQ StrUpCase(thisLegend)
;    test = StrUpCase(*(self.ptr_legend)) EQ StrUpCase(thisLegend[0])
;    if (debug) then print, infolabel, "test=", test
;    test = (*(self.ptr_legend)) EQ thisLegend
;    if (debug) then print, infolabel, "test=", test
    index1 = where( (StrUpCase(*(self.ptr_legend)) EQ StrUpCase(thisLegend)), $
    		 count1)
;    index1 = where( test, count1)
    if (count1 gt 0) then begin	
      thisPos = index1[0]
      return, 1
    endif else begin
       if (debug) then print, infolabel, '"', thisLegend, '"', $
       	" legend not found in 1-D array ",  $
		 '"',(*(self.ptr_legend))[0],  '", ', $
		 '"',(*(self.ptr_legend))[1],  '", ', $
		 '"',(*(self.ptr_legend))[2],  '"'
	
;       if (debug) then print, infolabel, n_elements(thisLegend), thisLegend, $
;       	" legend not found in 1-D array ",  $
;		n_elements((*(self.ptr_legend))[1]),*(self.ptr_legend)
;	
;      if (debug) then print, "Name of self is ", self.name
      thisPos = -2
      return, 0
    endelse
  endif
  
  if (self.rank EQ 2) then begin
    if ( thisLegend EQ '*') then begin
      thisPos = [0,0]
      return, 1		; First item in the list, by default.
    endif
    test = StrUpCase(*(self.ptr_legend)) EQ StrUpCase(thisLegend)
    index1 = where( test, count1)
    ncols = (self.dim)[0]
    nrows = (self.dim)[1]
;    ncols = *(self.dim)[0]
;    nrows = *(self.dim)[1]
    index_col = index1 mod ncols
    index_row = index1 / nrows
    if (count1 gt 0) then begin	
      thisPos = [index_col, index_row]        
      return, 1
    endif else begin
      thisPos = [-2, -2]
      return, 0
    endelse
  endif

  if (self.rank GT 2) then begin
    print, infolabel, "Can not calculate position of legend for", $
    	" arrays with dimensions > 2."
   print, infolabel, "Use 'forPos' keyword instead of 'forLegend'"
   return, 0
  endif

  
end


;=============================================================================
;*
pro qtyObj::show, $
  lun=thisLun, $ 	; Optionally, write to a file. If absent, to screen
  strArray = thisStrArray, $	; Optionally, write to an array instead of file 
  noTrailingBlank = thisNoTrailingBlank, $
  prefix = thisPrefix, $
  level = thisLevel, $		; how deep in tree
  recursive = thisRecursive, $	;; ignore. End of the tree here.
  heapNum = thisHeapNum		;  ignore. End of the tree here.
;/
;  showObj = thisShowObj, $	; Boolean Keyword. ignore
;  showClassSize = thisShowClassSize, $

  infolabel = "qtyObj::show >> "  

  if (n_elements(thisPrefix) EQ 0) then begin
    localPrefix = ''
  endif else begin
    localPrefix = thisPrefix
  endelse
;;  if (n_elements(thisPrefix) EQ 0) then thisPrefix = ''


;Moved localClassSize to ::Get(everything)
;  localClassSize = ''
;;  if (keyword_set(thisShowClassSize)) then begin
;  	localClassSize = '{' + obj_class(self) +  '}' + $
;	 format_size(ptr_array=self.ptr_qty)
;;;;	 format_size(self.dim)
;;  endif

  localPrefix = localPrefix		;; + localClassSize
  
  offset = ''
  if (n_elements(thisLevel) GT 0) then begin
    for i = 1, thisLevel do begin
      offset = offset + '  '		; 2 spaces
    endfor
  endif

  extraOffset = ''
    for i = 1, (strlen(localPrefix)) do begin
      extraOffset = extraOffset + ' '	; one space
  endfor 
;  print, infolabel, 'strlen(thisPrefix) =', strlen(thisPrefix)   
;  print, infolabel, '"', extraOffset, '"', '=extraOffset'
;  print, infolabel, '"', thisPrefix, '"', '=thisPrefix'
  
  strAr = ''
  if (n_elements(thisStrArray) GT 0) then begin
;    lun = -666		; Don't write to a file
  endif else begin
	;Unit Attributes Name
	;-2   Write, Truncate, Tty, Reserved   <stderr>
	;-1   Write, Truncate, Tty, Reserved   <stdout>
	;0   Read, Tty, Reserved              <stdin>
    if (n_elements(thisLun) eq 0) then begin
      lun= -1
    endif else begin
      lun = thisLun
    endelse
  endelse
  
  OK = self->get(everything=astringarray)
  nlen = n_elements(astringarray)
  strAr = [strAr, string( offset, localPrefix, astringarray[0])]
;  printf, lun, offset, localPrefix, $			;;;localClassSize, ' ', 
;  		astringarray[0]
  for i = 1, (nlen -1) do begin
        strAr = [strAr, string( offset, extraOffset, astringarray[i])]
;        printf, lun, offset, extraOffset, $		;; localClassSize, ' ', 
;		astringarray[i]
  endfor
  
  if (not keyword_set(thisNoTrailingBlank)) then  strAr = [strAr, ' ']
;  if (not keyword_set(thisNoTrailingBlank)) then printf, lun
  
  ;----------------------------------------------
  if (arg_present(thisStrArray)) then begin
     thisStrArray = strAr; 
  endif else begin
    nelm = n_elements(strAr)
    for i = 0, (nelm-1) do begin
      printf, lun, strAr[i]
    endfor 
  endelse  

end		; ::show 
 

;=============================================================================
;=================================================================
;*
function qtyObj__prepErrors, $	; returns 0/1 for failure/success
	thisObj1, $		; Either obj in class qtyObj or scalar
	;----------------------------------------------
	ptr_qty=ptr_qty1, $		; output
	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg1, $		; output
	units=units1, $ 		; output
	name=name1, $ 			; output
	;----------------------------------------------------
	errMSg = ErrMsg1		; output. If there has been an error
;/

;  success = 1
  infolabel = "qtyObj::PrepErrors >> "
  debug = 0
  
  OK1 = 1

  if (n_elements(thisObj1) EQ 0) then begin
    errMsg1 = string(infolabel, "Missing required parameter thisObj")
    return, 0
  endif
    
  errMsg1 = ' '
  if (obj_valid(thisobj1)) then begin
    OK1 = thisobj1 -> get($
	ptr_qty=ptr_qty1, $		; output
	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg1, $		; output
	units=units1, $ 		; output
	name=name1)		; output.
    if (NOT OK1) then ErrMsg1 = string(infolabel, "thisobj1 -> get() FAILED")

    if (ptr_valid(ptr_err1) AND ptr_valid(ptr_nerr1)) then begin
        ; Make the errors symmetric	
	  (*ptr_err1) = ((*ptr_err1) + (*ptr_nerr1))/2.0
	  ptr_free, ptr_nerr1
    endif

  endif else begin
    ; Assume that obj1 is a numeric scalar
    ptr_qty1 = ptr_new(thisObj1)
    if (n_elements(thisObj1) EQ 1) then begin
      name1 = strtrim(string(thisObj1), 2)
      units = ''
;Wrong      units1 = strtrim(string(thisObj1), 2)
    endif else begin
      name1 = format_list(thisObj1)
      units = ''
;Wrong      units1 = format_list(thisObj1)
    endelse
  endelse

  ; Use scalar zero for those errors not being used.
    if (NOT ptr_valid(ptr_err1)) then begin
	  if (n_elements((*ptr_qty1)) EQ 1) then begin
	    tempZero = (*ptr_qty1) - (*ptr_qty1) 
	  endif else begin
	    tempZero = ((*ptr_qty1)[0]) - ((*ptr_qty1)[0]) 
	  endelse
	  ptr_err1 = ptr_new(tempZero)	; same type as (*ptr_qty1)
    endif
    if (ptr_valid(ptr_nerr1)) then ptr_free, ptr_nerr1


  return, OK1
  
end		;     qtyObj__prepErrors

;=================================================================
;*
function qtyObj__listBinaryOp		; returns list of recognized operators
;/

  return, ['+', '-', '*', '/', '^']
end					; qtyObj__listBinaryOp

;=================================================================
;*
function qtyObj__BinaryOp, $	; returns object of type qtyObj.  Null obj if error.
	; returns (thisObj1 thisOp thisObj2)
	thisObj1, $		; Either obj in class qtyObj or scalar
	thisOp, $		; '+', '-', '*', '/'
	thisObj2, $ 		; Either obj in class qtyObj or scalar
	;------------------------
	; If name, units or legend are not provided, they are taken from tags1.
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend, $	; legend for new results
	;----------------------------------------------
	estimate = thisEstimate, $	; Estimate the errors if operator not recognized
	errMsg = thisErrMsg		; Optional error message if failure
;/

;  success = 1
  infolabel = "qtyObj__BinaryOp >> "
  debug = 0

  OK1 = qtyObj__prepErrors( $
	thisObj1, $		; Either obj in class qtyObj or scalar
	;----------------------------------------------
	ptr_qty=ptr_qty1, $		; output
	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg1, $		; output
	units=units1, $ 		; output
	name=name1, $ 			; output
	;----------------------------------------------------
	errMSg = ErrMsg1)		; output. If there has been an error

  if (NOT OK1) then begin
    thisErrmsg = [errMsg1, string(infolabel, "Failure with thisObj1")]
    newObj = obj_new()		; failure object
    goto, the_cleanup
  endif

  ;---------------------------------------------------------------     
  OK2 = qtyObj__prepErrors( $
	thisObj2, $		; Either obj in class qtyObj or scalar
	;----------------------------------------------
	ptr_qty=ptr_qty2, $		; output
	ptr_err=ptr_err2, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg2, $		; output
	units=units2, $ 		; output
	name=name2, $ 			; output
	;----------------------------------------------------
	errMSg = ErrMsg2)		; output. If there has been an error
  
  if (NOT OK2) then begin
    thisErrmsg = [errMsg2, string(infolabel, "Failure with thisObj2")]
;     thisErrmsg = errMsg2
    newObj = obj_new()		; failure object
    goto, the_cleanup
  endif
  ;---------------------------------------------------------------  
;  errMsg = ''
  case (thisOp) of 
    '+': begin
  	 qtyout  = (*(ptr_qty1)) + (*(ptr_qty2))
    	 end
    '-': begin
 	 qtyout  = (*(ptr_qty1)) - (*(ptr_qty2))
     	 end
    '*': begin
 	 qtyout  = (*(ptr_qty1)) * (*(ptr_qty2))
     	 end
    '/': begin
 	 qtyout  = (*(ptr_qty1)) / (*(ptr_qty2))
     	 end
    '^': begin
 	 qtyout  = (*(ptr_qty1)) ^ (*(ptr_qty2))
     	 end
    else: begin
         if (keyword_set(thisEstimate)) then begin
	    exStr = 'qtyout = ' + "( (*ptr_qty1) )" + thisOp + "( (*ptr_qty2) )" 
	    OK = execute(exStr)
	    if (NOT OK) then begin
              message, /info, string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' with qty")
	      thisErrMsg = string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' with qty")
    	      newObj = obj_new()		; failure object
    	      goto, the_cleanup
;	     return, 0
	     
	    endif
	    if (debug) then print, infolabel,  "estimated qtyout=", qtyout
	 endif else begin
           message, /info, string("Unrecognised binary operator '", thisOp, "'")
	   thisErrMsg = string("Unrecognised binary operator '", thisOp, "'")
    	   newObj = obj_new()		; failure object
    	   goto, the_cleanup
;	   return, obj_new()
;;	   return, 0
	 endelse
    	 end
   endcase
    
  ;---------------------------------------------------------------  
  errIsUsed1 = 1
  if (n_elements(*ptr_err1) eq 1) then begin
    if ((*ptr_err1) eq 0.0) then begin
      errIsUsed1 = 0
    endif
  endif

  errIsUsed2 = 1
  if (n_elements(*ptr_err2) eq 1) then begin
    if ((*ptr_err2) eq 0.0) then begin
      errIsUsed2 = 0
    endif
  endif

  if ( errIsUsed1 OR  errIsUsed2 ) then begin
   ; Unused errors are currently a scalar zero.
    case (thisOp) of 
      '+': begin
         errout = sqrt( (*(ptr_err1))^2 + (*ptr_err2)^2)
    	 end
      '-': begin
         errout = sqrt( (*(ptr_err1))^2 + (*ptr_err2)^2)
     	 end
      '*': begin
         errout = sqrt( ( (*ptr_err1) * (*ptr_qty2) )^2 + $
      		     ( (*ptr_err2) * (*ptr_qty1) )^2  ) 
     	 end
      '/': begin
         errout = sqrt( ( (*ptr_err1) / (*ptr_qty2) )^2 + $
       		     ( (*ptr_err2) * (*ptr_qty1) / ((*ptr_qty2)^2) )^2  ) 
     	 end
      '^': begin
         errout = sqrt( ( (*ptr_err1) * $
		(*ptr_qty2) * ((*ptr_err1) ^ (*ptr_qty2 - 1.0)) )^2 + $
       		( (*ptr_err2) * (alog((*ptr_qty1)) * (*ptr_qty1) ^(*ptr_qty2)) $
			 )^2  ) 
     	 end
      else: begin
         if (keyword_set(thisEstimate)) then begin
	    ; this is a hack. Need to discuss with others if this is a reasonable
	    ; approach to estimating the error.
	    OK = 1
	    exStr_pp = "errout_pp =  " $
	     + "( ((*ptr_qty1) + (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) + (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_pp)
	    if (NOT OK) then goto, ex_fail
            errout_pp = abs(qtyout - errout_pp )
	    
	    exStr_pm = "errout_pm =  " $
	     + "( ((*ptr_qty1) + (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) - (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_pm)	    
	    if (NOT OK) then goto, ex_fail
            errout_pm = abs( qtyout - errout_pm )
	    
	    exStr_mp = "errout_mp =  " $
	     + "( ((*ptr_qty1) - (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) + (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_mp)
	    if (NOT OK) then goto, ex_fail
            errout_mp = abs( qtyout - errout_mp )
	    
	    exStr_mm = "errout_mm =  " $
	     + "( ((*ptr_qty1) - (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) - (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_mm)
	    if (NOT OK) then goto, ex_fail
            errout_mm = abs( qtyout - errout_mm )

 ex_fail:
	    if (NOT OK) then begin
              message, /info, string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' on estimated errors")
	      thisErrMsg = string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' on estimated errors")
    	      newObj = obj_new()		; failure object
    	      goto, the_cleanup
;	     return, obj_new()
;;	     return, 0
	    endif

            n_nonzero = 0
	    if (norm(errout_pp) GT 0.0) then  n_nonzero = n_nonzero + 1
	    if (norm(errout_pm) GT 0.0) then  n_nonzero = n_nonzero + 1
	    if (norm(errout_mp) GT 0.0) then  n_nonzero = n_nonzero + 1
	    if (norm(errout_mm) GT 0.0) then  n_nonzero = n_nonzero + 1

	    
	    errout = 0.0
	    if (n_nonzero GT 0) then begin
	     errout = (errout_pp + errout_pm + errout_mp + errout_mm)
	     errout = errout / n_nonzero
	    endif

	    if (debug) then begin
              print, infolabel, "errout_pp = ", errout_pp
              print, infolabel, "errout_pm = ", errout_pm
              print, infolabel, "errout_mp = ", errout_mp
              print, infolabel, "errout_mm = ", errout_mm
	      print, infolabel, " n_nonzero = ",  n_nonzero
	      print, infolabel,  "estimated errout=", errout 
	    endif

	 endif else begin
           message, /info, string("Unrecognised binary operator '", thisOp, "'")
	   thisErrMsg = string("Unrecognised binary operator '", thisOp, "'")
    	   newObj = obj_new()		; failure object
    	   goto, the_cleanup
;	   return, obj_new()
;;	   return, 0
	 endelse
   	 end
     endcase
  endif else begin
    if (debug) then  print, infolabel, " No errors"
    errout = 0.0
  endelse
  ;---------------------------------------------------------------  

  if (n_elements(errout) eq 1) then begin  
    if (errout eq 0.0) then begin
      ; ptr_errout is undefined
    endif else begin
      ptr_errout = ptr_new(errout)
    endelse
  endif else begin
    ptr_errout = ptr_new(errout)
  endelse

  ;---------------------------------------------------------------  
  if (n_elements(thisName) GT 0) then begin
    resName = thisName
  endif else begin
    if (n_elements(Name1) GT 0) then begin
      resName = Name1 + ' ' + thisOp + ' ' + Name2
    endif
  endelse

  if (n_elements(thisUnits) GT 0) then begin
    resUnits = thisUnits
  endif else begin
   if (n_elements(Units1) GT 0) then begin
     case (thisOp) of 
      '+': begin
         resUnits = Units1 
   	 end
      '-': begin
         resUnits = Units1 
     	 end
;      '^': begin
;         resUnits = Units1 
;     	 end
      '*': begin
         resUnits = Units1 + ' ' + Units2
     	 end
;      '/': begin
;         resUnits = Units1 + ' ' + thisOp + ' ' + Units2
;      	 end
       else: begin
         resUnits = Units1 + ' ' + thisOp + ' ' + Units2
         end
      endcase
   endif
  endelse

  if (n_elements(thislegend) GT 0) then begin
    reslegend = thisLegend
  endif else begin
    if (ptr_valid(ptr_leg1)) then resLegend = (*ptr_leg1)
  endelse

  newObj = obj_new('qtyObj', name = resName, $
  	units = resUnits, legend = resLegend, $
	qty = qtyout, ptr_err=ptr_errout)	

  if (NOT obj_valid(newObj)) then begin		; New Aug 18, 2003. TMR.
     thisErrMsg = string("Object failed to be constructed.") 
     newObj = obj_new()		; failure object
     goto, the_cleanup
;      return, obj_new()
  endif
 	     	     
  the_cleanup:
  
  if (ptr_valid(ptr_qty1))  then ptr_free, ptr_qty1
  if (ptr_valid(ptr_qty2))  then ptr_free, ptr_qty2
  if (ptr_valid(ptr_err1))  then ptr_free, ptr_err1
  if (ptr_valid(ptr_err2))  then ptr_free, ptr_err2
  if (ptr_valid(ptr_leg1)) then ptr_free, ptr_leg1
  if (ptr_valid(ptr_leg2)) then ptr_free, ptr_leg2
   
  if (ptr_valid(ptr_errout))  then ptr_free, ptr_errout
  		
  return, newObj

end		; qtyObj__BinaryOp

;=================================================================
;*
function qtyObj__listUnitaryOp		; returns list of recognized operators
;/

  return, ['-', 'cos', 'sin', 'exp', 'alog']
end					; qtyObj__listUnitaryOp

;=================================================================
;*
function qtyObj__UnitaryOp, $	; returns object of type qtyObj.  Null obj if error.
	; returns (thisObj1 thisOp thisObj2)
	thisOp, $		;  '-', 'cos', 'sin', 'exp', 'alog'
	thisObj1, $		; Either obj in class qtyObj or scalar
	; Operates upon item stored at field1.
	;------------------------
	; If name, units or legend are not provided, they are taken from tags1.
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend, $	; legend for new results
	;-------------------------------------------------
	estimate = thisEstimate,$	; Estimate the errors if operator not recognized
	;----------------------------------------------
	errMsg = thisErrMsg		; Optional error message if failure
;/
	;----------------------------------------------

;  success = 1
  infolabel = "qtyObj__UnitaryOp >> "
  debug = 0	;1
  
  errMsg1 = ' '

  OK1 = qtyObj__prepErrors( $
	thisObj1, $		; Either obj in class qtyObj or scalar
	;----------------------------------------------
	ptr_qty=ptr_qty1, $		; output
	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg1, $		; output
	units=units1, $ 		; output
	name=name1, $ 			; output
	;----------------------------------------------------
	errMSg = ErrMsg1)		; output. If there has been an error

  if (NOT OK1) then begin
    thisErrmsg = errMsg1
    newObj = obj_new()		; failure object
    goto, the_cleanup
  endif

  ;---------------------------------------------------------------  
;  errMsg = ''
  case (thisOp) of 
    '-': begin
 	 qtyout  = -(*(ptr_qty1))
     	 end
    'cos': begin
 	 qtyout  = cos(*(ptr_qty1))
     	 end
    'sin': begin
 	 qtyout  = sin(*(ptr_qty1))
     	 end
    'exp': begin
 	 qtyout  = exp(*(ptr_qty1))
     	 end
    'alog': begin
 	 qtyout  = alog(*(ptr_qty1))
     	 end
    else: begin
         if (keyword_set(thisEstimate)) then begin
	    exStr = 'qtyout = ' + thisOp + "( (*ptr_qty1) )" 
	    OK = execute(exStr)
	    if (NOT OK) then begin
              message, /info, string("Failure to EXECUTE unitary operator'", $
	      		thisOp, "'")
    	     newObj = obj_new()		; failure object
    	     goto, the_cleanup
;	     return, 0
	    endif
	 endif else begin
           message, /info, string("Unrecognised unitary operator '", thisOp, "'")
    	   newObj = obj_new()		; failure object
    	   goto, the_cleanup
;	   return, 0
	 endelse
    	 end
   endcase
    
  ;---------------------------------------------------------------  
  errIsUsed1 = 1
  if (n_elements(*ptr_err1) eq 1) then begin
    if ((*ptr_err1) eq 0.0) then begin
      errIsUsed1 = 0
    endif
  endif


  if ( errIsUsed1 ) then begin
   ; Unused errors are currently a scalar zero.
    case (thisOp) of 
      '-': begin
         errout = *(ptr_err1)
     	 end
      'cos': begin
         errout = abs((*ptr_err1) *  sin(*(ptr_qty1)))
     	 end
      'sin': begin
         errout = abs((*ptr_err1) * cos(*(ptr_qyt1)))
     	 end
      'exp': begin
         errout = (*ptr_err1) * exp(*(ptr_qty1))
     	 end
      'alog': begin
         errout = (*ptr_err1) / (*(ptr_qty1))
     	 end
      else: begin
         if (keyword_set(thisEstimate)) then begin
	    exStr_plus = "errout_plus = " +  thisOp + $
	    	'( ( (*ptr_qty1) + (*ptr_err1) ) ) '
	    OK_plus = execute(exStr_plus)
	    if (NOT OK_plus) then begin
              message, /info, string("Failure to EXECUTE unitary operator'", $
	      		thisOp, "' on positive errors")
	     return, 0
	    endif
            errout_plus = abs( (*ptr_qty1) - errout_plus )
	    
	    exStr_minus = "errout_minus = " + thisOp + $
	    	'( ( (*ptr_qty1) - (*ptr_err1) ) )'
	    OK_minus = execute(exStr_minus)
	    errout = abs(errout_plus - errout_minus)/2.0
	    if (NOT OK_minus) then begin
              message, /info, string("Failure to EXECUTE unitary operator'", $
	      		thisOp, "' on negative errors")
	     return, 0
	    endif
            errout_minus = abs( (*ptr_qty1) - errout_minus )

	 endif else begin
           message, /info, string("Unrecognised unitary operator '", thisOp, "'")
    	   newObj = obj_new()		; failure object
    	   goto, the_cleanup
;	   return, 0
	 endelse
    	 end
     endcase
   endif else begin
;    if (debug) then  print, infolabel, " No errors"
    errout = 0.0
  endelse
  ;---------------------------------------------------------------  

  if (n_elements(errout) eq 1) then begin  
    if (errout eq 0.0) then begin
      ; ptr_errout is undefined
    endif else begin
      ptr_errout = ptr_new(errout)
    endelse
  endif else begin
    ptr_errout = ptr_new(errout)
  endelse

  ;---------------------------------------------------------------  
  if (n_elements(thisName) GT 0) then begin
    resName = thisName
  endif else begin
    if (n_elements(Name1) GT 0) then begin
      resName = string(thisOp, '(', Name1, ')')
    endif
  endelse

  if (n_elements(thisUnits) GT 0) then begin
    resUnits = thisUnits
  endif else begin
   if (n_elements(Units1) GT 0) then begin
     case (thisOp) of 
      '-': begin
         resUnits = Units1 
     	 end
      else: begin
         resUnits = string(thisOp, '(', Units1, ')')
         end
      endcase
   endif
  endelse
  

  if (n_elements(thislegend) GT 0) then begin
    reslegend = thisLegend
  endif else begin
    if (ptr_valid(ptr_leg1)) then resLegend = (*ptr_leg1)
  endelse

  newObj = obj_new('qtyObj', name = resName, $
  	units = resUnits, legend = resLegend, $
	qty = qtyout, ptr_err=ptr_errout)	
  	     	     
  if (NOT obj_valid(newObj)) then begin		; New Aug 18, 2003. TMR.
     thisErrMsg = string("Object failed to be constructed.") 
     newObj = obj_new()		; failure object
     goto, the_cleanup
  endif
 	     	     
  the_cleanup:

  if (ptr_valid(ptr_qty1))  then ptr_free, ptr_qty1
  if (ptr_valid(ptr_err1))  then ptr_free, ptr_err1
  if (ptr_valid(ptr_leg1)) then ptr_free, ptr_leg1
   
  if (ptr_valid(ptr_errout))  then ptr_free, ptr_errout
		
  return, newObj

end		; qtyObj__UnitaryOp



;================================================================================
;*
pro qtyObj::Cleanup
;/

  infolabel = "qtyObj::cleanup >> "  
; Note: currently can not deal with any of these pointers containg pointers.
; Only STRING, numeric types. 
; Nothing is allowed to be type OBJREF.

;Old  ptr_free, self.ptr_dim			; type integer
  	
  ptr_free, self.ptr_legend			; type STRING		

  if (ptr_valid(self.ptr_qty)) then begin
    ; If type is STRING or numeric, no special cleanup neccessary.
    tname = size(*(self.ptr_qty), /tname)
;    if (tname EQ 'OBJREF') then begin
;      ; destroys single object or array of objects.
;      obj_destroy, *(self.ptr_qty)
;    endif
  endif
  ptr_free, self.ptr_qty		; STRING or numeric or OBJREF type.

  if (ptr_valid(self.ptr_err)) then begin
    ; If type is STRING or numeric, no special cleanup neccessary.
    tname = size(*(self.ptr_err), /tname)
;    if (tname EQ 'OBJREF') then begin
;      ; destroys single object or array of objects.
;      obj_destroy, *(self.ptr_err)
;    endif
  endif
  ptr_free, self.ptr_err		; numeric type

  if (ptr_valid(self.ptr_nerr)) then begin
    ; If type is STRING or numeric, no special cleanup neccessary.
    tname = size(*(self.ptr_nerr), /tname)
;    if (tname EQ 'OBJREF') then begin
;      ; destroys single object or array of objects.
;      obj_destroy, *(self.ptr_nerr)
;    endif
  endif
  ptr_free, self.ptr_nerr		; numeric type
  
  
end		; :: cleanup

;================================================================================
;*
pro qtyObj__define
  infolabel = "qtyObj__define >> "  
  define = {qtyObj, $
    name: '', $	; Long name or description wuitable for plot axis.
    		; Example: "Sample temperature" 
    units:'', $	; Units
    tname: '', $; Type of data ('INT', 'FLOAT', 'DOUBLE', 'STRING',, etc.
    		; Not allowed: 'POINTER' and  'OBJREF'.
		; This is the type of *(self.ptr_qty).
		; The errors  *(self.ptr_err) and  *(self.ptr_nerr), 
		; might be of another type.
    		; Only need to specify self.tname upon init when want to 
		; initialize with 0/null values. Othersize, if self.ptr_qty is 
		; non-trivially initialized (e.g. not a null pointer)
		; then self.tname is automatically determined by
		; the type of what self.ptr_qty is pointing to.
    StyleErr: '', $ ; '' Default normal-Gaussian errors.
    		; 'poisson'
    		; 'center_step'. The ptr_err is really a step size.
		; 'start_step'. The ptr_err is really a step size.
		; 'cov_matrix'. ptr_err has dimensions of dim^2 rather than dim
    rank: 0, $	; Number of dimensions size(*(self.ptr_qty), /n_dimensions)
    		; 0 for scalar. 1 for 1-D, 2 for 2-D, etc.
    dim: [0,0,0,0], $ ; Pointer to an array containing
    		; size(*(self.ptr_qty), /dimensions)
		; [0,0,0,0] for scalar,
		; [#,0,0,0] for 1-D,
		; [#,#,0,0] for 2-D, etc.
    ptr_legend: ptr_new(), $ ; Pointer to array of type string.
    		; Holds some descriptive info, e.g. 'Average Temp' or 'Quench'
                ; Note: use null ptr if legends are not needed.
    ptr_qty: ptr_new(), $ ; Pointer to array of type self.tname.
    		; Size of array is given by self.rank and self.dim.
		; Note; data is stored via this pointer, but in methods
		; SET and GET you can use the keywords ptr_qty (by reference)
		; or qty (by value).
    ptr_err: ptr_new(), $ ; symmetric errors or positive errors (if use ptr_nerr)
    		; Only makes sense for numerical types, but no restriction. 
		; Need not have the same type as *(self.ptr_qty).
		; Note: use null ptr if errors are not needed.
    		; Size of array is given by self.rank and self.dim.
		; Note; data is stored via this pointer, but in methods
		; SET and GET you can use the keywords ptr_err (by reference)
		; or err (by value).
    ptr_nerr: ptr_new() $ ; Optional negative errors.
    		; Only makes sense for numerical types, but no restriction. 
		; Need not have the same type as *(self.ptr_qty).
		; Note: use null ptr if errors are not needed.
    		; Size of array is given by self.rank and self.dim.
		; Note; data is stored via this pointer, but in methods
		; SET and GET you can use the keywords ptr_nerr (by reference)
		; or nerr (by value).
    }
;/

; Not needed. In class DAVE_tagContainer, use self.myTags as a link to
; the self.tags in the next level up.
;    intendedTag: '', $	; short name, to be used by class branchObj's self.ptr_tag
;    		; as the "column" label. Should be unique.
;    		; Example: "T_s" or "temp" 
; Silly.
;    ptr_Dim: ptr_new(), $ ; Pointer to an array containing
;    		; size(*(self.ptr_qty), /dimensions)
;		; 0 for scalar, [#] (or #?)  for 1-D, [#,#] for 2-D, etc.
;    ptr_type: ptr_new() $ ; Optional pointer to an array of "type" strings.
;    		; Like xtyle/ytype in DAVE: "points", "histo"
;		; Use self.ptr_err as a step size rather than true error for 
;		; "center_step" and "start_step".
;    ptr_junior: ptr_new() ; Pointer to an anonynous structure
;    		; containing helper objects or what-have-you.

end
