; $Id: reducespecobj.pro,v 1.9 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
;       reduceSpecObj
;
; PURPOSE:
;
;	This is the DAVE program button which launches the specObj reduction
; programs, for either raw triple axis ICP data or for pre-existing DAVE files.
; The pre-existing DAVE files may either be saved reduced triple axis ICP data
; (containing the specific object in the specific area) or other dave files
; which may or may not have a specific object. 
;
; AUTHOR:
;	Richard Azuah, Ph.D. and
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       DAVE application button.
;
; SEE ALSO:
; 	<LI><A HREF="#LAUNCHSPECOBJ">LAUNCHSPECOBJ</A>
;	<LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
; MODIFICATION HISTORY:
;	Richard Azuah and Tanya Riseman, July 23, 2003. 
;	If no files selected, return control to main DAVE. TMR Aug 1, 2003.
;	TMR. Aug 29, 2003. Changed "state={...instr: 'bt2'..",  to 
;		"state={...instr: '*.bt*'.." to help pickfile filter.
;	TMR. Sept 10, 2003. Renamed reduce_frgd_bkgd_specobj.pro reducespecobj.pro.
;	Now this does both raw data files and DAVE files.
;	TMR. Sept 15, 2003. changed usingGUI (0/1) to usingGroupLeader (0/#)
;-

;===============================================================================
; reduceSpecObj
;
;===============================================================================



;===============================================================================
pro reduceSpecObj_cleanup, tlbID

; message, /info, 'Cleaning up...'
widget_control, tlbID, get_uvalue=sPtr


if (!debug) then begin
  message, /info, "Entered"
  print,'Validity of sPtr is ',ptr_valid(sPtr)
  if (ptr_valid(sPtr)) then print, sPtr
endif

if (obj_valid((*sPtr).GUIObj1)) then begin 
  if (!debug) then begin
     message, /info,  string("obj_destroy, (*sPtr).guiObj1 of class ", $
    		obj_class((*sPtr).guiObj1))
  endif
  obj_destroy, (*sPtr).GUIobj1
endif

if (obj_valid((*sPtr).GUIObj2)) then begin 
  if (!debug) then begin
    message, /info,  string("obj_destroy, (*sPtr).guiObj2 of class ", $
    		obj_class((*sPtr).guiObj2))
  endif
  obj_destroy, (*sPtr).GUIobj2
endif


;---- BEGIN DAVE specific cleanup section
icpInfo = {reduceSpecObjEVENT,$
          id:(*sPtr).notifyIDs[0],$
          top:(*sPtr).notifyIDs[1],$
          handler:0L}
if widget_info((*sPtr).notifyIDs[0],/valid_id) then begin
  message, /info, string($
  	"Changing GUI focus back to main DAVE program, id=", $
  	(*sPtr).notifyIDs[0])
  widget_control,(*sPtr).notifyIDs[0],send_event = icpInfo
endif

;; Not normal. Just experimenting.
;if widget_info((*sPtr).notifyIDs[1],/valid_id) then begin
;  message, /info, string($
;  	"Changing GUI focus back to main DAVE program. top=",$
;  	(*sPtr).notifyIDs[1] )
;  widget_control,(*sPtr).notifyIDs[1],send_event = icpInfo
;endif

;---- END DAVE specific section

if (!debug) then begin
  help, sPtr
  print, (*sPtr)
  print,'Validity of sPtr is ',ptr_valid(sPtr)
endif
if (!debug) then print,'Validity of davePtr is ',ptr_valid((*sPtr).davePtr)

ptr_free, sPtr		; Aug 1st, 2003

if (!debug) then print,'Validity of sPtr is ',ptr_valid(sPtr)

end



;;===============================================================================
;pro reduceSpecObj_event,event
;
;uname = widget_info(event.id,/uname)
;
;case uname of
;    'DISMISS': widget_control,event.top,/destroy
;    else:
;endcase
;
;end



;===============================================================================
pro reduceSpecObj, davePtr,notifyIds=notifyIds,group_leader=group_leader

  compile_opt idl2

  debug = 0
  debug2 = 0
  
  infolabel = 'reduceSpecObj >> ' 
  
  if (n_elements(group_leader) NE 0) then begin
    usingGroupLeader = group_leader
  endif else begin
    usingGroupLeader = 0L
  endelse
  
;  usingGUI = 1		; Only used by 'FRGD_BKGD' currently
;  if (usingGUI eq 1) then begin
;    usingGroupLeader = group_leader
;  endif else begin
;    usingGroupLeader = 0L
;  endelse
;;  help, usingGUI
;;  help, usingGroupLeader

      if (!debug) then begin
        message, /info, "Entered reduceSpecObj"
;        help, /heap
        message, /info, "help, davePtr ="
        help, davePtr
      endif

;-----------------------------------------------
; file dialog for selecting files
title = 'raw ICP or DAVE file selection'

datDir = (*!dave_defaults).datdir         ; get current data directory from DAVE env var

cd, datDir[0], cur=curDir       ; cd to fans data dir

  if (debug2) then begin
    message, /info, string("DavePtr = ", davePtr)
    print, *(davePtr)
    print, "help, /heap"
    help, /heap
    print
  endif
;----------------------------------------------------
; state variable...
state = { $
        davePtr:davePtr, $      ; the davePtr
        notifyIds:notifyIds, $	; event structure used to communicate with dave.pro through launchicp.pro
	; below: specific to this application:
        GUIobj1: obj_new(), $	; fill in later.
        GUIobj2: obj_new(), $	; fill in later.
	instr: '*.*', $		; which instrument bt2, etc.
	dataReader: 'func_icp_data_init', $	; which subroutine to use to read the data.
	action: 'SINGLE' $	; which method to use with specObj.
;	secondAction: '' $	; Second action 'FRGD_BKGD'
        }
sPtr = ptr_new(state)

;----------------------------------------------------

  GUIObj1 = obj_new('dialog_ow_selectFileType', $
  		groupLeader=group_leader,  $
		; instrOptions = ['', '', '',...], $	; Has triple axis defaults.
		; dataReaderOptions = ['', '', '',..], $; Has triple axis defaults.
		; actionOptions = ['', '', '',..], $	; Has triple axis defaults.		
;		; secondActionOps = ['', ''], $		; has triple axis defaults
  		statePtr = sPtr)   

; update state variable...
  (*sPtr).GUIobj1 = GUIObj1	

; create the GUI
  OK = ((*sPtr).GUIobj1) -> createWidgets()

  ;------------------------------------------------------
  message, /info, String("(*sPtr).instr =", (*sPtr).instr)
  if ((*sPtr).instr EQ 'IN_MEMORY') then begin
    goto, skip_file_selection
  endif
  ;----------------------------------------------------

  allowed_extensions = [(*sPtr).instr]	; '*.*', '*.bt*', '*.bt2', '*.dave', etc.

  theMultiMethod = (*sPtr).action	; 'SINGLE' 'CONCATENATE', 'COMPACTCONCATENATE',
		; 'ZIP', 'MERGE', 'FRGD_BKGD'
  theDataReader = (*sPtr).dataReader

  if (theMultiMethod EQ 'SINGLE') then begin
    ; Only a single file may be selected
    long_fileList = dialog_pickfile(title=title,$
  	get_path=newDir, $
        filter=allowed_extensions, $
	/must_exist,dialog_parent=group_leader)
  endif else begin
    ; Exactly 2 files may be selected
    if (theMultiMethod EQ 'FRGD_BKGD') then begin
      long_fileList1 = dialog_pickfile(title=string('Foreground ', title),$
  	get_path=newDir1, $
        filter=allowed_extensions, $
	/must_exist,dialog_parent=group_leader)
      long_fileList2 = dialog_pickfile(title=string('Background ', title),$
  	get_path=newDir2, $
        filter=allowed_extensions, $
	/must_exist,dialog_parent=group_leader)
      long_filelist = [long_filelist1,long_fileList2]
      newDir = newDir1	; Limitation: 
       		; Both foreground and background files must be in same directory
    endif else begin
      ; 2 or more files may be selected for 'CONCATENATE', 'ZIP', 'SUM', etc.  
      long_fileList = dialog_pickfile(title=title, /multiple_files, $
  	get_path=newDir, $
        filter=allowed_extensions, $
	/must_exist,dialog_parent=group_leader)
    endelse
  endelse

  nfiles = (strtrim(long_fileList[0],2) eq '') ? 0 : n_elements(long_fileList)
;  nfiles2 = (strtrim(long_fileList2[0],2) eq '') ? 0 : n_elements(long_fileList2)

  if (nfiles le 0) then begin
      message, /info, "No files to initialize"
      ; return control to main DAVE
      widget_control, notifyIDs[0], set_uvalue=sPtr
      reduceSpecObj_cleanup, notifyIDs[0]
    return     ; no need to carry on!
  endif

;  if (nfiles2 le 0) then begin
;      message, /info, "No Background files to initialize. Ignore request."
;      (*sPtr).secondAction = ''
;  endif

  short_fileList = strarr(nfiles)
  extList = strarr(nfiles)
  max_filename_len = 0
  for i = 0, (nfiles-1) do begin
    max_filename_len = max_filename_len > strlen(long_fileList[i]) ; keep track of length of filename
    pos = strpos(long_fileList[i],!delimiter,/reverse_search) + 1
    length = strlen(long_fileList[i]) - pos
    short_fileList[i] = strmid(long_fileList[i],pos,length)
    extStarts  =  strpos(short_fileList[i], '.')
    extList[i] = strmid(short_fileList[i], (extStarts+1))
  endfor

  (*sPtr).instr = extList[0]	; just use the first one

  ;------------------------------------------------------
  if (nfiles EQ 1) then (*sPtr).action = 'SINGLE'	; can't do other options
 							; if only 1 file. 
  ;------------------------------------------------------
  skip_file_selection:

  ;------------------ initialize specObj ------------
  ErrMsg = ''

  if (debug2) then begin
;    help, usingGroupLeader
    print, '~~~~~~  ', infolabel, (*sPtr).action, " usingGroupLeader = ",$
    	usingGroupLeader
  endif

  ;------- Use what is in memory ------------------------
  if ((*sPtr).instr EQ 'IN_MEMORY') then begin
    if (DavePtr_containsSpecObj(DavePtr)) then begin
      message, /info, "SpecObj is already contained in davePtr."
      specObj = davePtr_getSpecObj(davePtr, from='reduceSpySpecObj')
;Debug      help, specobj		; null
;Debug      davePtr_show, davePtr	; OK 

      resultStr = "DavePtr already contains SpecObj"
      message, /info, resultStr
    endif else begin
      ;------------------ initialize specObj ------------
      specObj = obj_new('specObj', $	; create from a DAVE data structure 
    	davePtr = davePtr, $
        ownerDavePtr = 'main', $
  	label='IN_MEMORY', $
	codeSource = 'reduceSpySpecObj', $
	/convert)
        resultStr = "Converted DavePtr to SpecObj"
        message, /info, resultStr
    endelse
    
    goto, dataIsLoaded
  endif 		;; if ((*sPtr).instr EQ 'IN_MEMORY') then begin


  if (debug) then message, /info, string("(*sPtr).instr=", (*sPtr).instr )

  
  ;-- Read in a DAVE file and convert if necessary for backward compatibility --
  if (((*sPtr).instr EQ '*.dave') OR ((*sPtr).instr EQ 'dave')) then begin
   ;------------------------------------------------------
   ; Comment out Sept 15, 2003.
;    ; Empty out existing davePtr only if not using "IN_MEMORY" option.
;    davePtr_empty, davePtr		; Aug 29, 2003
;    heap_free, (*DavePtr)
    ;------------------------------------------------------

    if ( ((*sPtr).action) EQ 'SINGLE') then begin
       specObj = combo_restore(theMultiMethod, 'specObj', $
        usingGroupLeader = usingGroupLeader, $	; Sept 15, 2003. TMR.
;        usingGUI = usingGUI, $			; Sept 9, 2003. TMR.
  	davePtr = davePtr, $
        ownerDavePtr = 'main', $
	codeSource = 'reduceSpecObj', $
	listInstrument=extList[0], $
  	listPath = newDir, $
	listFilename = short_fileList[0], $
	errMsg = ErrMsg)
        resultStr = "Read in DAVE data file: short_fileList"

    endif else begin
  	; Multiple files
     specObj = combo_restore(theMultiMethod, 'specObj', $
        usingGroupLeader = usingGroupLeader, $	; Sept 15, 2003. TMR.
;        usingGUI = usingGUI, $			; Sept 9, 2003. TMR.
   	davePtr = davePtr, $
        ownerDavePtr = 'main', $
	listInstrument=extList, $
  	listPath = newDir, $
	listFilename = short_fileList, $
	depField = ['qtyObj', 'AvTemp', '*'], $	;only used by zip.
	errMsg = ErrMsg)    
        resultStr = "Read in multiple DAVE data files: short_fileList"
    endelse
    goto, dataIsLoaded
  endif		;   if ((*sPtr).instr EQ '*.dave') then begin   

  ;-------- If get here, assume that the file is a raw datafile, e.g. *.bt* --
  if ( ((*sPtr).action) EQ 'SINGLE') then begin
    ; Single file
    specObj = obj_new('specObj', $	; create from a TAS data file 
    	davePtr = davePtr, $
        ownerDavePtr = 'main', $
  	label=short_fileList[0], $
	codeSource = 'reduceSpecObj', $
	instrument=extList[0], $
  	path = newDir, $
	filename = short_fileList[0], $
	datareader='func_icp_data_init', $
	errMsg = ErrMsg)
        resultStr = "Read in TAS data file: short_fileList"
;        message, /info, resultStr

  endif else begin
  	; Multiple files
   specObj = combo_obj_new(theMultiMethod, 'specObj', $
   	davePtr = davePtr, $
        ownerDavePtr = 'main', $
        usingGroupLeader = usingGroupLeader, $	; Sept 15, 2003. TMR.
;        usingGUI = usingGUI, $			; Sept 9, 2003. TMR.
;Unused  	label=theMultiMethod, $
;Unused	codeSource = 'reduceSpecObj', $
	listInstrument=extList, $
  	listPath = newDir, $
	listFilename = short_fileList, $
	datareader='func_icp_data_init', $
	depField = ['qtyObj', 'AvTemp', '*'], $	;only used by zip.
	errMsg = ErrMsg)    
  endelse
  goto, dataIsLoaded
  
  ;--------------------------------------------------------
  dataIsLoaded:
  
  ;--------------------------------------------------------
   if (NOT obj_valid(specObj)) then begin
      message, /info, "Object initialization failed with DavePtr"
      ; return control to main DAVE
     voidID = dialog_message($
    	string('Object initialization failed with DavePtr (empty?).', $
	 	' Please load a Dave save file or reduce raw data.'),$
    	/info, title='reduceSpySpecObj')
      widget_control, notifyIDs[0], set_uvalue=sPtr
      reduceSpecObj_cleanup, notifyIDs[0]
      return
   endif

;  if (NOT obj_valid(specObj)) then begin
;       if (strlen(errmsg) EQ 0) then begin
;          errmsg = string("Failure to initialize data container")
;       endif
;       message, /info, errMsg
;;       davePtr_addTreatment, DavePtr, errMsg
;;       reduceSpecObj_cleanup, notifyIDs[0]
;;       return     ; no need to carry on!
;  endif 
 
  ;------------------ Here's where we call specObj menu ------------
  ; Should gracefully abort if davePtr and specObj are invalid.
  GUIObj2 = obj_new('dialog_ow_menu', specObj,$
  	groupLeader=group_leader,  $
	/no_block, $
  	statePtr = sPtr, cleanup = 'reduceSpecObj_cleanup')   
  
  ;----------------------------------------------------------------

; Never get here. Above object does the cleanup.

end		; reduceSpecObj


