; $Id: report_obj.pro,v 1.2 2003/09/12 15:26:20 tanya Exp $
;+
; NAME:
;       report_obj
;
; PURPOSE:
;
;	This is a generic routine to report information about an object
; to the screen or a file.
; It reports on the structure of the object, its methods and their arguments,
; and upon the contents of the object. 
;
;	Since the object in question is the first argument, this routine
; can be considered a "friend" method for all objects.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Objects. DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED:
;	Requires that the object have the following methods defined:
;		print_obj
;		help_obj
;		show
;
; EXAMPLE
;	...
;
; MODIFICATION HISTORY:
;	Created Spring, 2003. Tanya Riseman
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;*
pro report_obj, $
  thisObj, $			; Object to report on. 
  newfile, $			; String. Output info to this file.
  heapNum = thisHeapNum		; Boolean keyword. 
  				; If set, show the heap number via HELP
;/
; Object thisObj must have the methods print_obj, help_obj and show
 
    ;-------------- write info to a file ----------------------    	
 
;    thisObj = obj_new('qtyObj', name='Lattice Dimensions', $
;	  qty=[4.141, 4.141, 4.141],units='AA', legend = ['a', 'b', 'c'] )
   ;-------------- write info to a file ----------------------    	
;    newfile = 'info_qtyObj.txt'
;    print, "newfile = ", newfile

 if (n_elements(newFile) GT 0) then begin
    openw, localLun, newFile, /get_lun, error = errCode
    IF (errCode NE 0) then begin
      ; the standard error file (logical unit -2):
;      PRINT,  !ERROR_STATE.MSG
      PRINTF, locallun,  !ERROR_STATE.MSG
      return
    endif

 endif else begin
	;Unit Attributes Name
	;-2   Write, Truncate, Tty, Reserved   <stderr>
	;-1   Write, Truncate, Tty, Reserved   <stdout>
	;0   Read, Tty, Reserved              <stdin>
      locallun= -1
      newfile = 'screen'
 endelse
 
;;;;    printf, locallun, 'IDL> DAVE_tagContainer_example, 6'

;    printf, localLun, localLun, '=localLun. output file = ', newFile
;    printf, locallun, '------------------------------------------'
;    printf, locallun
;    printf, locallun, 'thisObj ->print_obj, lun=locallun'
;    printf, locallun
;    thisObj -> print_obj, lun=locallun	
;    printf, locallun, '-------------------------------------'
;    printf, locallun	

    printf, localLun, localLun, '=localLun. output file = ', newFile
    printf, locallun, '------------------------------------------'
    printf, locallun
    printf, locallun, 'thisObj ->print_obj, lun=locallun, /alsoPtrs'
    printf, locallun
    thisObj -> print_obj, lun=locallun, /alsoPtrs	
    printf, locallun, '-------------------------------------'
    printf, locallun	
;    print, 'thisObj ->print_obj, lun=locallun, /alsoPtrs'
;    print, '-------------------------------------'
;    print
        	
        	
    printf, locallun, 'thisObj ->help_obj,lun=locallun
;    printf, locallun, 'thisObj ->help_obj,lun=locallun, /methodsOnly
    printf, locallun
    thisObj -> help_obj,  lun=locallun
;    thisObj -> help_obj,  lun=locallun, /methodsOnly	
    printf, locallun, '-------------------------------------'
    printf, locallun	
;    print, 'thisObj ->help_obj,lun=locallun, /methodsOnly'
;    print, '-------------------------------------'
;    print

;    print, '-------------------------------------'
    printf, locallun, "thisObj -> show, /recursive, lun=locallun, heapNum=",$
    		keyword_set(thisHeapNum)
    printf, locallun
    thisObj -> show, /recursive, lun=locallun, heapNum=thisHeapNum
    printf, locallun, '-------------------------------------'
    printf, locallun	
;    print, 'thisObj -> show, /recursive, lun=locallun'
;    print, '-------------------------------------'

    if (locallun NE -1) then begin
      close, locallun, exit_status=exit_status
      free_lun, locallun
      if (exit_status NE 0) then begin
;        print, infolabel, localLun, 'CLOSE exit_status=', exit_status
        printf, locallun, infolabel, localLun, 'CLOSE exit_status=', exit_status
      endif
    endif
    locallun = 0
        	
    ;-------------- write info to a file ----------------------    	

end
