; $Id: tagobj__define.pro,v 1.3 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
;        tagObj
;
; PURPOSE:
;
;	 tagObj is a "helper" object used by the container class 
; DAVE_tagContainer. In DAVE_tagContainer, items can be referred to by a 
; "tag" string as an alternative to a positition integer.
; 
;	Note: In IDL, classes using inheritance basically construct
; one huge structure including all the data. The inherited methods
; have access to ALL of the data in the structure, not just the parts that
; are relevent to the inherited class. Therefore it is best to use these
; generic code fragements for methods ::print_obj and ::help_obj
; ONLY FOR THE TOP MOST CLASS which has inherited classes.
;
; I have found that it is preferable to make a container class
; DAVE_tagContainer with basically two trees in parallel,
; one for the basic "data" using the class qtyObj and 
; another for I/O and GUI issues using the class tagObj.
; This is specifically to avoid creating a new class which inherits
; qtyObj and tagObj. This has several purposes:
; 	1. Avoid confusion by avoiding mixing the "pure" data and how the
;	   application manages it. 
;	2. Avoid as much as possible the use of EXTRA keywords parameters.
;	   Use the same methods and keywords as much as possible
;	   for different classes instead and use them explicitly.
;	   EXTRA and EXTRA_REF can not be mixed!
;	3. Previously, I had problems with infinite loops in
; 	   GUIs from calling get/set with inherited keywords 
;	   in complex inherited classes. This would occur when a 
;	   typo created too few valid keywords. Then some sub-classes methods
;	   would be called with no valid keywords. 
;
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Objects. DAVE application programming.
;
; OBJECT DEFINTION
;	<A HREF="#TAGOBJ__DEFINE_arguments">TAGOBJ__DEFINE_arguments</A>
;
;Class DAVE_TAGCONTAINER has 7 tags of type: 
;self.(0) =      self.CLASS=     	string
;self.(1) =      self.LABEL=     	string
;self.(2) =      self.CODESOURCE=       string
;self.(3) =      self.HIDEMASK=         int (really Boolean)
;self.(4) =      self.EDITABLEMASK=     int (really Boolean)
;self.(5) =      self.ANNOTATEMASK=     int (really Boolean)
;self.(6) =      self.PENDENT=   	string
;
;	The first three taken together [self.class, self.label, self.codeSource]
; act as a string labels for the contents of the container class
; DAVE_tagContainer. 
;
; 	Self.hideMask is intended to mark the item in DAVE_tagContainer
; as "hidden", so it is not shown in a GUI. Not developed yet.
;
; 	Self.editableMask is intended to mark the item in DAVE_tagContainer
; as "editable", so that the user is allowed to edit it in a GUI. 
; Not developed yet.
;
; 	Self.annotate is intended to mark the item in DAVE_tagContainer
; as "annotation", so that if user or software marks it TRUE, that the 
; correpsonding information in the DAVE_tagContainer
; appears as a footnote or annotation on the bottom of plots, when 
; printed out. Goal: to have enough info on printouts, that 
; the analysis can be reconstructed.
; Not implimented fully yet.
;
; Key methods: 
;	Since this is a "helper" class for DAVE_tagContainer,
; there are no "key" external use methods.
; 
; OTHER FILES AND ROUTINES USED: none?
;
; INCLUDE FILES:
;	<A HREF="#GENERIC__HELCLASS_FRAG.PRO">GENERIC__HELCLASS_FRAG.PRO</A>
;	<A HREF="#GENERIC__HELP_OBJ_FRAG.PRO">GENERIC__HELP_OBJ_FRAG.PRO</A>
;	<A HREF="#GENERIC__PRINT_FRAG.PRO">GENERIC__PRINT_FRAG.PRO</A>
;
; EXAMPLE
;	...
;
; MODIFICATION HISTORY:
;	Tanya Riseman, Spring, 2003. 
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;=============================================================================
;+
; NAME:
; 	tagObj::init
;
; PURPOSE:
;	Initialize the object.
;-
;*
function tagObj::init, $		; Returns 0/1 for failure/success
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $		;; Boolean. /annotate is OK.
	pendent = thisPendent		;; Will be C: constant, D: dependent, I: independent
;/
	
  return, self->set($
  	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $		;; Boolean. /annotate is OK.
	pendent = thisPendent)		;; Will be C: constant, D: dependent, I: independent
		  
end		;; tagObj::init,

	
;=============================================================================
;+
; NAME:
; 	tagObj::clone
;
; PURPOSE:
;	Copy the object into another object
;-
;*
function tagObj::clone		; Returns an object owhcih is a copy of self
;/
	
  success = self->get($
  	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $		;; Boolean. /annotate is OK.
	pendent = thisPendent)		;; Will be C: constant, D: dependent, I: independent
	
  clone = obj_new('tagObj', $
  	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $		;; Boolean. /annotate is OK.
	pendent = thisPendent)		;; Will be C: constant, D: dependent, I: independent

  return, clone	
  	  
end		;; tagObj::clone

	
;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	tagObj::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro tagObj::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $		; string.
  lun=thisLun, $	; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs	 ; Optionally print contents of pointers.
;/
	  @generic__print_frag.pro
  
end		; ::print_obj

;======== HELP method common to all classes ===============================
;+
; NAME:
; 	tagObj::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;-
;*
pro tagObj::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;/
	  @generic__help_obj_frag.pro
end		; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	tagObj::helpclass
;
; PURPOSE:
;	Helper method for tagObj::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro tagObj::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/
	  @generic__helpClass_frag.pro
end		; ::HelpClass

;========= End of methods common to all classes =============================
;=======================================================================
;+
; NAME:
; 	splitTags
;
; PURPOSE:
;	Helper function for tagObj class.
;-
;*
function splitTags, $		; Returns 0/1 for failure/success
	class = thisClass, $		; OUTPUT. string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; OUTPUT. string "Counts"
	codeSource = thisCodeSource, $	; OUTPUT. string "ICP", "PAN"
	tags = thisTags		;; INPUT.string array of class, label, codeSource.
;/
  infolabel = "splitTags >> "  
  ;-------------------------------------------------------------
  nfail = 0
  if NOT arg_present(thisClass) then nfail = nfail + 1
  if NOT arg_present(thisLabel) then nfail = nfail + 1
  if NOT arg_present(thisCodeSource) then nfail = nfail + 1
  nTags = n_elements(thisTags)
  if (nTags EQ 0) then nfail = nfail + 1
  
  if (nfail GT 0) then begin
    print, infolabel, "Missing required INPUT keyword tags or ", $
    	'OUTPUT keywords class, label, and codeSource'
    return, 0
  endif

 ;-------------------------------------------------------------
   if (nTags GT 3) then begin
     print, infolabel, nTags, " > 3. Too many strings in keyword tags"
   endif

     ; default values
     thisClass = ''	
     thisLabel = ''
     thisCodeSource = ''
     if (nTags EQ 1) then begin
       thisClass = thisTags
     endif else begin
       thisClass = thisTags[0]
       if (nTags GE 2) then thisLabel = thisTags[1]
       if (nTags GE 3) then thisCodeSource = thisTags[2]
     endelse
     
     return, 1

end	; splitTags


;=======================================================================
;+
; NAME:
; 	combineTags
;
; PURPOSE:
;	Helper function for tagObj class.
;-
;*
function combineTags, $		; Returns array of 3 strings, e.g. ['', '', '']
	class = thisClass, $		; INPUT. string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; INPUT. string "Counts"
	codeSource = thisCodeSource, $	; INPUT. string "ICP", "PAN"
	tags = thisTags		;; OUTPUT.string array of class, label, codeSource.
;/
  infolabel = "combineTags >> "  

  ;-------------------------------------------------------------
  nfail = 0
  if NOT arg_present(thisTags) then nfail = nfail + 1  
;  if (n_elements(thisClass) EQ 0) then nfail = nfail + 1
;  if (n_elements(thislabel) EQ 0) then nfail = nfail + 1
;  if (n_elements(thisCodeSource) EQ 0) then nfail = nfail + 1
  
  if (nfail GT 0) then begin
    print, infolabel, "Missing required OUTPUT keyword tags'
    return, 0
  endif

  ;-------------------------------------------------------------
  ; default values
  thisTags = ['', '', '']
  if (n_elements(thisClass) GT 0)  then thisTags[0] = thisClass
  if (n_elements(thislabel) GT 0)  then thisTags[1] = thisLabel
  if (n_elements(thisCodeSource) GT 0)  then thisTags[2] = thisCodeSource

end	; tagObj::combineTags

;=============================================================================
;+
; NAME:
; 	tagObj::set
;
; PURPOSE:
;	The usual. 
;-
;*
function tagObj::Set, $			; Returns 0/1 for failure/success
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $		;; Boolean. /annotate is OK.
	pendent = thisPendent		;; Will be C: constant, D: dependent, I: independent
;/

  infolabel = "tagObj::Set >> "
  nsuc = 0
  
 ;-------------------------------------------------------------
 nTags = n_elements(thisTags)
 if (nTags GT 0) then begin
   OK = splitTags(class = thisClass, $
		  label = thisLabel, $
		  codeSource = thisCodeSource, $	
		  tags = thisTags)
   if (NOT OK)  then begin
      print, infolabel, 'Failed splitTags'
     return, 0
   endif
 endif
 ;----------------------------------------
 
  if (n_elements(thisClass) NE 0) then begin
   self.class = thisClass
    nsuc = nsuc + 1
  endif

  if (n_elements(thisLabel) NE 0) then begin
    self.label = thisLabel
    nsuc = nsuc + 1
  endif
  	

  if (n_elements(thisCodeSource) NE 0) then begin
    self.codeSource = thisCodeSource
    nsuc = nsuc + 1
  endif

  if (n_elements(thisHide) NE 0) then begin
    self.hideMask = thisHide
    nsuc = nsuc + 1
  endif

  if (n_elements(thisEditable) NE 0) then begin
    self.editableMask = thisEditable
    nsuc = nsuc + 1
  endif

  if (n_elements(thisAnnotate) NE 0) then begin
    self.annotateMask = thisAnnotate
    nsuc = nsuc + 1
  endif

  if (n_elements(thisPendent) NE 0) then begin
    self.pendent = thisPendent
    nsuc = nsuc + 1
  endif

 success = (nsuc NE 0)
; if (NOT success) then begin
;   print, infolabel, "No recognised keywords"
; endif
 
 return, success

end		;; tagObj::Set
	
;=============================================================================
;+
; NAME:
; 	tagObj::get
;
; PURPOSE:
;	The usual. 
;-
;*
function tagObj::Get, $			; Returns 0/1 for failure/success
  	argPresent = thisArgPresent, $; ARG_PRESENT() function does not work on 
   		;propagated keywords from other calling routines.
		; Needed for get* methods.
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $	;; Boolean. /annotate is OK.
	pendent = thisPendent, $	;; Will be C: constant, D: dependent, I: independent
	everything = thisEverything
;/
	
  infolabel = "tagObj::get >> "  
  nsuc = 0
  
  if (NOT arg_present(thisArgPresent)) then begin   
    ArgPresent = {$			; make this be anonymous structure
	class : arg_present(thisClass), $
	label : arg_present(thisLabel), $	
	codeSource : arg_present(thisCodeSource), $
	tags : arg_present(thisTags), $
	hide : arg_present(thisHide), $
	editable : arg_present(thisEditable), $
	annotate : arg_present(thisAnnotate), $
	pendent : arg_present(thisPendent), $
	everything: arg_present(thisEverything) }
  endif else begin
    ; ArgPresent had better include the appropriate tags.
    ArgPresent = thisArgPresent
  endelse


  ;-------------------------------------------------------
  if (argPresent.everything) then begin
    thisEverything = string('[', self.class, ',', self.label, ',',$
    	 self.codeSource, '] ')
    if (self.annotateMask) then begin
      thisEverything = thisEverything + '* '
    endif 
    if (self.editableMask) then begin
      thisEverything = thisEverything + 'Editable '
    endif 
    if (self.hideMask) then begin
      thisEverything = thisEverything + 'Hidden '
    endif 
    if (self.pendent) then begin
      thisEverything = thisEverything + self.pendent + ' '
    endif 

    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.tags) then begin
    thisTags = [self.class, self.label, self.codeSource]
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.class) then begin
    thisClass = self.class
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.label) then begin
    thisLabel = self.label
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.codeSource) then begin
    thisCodeSource = self.codeSource
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.hide) then begin
    thisHide = self.hideMask
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.editable) then begin
    thisEditable = self.editableMask
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.annotate) then begin
    thisAnnotate= self.annotateMask
    nsuc = nsuc + 1
  endif
  
  ;-------------------------------------------------------
  if (argPresent.pendent) then begin
    thisPendent= self.pendent
    nsuc = nsuc + 1
  endif
  

  ;-----------------------------------------
  success = (nsuc gt 0)

  if (success EQ 0) then begin
;debug     print, infolabel, ' No recognized keywords provided, so return self'
;;;     printArgPresent, ArgPresent  
     return, self
  endif

  return, success 

end		;; tagObj::Get

;=======================================================================
;+
; NAME:
; 	tagObj::show
;
; PURPOSE:
;	The usual. 
;-
;*
pro tagObj::show, $
  	lun=thisLun, $ ; Optionally, write to a file. If absent, to screen
	prefix = thisPrefix, $
	level = thisLevel, $		; how deep in tree
	ignoreHide = thisIgnoreHide, $ 
	showObj = thisShowObj, $	; Boolean Keyword. ignore
	recursive = thisRecursive, $	;; ignore. . End of the tree here.
  	heapNum = thisHeapNum		;  ignore. End of the tree here.
		; This is a leaf-style node, so ignore keywords like
		; "/showObj" and "/recursive". It's the end of the road.
;/
  if ((self.hideMask) AND (NOT keyword_set(thisIgnoreHide))) then begin
    return
  endif
  
  ;--------------------------------------
;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  ;--------------------------------------
  if (n_elements(thisLevel) EQ 0) then begin
    level = 0
  endif else begin
    level = thisLevel
  endelse
  
  ;---------------------------------  
  offset = ''
  for i = 0, level do begin
    offset = offset + '  '		; 2 spaces
  endfor
  
  localPrefix = ''
  if (n_elements(prefix) gt 0) then begin
    localPrefix = thisPrefix
  endif

  if (self.codeSource NE '') then begin
    fromCode = '{' + self.codeSource +  '}'
  endif else begin
    fromCode = ''
  endelse

  if (self.annotateMask) then begin
    annote = "*"
  endif else begin
    annote = " "
  endelse
  
  printf, lun, $
  	 offset, $
  	 localPrefix, $
	 annote, $
	self.pendent, ' ',  $
  	self.class, fromCode,  '.', $
  	self.label
  	
end

;=======================================================================
;+
; NAME:
; 	tagObj::match
;
; PURPOSE:
;	The usual. 
;-
;*
function tagObj::match, $		; Returns 0/1 for no match/match
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags		;; string array of class, label, codeSource.
;/
;;	exact = thisExact		;; BOOLEAN keyword. /exact
	
  infolabel = "tagObj::match >> "  
  debug = 0	;1
  
  nMatch = 0
  nWildcard = 0
  nKeywords = 0	

 ;-------------------------------------------------------------
 nTags = n_elements(thisTags)
 if (nTags GT 0) then begin
   OK = splitTags(class = thisClass, $
		  label = thisLabel, $
		  codeSource = thisCodeSource, $	
		  tags = thisTags)
;   print, infolabel, 'nTags = ', ntags
;   print, infolabel, 'split = ', thisClass, thisLabel, thisCodeSource
 endif
 
 ;-------------------------------------
  if (n_elements(thisClass) GT 0) then begin
    nKeywords = nKeywords + 1
    if (thisClass EQ '*') then begin
      nWildcard = nWildcard + 1
    endif
    if (StrUpCase(thisClass) EQ StrUpCase(self.class)) then begin
      nMatch = nMatch + 1
    endif    
;   if (debug) then print, infolabel, 'Compare = ', thisClass, ', ', self.class
  endif
  
 ;-------------------------------------
  if (n_elements(thisLabel) GT 0) then begin
    nKeywords = nKeywords + 1
    if (thislabel EQ '*')then begin
      nWildcard = nWildcard + 1
    endif
    if (StrUpCase(thisLabel) EQ StrUpCase(self.label)) then begin
      nMatch = nMatch + 1
    endif    
;   if (debug) then print, infolabel, 'Compare = ', thisLabel, ', ', self.label
  endif

 ;-------------------------------------
  if (n_elements(thisCodeSource) GT 0) then begin
    nKeywords = nKeywords + 1
    if (thisCodeSource EQ '*') then begin
      nWildcard = nWildcard + 1
    endif
    if (StrUpCase(thisCodeSource) EQ StrUpCase(self.codeSource)) then begin
      nMatch = nMatch + 1
    endif    
;   if (debug) then print, infolabel, 'Compare = ', thisCodeSource, $
;   		', ', self.CodeSource,', ',  nmatch
  endif
  
  ;------------------------------------
  success = (nKeywords EQ (nMatch + nWildcard))
  if ((nMatch+nWildcard) EQ 0) then success = 0
  
  return, success
  
end	;; tagObj::match

;=======================================================================
;+
; NAME:
; 	tagObj::isHidden
;
; PURPOSE:
;	The usual. 
;-
;*
function tagObj::isHidden		; Returns 0/1 for true/false
;/
  return, self.hideMask
end		;;  tagObj::isHidden

;=======================================================================
;+
; NAME:
; 	tagObj::isEditable	
;
; PURPOSE:
;	The usual. 
;-
;*
function tagObj::isEditable		; Returns 0/1 for true/false
;/
  return, self.editableMask
end		;;  tagObj::isEditable

;=======================================================================
;+
; NAME:
; 	tagObj::isAnnotated
;
; PURPOSE:
;	The usual. 
;-
;*
function tagObj::isAnnotated		; Returns 0/1 for true/false
;/
  return, self.annotateMask
end		;;  tagObj::isAnnotated

;=============================================================================
;+
; NAME:
; 	tagObj::count
;
; PURPOSE:
;	The usual. 
;-
;*
function tagObj::count			; Returns the number of contents (1, trivially)
;/
;; Just for consistency with other classes, esp. DAVE_tagContainer.
   return, 1   
end		; tagObj::count

;=======================================================================
pro tagObj::cleanup
;+
; NAME:
; 	tagObj::cleanup
;
; PURPOSE:
;	The usual. 
;-
  ; Nothing dangerous that needs cleaning up.
end
	
;=======================================================================
;*
pro tagObj__define
  define = {tagObj, $
	class: '', $		; type string. "DVHeader", DVData", "qtyObj"
	label: '', $		; type string. Short name
	codeSource: '', $	; type string. "ICP", "FANS", "PAN", etc.
	hideMask: 0, $		' Integer boolean. Hide from user the portion	
				; in <DAVE_tagContainer>.contents.
	editableMask: 0, $	; Integer boolean. In GUI allowed to modify 
				; the portion in <DAVE_tagContainer>.contents.
	annotateMask: 0,  $	; Integer boolean. Store to ASCII file or print
				; bottom of a plot information marked annotate
	pendent: '' $	;Will be C: constant, D: dependent, I: independent 
	}
;/
;	ptr_publicMethods: ptr_new(), $	; ptr to string array ?
;		; use via <DAVE_tagContainer> -> callMethod
;	ptr_publicFunctions: ptr_new(),$ 	; ptr to string array ?
;		; Use via  <DAVE_tagContainer> ->callFunction
;	ptr_publicProcedures: ptr_new(),$ 	; ptr to string array ?
;		; Use via  <DAVE_tagContainer> ->callProcedure	
;			; stored in the portion	<DAVE_tagContainer>.contents.
;	Question: Put this stuff here <DAVE_tagContainer>.tags or in 
;		<DAVE_tagContainer>.contents????
;;;	ptr_fakeMethods: ptr_new(), $;	string array of faked methods/functions
;;;				; that work on <DAVE_tagContainer>.contents.
;;;	ptr_stash: ptr_new() $	; pointer to an anonyomous structure
;;;				; containing whatever you want. Should not need.	
;;;     ptr_Comments: ptr_new(), $ ; pointer to string array containing 
;;;				; comments which can be included in the annotation. 

end		;  tagObj__define
