; $Id: triasubs_irreduc.pro,v 1.2 2003/09/12 15:24:51 tanya Exp $
;+
; NAME:
;       triasubs_irreduc
;
; PURPOSE:
;
; Based on subroutine irreduc(u,v,uirr,virr) in spurion file calclattice.f
;C  generate irreducible pair of vectors uirr,virr as close
;C  to right angles as possible that are still in the
;C  same plane as u and v
;
; 	Based on Fortran routines used by spurion program.
;	These routines are used by func_icp_header_init and func_icp_data_init.
;
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;	<LI><A HREF="#FUNC_ICP_DATA_INIT">FUNC_ICP_DATA_INIT</A>
;	<LI><A HREF="#FUNC_ICP_HEADER_INIT">FUNC_ICP_HEADER_INIT</A>
;	
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Dec 18, 2002.
;	Split func_icp_subs.pro into separate files. TMR. Aug 1, 2003
;-

;=======================================================================
;*
pro triasubs_irreduc, $
	uin, vin, $		; input vectors
	uirr, virr		; output vectors, which are irreducible.
;/
; Based on subroutine irreduc(u,v,uirr,virr) in spurion file calclattice.f
;C  generate irreducible pair of vectors uirr,virr as close
;C  to right angles as possible that are still in the
;C  same plane as u and v

  debug = 0

;C  first check for any common denominators using division by primes
   uirr = uin
   virr = vin
  ; the prime "factors" by which uirr, virr has been changed
   triasubs_lcd, uirr, factors
   triasubs_lcd, virr, factors

;C what is the possibility that an integer sum of the peaks will yield
;C zero for one of the indices?

   check_again:
   for j = 0, 2    do begin
     nonZero = triasubs_remove_lcd(uirr[j], virr[j], ucoeff, vcoeff)
     if (nonZero) then begin
       diff = ucoeff*uirr + vcoeff*virr		; 3-vector
      	 triasubs_lcd, diff, n
       if (norm(diff) lt norm(virr)) then begin
         virr = diff			; remove the common factor
	 goto, check_again
       endif
     endif		; if (nonZero)
   endfor

end		; triasubs_irreduc
