; $Id: triasubs_transform.pro,v 1.2 2003/09/12 15:24:51 tanya Exp $
;+
; NAME:
;       triasubs_transform
;
; PURPOSE:
;
;;;      subroutine transform(ui,vi,bmat,u3,v3,b1mat) from file transform.f
;
; 	Based on Fortran routines used by spurion program.
;	These routines are used by func_icp_header_init and func_icp_data_init.
;
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;	<LI><A HREF="#FUNC_ICP_DATA_INIT">FUNC_ICP_DATA_INIT</A>
;	<LI><A HREF="#FUNC_ICP_HEADER_INIT">FUNC_ICP_HEADER_INIT</A>
;	
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Dec 18, 2002.
;	Split func_icp_subs.pro into separate files. TMR. Aug 1, 2003
;-


;=======================================================================
;*
pro triasubs_transform, $
	ui,vi,$
	bmat, $
	u3,v3, $
	b1mat
;/

;scatPlane_u, scatPlane_v, bmat, uuv, vuv,  bmatuv
;;;      subroutine transform(ui,vi,bmat,u3,v3,b1mat) from file transform.f

  infolabel = "triasubs_transform>> "  
  debug = 0
  
;C     The desired transformation is to a coordinate system where
;C     x or zeta is perpendicular to uu, vv is a postive rotation
;C     of A3 away from uu, and z is hence parallel to uu x vv.
;C
;C     Currently b[1] is parallel to x and b[2] lies in the x-y
;C     plane because of the generating routine

;C     Generate u and v in the current coordinate system.

;   u = fltarr(3)
;   v = fltarr(3)
   u = ui[0]*bmat[*,0] + ui[1]*bmat[*,1] + ui[2]*bmat[*,2] 
   v = vi[0]*bmat[*,0] + vi[1]*bmat[*,1] + vi[2]*bmat[*,2] 
   if (debug) then begin
     print, infolabel, 'ui=', ui
     print, infolabel, 'vi=', vi
     print, infolabel, 'bmat =', bmat
     print, infolabel, 'u=', u
     print, infolabel, 'v=', v
   endif

;C     What is one of the transformation matrices that will align
;C     u and x?  Will take 2 steps.
;C     Rotation about z-axis with quadrant conventions.

   thetaRad = triasubs_calcThetaRad(u[0], u[1])
   if (debug) then begin
     print, infolabel, 'thetaRad =', thetaRad
   endif
   
;C     -theta appears in the transformation matix because we
;C     want the transformation rotate the coordinate axies
;C     so that u will lie in the x'-z' plane
   tmat = triasubs_rotate(-thetaRad,2)
   b1mat = tmat ## bmat	;Fortran: call mmmult(tmat,bmat,3,3,3,b1mat)
   u1 = tmat ## u		;Fortran: call mvmult(tmat,u,3,3,u1)
   v1 = tmat ## v		;Fortran: call mvmult(tmat,v,3,3,v1)

   if (debug) then begin
     print, infolabel, 'tmat =', tmat
     print, infolabel, 'b1mat=', b1mat
     print, infolabel, 'u1=', u1
     print, infolabel, 'v1=', v1
   endif
      
;C     Rotation about y1-axis
;C     This one is a bit different since with the y1-axis coming
;C     out of the page x1-z1 are left-handed.  Best plan is to
;C     choose theta such that it is correct for the +z1 axis
;C     and then rotate pi/2 further to +x1.

   thetaRad = triasubs_calcThetaRad(u1[2], u1[0])
;   temp = -thetaRad + !pi/2.0
   tmat = triasubs_rotate( (-thetaRad + !pi/2.0), 1)

   b2mat = tmat ## b1mat	;Fortran: call mmmult(tmat,b1mat,3,3,3,b2mat)
   u2 = tmat ## u		;Fortran: call mvmult(tmat,u,3,3,u2)
   v2 = tmat ## v		;Fortran: call mvmult(tmat,v,3,3,v2)

   if (debug) then begin
     print, infolabel, 'thetaRad =', thetaRad
     print, infolabel, 'tmat =', tmat
     print, infolabel, 'b2mat=', b2mat
     print, infolabel, 'u2=', u2
     print, infolabel, 'v2=', v2
   endif

;C     final rotation about x''-axis
   thetaRad = triasubs_calcThetaRad(v2[1], v2[2])
      
   tmat = triasubs_rotate( (-thetaRad), 0)
   b1mat = tmat ## b2mat	;Fortran: call mmmult(tmat,b2mat,3,3,3,b1mat)
   u3 = transpose(tmat ## u2)		;Fortran: call mvmult(tmat,u2,3,3,u3)
   v3 = transpose(tmat ## v2)		;Fortran: call mvmult(tmat,v2,3,3,v3)

   if (debug) then begin
     print, infolabel, 'thetaRad =', thetaRad
     print, infolabel, 'tmat =', tmat
     print, infolabel, 'b1mat=', b1mat
     print, infolabel, 'u3=', u3
     print, infolabel, 'v3=', v3
   endif


end			; triasubs_transform

